@@ -27,15 +27,23 @@ declare -a DEPS_LIST=(
     valgrind
 )
 
+# "Integration tests dependencies satisfied" flag
+declare DEPS_INTGCHECK_SATISFIED=true
+
 if [[ "$DISTRO_BRANCH" == -redhat-* ]]; then
     declare _DEPS_LIST_SPEC
     DEPS_LIST+=(
         clang-analyzer
+        fakeroot
         libcmocka-devel
         mock
+        nss_wrapper
+        openldap-clients
+        openldap-servers
+        pytest
+        python-ldap
         rpm-build
         uid_wrapper
-        nss_wrapper
     )
     _DEPS_LIST_SPEC=`
         sed -e 's/@PACKAGE_VERSION@/0/g' \
@@ -84,19 +92,30 @@ if [[ "$DISTRO_BRANCH" == -debian-* ]]; then
         libselinux1-dev
         libsemanage1-dev
         libsmbclient-dev
-        libsystemd-journal-dev
+        libsystemd-dev
         libtalloc-dev
         libtdb-dev
         libtevent-dev
         libtool
+        libtool-bin
         libxml2-utils
         make
         python-dev
+        python3-dev
         samba-dev
         systemd
         xml-core
         xsltproc
+        libssl-dev
+        fakeroot
+        libnss-wrapper
+        libuid-wrapper
+        python-pytest
+        python-ldap
+        ldap-utils
+        slapd
     )
+    DEPS_INTGCHECK_SATISFIED=true
 fi
 
 declare -a -r DEPS_LIST
