@@ -6,6 +6,8 @@ Created on Sep 18, 2009
 '''
 import unittest
 import os
+import shutil
+import tempfile
 from stat import *
 
 import sys
@@ -19,12 +21,17 @@ else:
 import SSSDConfig
 
 
+def create_temp_dir():
+    test_dir = os.environ.get('SSS_TEST_DIR') or "."
+    return tempfile.mkdtemp(dir=test_dir)
+
+
 class SSSDConfigTestValid(unittest.TestCase):
     def setUp(self):
-        pass
+        self.tmp_dir = create_temp_dir()
 
     def tearDown(self):
-        pass
+        shutil.rmtree(self.tmp_dir)
 
     def testServices(self):
         sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
@@ -132,7 +139,7 @@ class SSSDConfigTestValid(unittest.TestCase):
         local_domain.set_active(True)
         sssdconfig.save_domain(local_domain)
 
-        of = '/tmp/testCreateNewLocalConfig.conf'
+        of = self.tmp_dir + '/testCreateNewLocalConfig.conf'
 
         #Ensure the output file doesn't exist
         try:
@@ -150,10 +157,14 @@ class SSSDConfigTestValid(unittest.TestCase):
         #non-owners, and should not be executable by anyone
         self.assertFalse(S_IMODE(mode) & 0o177)
 
+        # try to import saved configuration file
+        config = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
+                                       srcdir + "/etc/sssd.api.d")
+        config.import_config(configfile=of)
+
         #Remove the output file
         os.unlink(of)
 
-
     def testCreateNewLDAPConfig(self):
         sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
                                 srcdir + "/etc/sssd.api.d")
@@ -166,7 +177,7 @@ class SSSDConfigTestValid(unittest.TestCase):
         ldap_domain.set_active(True)
         sssdconfig.save_domain(ldap_domain)
 
-        of = '/tmp/testCreateNewLDAPConfig.conf'
+        of = self.tmp_dir + '/testCreateNewLDAPConfig.conf'
 
         #Ensure the output file doesn't exist
         try:
@@ -184,9 +195,15 @@ class SSSDConfigTestValid(unittest.TestCase):
         #non-owners, and should not be executable by anyone
         self.assertFalse(S_IMODE(mode) & 0o177)
 
+        # try to import saved configuration file
+        config = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
+                                       srcdir + "/etc/sssd.api.d")
+        config.import_config(configfile=of)
+
         #Remove the output file
         os.unlink(of)
 
+
     def testModifyExistingConfig(self):
         sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
                                 srcdir + "/etc/sssd.api.d")
@@ -200,7 +217,7 @@ class SSSDConfigTestValid(unittest.TestCase):
         ldap_domain.set_active(True)
         sssdconfig.save_domain(ldap_domain)
 
-        of = '/tmp/testModifyExistingConfig.conf'
+        of = self.tmp_dir + '/testModifyExistingConfig.conf'
 
         #Ensure the output file doesn't exist
         try:
@@ -290,7 +307,8 @@ class SSSDConfigTestSSSDService(unittest.TestCase):
             'reconnection_retries',
             'fd_limit',
             'client_idle_timeout',
-            'description']
+            'description',
+            'certificate_verification']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -396,9 +414,6 @@ class SSSDConfigTestSSSDService(unittest.TestCase):
     def testGetOption(self):
         service = SSSDConfig.SSSDService('sssd', self.schema)
 
-        # Positive test - Single-valued
-        self.assertEqual(service.get_option('config_file_version'), 2)
-
         # Positive test - List of values
         self.assertEqual(service.get_option('services'), ['nss', 'pam'])
 
@@ -410,9 +425,7 @@ class SSSDConfigTestSSSDService(unittest.TestCase):
 
         #Positive test
         options = service.get_all_options()
-        control_list = [
-            'config_file_version',
-            'services']
+        control_list = ['services']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -494,6 +507,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'command',
             'enumerate',
             'cache_credentials',
+            'cache_credentials_minimal_first_factor_length',
             'store_legacy_passwords',
             'use_fully_qualified_names',
             'ignore_group_members',
@@ -519,6 +533,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'dyndns_update_ptr',
             'dyndns_force_tcp',
             'dyndns_auth',
+            'dyndns_server',
             'subdomain_enumerate',
             'override_gid',
             'case_sensitive',
@@ -539,7 +554,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'subdomains_provider',
             'realmd_tags',
             'subdomain_refresh_interval',
-            'subdomain_inherit']
+            'subdomain_inherit',
+            'cached_auth_timeout']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -736,7 +752,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         control_provider_dict = {
             'ipa': ['id', 'auth', 'access', 'chpass', 'sudo', 'autofs',
                     'session', 'hostid', 'subdomains'],
-            'ad': ['id', 'auth', 'access', 'chpass', 'sudo', 'subdomains'],
+            'ad': ['id', 'auth', 'access', 'chpass', 'sudo', 'autofs',
+                   'subdomains'],
             'local': ['id', 'auth', 'chpass'],
             'ldap': ['id', 'auth', 'access', 'chpass', 'sudo', 'autofs'],
             'krb5': ['auth', 'access', 'chpass'],
@@ -856,6 +873,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'command',
             'enumerate',
             'cache_credentials',
+            'cache_credentials_minimal_first_factor_length',
             'store_legacy_passwords',
             'use_fully_qualified_names',
             'ignore_group_members',
@@ -881,6 +899,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'dyndns_update_ptr',
             'dyndns_force_tcp',
             'dyndns_auth',
+            'dyndns_server',
             'subdomain_enumerate',
             'override_gid',
             'case_sensitive',
@@ -901,7 +920,8 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'subdomains_provider',
             'realmd_tags',
             'subdomain_refresh_interval',
-            'subdomain_inherit']
+            'subdomain_inherit',
+            'cached_auth_timeout']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -1137,10 +1157,10 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
 
 class SSSDConfigTestSSSDConfig(unittest.TestCase):
     def setUp(self):
-        pass
+        self.tmp_dir = create_temp_dir()
 
     def tearDown(self):
-        pass
+        shutil.rmtree(self.tmp_dir)
 
     def testInit(self):
         # Positive test
@@ -1218,11 +1238,6 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
                                            srcdir + "/etc/sssd.api.d")
         self.assertRaises(SSSDConfig.ParsingError, sssdconfig.import_config, srcdir + "/testconfigs/sssd-badversion.conf")
 
-        # Negative Test - No config file version
-        sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
-                                           srcdir + "/etc/sssd.api.d")
-        self.assertRaises(SSSDConfig.ParsingError, sssdconfig.import_config, srcdir + "/testconfigs/sssd-noversion.conf")
-
         # Negative Test - Already initialized
         sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
                                            srcdir + "/etc/sssd.api.d")
@@ -1230,6 +1245,91 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         self.assertRaises(SSSDConfig.AlreadyInitializedError,
                           sssdconfig.import_config, srcdir + "/testconfigs/sssd-valid.conf")
 
+    def testImportConfigNoVersion(self):
+        # Positive Test
+        sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
+                                           srcdir + "/etc/sssd.api.d")
+        sssdconfig.import_config(
+            srcdir + "/testconfigs/sssd-noversion.conf"
+        )
+
+        # Validate services
+        services = sssdconfig.list_services()
+        self.assertTrue('sssd' in services)
+        self.assertTrue('nss' in services)
+        self.assertTrue('pam' in services)
+        self.assertTrue('dp' in services)
+
+        #Verify service attributes
+        sssd_service = sssdconfig.get_service('sssd')
+        service_opts = sssd_service.list_options()
+
+        self.assertTrue('services' in service_opts.keys())
+        service_list = sssd_service.get_option('services')
+        self.assertTrue('nss' in service_list)
+        self.assertTrue('pam' in service_list)
+        self.assertTrue('reconnection_retries' in service_opts)
+
+        #Validate domain list
+        domains = sssdconfig.list_domains()
+        self.assertTrue('LOCAL' in domains)
+        self.assertTrue('LDAP' in domains)
+        self.assertTrue('PROXY' in domains)
+        self.assertTrue('IPA' in domains)
+
+        # Verify domain attributes
+        ipa_domain = sssdconfig.get_domain('IPA')
+        domain_opts = ipa_domain.list_options()
+        self.assertTrue('debug_level' in domain_opts.keys())
+        self.assertTrue('id_provider' in domain_opts.keys())
+        self.assertTrue('auth_provider' in domain_opts.keys())
+
+        # Verify domain attributes
+        proxy_domain = sssdconfig.get_domain('PROXY')
+        domain_opts = proxy_domain.list_options()
+        self.assertTrue('debug_level' in domain_opts.keys())
+        self.assertTrue('id_provider' in domain_opts.keys())
+        self.assertTrue('auth_provider' in domain_opts.keys())
+
+        # Verify domain attributes
+        local_domain = sssdconfig.get_domain('LOCAL')
+        domain_opts = local_domain.list_options()
+        self.assertTrue('debug_level' in domain_opts.keys())
+        self.assertTrue('id_provider' in domain_opts.keys())
+        self.assertTrue('auth_provider' in domain_opts.keys())
+
+        # Verify domain attributes
+        ldap_domain = sssdconfig.get_domain('LDAP')
+        domain_opts = ldap_domain.list_options()
+        self.assertTrue('debug_level' in domain_opts.keys())
+        self.assertTrue('id_provider' in domain_opts.keys())
+        self.assertTrue('auth_provider' in domain_opts.keys())
+
+        domain_control_list = [
+            'cache_credentials',
+            'id_provider',
+            'auth_provider',
+            'access_provider',
+            'default_shell',
+            'fallback_homedir',
+            'cache_credentials',
+            'use_fully_qualified_names',
+        ]
+
+        ad_domain = sssdconfig.get_domain("ad.example.com")
+
+        for option in ad_domain.get_all_options():
+            self.assertTrue(option in domain_control_list)
+
+        negative_domain_control_list = [
+            'ad_server',
+            'ldap_id_mapping',
+            'ldap_sasl_authid',
+        ]
+
+        for option in ad_domain.get_all_options():
+            self.assertFalse(option in negative_domain_control_list)
+
     def testNewConfig(self):
         # Positive Test
         sssdconfig = SSSDConfig.SSSDConfig(srcdir + "/etc/sssd.api.conf",
@@ -1253,9 +1353,7 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         for section in sssdconfig.sections():
             self.assertTrue(section['name'] in control_list)
 
-        control_list = [
-            'config_file_version',
-            'services']
+        control_list = ['services']
         for option in control_list:
             self.assertTrue(sssdconfig.has_option('sssd', option),
                             "Option [%s] missing from [sssd]" %
@@ -1288,6 +1386,7 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
             'nss',
             'pam']
         active_services = sssdconfig.list_active_services()
+        self.assertTrue(isinstance(active_services, list))
 
         for service in control_list:
             self.assertTrue(service in active_services,
@@ -1510,6 +1609,7 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
             'IPA',
             'LOCAL']
         active_domains = sssdconfig.list_active_domains()
+        self.assertTrue(isinstance(active_domains, list))
 
         for domain in control_list:
             self.assertTrue(domain in active_domains,
@@ -1756,7 +1856,7 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
 
         sssdconfig.save_domain(domain)
 
-        of = '/tmp/testSaveDomain.out'
+        of = self.tmp_dir + '/testSaveDomain.out'
 
         #Ensure the output file doesn't exist
         try:
