@@ -46,8 +46,7 @@ int sysdb_getpwnam(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
-                             SYSDB_TMPL_USER_BASE, domain->name);
+    base_dn = sysdb_user_base_dn(tmp_ctx, domain);
     if (!base_dn) {
         ret = ENOMEM;
         goto done;
@@ -168,8 +167,7 @@ int sysdb_getpwuid(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
-                             SYSDB_TMPL_USER_BASE, domain->name);
+    base_dn = sysdb_user_base_dn(tmp_ctx, domain);
     if (!base_dn) {
         ret = ENOMEM;
         goto done;
@@ -257,12 +255,84 @@ done:
     return ret;
 }
 
-int sysdb_enumpwent(TALLOC_CTX *mem_ctx,
-                    struct sss_domain_info *domain,
-                    struct ldb_result **_res)
+static char *enum_filter(TALLOC_CTX *mem_ctx,
+                         const char *base_filter,
+                         const char *name_filter,
+                         const char *addtl_filter)
+{
+    char *filter;
+    TALLOC_CTX *tmp_ctx = NULL;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        return NULL;
+    }
+
+    if (name_filter == NULL && addtl_filter == NULL) {
+        filter = talloc_strdup(tmp_ctx, base_filter);
+    } else {
+        filter = talloc_asprintf(tmp_ctx, "(&%s", base_filter);
+
+        if (filter != NULL && name_filter != NULL) {
+            filter = talloc_asprintf_append(filter, "(%s=%s)",
+                                            SYSDB_NAME, name_filter);
+        }
+
+        if (filter != NULL && addtl_filter != NULL) {
+            filter = talloc_asprintf_append(filter, "%s", addtl_filter);
+        }
+
+        if (filter != NULL) {
+            filter = talloc_asprintf_append(filter, ")");
+        }
+    }
+
+    if (filter) {
+        talloc_steal(mem_ctx, filter);
+    }
+
+    talloc_free(tmp_ctx);
+    return filter;
+}
+
+int sysdb_getpwupn(TALLOC_CTX *mem_ctx,
+                   struct sss_domain_info *domain,
+                   const char *upn,
+                   struct ldb_result **_res)
+{
+    TALLOC_CTX *tmp_ctx;
+    struct ldb_result *res;
+    static const char *attrs[] = SYSDB_PW_ATTRS;
+    errno_t ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "talloc_new() failed\n");
+        return ENOMEM;
+    }
+
+    ret = sysdb_search_user_by_upn_res(tmp_ctx, domain, upn, attrs, &res);
+    if (ret != EOK && ret != ENOENT) {
+        DEBUG(SSSDBG_OP_FAILURE, "sysdb_search_user_by_upn_res() failed.\n");
+        goto done;
+    }
+
+    *_res = talloc_steal(mem_ctx, res);
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
+int sysdb_enumpwent_filter(TALLOC_CTX *mem_ctx,
+                           struct sss_domain_info *domain,
+                           const char *name_filter,
+                           const char *addtl_filter,
+                           struct ldb_result **_res)
 {
     TALLOC_CTX *tmp_ctx;
     static const char *attrs[] = SYSDB_PW_ATTRS;
+    char *filter = NULL;
     struct ldb_dn *base_dn;
     struct ldb_result *res;
     int ret;
@@ -272,15 +342,22 @@ int sysdb_enumpwent(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
-                             SYSDB_TMPL_USER_BASE, domain->name);
+    base_dn = sysdb_user_base_dn(tmp_ctx, domain);
     if (!base_dn) {
         ret = ENOMEM;
         goto done;
     }
 
+    filter = enum_filter(tmp_ctx, SYSDB_PWENT_FILTER,
+                         name_filter, addtl_filter);
+    if (filter == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+    DEBUG(SSSDBG_TRACE_LIBS, "Searching cache with [%s]\n", filter);
+
     ret = ldb_search(domain->sysdb->ldb, tmp_ctx, &res, base_dn,
-                     LDB_SCOPE_SUBTREE, attrs, SYSDB_PWENT_FILTER);
+                     LDB_SCOPE_SUBTREE, attrs, "%s", filter);
     if (ret) {
         ret = sysdb_error_to_errno(ret);
         goto done;
@@ -293,9 +370,18 @@ done:
     return ret;
 }
 
-int sysdb_enumpwent_with_views(TALLOC_CTX *mem_ctx,
-                               struct sss_domain_info *domain,
-                               struct ldb_result **_res)
+int sysdb_enumpwent(TALLOC_CTX *mem_ctx,
+                    struct sss_domain_info *domain,
+                    struct ldb_result **_res)
+{
+    return sysdb_enumpwent_filter(mem_ctx, domain, NULL, 0, _res);
+}
+
+int sysdb_enumpwent_filter_with_views(TALLOC_CTX *mem_ctx,
+                                      struct sss_domain_info *domain,
+                                      const char *name_filter,
+                                      const char *addtl_filter,
+                                      struct ldb_result **_res)
 {
     TALLOC_CTX *tmp_ctx;
     struct ldb_result *res;
@@ -308,7 +394,7 @@ int sysdb_enumpwent_with_views(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    ret = sysdb_enumpwent(tmp_ctx, domain, &res);
+    ret = sysdb_enumpwent_filter(tmp_ctx, domain, name_filter, addtl_filter, &res);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_enumpwent failed.\n");
         goto done;
@@ -334,6 +420,13 @@ done:
     return ret;
 }
 
+int sysdb_enumpwent_with_views(TALLOC_CTX *mem_ctx,
+                               struct sss_domain_info *domain,
+                               struct ldb_result **_res)
+{
+    return sysdb_enumpwent_filter_with_views(mem_ctx, domain, NULL, NULL, _res);
+}
+
 /* groups */
 
 static int mpg_convert(struct ldb_message *msg)
@@ -418,14 +511,16 @@ int sysdb_getgrnam_with_views(TALLOC_CTX *mem_ctx,
     /* If there are views we have to check if override values must be added to
      * the original object. */
     if (DOM_HAS_VIEWS(domain) && orig_obj->count == 1) {
-        el = ldb_msg_find_element(orig_obj->msgs[0], SYSDB_GHOST);
-        if (el != NULL && el->num_values != 0) {
-            DEBUG(SSSDBG_TRACE_ALL,
-                  "Group object [%s], contains ghost entries which must be " \
-                  "resolved before overrides can be applied.\n",
-                   ldb_dn_get_linearized(orig_obj->msgs[0]->dn));
-            ret = ENOENT;
-            goto done;
+        if (!is_local_view(domain->view_name)) {
+            el = ldb_msg_find_element(orig_obj->msgs[0], SYSDB_GHOST);
+            if (el != NULL && el->num_values != 0) {
+                DEBUG(SSSDBG_TRACE_ALL, "Group object [%s], contains ghost "
+                      "entries which must be resolved before overrides can be "
+                      "applied.\n",
+                      ldb_dn_get_linearized(orig_obj->msgs[0]->dn));
+                ret = ENOENT;
+                goto done;
+            }
         }
 
         ret = sysdb_add_overrides_to_object(domain, orig_obj->msgs[0],
@@ -489,8 +584,7 @@ int sysdb_getgrnam(TALLOC_CTX *mem_ctx,
                                  SYSDB_DOM_BASE, domain->name);
     } else {
         fmt_filter = SYSDB_GRNAM_FILTER;
-        base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
-                                 SYSDB_TMPL_GROUP_BASE, domain->name);
+        base_dn = sysdb_group_base_dn(tmp_ctx, domain);
     }
     if (!base_dn) {
         ret = ENOMEM;
@@ -571,14 +665,16 @@ int sysdb_getgrgid_with_views(TALLOC_CTX *mem_ctx,
     /* If there are views we have to check if override values must be added to
      * the original object. */
     if (DOM_HAS_VIEWS(domain) && orig_obj->count == 1) {
-        el = ldb_msg_find_element(orig_obj->msgs[0], SYSDB_GHOST);
-        if (el != NULL && el->num_values != 0) {
-            DEBUG(SSSDBG_TRACE_ALL,
-                  "Group object [%s], contains ghost entries which must be " \
-                  "resolved before overrides can be applied.\n",
-                   ldb_dn_get_linearized(orig_obj->msgs[0]->dn));
-            ret = ENOENT;
-            goto done;
+        if (!is_local_view(domain->view_name)) {
+            el = ldb_msg_find_element(orig_obj->msgs[0], SYSDB_GHOST);
+            if (el != NULL && el->num_values != 0) {
+                DEBUG(SSSDBG_TRACE_ALL, "Group object [%s], contains ghost "
+                      "entries which must be resolved before overrides can be "
+                      "applied.\n",
+                      ldb_dn_get_linearized(orig_obj->msgs[0]->dn));
+                ret = ENOENT;
+                goto done;
+            }
         }
 
         ret = sysdb_add_overrides_to_object(domain, orig_obj->msgs[0],
@@ -640,8 +736,7 @@ int sysdb_getgrgid(TALLOC_CTX *mem_ctx,
                                  SYSDB_DOM_BASE, domain->name);
     } else {
         fmt_filter = SYSDB_GRGID_FILTER;
-        base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
-                                 SYSDB_TMPL_GROUP_BASE, domain->name);
+        base_dn = sysdb_group_base_dn(tmp_ctx, domain);
     }
     if (!base_dn) {
         ret = ENOMEM;
@@ -667,13 +762,16 @@ done:
     return ret;
 }
 
-int sysdb_enumgrent(TALLOC_CTX *mem_ctx,
-                    struct sss_domain_info *domain,
-                    struct ldb_result **_res)
+int sysdb_enumgrent_filter(TALLOC_CTX *mem_ctx,
+                           struct sss_domain_info *domain,
+                           const char *name_filter,
+                           const char *addtl_filter,
+                           struct ldb_result **_res)
 {
     TALLOC_CTX *tmp_ctx;
     static const char *attrs[] = SYSDB_GRSRC_ATTRS;
-    const char *fmt_filter;
+    const char *filter = NULL;
+    const char *base_filter;
     struct ldb_dn *base_dn;
     struct ldb_result *res;
     int ret;
@@ -684,21 +782,28 @@ int sysdb_enumgrent(TALLOC_CTX *mem_ctx,
     }
 
     if (domain->mpg) {
-        fmt_filter = SYSDB_GRENT_MPG_FILTER;
+        base_filter = SYSDB_GRENT_MPG_FILTER;
         base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
                                  SYSDB_DOM_BASE, domain->name);
     } else {
-        fmt_filter = SYSDB_GRENT_FILTER;
-        base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
-                                 SYSDB_TMPL_GROUP_BASE, domain->name);
+        base_filter = SYSDB_GRENT_FILTER;
+        base_dn = sysdb_group_base_dn(tmp_ctx, domain);
     }
     if (!base_dn) {
         ret = ENOMEM;
         goto done;
     }
 
+    filter = enum_filter(tmp_ctx, base_filter,
+                         name_filter, addtl_filter);
+    if (filter == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+    DEBUG(SSSDBG_TRACE_LIBS, "Searching cache with [%s]\n", filter);
+
     ret = ldb_search(domain->sysdb->ldb, tmp_ctx, &res, base_dn,
-                     LDB_SCOPE_SUBTREE, attrs, "%s", fmt_filter);
+                     LDB_SCOPE_SUBTREE, attrs, "%s", filter);
     if (ret) {
         ret = sysdb_error_to_errno(ret);
         goto done;
@@ -716,9 +821,18 @@ done:
     return ret;
 }
 
-int sysdb_enumgrent_with_views(TALLOC_CTX *mem_ctx,
-                               struct sss_domain_info *domain,
-                               struct ldb_result **_res)
+int sysdb_enumgrent(TALLOC_CTX *mem_ctx,
+                    struct sss_domain_info *domain,
+                    struct ldb_result **_res)
+{
+    return sysdb_enumgrent_filter(mem_ctx, domain, NULL, 0, _res);
+}
+
+int sysdb_enumgrent_filter_with_views(TALLOC_CTX *mem_ctx,
+                                      struct sss_domain_info *domain,
+                                      const char *name_filter,
+                                      const char *addtl_filter,
+                                      struct ldb_result **_res)
 {
     TALLOC_CTX *tmp_ctx;
     struct ldb_result *res;
@@ -731,7 +845,7 @@ int sysdb_enumgrent_with_views(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    ret = sysdb_enumgrent(tmp_ctx, domain,&res);
+    ret = sysdb_enumgrent_filter(tmp_ctx, domain, name_filter, addtl_filter, &res);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_enumgrent failed.\n");
         goto done;
@@ -765,6 +879,13 @@ done:
     return ret;
 }
 
+int sysdb_enumgrent_with_views(TALLOC_CTX *mem_ctx,
+                               struct sss_domain_info *domain,
+                               struct ldb_result **_res)
+{
+    return sysdb_enumgrent_filter_with_views(mem_ctx, domain, NULL, NULL, _res);
+}
+
 int sysdb_initgroups(TALLOC_CTX *mem_ctx,
                      struct sss_domain_info *domain,
                      const char *name,
@@ -865,6 +986,70 @@ done:
     return ret;
 }
 
+int sysdb_initgroups_by_upn(TALLOC_CTX *mem_ctx,
+                             struct sss_domain_info *domain,
+                             const char *upn,
+                             struct ldb_result **_res)
+{
+    TALLOC_CTX *tmp_ctx;
+    struct ldb_message *msg;
+    struct ldb_result *res;
+    const char *sysdb_name;
+    static const char *attrs[] = SYSDB_INITGR_ATTRS;
+    size_t i;
+    errno_t ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "talloc_new() failed\n");
+        return ENOMEM;
+    }
+
+    ret = sysdb_search_user_by_upn(tmp_ctx, domain, upn, attrs, &msg);
+    if (ret != EOK && ret != ENOENT) {
+        DEBUG(SSSDBG_OP_FAILURE, "sysdb_search_user_by_upn() failed.\n");
+        goto done;
+    }
+
+    res = talloc_zero(tmp_ctx, struct ldb_result);
+    if (res == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "talloc_zero() failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    if (ret == ENOENT) {
+        res->count = 0;
+        res->msgs = NULL;
+    } else {
+        sysdb_name = ldb_msg_find_attr_as_string(msg, SYSDB_NAME, NULL);
+        if (sysdb_name == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, "Sysdb entry does not have a name.\n");
+            return EINVAL;
+        }
+
+        ret = sysdb_initgroups(tmp_ctx, domain, sysdb_name, &res);
+        if (ret == EOK && DOM_HAS_VIEWS(domain)) {
+            for (i = 0; i < res->count; i++) {
+                ret = sysdb_add_overrides_to_object(domain, res->msgs[i],
+                                                    NULL, NULL);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_OP_FAILURE,
+                        "sysdb_add_overrides_to_object() failed.\n");
+                    return ret;
+                }
+            }
+        }
+    }
+
+    *_res = talloc_steal(mem_ctx, res);
+    ret = EOK;
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
 int sysdb_initgroups_with_views(TALLOC_CTX *mem_ctx,
                                 struct sss_domain_info *domain,
                                 const char *name,
@@ -998,8 +1183,7 @@ int sysdb_get_user_attr(TALLOC_CTX *mem_ctx,
         return ENOMEM;
     }
 
-    base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
-                             SYSDB_TMPL_USER_BASE, domain->name);
+    base_dn = sysdb_user_base_dn(tmp_ctx, domain);
     if (!base_dn) {
         ret = ENOMEM;
         goto done;
@@ -1530,8 +1714,7 @@ errno_t sysdb_get_direct_parents(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    basedn = ldb_dn_new_fmt(tmp_ctx, sysdb_ctx_get_ldb(dom->sysdb),
-                            SYSDB_TMPL_GROUP_BASE, dom->name);
+    basedn = sysdb_group_base_dn(tmp_ctx, dom);
     if (!basedn) {
         ret = ENOMEM;
         goto done;
@@ -1621,10 +1804,11 @@ errno_t sysdb_get_real_name(TALLOC_CTX *mem_ctx,
                                                   &res);
                 if (ret == EOK && res->count == 1) {
                     msg = res->msgs[0];
-                } else {
+                } else if (ret != ENOENT) {
                     DEBUG(SSSDBG_OP_FAILURE,
-                          "sysdb_search_object_by_uuid did not return a " \
-                          "single result.\n");
+                          "sysdb_search_object_by_uuid failed or returned "
+                          "more than one result [%d][%s].\n",
+                          ret, sss_strerror(ret));
                     ret = ENOENT;
                     goto done;
                 }
