@@ -443,14 +443,12 @@ int sysdb_check_upgrade_02(struct sss_domain_info *domains,
             goto done;
         }
 
-        users_dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb,
-                                 SYSDB_TMPL_USER_BASE, dom->name);
+        users_dn = sysdb_user_base_dn(tmp_ctx, dom);
         if (!users_dn) {
             ret = ENOMEM;
             goto done;
         }
-        groups_dn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb,
-                                   SYSDB_TMPL_GROUP_BASE, dom->name);
+        groups_dn = sysdb_group_base_dn(tmp_ctx, dom);
         if (!groups_dn) {
             ret = ENOMEM;
             goto done;
@@ -1047,8 +1045,7 @@ int sysdb_upgrade_10(struct sysdb_ctx *sysdb, struct sss_domain_info *domain,
         return ret;
     }
 
-    basedn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb,
-                            SYSDB_TMPL_USER_BASE, domain->name);
+    basedn = sysdb_user_base_dn(tmp_ctx, domain);
     if (basedn == NULL) {
         ret = EIO;
         goto done;
@@ -1587,6 +1584,56 @@ done:
     return ret;
 }
 
+int sysdb_upgrade_16(struct sysdb_ctx *sysdb, const char **ver)
+{
+    struct ldb_message *msg;
+    struct upgrade_ctx *ctx;
+    errno_t ret;
+
+    ret = commence_upgrade(sysdb, sysdb->ldb, SYSDB_VERSION_0_17, &ctx);
+    if (ret) {
+        return ret;
+    }
+
+    msg = ldb_msg_new(ctx);
+    if (msg == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    msg->dn = ldb_dn_new(msg, sysdb->ldb, "@INDEXLIST");
+    if (msg->dn == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    /* add index for objectSIDString */
+    ret = ldb_msg_add_empty(msg, "@IDXATTR", LDB_FLAG_MOD_ADD, NULL);
+    if (ret != LDB_SUCCESS) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = ldb_msg_add_string(msg, "@IDXATTR", "objectSIDString");
+    if (ret != LDB_SUCCESS) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = ldb_modify(sysdb->ldb, msg);
+    if (ret != LDB_SUCCESS) {
+        ret = sysdb_error_to_errno(ret);
+        goto done;
+    }
+
+    /* conversion done, update version number */
+    ret = update_version(ctx);
+
+done:
+    ret = finish_upgrade(ret, &ctx, ver);
+    return ret;
+}
+
 /*
  * Example template for future upgrades.
  * Copy and change version numbers as appropriate.
