@@ -341,7 +341,7 @@ static int svc_destructor(void *mem)
         dbus_pending_call_cancel(svc->pending);
     }
 
-    /* svc is beeing freed, neutralize the spy */
+    /* svc is being freed, neutralize the spy */
     if (svc->conn_spy) {
         talloc_set_destructor((TALLOC_CTX *)svc->conn_spy, NULL);
         talloc_zfree(svc->conn_spy);
@@ -805,7 +805,7 @@ static int check_domain_ranges(struct sss_domain_info *domains)
     uint32_t id_min, id_max;
 
     while (dom) {
-        other = get_next_domain(dom, false);
+        other = get_next_domain(dom, 0);
         if (dom->id_max && dom->id_min > dom->id_max) {
             DEBUG(SSSDBG_CRIT_FAILURE,
                   "Domain '%s' does not have a valid ID range\n", dom->name);
@@ -821,9 +821,9 @@ static int check_domain_ranges(struct sss_domain_info *domains)
                       "Domains '%s' and '%s' overlap in range %u - %u\n",
                       dom->name, other->name, id_min, id_max);
             }
-            other = get_next_domain(other, false);
+            other = get_next_domain(other, 0);
         }
-        dom = get_next_domain(dom, false);
+        dom = get_next_domain(dom, 0);
     }
 
     return EOK;
@@ -844,7 +844,7 @@ static int check_local_domain_unique(struct sss_domain_info *domains)
             break;
         }
 
-        dom = get_next_domain(dom, false);
+        dom = get_next_domain(dom, 0);
     }
 
     if (count > 1) {
@@ -2157,13 +2157,13 @@ static int monitor_config_file(TALLOC_CTX *mem_ctx,
         err = errno;
         if (err == ENOENT && ignore_missing) {
             DEBUG(SSSDBG_MINOR_FAILURE,
-                    "file [%s] is missing. Will not update online status "
-                     "based on watching the file\n", file);
+                  "file [%s] is missing. Will not update online status "
+                  "based on watching the file\n", file);
             return EOK;
         } else {
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                 "Could not stat file [%s]. Error [%d:%s]\n",
-                 file, err, strerror(err));
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "Could not stat file [%s]. Error [%d:%s]\n",
+                  file, err, strerror(err));
 
             return err;
         }
@@ -2259,11 +2259,39 @@ static errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
+#define MISSING_RESOLV_CONF_POLL_TIME 10
+
+static void missing_resolv_conf(struct tevent_context *ev,
+                                struct tevent_timer *te,
+                                struct timeval tv, void *data)
+{
+    int ret;
+    struct mt_ctx *ctx = talloc_get_type(data, struct mt_ctx);
+
+    ret = monitor_config_file(ctx, ctx, RESOLV_CONF_PATH,
+                              monitor_update_resolv, false);
+    if (ret == EOK) {
+        signal_res_init(ctx);
+    } else if (ret == ENOENT) {
+        tv = tevent_timeval_current_ofs(MISSING_RESOLV_CONF_POLL_TIME, 0);
+        te = tevent_add_timer(ctx->ev, ctx, tv, missing_resolv_conf, ctx);
+        if (te == NULL) {
+            DEBUG(SSSDBG_FATAL_FAILURE,
+                  "tevent_add_timer failed. resolv.conf will be ignored.\n");
+        }
+    } else {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Monitor_config_file failed. resolv.conf will be ignored.\n");
+    }
+}
+
 static int monitor_process_init(struct mt_ctx *ctx,
                                 const char *config_file)
 {
     TALLOC_CTX *tmp_ctx;
     struct tevent_signal *tes;
+    struct timeval tv;
+    struct tevent_timer *te;
     struct sss_domain_info *dom;
     char *rcachedir;
     int num_providers;
@@ -2351,8 +2379,14 @@ static int monitor_process_init(struct mt_ctx *ctx,
 #endif
     /* Watch for changes to the DNS resolv.conf */
     ret = monitor_config_file(ctx, ctx, RESOLV_CONF_PATH,
-                             monitor_update_resolv,true);
-    if (ret != EOK) {
+                              monitor_update_resolv, false);
+    if (ret == ENOENT) {
+        tv = tevent_timeval_current_ofs(MISSING_RESOLV_CONF_POLL_TIME, 0);
+        te = tevent_add_timer(ctx->ev, ctx, tv, missing_resolv_conf, ctx);
+        if (te == NULL) {
+            DEBUG(SSSDBG_FATAL_FAILURE, "resolv.conf will be ignored\n");
+        }
+    } else if (ret != EOK) {
         return ret;
     }
 
@@ -2390,7 +2424,7 @@ static int monitor_process_init(struct mt_ctx *ctx,
 
     /* start providers */
     num_providers = 0;
-    for (dom = ctx->domains; dom; dom = get_next_domain(dom, false)) {
+    for (dom = ctx->domains; dom; dom = get_next_domain(dom, 0)) {
         ret = add_new_provider(ctx, dom->name, 0);
         if (ret != EOK && ret != ENOENT) {
             return ret;
@@ -2446,7 +2480,6 @@ static void init_timeout(struct tevent_context *ev,
  */
 static int monitor_service_init(struct sbus_connection *conn, void *data)
 {
-    struct sbus_interface *intf;
     struct mt_ctx *ctx;
     struct mon_init_conn *mini;
     struct timeval tv;
@@ -2477,12 +2510,8 @@ static int monitor_service_init(struct sbus_connection *conn, void *data)
         return ENOMEM;
     }
 
-    intf = sbus_new_interface(conn, MON_SRV_PATH, &monitor_methods.vtable, mini);
-    if (!intf) {
-        return ENOMEM;
-    }
-
-    return sbus_conn_add_interface(conn, intf);
+    return sbus_conn_register_iface(conn, &monitor_methods.vtable,
+                                    MON_SRV_PATH, mini);
 }
 
 /* service_send_ping
@@ -2501,7 +2530,7 @@ static int service_send_ping(struct mt_svc *svc)
         return ENXIO;
     }
 
-    DEBUG(SSSDBG_CONF_SETTINGS,"Pinging %s\n", svc->name);
+    DEBUG(SSSDBG_TRACE_INTERNAL, "Pinging %s\n", svc->name);
 
     /*
      * Set up identity request
@@ -2563,7 +2592,8 @@ static void ping_check(DBusPendingCall *pending, void *data)
         /* ok peer replied,
          * make sure we reset the failure counter in the service structure */
 
-        DEBUG(SSSDBG_CONF_SETTINGS,"Service %s replied to ping\n", svc->name);
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              "Service %s replied to ping\n", svc->name);
 
         svc->failed_pongs = 0;
         break;
