@@ -27,6 +27,26 @@ struct ad_access_ctx {
     struct dp_option *ad_options;
     struct sdap_access_ctx *sdap_access_ctx;
     struct ad_id_ctx *ad_id_ctx;
+    /* supported GPO access control modes */
+    enum gpo_access_control_mode {
+        GPO_ACCESS_CONTROL_DISABLED,
+        GPO_ACCESS_CONTROL_PERMISSIVE,
+        GPO_ACCESS_CONTROL_ENFORCING
+    } gpo_access_control_mode;
+    int gpo_cache_timeout;
+    /* supported GPO map options */
+    enum gpo_map_type {
+        GPO_MAP_INTERACTIVE = 0,
+        GPO_MAP_REMOTE_INTERACTIVE,
+        GPO_MAP_NETWORK,
+        GPO_MAP_BATCH,
+        GPO_MAP_SERVICE,
+        GPO_MAP_PERMIT,
+        GPO_MAP_DENY,
+        GPO_MAP_NUM_OPTS
+    } gpo_map_type;
+    hash_table_t *gpo_map_options_table;
+    enum gpo_map_type gpo_default_right;
 };
 
 void
