@@ -30,8 +30,8 @@ struct ad_server_data {
 };
 
 errno_t ad_set_search_bases(struct sdap_options *id_opts);
-static errno_t ad_set_ad_id_options(struct ad_options *ad_opts,
-                                    struct sdap_options *id_opts);
+static errno_t ad_set_sdap_options(struct ad_options *ad_opts,
+                                   struct sdap_options *id_opts);
 
 static struct sdap_options *
 ad_create_default_sdap_options(TALLOC_CTX *mem_ctx)
@@ -108,9 +108,7 @@ fail:
 }
 
 struct ad_options *
-ad_create_default_options(TALLOC_CTX *mem_ctx,
-                          const char *realm,
-                          const char *hostname)
+ad_create_default_options(TALLOC_CTX *mem_ctx)
 {
     struct ad_options *ad_options;
     errno_t ret;
@@ -134,21 +132,110 @@ ad_create_default_options(TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
+    return ad_options;
+}
+
+static errno_t
+set_common_ad_trust_opts(struct ad_options *ad_options,
+                         const char *realm,
+                         const char *ad_domain,
+                         const char *hostname)
+{
+    errno_t ret;
+
     ret = dp_opt_set_string(ad_options->basic, AD_KRB5_REALM, realm);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot set AD krb5 realm\n");
+        return ret;
+    }
+
+    ret = dp_opt_set_string(ad_options->basic, AD_DOMAIN, ad_domain);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Cannot set AD domain\n");
-        talloc_free(ad_options);
-        return NULL;
+        return ret;
     }
 
     ret = dp_opt_set_string(ad_options->basic, AD_HOSTNAME, hostname);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot set AD domain\n");
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot set AD hostname\n");
+        return ret;
+    }
+
+    return EOK;
+}
+
+struct ad_options *
+ad_create_2way_trust_options(TALLOC_CTX *mem_ctx,
+                             const char *realm,
+                             const char *ad_domain,
+                             const char *hostname)
+{
+    struct ad_options *ad_options;
+    errno_t ret;
+
+    ad_options = ad_create_default_options(mem_ctx);
+    if (ad_options == NULL) return NULL;
+
+    ret = set_common_ad_trust_opts(ad_options, realm, ad_domain, hostname);
+    if (ret != EOK) {
         talloc_free(ad_options);
         return NULL;
     }
 
-    ret = ad_set_ad_id_options(ad_options, ad_options->id);
+    ret = ad_set_sdap_options(ad_options, ad_options->id);
+    if (ret != EOK) {
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    return ad_options;
+}
+
+struct ad_options *
+ad_create_1way_trust_options(TALLOC_CTX *mem_ctx,
+                             const char *ad_domain,
+                             const char *hostname,
+                             const char *keytab,
+                             const char *sasl_authid)
+{
+    struct ad_options *ad_options;
+    const char *realm;
+    errno_t ret;
+
+    ad_options = ad_create_default_options(mem_ctx);
+    if (ad_options == NULL) return NULL;
+
+    realm = get_uppercase_realm(ad_options, ad_domain);
+    if (!realm) {
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    ret = set_common_ad_trust_opts(ad_options, realm,
+                                   ad_domain, hostname);
+    if (ret != EOK) {
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    /* Set AD_KEYTAB to the special 1way keytab */
+    ret = dp_opt_set_string(ad_options->basic, AD_KEYTAB, keytab);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot set trust keytab\n");
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    /* Set SDAP_SASL_AUTHID to the trust principal */
+    ret = dp_opt_set_string(ad_options->id->basic,
+                            SDAP_SASL_AUTHID, sasl_authid);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot set SASL authid\n");
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    ret = ad_set_sdap_options(ad_options, ad_options->id);
     if (ret != EOK) {
         talloc_free(ad_options);
         return NULL;
@@ -534,10 +621,6 @@ static int ad_user_data_cmp(void *ud1, void *ud2)
         return sd1 == sd2 ? 0 : 1;
     }
 
-    DEBUG(SSSDBG_TRACE_LIBS, "Comparing %s with %s\n",
-          sd1->gc ? "GC" : "LDAP",
-          sd2->gc ? "GC" : "LDAP");
-
     if (sd1->gc == sd2->gc) {
         return 0;
     }
@@ -831,8 +914,8 @@ done:
 }
 
 static errno_t
-ad_set_ad_id_options(struct ad_options *ad_opts,
-                     struct sdap_options *id_opts)
+ad_set_sdap_options(struct ad_options *ad_opts,
+                    struct sdap_options *id_opts)
 {
     errno_t ret;
     char *krb5_realm;
@@ -910,7 +993,7 @@ ad_get_id_options(struct ad_options *ad_opts,
         return ENOMEM;
     }
 
-    ret = ad_set_ad_id_options(ad_opts, id_opts);
+    ret = ad_set_sdap_options(ad_opts, id_opts);
     if (ret != EOK) {
         talloc_free(id_opts);
         return ret;
@@ -935,6 +1018,37 @@ ad_get_id_options(struct ad_options *ad_opts,
     return EOK;
 }
 
+errno_t
+ad_get_autofs_options(struct ad_options *ad_opts,
+                      struct confdb_ctx *cdb,
+                      const char *conf_path)
+{
+    errno_t ret;
+
+    /* autofs maps */
+    ret = sdap_get_map(ad_opts->id,
+                       cdb,
+                       conf_path,
+                       ad_autofs_mobject_map,
+                       SDAP_OPTS_AUTOFS_MAP,
+                       &ad_opts->id->autofs_mobject_map);
+    if (ret != EOK) {
+        return ret;
+    }
+
+    ret = sdap_get_map(ad_opts->id,
+                       cdb,
+                       conf_path,
+                       ad_autofs_entry_map,
+                       SDAP_OPTS_AUTOFS_ENTRY,
+                       &ad_opts->id->autofs_entry_map);
+    if (ret != EOK) {
+        return ret;
+    }
+
+    return EOK;
+}
+
 errno_t
 ad_set_search_bases(struct sdap_options *id_opts)
 {
@@ -1149,6 +1263,14 @@ ad_get_dom_ldap_conn(struct ad_id_ctx *ad_ctx, struct sss_domain_info *dom)
     subdom_id_ctx = talloc_get_type(sdom->pvt, struct ad_id_ctx);
     conn = subdom_id_ctx->ldap_ctx;
 
+    if (IS_SUBDOMAIN(sdom->dom) == true && conn != NULL) {
+        /* Regardless of connection types, a subdomain error must not be
+         * allowed to set the whole back end offline, rather report an error
+         * and let the caller deal with it (normally disable the subdomain
+         */
+        conn->ignore_mark_offline = true;
+    }
+
     return conn;
 }
 
@@ -1173,3 +1295,52 @@ ad_gc_conn_list(TALLOC_CTX *mem_ctx, struct ad_id_ctx *ad_ctx,
 
     return clist;
 }
+
+struct sdap_id_conn_ctx **
+ad_ldap_conn_list(TALLOC_CTX *mem_ctx,
+                  struct ad_id_ctx *ad_ctx,
+                  struct sss_domain_info *dom)
+{
+    struct sdap_id_conn_ctx **clist;
+
+    clist = talloc_zero_array(mem_ctx, struct sdap_id_conn_ctx *, 2);
+    if (clist == NULL) {
+        return NULL;
+    }
+
+    clist[0] = ad_get_dom_ldap_conn(ad_ctx, dom);
+
+    clist[1] = NULL;
+    return clist;
+}
+
+struct sdap_id_conn_ctx **
+ad_user_conn_list(TALLOC_CTX *mem_ctx,
+                  struct ad_id_ctx *ad_ctx,
+                  struct sss_domain_info *dom)
+{
+    struct sdap_id_conn_ctx **clist;
+    int cindex = 0;
+
+    clist = talloc_zero_array(ad_ctx, struct sdap_id_conn_ctx *, 3);
+    if (clist == NULL) {
+        return NULL;
+    }
+
+    /* Try GC first for users from trusted domains, but go to LDAP
+     * for users from non-trusted domains to get all POSIX attrs
+     */
+    if (dp_opt_get_bool(ad_ctx->ad_options->basic, AD_ENABLE_GC)
+            && IS_SUBDOMAIN(dom)) {
+        clist[cindex] = ad_ctx->gc_ctx;
+        clist[cindex]->ignore_mark_offline = true;
+        cindex++;
+    }
+
+    /* Users from primary domain can be just downloaded from LDAP.
+     * The domain's LDAP connection also works as a fallback
+     */
+    clist[cindex] = ad_get_dom_ldap_conn(ad_ctx, dom);
+
+    return clist;
+}
