@@ -62,6 +62,8 @@ enum ad_basic_opt {
     AD_GPO_DEFAULT_RIGHT,
     AD_SITE,
     AD_KRB5_CONFD_PATH,
+    AD_MAXIMUM_MACHINE_ACCOUNT_PASSWORD_AGE,
+    AD_MACHINE_ACCOUNT_PASSWORD_RENEWAL_OPTS,
 
     AD_OPTS_BASIC /* opts counter */
 };
@@ -103,9 +105,18 @@ ad_get_common_options(TALLOC_CTX *mem_ctx,
                       struct sss_domain_info *dom,
                       struct ad_options **_opts);
 
-struct ad_options *ad_create_default_options(TALLOC_CTX *mem_ctx,
-                                             const char *realm,
-                                             const char *hostname);
+struct ad_options *ad_create_default_options(TALLOC_CTX *mem_ctx);
+
+struct ad_options *ad_create_2way_trust_options(TALLOC_CTX *mem_ctx,
+                                                const char *realm,
+                                                const char *ad_domain,
+                                                const char *hostname);
+
+struct ad_options *ad_create_1way_trust_options(TALLOC_CTX *mem_ctx,
+                                                const char *ad_domain,
+                                                const char *hostname,
+                                                const char *keytab,
+                                                const char *sasl_authid);
 
 errno_t
 ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *ctx,
@@ -123,6 +134,10 @@ ad_get_id_options(struct ad_options *ad_opts,
                    const char *conf_path,
                    struct sdap_options **_opts);
 errno_t
+ad_get_autofs_options(struct ad_options *ad_opts,
+                      struct confdb_ctx *cdb,
+                      const char *conf_path);
+errno_t
 ad_get_auth_options(TALLOC_CTX *mem_ctx,
                     struct ad_options *ad_opts,
                     struct be_ctx *bectx,
@@ -139,6 +154,16 @@ struct sdap_id_conn_ctx **
 ad_gc_conn_list(TALLOC_CTX *mem_ctx, struct ad_id_ctx *ad_ctx,
                struct sss_domain_info *dom);
 
+struct sdap_id_conn_ctx **
+ad_ldap_conn_list(TALLOC_CTX *mem_ctx,
+                  struct ad_id_ctx *ad_ctx,
+                  struct sss_domain_info *dom);
+
+struct sdap_id_conn_ctx **
+ad_user_conn_list(TALLOC_CTX *mem_ctx,
+                  struct ad_id_ctx *ad_ctx,
+                  struct sss_domain_info *dom);
+
 struct sdap_id_conn_ctx *
 ad_get_dom_ldap_conn(struct ad_id_ctx *ad_ctx, struct sss_domain_info *dom);
 
@@ -152,4 +177,12 @@ int ad_sudo_init(struct be_ctx *be_ctx,
                  struct bet_ops **ops,
                  void **pvt_data);
 
+int ad_autofs_init(struct be_ctx *be_ctx,
+                  struct ad_id_ctx *id_ctx,
+                  struct bet_ops **ops,
+                  void **pvt_data);
+
+errno_t ad_machine_account_password_renewal_init(struct be_ctx *be_ctx,
+                                                 struct ad_options *ad_opts);
+
 #endif /* AD_COMMON_H_ */
