@@ -31,19 +31,16 @@
 #include "providers/ldap/sdap.h"
 #include "providers/ldap/sdap_async.h"
 #include "providers/ldap/sdap_idmap.h"
+#include "providers/ad/ad_domain_info.h"
+#include "providers/ad/ad_common.h"
 #include "util/util.h"
 
-#define AD_AT_OBJECT_SID "objectSID"
-#define AD_AT_DNS_DOMAIN "DnsDomain"
-#define AD_AT_NT_VERSION "NtVer"
-#define AD_AT_NETLOGON   "netlogon"
-
-#define MASTER_DOMAIN_SID_FILTER "objectclass=domain"
-
 static errno_t
 netlogon_get_domain_info(TALLOC_CTX *mem_ctx,
                          struct sysdb_attrs *reply,
-                         char **_flat_name, char **_forest)
+                         char **_flat_name,
+                         char **_site,
+                         char **_forest)
 {
     errno_t ret;
     struct ldb_message_element *el;
@@ -52,6 +49,7 @@ netlogon_get_domain_info(TALLOC_CTX *mem_ctx,
     enum ndr_err_code ndr_err;
     struct netlogon_samlogon_response response;
     const char *flat_name;
+    const char *site;
     const char *forest;
 
     ret = sysdb_attrs_get_el(reply, AD_AT_NETLOGON, &el);
@@ -128,6 +126,24 @@ netlogon_get_domain_info(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
+    /* get site name */
+    if (response.data.nt5_ex.client_site != NULL
+        && response.data.nt5_ex.client_site[0] != '\0') {
+        site = response.data.nt5_ex.client_site;
+    } else {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "No netlogon site name data available\n");
+        ret = ENOENT;
+        goto done;
+    }
+
+    *_site = talloc_strdup(mem_ctx, site);
+    if (*_site == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "talloc_strdup failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
     ret = EOK;
 done:
     talloc_free(ndr_pull);
@@ -145,6 +161,7 @@ struct ad_master_domain_state {
     int base_iter;
 
     char *flat;
+    char *site;
     char *forest;
     char *sid;
 };
@@ -361,14 +378,17 @@ ad_master_domain_netlogon_done(struct tevent_req *subreq)
     /* Exactly one flat name. Carry on */
 
     ret = netlogon_get_domain_info(state, reply[0], &state->flat,
-                                   &state->forest);
+                                   &state->site, &state->forest);
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE,
-              "Could not get the flat name or forest\n");
+              "Could not get the flat name or forest: %d:[%s]\n",
+              ret, sss_strerror(ret));
         /* Not fatal. Just quit. */
         goto done;
     }
+
     DEBUG(SSSDBG_TRACE_FUNC, "Found flat name [%s].\n", state->flat);
+    DEBUG(SSSDBG_TRACE_FUNC, "Found site [%s].\n", state->site);
     DEBUG(SSSDBG_TRACE_FUNC, "Found forest [%s].\n", state->forest);
 
 done:
@@ -381,6 +401,7 @@ ad_master_domain_recv(struct tevent_req *req,
                       TALLOC_CTX *mem_ctx,
                       char **_flat,
                       char **_id,
+                      char **_site,
                       char **_forest)
 {
     struct ad_master_domain_state *state = tevent_req_data(req,
@@ -392,6 +413,10 @@ ad_master_domain_recv(struct tevent_req *req,
         *_flat = talloc_steal(mem_ctx, state->flat);
     }
 
+    if (_site) {
+        *_site = talloc_steal(mem_ctx, state->site);
+    }
+
     if (_forest) {
         *_forest = talloc_steal(mem_ctx, state->forest);
     }
