@@ -154,7 +154,7 @@ static void ad_dyndns_nsupdate_done(struct tevent_req *req)
         return;
     }
 
-    DEBUG(SSSDBG_OP_FAILURE, "DNS update finished\n");
+    DEBUG(SSSDBG_TRACE_FUNC, "DNS update finished\n");
 }
 
 struct ad_dyndns_update_state {
@@ -183,7 +183,7 @@ ad_dyndns_update_send(struct ad_options *ctx)
 
     if (ctx->dyndns_ctx->last_refresh + 60 > time(NULL) ||
         ctx->dyndns_ctx->timer_in_progress) {
-        DEBUG(SSSDBG_FUNC_DATA, "Last periodic update ran recently or timer"
+        DEBUG(SSSDBG_FUNC_DATA, "Last periodic update ran recently or timer "
               "in progress, not scheduling another update\n");
         tevent_req_done(req);
         tevent_req_post(req, sdap_ctx->be->ev);
@@ -233,7 +233,6 @@ ad_dyndns_update_send(struct ad_options *ctx)
                                                        DP_OPT_DYNDNS_IFACE),
                                      dp_opt_get_string(ctx->basic,
                                                        AD_HOSTNAME),
-                                     NULL,
                                      dp_opt_get_string(ctx->basic,
                                                        AD_KRB5_REALM),
                                      state->servername,
