@@ -69,7 +69,7 @@ unpack_buffer(uint8_t *buf,
     if (len == 0) {
         return EINVAL;
     } else {
-        if ((p + len ) > size) return EINVAL;
+        if (len > size - p) return EINVAL;
         ibuf->smb_server = talloc_strndup(ibuf, (char *)(buf + p), len);
         if (ibuf->smb_server == NULL) return ENOMEM;
         DEBUG(SSSDBG_TRACE_ALL, "smb_server: %s\n", ibuf->smb_server);
@@ -82,7 +82,7 @@ unpack_buffer(uint8_t *buf,
     if (len == 0) {
         return EINVAL;
     } else {
-        if ((p + len ) > size) return EINVAL;
+        if (len > size - p) return EINVAL;
         ibuf->smb_share = talloc_strndup(ibuf, (char *)(buf + p), len);
         if (ibuf->smb_share == NULL) return ENOMEM;
         DEBUG(SSSDBG_TRACE_ALL, "smb_share: %s\n", ibuf->smb_share);
@@ -95,7 +95,7 @@ unpack_buffer(uint8_t *buf,
     if (len == 0) {
         return EINVAL;
     } else {
-        if ((p + len ) > size) return EINVAL;
+        if (len > size - p) return EINVAL;
         ibuf->smb_path = talloc_strndup(ibuf, (char *)(buf + p), len);
         if (ibuf->smb_path == NULL) return ENOMEM;
         DEBUG(SSSDBG_TRACE_ALL, "smb_path: %s\n", ibuf->smb_path);
@@ -108,7 +108,7 @@ unpack_buffer(uint8_t *buf,
     if (len == 0) {
         return EINVAL;
     } else {
-        if ((p + len ) > size) return EINVAL;
+        if (len > size - p) return EINVAL;
         ibuf->smb_cse_suffix = talloc_strndup(ibuf, (char *)(buf + p), len);
         if (ibuf->smb_cse_suffix == NULL) return ENOMEM;
         DEBUG(SSSDBG_TRACE_ALL, "smb_cse_suffix: %s\n", ibuf->smb_cse_suffix);
@@ -208,6 +208,7 @@ static errno_t prepare_gpo_cache(TALLOC_CTX *mem_ctx,
     char *last = NULL;
     char *smb_path_with_suffix = NULL;
     errno_t ret;
+    mode_t old_umask;
 
     smb_path_with_suffix = talloc_strdup(mem_ctx, input_smb_path_with_suffix);
     if (smb_path_with_suffix == NULL) {
@@ -229,11 +230,13 @@ static errno_t prepare_gpo_cache(TALLOC_CTX *mem_ctx,
 
     ptr = smb_path_with_suffix + 1;
 
+    old_umask = umask(SSS_DFL_X_UMASK);
     for (i = 0; i < num_dirs; i++) {
         first = ptr;
         last = strchr(first, delim);
         if (last == NULL) {
-            return EINVAL;
+            ret = EINVAL;
+            goto done;
         }
         *last = '\0';
         last++;
@@ -241,22 +244,27 @@ static errno_t prepare_gpo_cache(TALLOC_CTX *mem_ctx,
         current_dir = talloc_asprintf(mem_ctx, "%s/%s", current_dir, first);
         if (current_dir == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE, "talloc_asprintf failed.\n");
-            return ENOMEM;
+            ret = ENOMEM;
+            goto done;
         }
         DEBUG(SSSDBG_TRACE_FUNC, "Storing GPOs in %s\n", current_dir);
 
-        if ((mkdir(current_dir, 0644)) < 0 && errno != EEXIST) {
+        if ((mkdir(current_dir, 0700)) < 0 && errno != EEXIST) {
             ret = errno;
             DEBUG(SSSDBG_CRIT_FAILURE,
                   "mkdir(%s) failed: %d\n", current_dir, ret);
-            return ret;
+            goto done;
         }
 
         ptr = last;
     }
 
-    return EOK;
+    ret = EOK;
 
+done:
+    umask(old_umask);
+
+    return ret;
 }
 
 /*
@@ -273,10 +281,10 @@ static errno_t gpo_cache_store_file(const char *smb_path,
                                     int buflen)
 {
     int ret;
+    int fret;
     int fd = -1;
     char *tmp_name = NULL;
     ssize_t written;
-    mode_t old_umask;
     char *filename = NULL;
     char *smb_path_with_suffix = NULL;
     TALLOC_CTX *tmp_ctx = NULL;
@@ -318,13 +326,10 @@ static errno_t gpo_cache_store_file(const char *smb_path,
         goto done;
     }
 
-    old_umask = umask(077);
-    fd = mkstemp(tmp_name);
-    umask(old_umask);
+    fd = sss_unique_file(tmp_ctx, tmp_name, &ret);
     if (fd == -1) {
-        ret = errno;
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "mkstemp failed [%d][%s].\n", ret, strerror(ret));
+              "sss_unique_file failed [%d][%s].\n", ret, strerror(ret));
         goto done;
     }
 
@@ -353,14 +358,6 @@ static errno_t gpo_cache_store_file(const char *smb_path,
         goto done;
     }
 
-    ret = close(fd);
-    if (ret == -1) {
-        ret = errno;
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "close failed [%d][%s].\n", ret, strerror(ret));
-        goto done;
-    }
-
     ret = rename(tmp_name, filename);
     if (ret == -1) {
         ret = errno;
@@ -369,10 +366,19 @@ static errno_t gpo_cache_store_file(const char *smb_path,
         goto done;
     }
 
+    ret = EOK;
  done:
-
     if (ret != EOK) {
-      DEBUG(SSSDBG_CRIT_FAILURE, "Error encountered: %d.\n", ret);
+        DEBUG(SSSDBG_CRIT_FAILURE, "Error encountered: %d.\n", ret);
+    }
+
+    if (fd != -1) {
+        fret = close(fd);
+        if (fret == -1) {
+            fret = errno;
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "close failed [%d][%s].\n", fret, strerror(fret));
+        }
     }
 
     talloc_free(tmp_ctx);
@@ -465,8 +471,27 @@ ad_gpo_parse_ini_file(const char *smb_path,
 
     ret = ini_config_parse(file_ctx, INI_STOP_ON_NONE, 0, 0, ini_config);
     if (ret != 0) {
+        int lret;
+        char **errors;
+
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "ini_config_parse failed [%d][%s]\n", ret, strerror(ret));
+              "[%s]: ini_config_parse failed [%d][%s]\n",
+              ini_filename, ret, strerror(ret));
+
+        /* Now get specific errors if there are any */
+        lret = ini_config_get_errors(ini_config, &errors);
+        if (lret != 0) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Failed to get specific parse error [%d][%s]\n", lret,
+                  strerror(lret));
+            goto done;
+        }
+
+        for (int i = 0; errors[i]; i++) {
+             DEBUG(SSSDBG_CRIT_FAILURE, "%s\n", errors[i]);
+        }
+        ini_config_free_errors(errors);
+
         goto done;
     }
 
@@ -585,7 +610,7 @@ copy_smb_file_to_gpo_cache(SMBCCTX *smbc_ctx,
  * than the cached_gpt_version, thereby triggering a fresh download.
  *
  * Note that the backend will later do the following:
- * - backend will save the the sysvol_gpt_version to sysdb cache
+ * - backend will save the sysvol_gpt_version to sysdb cache
  * - backend will read the policy file from the GPO_CACHE
  */
 static errno_t
