@@ -91,8 +91,50 @@ static int ad_sasl_getopt(void *context, const char *plugin_name,
 
 typedef int (*sss_sasl_gen_cb_fn)(void);
 
+static int map_sasl2sssd_log_level(int sasl_level)
+{
+    int sssd_level;
+
+    switch(sasl_level) {
+    case SASL_LOG_ERR:       /* log unusual errors (default) */
+        sssd_level = SSSDBG_CRIT_FAILURE;
+        break;
+    case SASL_LOG_FAIL:      /* log all authentication failures */
+        sssd_level = SSSDBG_OP_FAILURE;
+        break;
+    case SASL_LOG_WARN:      /* log non-fatal warnings */
+        sssd_level = SSSDBG_MINOR_FAILURE;
+        break;
+    case SASL_LOG_NOTE:      /* more verbose than LOG_WARN */
+    case SASL_LOG_DEBUG:     /* more verbose than LOG_NOTE */
+    case SASL_LOG_TRACE:     /* traces of internal protocols */
+    case SASL_LOG_PASS:      /* traces of internal protocols, including */
+        sssd_level = SSSDBG_TRACE_ALL;
+        break;
+    default:
+        sssd_level = SSSDBG_TRACE_ALL;
+        break;
+    }
+
+    return sssd_level;
+}
+
+int ad_sasl_log(void *context, int level, const char *message)
+{
+    int sssd_level;
+
+    if (level == SASL_LOG_ERR || level == SASL_LOG_FAIL) {
+        sss_log(SSS_LOG_ERR, "%s\n", message);
+    }
+
+    sssd_level = map_sasl2sssd_log_level(level);
+    DEBUG(sssd_level, "SASL: %s\n", message);
+    return SASL_OK;
+}
+
 static const sasl_callback_t ad_sasl_callbacks[] = {
     { SASL_CB_GETOPT, (sss_sasl_gen_cb_fn)ad_sasl_getopt, NULL },
+    { SASL_CB_LOG, (sss_sasl_gen_cb_fn)ad_sasl_log, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 /* This is quite a hack, we *try* to fool openldap libraries by initializing
@@ -266,6 +308,13 @@ sssm_ad_id_init(struct be_ctx *bectx,
               "will not work [%d]: %s\n", ret, strerror(ret));
     }
 
+    ret = ad_machine_account_password_renewal_init(bectx, ad_options);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Cannot setup task for machine account "
+                                   "password renewal.\n");
+        goto done;
+    }
+
     *ops = &ad_id_ops;
     *pvt_data = ad_ctx;
 
@@ -538,3 +587,27 @@ int sssm_ad_sudo_init(struct be_ctx *bectx,
     return EOK;
 #endif
 }
+
+int sssm_ad_autofs_init(struct be_ctx *bectx,
+                        struct bet_ops **ops,
+                        void **pvt_data)
+{
+#ifdef BUILD_AUTOFS
+    struct ad_id_ctx *id_ctx;
+    int ret;
+
+    DEBUG(SSSDBG_TRACE_INTERNAL, "Initializing AD autofs handler\n");
+
+    ret = sssm_ad_id_init(bectx, ops, (void **) &id_ctx);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "sssm_ad_id_init failed.\n");
+        return ret;
+    }
+
+    return ad_autofs_init(bectx, id_ctx, ops, pvt_data);
+#else
+    DEBUG(SSSDBG_MINOR_FAILURE, "Autofs init handler called but SSSD is "
+                                "built without autofs support, ignoring\n");
+    return EOK;
+#endif
+}
