@@ -28,15 +28,14 @@
 #include "util/sss_ldap.h"
 #include "resolv/async_resolv.h"
 #include "providers/dp_backend.h"
+#include "providers/ad/ad_srv.h"
+#include "providers/ad/ad_common.h"
 #include "providers/fail_over.h"
 #include "providers/fail_over_srv.h"
 #include "providers/ldap/sdap.h"
 #include "providers/ldap/sdap_async.h"
 
 #define AD_SITE_DOMAIN_FMT "%s._sites.%s"
-#define AD_AT_DNS_DOMAIN "DnsDomain"
-#define AD_AT_NT_VERSION "NtVer"
-#define AD_AT_NETLOGON "netlogon"
 
 static errno_t ad_sort_servers_by_dns(TALLOC_CTX *mem_ctx,
                                       const char *domain,
@@ -119,7 +118,8 @@ static void ad_get_dc_servers_done(struct tevent_req *subreq);
 static struct tevent_req *ad_get_dc_servers_send(TALLOC_CTX *mem_ctx,
                                                  struct tevent_context *ev,
                                                  struct resolv_ctx *resolv_ctx,
-                                                 const char *domain)
+                                                 const char *discovery_domain,
+                                                 const char *site)
 {
     struct ad_get_dc_servers_state *state = NULL;
     struct tevent_req *req = NULL;
@@ -134,20 +134,38 @@ static struct tevent_req *ad_get_dc_servers_send(TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
-    domains = talloc_zero_array(state, const char *, 2);
+    domains = talloc_zero_array(state, const char *, 3);
     if (domains == NULL) {
         ret = ENOMEM;
         goto immediately;
     }
 
-    domains[0] = talloc_strdup(domains, domain);
-    if (domains[0] == NULL) {
-        ret = ENOMEM;
-        goto immediately;
-    }
+    if (site == NULL) {
+        DEBUG(SSSDBG_TRACE_FUNC, "Looking up domain controllers in domain "
+              "%s\n", discovery_domain);
 
-    DEBUG(SSSDBG_TRACE_FUNC, "Looking up domain controllers in domain %s\n",
-                              domain);
+        domains[0] = talloc_strdup(domains, discovery_domain);
+        if (domains[0] == NULL) {
+            ret = ENOMEM;
+            goto immediately;
+        }
+    } else {
+        DEBUG(SSSDBG_TRACE_FUNC, "Looking up domain controllers in domain "
+              "%s and site %s\n", discovery_domain, site);
+
+        domains[0] = talloc_asprintf(state, AD_SITE_DOMAIN_FMT,
+                                     site, discovery_domain);
+        if (domains[0] == NULL) {
+            ret = ENOMEM;
+            goto immediately;
+        }
+
+        domains[1] = talloc_strdup(domains, discovery_domain);
+        if (domains[1] == NULL) {
+            ret = ENOMEM;
+            goto immediately;
+        }
+    }
 
     subreq = fo_discover_srv_send(state, ev, resolv_ctx,
                                   "ldap", FO_PROTO_TCP, domains);
@@ -177,7 +195,7 @@ static void ad_get_dc_servers_done(struct tevent_req *subreq)
     req = tevent_req_callback_data(subreq, struct tevent_req);
     state = tevent_req_data(req, struct ad_get_dc_servers_state);
 
-    ret = fo_discover_srv_recv(state, subreq, &domain,
+    ret = fo_discover_srv_recv(state, subreq, &domain, NULL,
                                &state->servers, &state->num_servers);
     talloc_zfree(subreq);
     if (ret != EOK) {
@@ -541,8 +559,8 @@ done:
 
 int ad_get_client_site_recv(TALLOC_CTX *mem_ctx,
                             struct tevent_req *req,
-                            char **_site,
-                            char **_forest)
+                            const char **_site,
+                            const char **_forest)
 {
     struct ad_get_client_site_state *state = NULL;
     state = tevent_req_data(req, struct ad_get_client_site_state);
@@ -561,6 +579,7 @@ struct ad_srv_plugin_ctx {
     struct sdap_options *opts;
     const char *hostname;
     const char *ad_domain;
+    const char *ad_site_override;
 };
 
 struct ad_srv_plugin_ctx *
@@ -569,7 +588,8 @@ ad_srv_plugin_ctx_init(TALLOC_CTX *mem_ctx,
                        enum host_database *host_dbs,
                        struct sdap_options *opts,
                        const char *hostname,
-                       const char *ad_domain)
+                       const char *ad_domain,
+                       const char *ad_site_override)
 {
     struct ad_srv_plugin_ctx *ctx = NULL;
 
@@ -592,6 +612,13 @@ ad_srv_plugin_ctx_init(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
+    if (ad_site_override != NULL) {
+        ctx->ad_site_override = talloc_strdup(ctx, ad_site_override);
+        if (ctx->ad_site_override == NULL) {
+            goto fail;
+        }
+    }
+
     return ctx;
 
 fail:
@@ -606,9 +633,10 @@ struct ad_srv_plugin_state {
     const char *protocol;
     const char *discovery_domain;
 
-    char *site;
+    const char *site;
     char *dns_domain;
-    char *forest;
+    uint32_t ttl;
+    const char *forest;
     struct fo_server_info *primary_servers;
     size_t num_primary_servers;
     struct fo_server_info *backup_servers;
@@ -683,7 +711,8 @@ struct tevent_req *ad_srv_plugin_send(TALLOC_CTX *mem_ctx,
     DEBUG(SSSDBG_TRACE_FUNC, "About to find domain controllers\n");
 
     subreq = ad_get_dc_servers_send(state, ev, ctx->be_res->resolv,
-                                    state->discovery_domain);
+                                    state->discovery_domain,
+                                    state->ctx->ad_site_override);
     if (subreq == NULL) {
         ret = ENOMEM;
         goto immediately;
@@ -757,6 +786,23 @@ static void ad_srv_plugin_site_done(struct tevent_req *subreq)
 
     ret = ad_get_client_site_recv(state, subreq, &state->site, &state->forest);
     talloc_zfree(subreq);
+    /* Ignore AD site found by dns discovery if specific site is set in
+     * configuration file. */
+    if (state->ctx->ad_site_override != NULL) {
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              "Ignoring AD site found by DNS discovery: '%s', "
+              "using configured value: '%s' instead.\n",
+              state->site, state->ctx->ad_site_override);
+        state->site = state->ctx->ad_site_override;
+
+        if (state->forest == NULL) {
+            DEBUG(SSSDBG_TRACE_FUNC, "Missing forest information, using %s\n",
+                  state->discovery_domain);
+            state->forest = state->discovery_domain;
+        }
+
+        ret = EOK;
+    }
     if (ret == EOK) {
         if (strcmp(state->service, "gc") == 0) {
             primary_domain = talloc_asprintf(state, AD_SITE_DOMAIN_FMT,
@@ -820,6 +866,7 @@ static void ad_srv_plugin_servers_done(struct tevent_req *subreq)
     state = tevent_req_data(req, struct ad_srv_plugin_state);
 
     ret = fo_discover_servers_recv(state, subreq, &state->dns_domain,
+                                   &state->ttl,
                                    &state->primary_servers,
                                    &state->num_primary_servers,
                                    &state->backup_servers,
@@ -857,6 +904,7 @@ static void ad_srv_plugin_servers_done(struct tevent_req *subreq)
 errno_t ad_srv_plugin_recv(TALLOC_CTX *mem_ctx,
                            struct tevent_req *req,
                            char **_dns_domain,
+                           uint32_t *_ttl,
                            struct fo_server_info **_primary_servers,
                            size_t *_num_primary_servers,
                            struct fo_server_info **_backup_servers,
@@ -887,6 +935,9 @@ errno_t ad_srv_plugin_recv(TALLOC_CTX *mem_ctx,
         *_dns_domain = talloc_steal(mem_ctx, state->dns_domain);
     }
 
+    if (_ttl) {
+        *_ttl = state->ttl;
+    }
 
     return EOK;
 }
