@@ -43,6 +43,7 @@
 #include "sbus/sbus_client.h"
 #include "sss_client/sss_cli.h"
 #include "util/authtok.h"
+#include "providers/data_provider_req.h"
 #include "providers/data_provider_iface_generated.h"
 
 #define DATA_PROVIDER_VERSION 0x0001
@@ -128,26 +129,18 @@
 #define BE_FILTER_ENUM 3
 #define BE_FILTER_SECID 4
 #define BE_FILTER_UUID 5
-
-#define BE_REQ_USER          0x0001
-#define BE_REQ_GROUP         0x0002
-#define BE_REQ_INITGROUPS    0x0003
-#define BE_REQ_NETGROUP      0x0004
-#define BE_REQ_SERVICES      0x0005
-#define BE_REQ_SUDO_FULL     0x0006
-#define BE_REQ_SUDO_RULES    0x0007
-#define BE_REQ_AUTOFS        0x0009
-#define BE_REQ_HOST          0x0010
-#define BE_REQ_BY_SECID      0x0011
-#define BE_REQ_USER_AND_GROUP 0x0012
-#define BE_REQ_BY_UUID      0x0013
-#define BE_REQ_TYPE_MASK     0x00FF
-#define BE_REQ_FAST          0x1000
+#define BE_FILTER_CERT 6
+#define BE_FILTER_WILDCARD 7
 
 #define DP_SEC_ID "secid"
+#define DP_CERT "cert"
 /* sizeof() counts the trailing \0 so we must substract 1 for the string
  * length */
 #define DP_SEC_ID_LEN (sizeof(DP_SEC_ID) - 1)
+#define DP_CERT_LEN (sizeof(DP_CERT) - 1)
+
+#define DP_WILDCARD "wildcard"
+#define DP_WILDCARD_LEN (sizeof(DP_WILDCARD) - 1)
 
 #define EXTRA_NAME_IS_UPN "U"
 #define EXTRA_INPUT_MAYBE_WITH_VIEW "V"
@@ -189,6 +182,8 @@ struct pam_data {
     bool offline_auth;
     bool last_auth_saved;
     int priv;
+    int account_locked;
+
 #ifdef USE_KEYRING
     key_serial_t key_serial;
 #endif
