@@ -69,16 +69,6 @@ struct loaded_be {
     void *handle;
 };
 
-struct bet_queue_item {
-    struct bet_queue_item *prev;
-    struct bet_queue_item *next;
-
-    TALLOC_CTX *mem_ctx;
-    struct be_req *be_req;
-    be_req_fn_t fn;
-
-};
-
 struct bet_info {
     enum bet_type bet_type;
     struct bet_ops *bet_ops;
@@ -116,6 +106,8 @@ struct be_ctx {
     struct sss_domain_info *domain;
     const char *identity;
     const char *conf_path;
+    uid_t uid;
+    gid_t gid;
     struct be_failover_ctx *be_fo;
     struct be_resolv_ctx *be_res;
 
@@ -197,6 +189,7 @@ struct be_host_req {
 
 bool be_is_offline(struct be_ctx *ctx);
 void be_mark_offline(struct be_ctx *ctx);
+void be_mark_dom_offline(struct sss_domain_info *dom, struct be_ctx *ctx);
 
 int be_add_reconnect_cb(TALLOC_CTX *mem_ctx,
                         struct be_ctx *ctx,
@@ -265,12 +258,21 @@ struct tevent_req *be_resolve_server_send(TALLOC_CTX *memctx,
                                           struct be_ctx *ctx,
                                           const char *service_name,
                                           bool first_try);
-int be_resolve_server_recv(struct tevent_req *req, struct fo_server **srv);
+int be_resolve_server_recv(struct tevent_req *req,
+                           TALLOC_CTX *ref_ctx,
+                           struct fo_server **srv);
 
-void be_fo_set_port_status(struct be_ctx *ctx,
-                           const char *service_name,
-                           struct fo_server *server,
-                           enum port_status status);
+#define be_fo_set_port_status(ctx, service_name, server, status) \
+    _be_fo_set_port_status(ctx, service_name, server, status, \
+                           __LINE__, __FILE__, __FUNCTION__)
+
+void _be_fo_set_port_status(struct be_ctx *ctx,
+                            const char *service_name,
+                            struct fo_server *server,
+                            enum port_status status,
+                            int line,
+                            const char *file,
+                            const char *function);
 
 /*
  * Instruct fail-over to try next server on the next connect attempt.
@@ -283,6 +285,10 @@ int be_fo_run_callbacks_at_next_request(struct be_ctx *ctx,
                                         const char *service_name);
 
 void reset_fo(struct be_ctx *be_ctx);
+void be_fo_reset_svc(struct be_ctx *be_ctx, const char *svc_name);
+
+const char *be_fo_get_active_server_name(struct be_ctx *ctx,
+                                         const char *service_name);
 
 errno_t be_res_init(struct be_ctx *ctx);
 
