@@ -79,6 +79,8 @@ struct fo_service {
 };
 
 struct fo_server {
+    REFCOUNT_COMMON;
+
     struct fo_server *prev;
     struct fo_server *next;
 
@@ -90,6 +92,8 @@ struct fo_server {
     struct fo_service *service;
     struct timeval last_status_change;
     struct server_common *common;
+
+    TALLOC_CTX *fo_internal_owner;
 };
 
 struct server_common {
@@ -117,6 +121,7 @@ struct srv_data {
     struct fo_server *meta;
 
     int srv_lookup_status;
+    int ttl;
     struct timeval last_status_change;
 };
 
@@ -126,6 +131,7 @@ struct resolve_service_request {
 
     struct server_common *server_common;
     struct tevent_req *req;
+    struct tevent_context *ev;
 };
 
 struct status {
@@ -149,7 +155,7 @@ fo_context_init(TALLOC_CTX *mem_ctx, struct fo_options *opts)
         return NULL;
     }
 
-    ctx->opts->srv_retry_timeout = opts->srv_retry_timeout;
+    ctx->opts->srv_retry_neg_timeout = opts->srv_retry_neg_timeout;
     ctx->opts->retry_timeout = opts->retry_timeout;
     ctx->opts->family_order  = opts->family_order;
     ctx->opts->service_resolv_timeout = opts->service_resolv_timeout;
@@ -216,6 +222,15 @@ int fo_is_srv_lookup(struct fo_server *s)
     return s && s->srv_data;
 }
 
+static void fo_server_free(struct fo_server *server)
+{
+    if (server == NULL) {
+        return;
+    }
+
+    talloc_free(server->fo_internal_owner);
+}
+
 static struct fo_server *
 collapse_srv_lookup(struct fo_server **_server)
 {
@@ -230,12 +245,12 @@ collapse_srv_lookup(struct fo_server **_server)
         while (server->prev && server->prev->srv_data == meta->srv_data) {
             tmp = server->prev;
             DLIST_REMOVE(server->service->server_list, tmp);
-            talloc_zfree(tmp);
+            fo_server_free(tmp);
         }
         while (server->next && server->next->srv_data == meta->srv_data) {
             tmp = server->next;
             DLIST_REMOVE(server->service->server_list, tmp);
-            talloc_zfree(tmp);
+            fo_server_free(tmp);
         }
 
         if (server == server->service->active_server) {
@@ -248,7 +263,7 @@ collapse_srv_lookup(struct fo_server **_server)
         /* add back the meta server to denote SRV lookup */
         DLIST_ADD_AFTER(server->service->server_list, meta, server);
         DLIST_REMOVE(server->service->server_list, server);
-        talloc_zfree(server);
+        fo_server_free(server);
     }
 
     meta->srv_data->srv_lookup_status = SRV_NEUTRAL;
@@ -265,10 +280,16 @@ get_srv_data_status(struct srv_data *data)
     struct timeval tv;
     time_t timeout;
 
-    timeout = data->meta->service->ctx->opts->srv_retry_timeout;
     gettimeofday(&tv, NULL);
 
-    if (timeout && STATUS_DIFF(data, tv) > timeout) {
+    /* Determine timeout value based on state of previous lookup. */
+    if (data->srv_lookup_status == SRV_RESOLVE_ERROR) {
+        timeout = data->meta->service->ctx->opts->srv_retry_neg_timeout;
+    } else {
+        timeout = data->ttl;
+    }
+
+    if (STATUS_DIFF(data, tv) > timeout) {
         switch(data->srv_lookup_status) {
         case SRV_EXPIRED:
         case SRV_NEUTRAL:
@@ -280,6 +301,9 @@ get_srv_data_status(struct srv_data *data)
         case SRV_RESOLVE_ERROR:
             data->srv_lookup_status = SRV_NEUTRAL;
             data->last_status_change.tv_sec = 0;
+            DEBUG(SSSDBG_TRACE_FUNC,
+                  "Changing state of SRV lookup from 'SRV_RESOLVE_ERROR' to "
+                  "'SRV_NEUTRAL'.\n");
             break;
         default:
             DEBUG(SSSDBG_CRIT_FAILURE, "Unknown state for SRV server!\n");
@@ -492,12 +516,12 @@ create_server_common(TALLOC_CTX *mem_ctx, struct fo_ctx *ctx, const char *name)
     struct server_common *common;
 
     common = rc_alloc(mem_ctx, struct server_common);
-    if (common == NULL)
+    if (common == NULL) {
         return NULL;
+    }
 
     common->name = talloc_strdup(common, name);
     if (common->name == NULL) {
-        talloc_free(common);
         return NULL;
     }
 
@@ -515,6 +539,41 @@ create_server_common(TALLOC_CTX *mem_ctx, struct fo_ctx *ctx, const char *name)
     return common;
 }
 
+static struct fo_server *
+fo_server_alloc(struct fo_service *service, int port,
+                void *user_data, bool primary)
+{
+    static struct fo_server *server;
+    TALLOC_CTX *server_owner;
+
+    server_owner = talloc_new(service);
+    if (server_owner == NULL) {
+        return NULL;
+    }
+
+    server = rc_alloc(server_owner, struct fo_server);
+    if (server == NULL) {
+        return NULL;
+    }
+
+    server->fo_internal_owner = server_owner;
+
+    server->common = NULL;
+    server->next = NULL;
+    server->prev = NULL;
+    server->srv_data = NULL;
+    server->last_status_change.tv_sec = 0;
+    server->last_status_change.tv_usec = 0;
+
+    server->port = port;
+    server->user_data = user_data;
+    server->service = service;
+    server->port_status = DEFAULT_PORT_STATUS;
+    server->primary = primary;
+
+    return server;
+}
+
 int
 fo_add_srv_server(struct fo_service *service, const char *srv,
                   const char *discovery_domain, const char *sssd_domain,
@@ -548,14 +607,11 @@ fo_add_srv_server(struct fo_service *service, const char *srv,
         }
     }
 
-    server = talloc_zero(service, struct fo_server);
-    if (server == NULL)
+    /* SRV servers are always primary */
+    server = fo_server_alloc(service, 0, user_data, true);
+    if (server == NULL) {
         return ENOMEM;
-
-    server->user_data = user_data;
-    server->service = service;
-    server->port_status = DEFAULT_PORT_STATUS;
-    server->primary = true; /* SRV servers are never back up */
+    }
 
     /* add the SRV-specific data */
     server->srv_data = talloc_zero(service, struct srv_data);
@@ -599,7 +655,7 @@ create_fo_server(struct fo_service *service, const char *name,
     struct fo_server *server;
     int ret;
 
-    server = talloc_zero(service, struct fo_server);
+    server = fo_server_alloc(service, port, user_data, primary);
     if (server == NULL)
         return NULL;
 
@@ -614,11 +670,11 @@ create_fo_server(struct fo_service *service, const char *name,
         if (ret == ENOENT) {
             server->common = create_server_common(server, service->ctx, name);
             if (server->common == NULL) {
-                talloc_free(server);
+                fo_server_free(server);
                 return NULL;
             }
         } else if (ret != EOK) {
-            talloc_free(server);
+            fo_server_free(server);
             return NULL;
         }
     }
@@ -751,7 +807,6 @@ static errno_t fo_add_server_list(struct fo_service *service,
         server = create_fo_server(service, servers[i].host, servers[i].port,
                                   user_data, primary);
         if (server == NULL) {
-            talloc_free(srv_list);
             return ENOMEM;
         }
 
@@ -760,7 +815,7 @@ static errno_t fo_add_server_list(struct fo_service *service,
         ret = fo_add_server_to_list(&srv_list, service->server_list,
                                     server, service->name);
         if (ret != EOK) {
-            talloc_zfree(server);
+            fo_server_free(server);
             continue;
         }
 
@@ -794,12 +849,20 @@ fo_add_server(struct fo_service *service, const char *name, int port,
     ret = fo_add_server_to_list(&service->server_list, service->server_list,
                                 server, service->name);
     if (ret != EOK) {
-        talloc_free(server);
+        fo_server_free(server);
     }
 
     return ret;
 }
 
+void fo_ref_server(TALLOC_CTX *ref_ctx,
+                   struct fo_server *server)
+{
+    if (server) {
+        server = rc_reference(ref_ctx, struct fo_server, server);
+    }
+}
+
 static int
 get_first_server_entity(struct fo_service *service, struct fo_server **_server)
 {
@@ -878,7 +941,9 @@ resolve_service_request_destructor(struct resolve_service_request *request)
 }
 
 static int
-set_lookup_hook(struct fo_server *server, struct tevent_req *req)
+set_lookup_hook(struct tevent_context *ev,
+                struct fo_server *server,
+                struct tevent_req *req)
 {
     struct resolve_service_request *request;
 
@@ -894,6 +959,7 @@ set_lookup_hook(struct fo_server *server, struct tevent_req *req)
         talloc_free(request);
         return ENOMEM;
     }
+    request->ev = ev;
     request->req = req;
     DLIST_ADD(server->common->request_list, request);
     talloc_set_destructor(request, resolve_service_request_destructor);
@@ -1022,7 +1088,7 @@ fo_resolve_service_activate_timeout(struct tevent_req *req,
     tv = tevent_timeval_current();
     tv = tevent_timeval_add(&tv, timeout_seconds, 0);
     state->timeout_handler = tevent_add_timer(ev, state, tv,
-                                           fo_resolve_service_timeout, req);
+                                              fo_resolve_service_timeout, req);
     if (state->timeout_handler == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "tevent_add_timer failed.\n");
         return ENOMEM;
@@ -1080,7 +1146,7 @@ fo_resolve_service_server(struct tevent_req *req)
     case SERVER_RESOLVING_NAME:
         /* Name resolution is already under way. Just add ourselves into the
          * waiting queue so we get notified after the operation is finished. */
-        ret = set_lookup_hook(state->server, req);
+        ret = set_lookup_hook(state->ev, state->server, req);
         if (ret != EOK) {
             tevent_req_error(req, ret);
             return true;
@@ -1132,6 +1198,13 @@ fo_resolve_service_done(struct tevent_req *subreq)
     /* Take care of all requests for this server. */
     while ((request = common->request_list) != NULL) {
         DLIST_REMOVE(common->request_list, request);
+
+        /* If the request callback decresed refcount on the returned
+         * server, we would have crashed as common would not be valid
+         * anymore. Rather schedule the notify for next tev iteration
+         */
+        tevent_req_defer_callback(request->req, request->ev);
+
         if (ret) {
             tevent_req_error(request->req, ret);
         } else {
@@ -1141,7 +1214,9 @@ fo_resolve_service_done(struct tevent_req *subreq)
 }
 
 int
-fo_resolve_service_recv(struct tevent_req *req, struct fo_server **server)
+fo_resolve_service_recv(struct tevent_req *req,
+                        TALLOC_CTX *ref_ctx,
+                        struct fo_server **server)
 {
     struct resolve_service_state *state;
 
@@ -1149,8 +1224,10 @@ fo_resolve_service_recv(struct tevent_req *req, struct fo_server **server)
 
     /* always return the server if asked for, otherwise the caller
      * cannot mark it as faulty in case we return an error */
-    if (server)
+    if (server != NULL) {
+        fo_ref_server(ref_ctx, state->server);
         *server = state->server;
+    }
 
     TEVENT_REQ_RETURN_ON_ERROR(req);
 
@@ -1275,8 +1352,9 @@ resolve_srv_done(struct tevent_req *subreq)
     size_t num_backup_servers = 0;
     char *dns_domain = NULL;
     int ret;
+    uint32_t ttl;
 
-    ret = state->fo_ctx->srv_recv_fn(state, subreq, &dns_domain,
+    ret = state->fo_ctx->srv_recv_fn(state, subreq, &dns_domain, &ttl,
                                      &primary_servers, &num_primary_servers,
                                      &backup_servers, &num_backup_servers);
     talloc_free(subreq);
@@ -1290,6 +1368,7 @@ resolve_srv_done(struct tevent_req *subreq)
             goto done;
         }
 
+        state->meta->srv_data->ttl = ttl;
         talloc_zfree(state->meta->srv_data->dns_domain);
         state->meta->srv_data->dns_domain = talloc_steal(state->meta->srv_data,
                                                          dns_domain);
@@ -1389,11 +1468,6 @@ resolve_srv_recv(struct tevent_req *req, struct fo_server **server)
 /*******************************************************************
  *     Get Fully Qualified Domain Name of the host machine         *
  *******************************************************************/
-struct resolve_get_domain_state {
-    char *fqdn;
-    char hostname[HOST_NAME_MAX];
-};
-
 static void
 set_server_common_status(struct server_common *common,
                          enum server_status status)
@@ -1450,6 +1524,11 @@ fo_set_port_status(struct fo_server *server, enum port_status status)
     }
 }
 
+struct fo_server *fo_get_active_server(struct fo_service *service)
+{
+    return service->active_server;
+}
+
 void fo_try_next_server(struct fo_service *service)
 {
     struct fo_server *server;
@@ -1540,21 +1619,33 @@ time_t fo_get_service_retry_timeout(struct fo_service *svc)
     return svc->ctx->opts->retry_timeout;
 }
 
+void fo_reset_servers(struct fo_service *service)
+{
+    struct fo_server *server;
+
+    DLIST_FOR_EACH(server, service->server_list) {
+        if (server->srv_data != NULL) {
+            set_srv_data_status(server->srv_data, SRV_NEUTRAL);
+        }
+
+        if (server->common) {
+            fo_set_server_status(server, SERVER_NAME_NOT_RESOLVED);
+        }
+
+        fo_set_port_status(server, PORT_NEUTRAL);
+    }
+}
+
+
 void fo_reset_services(struct fo_ctx *fo_ctx)
 {
     struct fo_service *service;
-    struct fo_server *server;
+
+    DEBUG(SSSDBG_TRACE_LIBS,
+          "Resetting all servers in all services\n");
 
     DLIST_FOR_EACH(service, fo_ctx->service_list) {
-        DLIST_FOR_EACH(server, service->server_list) {
-            if (server->srv_data != NULL) {
-                set_srv_data_status(server->srv_data, SRV_NEUTRAL);
-            } else {
-                fo_set_server_status(server, SERVER_NAME_NOT_RESOLVED);
-            }
-
-            fo_set_port_status(server, PORT_NEUTRAL);
-        }
+        fo_reset_servers(service);
     }
 }
 
