@@ -34,8 +34,6 @@
 #define FO_PROTO_TCP "tcp"
 #define FO_PROTO_UDP "udp"
 
-#define FO_DEFAULT_SVC_TIMEOUT 10
-
 /* Some forward declarations that don't have to do anything with fail over. */
 struct hostent;
 struct tevent_context;
@@ -73,11 +71,14 @@ struct fo_server;
  * The 'srv_retry_timeout' member specifies how long a SRV lookup
  * is considered valid until we ask the server again.
  *
+ * The 'srv_retry_neg_timeout' member specifies how long a SRV lookup
+ * waits before previously failed lookup is tried again.
+ *
  * The family_order member specifies the order of address families to
  * try when looking up the service.
  */
 struct fo_options {
-    time_t srv_retry_timeout;
+    time_t srv_retry_neg_timeout;
     time_t retry_timeout;
     int service_resolv_timeout;
     enum restrict_family family_order;
@@ -127,7 +128,6 @@ int fo_add_server(struct fo_service *service,
                   const char *name, int port,
                   void *user_data, bool primary);
 
-
 int fo_add_srv_server(struct fo_service *service,
                       const char *srv,
                       const char *discovery_domain,
@@ -147,8 +147,17 @@ struct tevent_req *fo_resolve_service_send(TALLOC_CTX *mem_ctx,
                                            struct fo_service *service);
 
 int fo_resolve_service_recv(struct tevent_req *req,
+                            TALLOC_CTX *ref_ctx,
                             struct fo_server **server);
 
+
+/* To be used by async consumers of fo_resolve_service. If a server should be returned
+ * to an outer request, it should be referenced by a memory from that outer request,
+ * because the failover's server list might change with a subsequent call (see upstream
+ * bug #2829)
+ */
+void fo_ref_server(TALLOC_CTX *ref_ctx, struct fo_server *server);
+
 /*
  * Set feedback about 'server'. Caller should use this to indicate a problem
  * with the server itself, not only with the service on that server. This
@@ -197,6 +206,10 @@ time_t fo_get_service_retry_timeout(struct fo_service *svc);
 
 void fo_reset_services(struct fo_ctx *fo_ctx);
 
+void fo_reset_servers(struct fo_service *svc);
+
+struct fo_server *fo_get_active_server(struct fo_service *service);
+
 bool fo_svc_has_server(struct fo_service *service, struct fo_server *server);
 
 /*
