@@ -208,6 +208,7 @@ void ipa_auth(struct be_req *be_req)
 
     switch (state->pd->cmd) {
         case SSS_PAM_AUTHENTICATE:
+        case SSS_PAM_PREAUTH:
             state->ipa_auth_ctx = talloc_get_type(
                                     be_ctx->bet_info[BET_AUTH].pvt_bet_data,
                                     struct ipa_auth_ctx);
@@ -223,8 +224,8 @@ void ipa_auth(struct be_req *be_req)
             goto fail;
     }
 
-    req = krb5_auth_send(state, state->ev, be_ctx, state->pd,
-                         state->ipa_auth_ctx->krb5_auth_ctx);
+    req = krb5_auth_queue_send(state, state->ev, be_ctx, state->pd,
+                               state->ipa_auth_ctx->krb5_auth_ctx);
     if (req == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "krb5_auth_send failed.\n");
         goto fail;
@@ -247,7 +248,7 @@ static void ipa_auth_handler_done(struct tevent_req *req)
     int pam_status = PAM_SYSTEM_ERR;
     int dp_err;
 
-    ret = krb5_auth_recv(req, &pam_status, &dp_err);
+    ret = krb5_auth_queue_recv(req, &pam_status, &dp_err);
     talloc_zfree(req);
     state->pd->pam_status = pam_status;
     if (ret != EOK && pam_status != PAM_CRED_ERR) {
@@ -330,6 +331,7 @@ static void ipa_migration_flag_connect_done(struct tevent_req *req)
     const char *dn;
     int dp_err = DP_ERR_FATAL;
     int ret;
+    int auth_timeout;
 
     ret = sdap_cli_connect_recv(req, state, NULL, &state->sh, NULL);
     talloc_zfree(req);
@@ -369,8 +371,12 @@ static void ipa_migration_flag_connect_done(struct tevent_req *req)
         goto done;
     }
 
+    auth_timeout = dp_opt_get_int(
+                            state->ipa_auth_ctx->sdap_auth_ctx->opts->basic,
+                            SDAP_OPT_TIMEOUT);
+
     req = sdap_auth_send(state, state->ev, state->sh, NULL, NULL, dn,
-                         state->pd->authtok);
+                         state->pd->authtok, auth_timeout);
     if (req == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "sdap_auth_send failed.\n");
         goto done;
@@ -417,8 +423,8 @@ static void ipa_auth_ldap_done(struct tevent_req *req)
     DEBUG(SSSDBG_TRACE_FUNC, "LDAP authentication succeded, "
                               "trying Kerberos authentication again.\n");
 
-    req = krb5_auth_send(state, state->ev, be_ctx, state->pd,
-                         state->ipa_auth_ctx->krb5_auth_ctx);
+    req = krb5_auth_queue_send(state, state->ev, be_ctx, state->pd,
+                               state->ipa_auth_ctx->krb5_auth_ctx);
     if (req == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "krb5_auth_send failed.\n");
         goto done;
@@ -439,7 +445,7 @@ static void ipa_auth_handler_retry_done(struct tevent_req *req)
     int pam_status;
     int dp_err;
 
-    ret = krb5_auth_recv(req, &pam_status, &dp_err);
+    ret = krb5_auth_queue_recv(req, &pam_status, &dp_err);
     talloc_zfree(req);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "krb5_auth_recv request failed.\n");
