@@ -47,7 +47,6 @@ enum ipa_basic_opt {
     IPA_KRB5_REALM,
     IPA_HBAC_REFRESH,
     IPA_SELINUX_REFRESH,
-    IPA_HBAC_DENY_METHOD,
     IPA_HBAC_SUPPORT_SRCHOST,
     IPA_AUTOMOUNT_LOCATION,
     IPA_RANGES_SEARCH_BASE,
@@ -134,6 +133,54 @@ enum ipa_override_attrs {
     IPA_OPTS_OVERRIDE
 };
 
+enum ipa_sudorule_attrs {
+    IPA_OC_SUDORULE = 0,
+    IPA_AT_SUDORULE_NAME,
+    IPA_AT_SUDORULE_UUID,
+    IPA_AT_SUDORULE_ENABLED,
+    IPA_AT_SUDORULE_OPTION,
+    IPA_AT_SUDORULE_RUNASUSER,
+    IPA_AT_SUDORULE_RUNASGROUP,
+    IPA_AT_SUDORULE_ALLOWCMD,
+    IPA_AT_SUDORULE_DENYCMD,
+    IPA_AT_SUDORULE_HOST,
+    IPA_AT_SUDORULE_USER,
+    IPA_AT_SUDORULE_NOTAFTER,
+    IPA_AT_SUDORULE_NOTBEFORE,
+    IPA_AT_SUDORULE_SUDOORDER,
+    IPA_AT_SUDORULE_CMDCATEGORY,
+    IPA_AT_SUDORULE_HOSTCATEGORY,
+    IPA_AT_SUDORULE_USERCATEGORY,
+    IPA_AT_SUDORULE_RUNASUSERCATEGORY,
+    IPA_AT_SUDORULE_RUNASGROUPCATEGORY,
+    IPA_AT_SUDORULE_RUNASEXTUSER,
+    IPA_AT_SUDORULE_RUNASEXTGROUP,
+    IPA_AT_SUDORULE_RUNASEXTUSERGROUP,
+    IPA_AT_SUDORULE_EXTUSER,
+    IPA_AT_SUDORULE_ENTRYUSN,
+
+    IPA_OPTS_SUDORULE
+};
+
+enum ipa_sudocmdgroup_attrs {
+    IPA_OC_SUDOCMDGROUP = 0,
+    IPA_AT_SUDOCMDGROUP_UUID,
+    IPA_AT_SUDOCMDGROUP_NAME,
+    IPA_AT_SUDOCMDGROUP_MEMBER,
+    IPA_AT_SUDOCMDGROUP_ENTRYUSN,
+
+    IPA_OPTS_SUDOCMDGROUP
+};
+
+enum ipa_sudocmd_attrs {
+    IPA_OC_SUDOCMD = 0,
+    IPA_AT_SUDOCMD_UUID,
+    IPA_AT_SUDOCMD_CMD,
+    IPA_AT_SUDOCMD_MEMBEROF,
+
+    IPA_OPTS_SUDOCMD
+};
+
 struct ipa_auth_ctx {
     struct krb5_ctx *krb5_auth_ctx;
     struct sdap_id_ctx *sdap_id_ctx;
