@@ -25,13 +25,14 @@
 #ifndef IPA_DYNDNS_H_
 #define IPA_DYNDNS_H_
 
+#include "util/util_errors.h"
+#include "providers/ipa/ipa_common.h"
+#include "providers/dp_backend.h"
+
 void ipa_dyndns_update(void *pvt);
 void ipa_dyndns_timer(void *pvt);
 
 errno_t ipa_dyndns_init(struct be_ctx *be_ctx,
                         struct ipa_options *ctx);
 
-void ipa_dyndns_update(void *pvt);
-
-
 #endif /* IPA_DYNDNS_H_ */
