@@ -30,6 +30,27 @@
 #include "providers/ldap/sdap_async.h"
 #include "providers/ipa/ipa_id.h"
 
+static bool is_object_overridable(struct be_acct_req *ar)
+{
+    bool ret = false;
+
+    switch (ar->entry_type & BE_REQ_TYPE_MASK) {
+    case BE_REQ_USER:
+    case BE_REQ_GROUP:
+    case BE_REQ_INITGROUPS:
+    case BE_REQ_BY_SECID:
+    case BE_REQ_USER_AND_GROUP:
+    case BE_REQ_BY_UUID:
+    case BE_REQ_BY_CERT:
+        ret = true;
+        break;
+    default:
+        break;
+    }
+
+    return ret;
+}
+
 static const char *ipa_account_info_error_text(int ret, int *dp_error,
                                                const char *default_text)
 {
@@ -384,7 +405,10 @@ static int ipa_initgr_get_overrides_step(struct tevent_req *req)
         /* This should never happen, the search filter used to get the list
          * of groups includes "uuid=*"
          */
-        DEBUG(SSSDBG_OP_FAILURE, "A group with no UUID, error!\n");
+        DEBUG(SSSDBG_OP_FAILURE,
+              "The group %s has no UUID attribute %s, error!\n",
+              ldb_dn_get_linearized(state->groups[state->group_idx]->dn),
+              state->groups_id_attr);
         return EINVAL;
     }
 
@@ -445,7 +469,7 @@ static void ipa_initgr_get_overrides_override_done(struct tevent_req *subreq)
         return;
     }
 
-    if (strcmp(state->ipa_ctx->view_name, SYSDB_DEFAULT_VIEW_NAME) == 0) {
+    if (is_default_view(state->ipa_ctx->view_name)) {
         ret = sysdb_apply_default_override(state->user_dom, override_attrs,
                                        state->groups[state->group_idx]->dn);
     } else {
@@ -493,7 +517,6 @@ int ipa_initgr_get_overrides_recv(struct tevent_req *req, int *dp_error)
  */
 static errno_t ipa_id_get_group_uuids(TALLOC_CTX *mem_ctx,
                                       struct sysdb_ctx *sysdb,
-                                      const char *username,
                                       size_t *_msgs_count,
                                       struct ldb_message ***_msgs)
 {
@@ -635,13 +658,12 @@ ipa_id_get_account_info_send(TALLOC_CTX *memctx, struct tevent_context *ev,
      * - there is no view set of it is the default view
      * - if the EXTRA_INPUT_MAYBE_WITH_VIEW flag is not set
      */
-    if (state->ipa_ctx->view_name == NULL
+    if (is_default_view(state->ipa_ctx->view_name)
             || state->ar->filter_type == BE_FILTER_SECID
-            || strcmp(state->ipa_ctx->view_name,
-                      SYSDB_DEFAULT_VIEW_NAME) == 0
             || state->ar->extra_value == NULL
             || strcmp(state->ar->extra_value,
-                      EXTRA_INPUT_MAYBE_WITH_VIEW) != 0 ) {
+                      EXTRA_INPUT_MAYBE_WITH_VIEW) != 0
+            || ! is_object_overridable(state->ar)) {
         ret = ipa_id_get_account_info_get_original_step(req, ar);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE,
@@ -823,6 +845,12 @@ static void ipa_id_get_account_info_orig_done(struct tevent_req *subreq)
         goto fail;
     }
 
+    if (! is_object_overridable(state->ar)) {
+        state->dp_error = DP_ERR_OK;
+        tevent_req_done(req);
+        return;
+    }
+
     ret = get_object_from_cache(state, state->domain, state->ar,
                                 &state->obj_msg);
     if (ret == ENOENT) {
@@ -843,8 +871,7 @@ static void ipa_id_get_account_info_orig_done(struct tevent_req *subreq)
     }
 
 
-    if (state->ipa_ctx->view_name != NULL &&
-            strcmp(state->ipa_ctx->view_name, SYSDB_DEFAULT_VIEW_NAME) != 0) {
+    if (!is_default_view(state->ipa_ctx->view_name)) {
 
         if ((state->ar->entry_type & BE_REQ_TYPE_MASK) == BE_REQ_GROUP
                 || ((state->ar->entry_type & BE_REQ_TYPE_MASK) == BE_REQ_BY_UUID
@@ -856,7 +883,6 @@ static void ipa_id_get_account_info_orig_done(struct tevent_req *subreq)
                         BE_REQ_INITGROUPS) {
             /* Get UUID list of groups that have no overrideDN set. */
             ret = ipa_id_get_group_uuids(state, state->sysdb,
-                                         state->ar->filter_value,
                                          &state->group_cnt,
                                          &state->user_groups);
             if (ret != EOK) {
