@@ -1034,8 +1034,7 @@ static void ipa_s2n_get_fqlist_next(struct tevent_req *subreq)
         goto fail;
     }
 
-    if (state->ipa_ctx->view_name == NULL ||
-            strcmp(state->ipa_ctx->view_name, SYSDB_DEFAULT_VIEW_NAME) == 0) {
+    if (is_default_view(state->ipa_ctx->view_name)) {
         ret = ipa_s2n_get_fqlist_save_step(req);
         if (ret == EOK) {
             tevent_req_done(req);
@@ -1613,10 +1612,7 @@ static void ipa_s2n_get_user_done(struct tevent_req *subreq)
         ret = ENOENT;
     }
 
-    if (ret == ENOENT
-            || state->ipa_ctx->view_name == NULL
-            || strcmp(state->ipa_ctx->view_name,
-                      SYSDB_DEFAULT_VIEW_NAME) == 0) {
+    if (ret == ENOENT || is_default_view(state->ipa_ctx->view_name)) {
         ret = ipa_s2n_save_objects(state->dom, state->req_input, state->attrs,
                                    state->simple_attrs, NULL, NULL, true);
         if (ret != EOK) {
@@ -1747,7 +1743,6 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
 {
     int ret;
     time_t now;
-    uint64_t timeout = 10*60*60; /* FIXME: find a better timeout ! */
     struct sss_nss_homedir_ctx homedir_ctx;
     char *name = NULL;
     char *realm;
@@ -1768,6 +1763,9 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
     int tret;
     struct sysdb_attrs *gid_override_attrs = NULL;
     char ** exop_grouplist;
+    struct ldb_message *msg;
+    struct ldb_message_element *el = NULL;
+    const char *missing[] = {NULL, NULL};
 
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
@@ -1807,10 +1805,11 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
         ret = sysdb_attrs_get_string(attrs->sysdb_attrs,
                                      SYSDB_DEFAULT_OVERRIDE_NAME, &tmp_str);
         if (ret == EOK) {
-            ret = sysdb_attrs_add_lc_name_alias(attrs->sysdb_attrs, tmp_str);
+            ret = sysdb_attrs_add_lc_name_alias_safe(attrs->sysdb_attrs,
+                                                     tmp_str);
             if (ret != EOK) {
                 DEBUG(SSSDBG_OP_FAILURE,
-                      "sysdb_attrs_add_lc_name_alias failed.\n");
+                      "sysdb_attrs_add_lc_name_alias_safe failed.\n");
                 goto done;
             }
         } else if (ret != ENOENT) {
@@ -1879,10 +1878,10 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
                 }
             }
 
-            ret = sysdb_attrs_add_lc_name_alias(attrs->sysdb_attrs, name);
+            ret = sysdb_attrs_add_lc_name_alias_safe(attrs->sysdb_attrs, name);
             if (ret != EOK) {
                 DEBUG(SSSDBG_OP_FAILURE,
-                      "sysdb_attrs_add_lc_name_alias failed.\n");
+                      "sysdb_attrs_add_lc_name_alias_safe failed.\n");
                 goto done;
             }
 
@@ -1949,7 +1948,7 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
                  * SYSDB_INITGR_EXPIRE will be set.*/
                 ret = sysdb_attrs_add_time_t(attrs->sysdb_attrs,
                                              SYSDB_INITGR_EXPIRE,
-                                             time(NULL) + timeout);
+                                             time(NULL) + dom->user_timeout);
                 if (ret != EOK) {
                     DEBUG(SSSDBG_OP_FAILURE,
                           "sysdb_attrs_add_time_t failed.\n");
@@ -1996,6 +1995,12 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
                 }
             }
 
+            ret = sysdb_attrs_get_el_ext(attrs->sysdb_attrs,
+                                         SYSDB_ORIG_MEMBEROF, false, &el);
+            if (ret == ENOENT) {
+                missing[0] = SYSDB_ORIG_MEMBEROF;
+            }
+
             ret = sysdb_transaction_start(dom->sysdb);
             if (ret != EOK) {
                 DEBUG(SSSDBG_CRIT_FAILURE, "Failed to start transaction\n");
@@ -2007,10 +2012,48 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
                                    attrs->a.user.pw_uid,
                                    gid, attrs->a.user.pw_gecos,
                                    attrs->a.user.pw_dir, attrs->a.user.pw_shell,
-                                   NULL, attrs->sysdb_attrs, NULL,
-                                   timeout, now);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_OP_FAILURE, "sysdb_store_user failed.\n");
+                                   NULL, attrs->sysdb_attrs,
+                                   missing[0] == NULL ? NULL
+                                                      : discard_const(missing),
+                                   dom->user_timeout, now);
+            if (ret == EEXIST && dom->mpg == true) {
+                /* This handles the case where getgrgid() was called for
+                 * this user, so a group was created in the cache
+                 */
+                ret = sysdb_search_group_by_name(tmp_ctx, dom, name, NULL, &msg);
+                if (ret != EOK) {
+                    /* Fail even on ENOENT, the group must be around */
+                    DEBUG(SSSDBG_OP_FAILURE,
+                          "Could not delete MPG group [%d]: %s\n",
+                          ret, sss_strerror(ret));
+                    goto done;
+                }
+
+                ret = sysdb_delete_group(dom, NULL, attrs->a.user.pw_uid);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_OP_FAILURE,
+                          "sysdb_delete_group failed for MPG group [%d]: %s\n",
+                          ret, sss_strerror(ret));
+                    goto done;
+                }
+
+                ret = sysdb_store_user(dom, name, NULL,
+                                       attrs->a.user.pw_uid,
+                                       gid, attrs->a.user.pw_gecos,
+                                       attrs->a.user.pw_dir,
+                                       attrs->a.user.pw_shell,
+                                       NULL, attrs->sysdb_attrs, NULL,
+                                       dom->user_timeout, now);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_OP_FAILURE,
+                          "sysdb_store_user failed for MPG user [%d]: %s\n",
+                          ret, sss_strerror(ret));
+                    goto done;
+                }
+            } else if (ret != EOK) {
+                DEBUG(SSSDBG_OP_FAILURE,
+                      "sysdb_store_user failed [%d]: %s\n",
+                      ret, sss_strerror(ret));
                 goto done;
             }
 
@@ -2100,10 +2143,10 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
             }
             DEBUG(SSSDBG_TRACE_FUNC, "Processing group %s\n", name);
 
-            ret = sysdb_attrs_add_lc_name_alias(attrs->sysdb_attrs, name);
+            ret = sysdb_attrs_add_lc_name_alias_safe(attrs->sysdb_attrs, name);
             if (ret != EOK) {
                 DEBUG(SSSDBG_OP_FAILURE,
-                      "sysdb_attrs_add_lc_name_alias failed.\n");
+                      "sysdb_attrs_add_lc_name_alias_safe failed.\n");
                 goto done;
             }
 
@@ -2140,7 +2183,8 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
             }
 
             ret = sysdb_store_group(dom, name, attrs->a.group.gr_gid,
-                                    attrs->sysdb_attrs, timeout, now);
+                                    attrs->sysdb_attrs, dom->group_timeout,
+                                    now);
             if (ret != EOK) {
                 DEBUG(SSSDBG_OP_FAILURE, "sysdb_store_group failed.\n");
                 goto done;
@@ -2167,7 +2211,7 @@ static errno_t ipa_s2n_save_objects(struct sss_domain_info *dom,
         goto done;
     }
 
-    if (view_name != NULL && strcmp(view_name, SYSDB_DEFAULT_VIEW_NAME) != 0) {
+    if (!is_default_view(view_name)) {
         /* For the default view the data return by the extdom plugin already
          * contains all needed data and it is not expected to have a separate
          * override object. */
@@ -2233,9 +2277,7 @@ static void ipa_s2n_get_fqlist_done(struct tevent_req  *subreq)
     }
 
     if (state->override_attrs == NULL
-            && state->ipa_ctx->view_name != NULL
-            && strcmp(state->ipa_ctx->view_name,
-                      SYSDB_DEFAULT_VIEW_NAME) != 0) {
+            && !is_default_view(state->ipa_ctx->view_name)) {
         subreq = ipa_get_ad_override_send(state, state->ev,
                            state->ipa_ctx->sdap_id_ctx,
                            state->ipa_ctx->ipa_options,
