@@ -1058,8 +1058,8 @@ static errno_t selinux_fork_child(struct selinux_child_state *state)
         close(pipefd_from_child[1]);
         state->io->write_to_child_fd = pipefd_to_child[1];
         close(pipefd_to_child[0]);
-        fd_nonblocking(state->io->read_from_child_fd);
-        fd_nonblocking(state->io->write_to_child_fd);
+        sss_fd_nonblocking(state->io->read_from_child_fd);
+        sss_fd_nonblocking(state->io->write_to_child_fd);
 
         ret = child_handler_setup(state->ev, pid, NULL, NULL, NULL);
         if (ret != EOK) {
@@ -1557,7 +1557,7 @@ static void ipa_get_selinux_maps_done(struct tevent_req *subreq)
 
         DEBUG(SSSDBG_TRACE_FUNC, "SELinux maps referenced an HBAC rule. "
               "Need to refresh HBAC rules\n");
-        subreq = ipa_hbac_rule_info_send(state, false, state->be_ctx->ev,
+        subreq = ipa_hbac_rule_info_send(state, state->be_ctx->ev,
                                          sdap_id_op_handle(state->op),
                                          id_ctx->sdap_id_ctx->opts,
                                          state->selinux_ctx->hbac_search_bases,
