@@ -27,8 +27,16 @@
 
 #include "providers/ldap/ldap_common.h"
 
+#ifdef HAVE_SELINUX_LOGIN_DIR
+
+#define ALL_SERVICES "*"
+#define selogin_path(mem_ctx, username) \
+    talloc_asprintf(mem_ctx, "%s/logins/%s", selinux_policy_root(), username)
+#endif /* HAVE_SELINUX_LOGIN_DIR */
+
 struct ipa_selinux_ctx {
     struct ipa_id_ctx *id_ctx;
+    time_t last_update;
 
     struct sdap_search_base **selinux_search_bases;
     struct sdap_search_base **host_search_bases;
