@@ -105,230 +105,14 @@ struct be_ctx *ipa_get_subdomains_be_ctx(struct be_ctx *be_ctx)
     return subdom_ctx->be_ctx;
 }
 
-static errno_t
-ipa_ad_ctx_new(struct be_ctx *be_ctx,
-               struct ipa_id_ctx *id_ctx,
-               struct sss_domain_info *subdom,
-               struct ad_id_ctx **_ad_id_ctx)
-{
-    struct ad_options *ad_options;
-    struct ad_id_ctx *ad_id_ctx;
-    const char *gc_service_name;
-    struct ad_srv_plugin_ctx *srv_ctx;
-    char *ad_domain;
-    const char *ad_site_override;
-    struct sdap_domain *sdom;
-    errno_t ret;
-    const char *extra_attrs;
-
-    ad_options = ad_create_default_options(id_ctx, id_ctx->server_mode->realm,
-                                           id_ctx->server_mode->hostname);
-    if (ad_options == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot initialize AD options\n");
-        talloc_free(ad_options);
-        return ENOMEM;
-    }
-
-    ad_domain = subdom->name;
-
-    ret = dp_opt_set_string(ad_options->basic, AD_DOMAIN, ad_domain);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot set AD domain\n");
-        talloc_free(ad_options);
-        return ret;
-    }
-
-    ret = dp_opt_set_string(ad_options->basic, AD_KRB5_REALM,
-                            id_ctx->server_mode->realm);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot set AD realm\n");
-        talloc_free(ad_options);
-        return ret;
-    }
-
-    extra_attrs = dp_opt_get_string(id_ctx->sdap_id_ctx->opts->basic,
-                            SDAP_USER_EXTRA_ATTRS);
-    if (extra_attrs != NULL) {
-        DEBUG(SSSDBG_TRACE_ALL,
-              "Setting extra attrs for subdomain [%s] to [%s].\n", ad_domain,
-                                                                   extra_attrs);
-
-        ret = dp_opt_set_string(ad_options->id->basic, SDAP_USER_EXTRA_ATTRS,
-                                extra_attrs);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE, "dp_opt_get_string failed.\n");
-            talloc_free(ad_options);
-            return ret;
-        }
-
-        ret = sdap_extend_map_with_list(ad_options->id, ad_options->id,
-                                        SDAP_USER_EXTRA_ATTRS,
-                                        ad_options->id->user_map,
-                                        SDAP_OPTS_USER,
-                                        &ad_options->id->user_map,
-                                        &ad_options->id->user_map_cnt);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE, "sdap_extend_map_with_list failed.\n");
-            talloc_free(ad_options);
-            return ret;
-        }
-    } else {
-        DEBUG(SSSDBG_TRACE_ALL, "No extra attrs set.\n");
-    }
-
-    gc_service_name = talloc_asprintf(ad_options, "%s%s", "gc_", subdom->name);
-    if (gc_service_name == NULL) {
-        talloc_free(ad_options);
-        return ENOMEM;
-    }
-
-    /* Set KRB5 realm to same as the one of IPA when IPA
-     * is able to attach PAC. For testing, use hardcoded. */
-    ret = ad_failover_init(ad_options, be_ctx, NULL, NULL,
-                           id_ctx->server_mode->realm,
-                           subdom->name, gc_service_name,
-                           subdom->name, &ad_options->service);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot initialize AD failover\n");
-        talloc_free(ad_options);
-        return ret;
-    }
-
-    ad_id_ctx = ad_id_ctx_init(ad_options, be_ctx);
-    if (ad_id_ctx == NULL) {
-        talloc_free(ad_options);
-        return ENOMEM;
-    }
-    ad_id_ctx->sdap_id_ctx->opts = ad_options->id;
-    ad_options->id_ctx = ad_id_ctx;
-
-    ad_site_override = dp_opt_get_string(ad_options->basic, AD_SITE);
-
-    /* use AD plugin */
-    srv_ctx = ad_srv_plugin_ctx_init(be_ctx, be_ctx->be_res,
-                                     default_host_dbs,
-                                     ad_id_ctx->ad_options->id,
-                                     id_ctx->server_mode->hostname,
-                                     ad_domain,
-                                     ad_site_override);
-    if (srv_ctx == NULL) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory?\n");
-        return ENOMEM;
-    }
-    be_fo_set_srv_lookup_plugin(be_ctx, ad_srv_plugin_send,
-                                ad_srv_plugin_recv, srv_ctx, "AD");
-
-    ret = sdap_domain_subdom_add(ad_id_ctx->sdap_id_ctx,
-                                 ad_id_ctx->sdap_id_ctx->opts->sdom,
-                                 subdom->parent);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Cannot initialize sdap domain\n");
-        talloc_free(ad_options);
-        return ret;
-    }
-
-    sdom = sdap_domain_get(ad_id_ctx->sdap_id_ctx->opts, subdom);
-    if (sdom == NULL) {
-        return EFAULT;
-    }
-
-    sdap_inherit_options(subdom->parent->sd_inherit,
-                         id_ctx->sdap_id_ctx->opts,
-                         ad_id_ctx->sdap_id_ctx->opts);
-
-    ret = sdap_id_setup_tasks(be_ctx,
-                              ad_id_ctx->sdap_id_ctx,
-                              sdom,
-                              ldap_enumeration_send,
-                              ldap_enumeration_recv,
-                              ad_id_ctx->sdap_id_ctx);
-    if (ret != EOK) {
-        talloc_free(ad_options);
-        return ret;
-    }
-
-    sdom->pvt = ad_id_ctx;
-
-    /* Set up the ID mapping object */
-    ad_id_ctx->sdap_id_ctx->opts->idmap_ctx =
-        id_ctx->sdap_id_ctx->opts->idmap_ctx;
-
-    *_ad_id_ctx = ad_id_ctx;
-    return EOK;
-}
-
-static errno_t
-ipa_server_trust_add(struct be_ctx *be_ctx,
-                     struct ipa_id_ctx *id_ctx,
-                     struct sss_domain_info *subdom)
-{
-    struct ipa_ad_server_ctx *trust_ctx;
-    struct ad_id_ctx *ad_id_ctx;
-    errno_t ret;
-
-    ret = ipa_ad_ctx_new(be_ctx, id_ctx, subdom, &ad_id_ctx);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE,
-              "Cannot create ad_id_ctx for subdomain %s\n", subdom->name);
-        return ret;
-    }
-
-    trust_ctx = talloc(id_ctx->server_mode, struct ipa_ad_server_ctx);
-    if (trust_ctx == NULL) {
-        return ENOMEM;
-    }
-    trust_ctx->dom = subdom;
-    trust_ctx->ad_id_ctx = ad_id_ctx;
-
-    DLIST_ADD(id_ctx->server_mode->trusts, trust_ctx);
-    return EOK;
-}
-
-static errno_t
-ipa_ad_subdom_refresh(struct be_ctx *be_ctx,
-                      struct ipa_id_ctx *id_ctx,
-                      struct sss_domain_info *parent)
-{
-    struct sss_domain_info *dom;
-    struct ipa_ad_server_ctx *trust_iter;
-    errno_t ret;
-
-    if (dp_opt_get_bool(id_ctx->ipa_options->basic,
-                        IPA_SERVER_MODE) == false) {
-        return EOK;
-    }
-
-    for (dom = get_next_domain(parent, true);
-         dom && IS_SUBDOMAIN(dom); /* if we get back to a parent, stop */
-         dom = get_next_domain(dom, false)) {
-
-        /* Check if we already have an ID context for this subdomain */
-        DLIST_FOR_EACH(trust_iter, id_ctx->server_mode->trusts) {
-            if (trust_iter->dom == dom) {
-                break;
-            }
-        }
-
-        /* Newly detected trust */
-        if (trust_iter == NULL) {
-            ret = ipa_server_trust_add(be_ctx, id_ctx, dom);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_OP_FAILURE,
-                      "Cannot create ad_id_ctx for subdomain %s\n",
-                       dom->name);
-                continue;
-            }
-        }
-    }
-
-    return EOK;
-}
-
 static errno_t
 ipa_subdom_reinit(struct ipa_subdomains_ctx *ctx)
 {
     errno_t ret;
 
+    DEBUG(SSSDBG_TRACE_INTERNAL,
+          "Re-initializing domain %s\n", ctx->be_ctx->domain->name);
+
     ret = sss_write_krb5_conf_snippet(
                               dp_opt_get_string(ctx->id_ctx->ipa_options->basic,
                                                 IPA_KRB5_CONFD_PATH));
@@ -337,6 +121,12 @@ ipa_subdom_reinit(struct ipa_subdomains_ctx *ctx)
         /* Just continue */
     }
 
+    ret = sysdb_master_domain_update(ctx->be_ctx->domain);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "sysdb_master_domain_update failed.\n");
+        return ret;
+    }
+
     ret = sysdb_update_subdomains(ctx->be_ctx->domain);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_update_subdomains failed.\n");
@@ -353,62 +143,6 @@ ipa_subdom_reinit(struct ipa_subdomains_ctx *ctx)
     return EOK;
 }
 
-static void
-ipa_ad_subdom_remove(struct ipa_subdomains_ctx *ctx,
-                     struct sss_domain_info *subdom)
-{
-    struct ipa_ad_server_ctx *iter;
-    struct sdap_domain *sdom;
-
-    if (dp_opt_get_bool(ctx->id_ctx->ipa_options->basic,
-                        IPA_SERVER_MODE) == false) {
-        return;
-    }
-
-    DLIST_FOR_EACH(iter, ctx->id_ctx->server_mode->trusts) {
-        if (iter->dom == subdom) break;
-    }
-
-    if (iter == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "No IPA-AD context for subdomain %s\n",
-              subdom->name);
-        return;
-    }
-
-    sdom = sdap_domain_get(iter->ad_id_ctx->sdap_id_ctx->opts, subdom);
-    if (sdom == NULL) return;
-    be_ptask_destroy(&sdom->enum_task);
-    be_ptask_destroy(&sdom->cleanup_task);
-
-    sdap_domain_remove(iter->ad_id_ctx->sdap_id_ctx->opts, subdom);
-    DLIST_REMOVE(ctx->id_ctx->server_mode->trusts, iter);
-
-    /* terminate all requests for this subdomain so we can free it */
-    be_terminate_domain_requests(ctx->be_ctx, subdom->name);
-    talloc_zfree(sdom);
-}
-
-const char *get_flat_name_from_subdomain_name(struct be_ctx *be_ctx,
-                                              const char *name)
-{
-    struct ipa_subdomains_ctx *ctx;
-    struct sss_domain_info *dom;
-
-    ctx = talloc_get_type(be_ctx->bet_info[BET_SUBDOMAINS].pvt_bet_data,
-                          struct ipa_subdomains_ctx);
-    if (ctx == NULL) {
-        DEBUG(SSSDBG_TRACE_ALL, "Subdomains are not configured.\n");
-        return NULL;
-    }
-
-    dom = find_domain_by_name(ctx->be_ctx->domain, name, true);
-    if (dom) {
-        return dom->flat_name;
-    }
-
-    return NULL;
-}
-
 static errno_t ipa_ranges_parse_results(TALLOC_CTX *mem_ctx,
                                         char *domain_name,
                                         size_t count,
@@ -530,7 +264,7 @@ static errno_t ipa_ranges_parse_results(TALLOC_CTX *mem_ctx,
         ret = get_idmap_data_from_range(r, domain_name, &name1, &sid1, &rid1,
                                         &range1, &mapping1);
         if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE, ("get_idmap_data_from_range failed.\n"));
+            DEBUG(SSSDBG_OP_FAILURE, "get_idmap_data_from_range failed.\n");
             goto done;
         }
         for (d = 0; d < c; d++) {
@@ -538,7 +272,7 @@ static errno_t ipa_ranges_parse_results(TALLOC_CTX *mem_ctx,
                                             &sid2, &rid2, &range2, &mapping2);
             if (ret != EOK) {
                 DEBUG(SSSDBG_OP_FAILURE,
-                      ("get_idmap_data_from_range failed.\n"));
+                      "get_idmap_data_from_range failed.\n");
                 goto done;
             }
 
@@ -594,56 +328,33 @@ static errno_t ipa_subdom_get_forest(TALLOC_CTX *mem_ctx,
                                      char **_forest)
 {
     int ret;
-    const char *orig_dn;
     struct ldb_dn *dn = NULL;
+    const char *name;
     const struct ldb_val *val;
     char *forest = NULL;
 
-    ret = sysdb_attrs_get_string(attrs, SYSDB_ORIG_DN, &orig_dn);
-    if (ret) {
-        DEBUG(SSSDBG_OP_FAILURE, "sysdb_attrs_get_string failed.\n");
-        goto done;
-    }
-    DEBUG(SSSDBG_TRACE_ALL, "Checking if we need the forest name for [%s].\n",
-                             orig_dn);
-
-    dn = ldb_dn_new(mem_ctx, ldb_ctx, orig_dn);
+    dn = ipa_subdom_ldb_dn(mem_ctx, ldb_ctx, attrs);
     if (dn == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "ldb_dn_new failed.\n");
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_subdom_ldb_dn failed.\n");
+        ret = EIO;
         goto done;
     }
 
-    if (!ldb_dn_validate(dn)) {
-        DEBUG(SSSDBG_OP_FAILURE, "Original DN [%s] is not a valid DN.\n",
-                                  orig_dn);
-        ret = EINVAL;
-        goto done;
-    }
+    if (ipa_subdom_is_member_dom(dn) == false) {
+        ret = sysdb_attrs_get_string(attrs, IPA_CN, &name);
+        if (ret) {
+            DEBUG(SSSDBG_OP_FAILURE, "sysdb_attrs_get_string failed.\n");
+            goto done;
+        }
 
-    if (ldb_dn_get_comp_num(dn) < 5) {
-        /* We are only interested in the member domain objects. In IPA the
-         * forest root object is stored as e.g.
-         * cn=AD.DOM,cn=ad,cn=trusts,dc=example,dc=com. Member domains in the
-         * forest are children of the forest root object e.g.
-         * cn=SUB.AD.DOM,cn=AD.DOM,cn=ad,cn=trusts,dc=example,dc=com. Since
-         * the forest name is not stored in the member objects we derive it
-         * from the RDN of the forest root object. */
-        ret = EOK;
-        goto done;
-    }
+        forest = talloc_strdup(mem_ctx, name);
+        if (forest == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, "talloc_strndup failed.\n");
+            ret = ENOMEM;
+            goto done;
+        }
 
-    val = ldb_dn_get_component_val(dn, 3);
-    if (strncasecmp("trusts", (const char *) val->data, val->length) != 0) {
-        DEBUG(SSSDBG_TRACE_FUNC,
-              "4th component is not 'trust', nothing to do.\n");
-        ret = EOK;
-        goto done;
-    }
-
-    val = ldb_dn_get_component_val(dn, 2);
-    if (strncasecmp("ad", (const char *) val->data, val->length) != 0) {
-        DEBUG(SSSDBG_TRACE_FUNC,
-              "3rd component is not 'ad', nothing to do.\n");
+        DEBUG(SSSDBG_TRACE_INTERNAL, "The forest name is %s\n", forest);
         ret = EOK;
         goto done;
     }
@@ -656,6 +367,7 @@ static errno_t ipa_subdom_get_forest(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
+    ret = EOK;
 done:
     talloc_free(dn);
 
@@ -666,10 +378,26 @@ done:
     return ret;
 }
 
+static errno_t ipa_get_sd_trust_direction(struct sysdb_attrs *sd,
+                                          struct ipa_id_ctx *id_ctx,
+                                          struct ldb_context *ldb_ctx,
+                                          uint32_t *_direction)
+{
+    if (id_ctx->server_mode != NULL) {
+        return ipa_server_get_trust_direction(sd, ldb_ctx, _direction);
+    } else {
+        /* Clients do not have access to the trust objects's trust direction
+         * and don't generally care
+         */
+        *_direction = 0;
+        return EOK;
+    }
+}
+
 static errno_t ipa_subdom_store(struct sss_domain_info *parent,
+                                struct ipa_id_ctx *id_ctx,
                                 struct sdap_idmap_ctx *sdap_idmap_ctx,
-                                struct sysdb_attrs *attrs,
-                                bool enumerate)
+                                struct sysdb_attrs *attrs)
 {
     TALLOC_CTX *tmp_ctx;
     const char *name;
@@ -679,6 +407,8 @@ static errno_t ipa_subdom_store(struct sss_domain_info *parent,
     char *forest = NULL;
     int ret;
     bool mpg;
+    bool enumerate;
+    uint32_t direction;
 
     tmp_ctx = talloc_new(parent);
     if (tmp_ctx == NULL) {
@@ -717,8 +447,25 @@ static errno_t ipa_subdom_store(struct sss_domain_info *parent,
         goto done;
     }
 
+    ret = ipa_subdom_enumerates(parent, attrs, &enumerate);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    ret = ipa_get_sd_trust_direction(attrs, id_ctx,
+                                     sysdb_ctx_get_ldb(parent->sysdb),
+                                     &direction);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "ipa_get_sd_trust_direction failed: %d\n", ret);
+        goto done;
+    }
+
+    DEBUG(SSSDBG_FUNC_DATA,
+          "Trust direction of %s is %s\n", name, ipa_trust_dir2str(direction));
     ret = sysdb_subdomain_store(parent->sysdb, name, realm, flat,
-                                id, mpg, enumerate, forest);
+                                id, mpg, enumerate, forest,
+                                direction);
     if (ret) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_subdomain_store failed.\n");
         goto done;
@@ -730,6 +477,23 @@ done:
     return ret;
 }
 
+static void ipa_subdom_store_step(struct sss_domain_info *parent,
+                                  struct ipa_id_ctx *id_ctx,
+                                  struct sdap_idmap_ctx *sdap_idmap_ctx,
+                                  struct sysdb_attrs *attrs)
+{
+    int ret;
+
+    ret = ipa_subdom_store(parent, id_ctx, sdap_idmap_ctx, attrs);
+    if (ret == ERR_TRUST_NOT_SUPPORTED) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "Unsupported trust type, skipping\n");
+    } else if (ret) {
+        /* Nothing we can do about the error. */
+        DEBUG(SSSDBG_MINOR_FAILURE, "Failed to parse subdom data, "
+              "will try to use cached subdomain\n");
+    }
+}
+
 static errno_t ipa_subdomains_refresh(struct ipa_subdomains_ctx *ctx,
                                       int count, struct sysdb_attrs **reply,
                                       bool *changes)
@@ -739,16 +503,15 @@ static errno_t ipa_subdomains_refresh(struct ipa_subdomains_ctx *ctx,
     const char *value;
     int c, h;
     int ret;
-    bool enumerate;
 
     parent = ctx->be_ctx->domain;
     memset(handled, 0, sizeof(bool) * count);
     h = 0;
 
     /* check existing subdomains */
-    for (dom = get_next_domain(parent, true);
+    for (dom = get_next_domain(parent, SSS_GND_DESCEND);
          dom && IS_SUBDOMAIN(dom); /* if we get back to a parent, stop */
-         dom = get_next_domain(dom, false)) {
+         dom = get_next_domain(dom, 0)) {
         for (c = 0; c < count; c++) {
             if (handled[c]) {
                 continue;
@@ -765,30 +528,19 @@ static errno_t ipa_subdomains_refresh(struct ipa_subdomains_ctx *ctx,
 
         if (c >= count) {
             /* ok this subdomain does not exist anymore, let's clean up */
-            dom->disabled = true;
+            sss_domain_set_state(dom, DOM_DISABLED);
             ret = sysdb_subdomain_delete(dom->sysdb, dom->name);
             if (ret != EOK) {
                 goto done;
             }
 
             /* Remove the AD ID ctx from the list of LDAP domains */
-            ipa_ad_subdom_remove(ctx, dom);
+            ipa_ad_subdom_remove(ctx->be_ctx, ctx->id_ctx, dom);
         } else {
             /* ok let's try to update it */
-            ret = ipa_subdom_enumerates(parent, reply[c], &enumerate);
-            if (ret != EOK) {
-                goto done;
-            }
-
-            ret = ipa_subdom_store(parent, ctx->sdap_id_ctx->opts->idmap_ctx,
-                                   reply[c], enumerate);
-            if (ret) {
-                /* Nothing we can do about the errorr. Let's at least try
-                 * to reuse the existing domain
-                 */
-                DEBUG(SSSDBG_MINOR_FAILURE, "Failed to parse subdom data, "
-                      "will try to use cached subdomain\n");
-            }
+            ipa_subdom_store_step(parent, ctx->id_ctx,
+                                  ctx->sdap_id_ctx->opts->idmap_ctx,
+                                  reply[c]);
             handled[c] = true;
             h++;
         }
@@ -807,24 +559,13 @@ static errno_t ipa_subdomains_refresh(struct ipa_subdomains_ctx *ctx,
         if (handled[c]) {
             continue;
         }
-        /* Nothing we can do about the errorr. Let's at least try
-         * to reuse the existing domain.
-         */
-        ret = ipa_subdom_enumerates(parent, reply[c], &enumerate);
-        if (ret != EOK) {
-            goto done;
-        }
 
-        ret = ipa_subdom_store(parent, ctx->sdap_id_ctx->opts->idmap_ctx,
-                               reply[c], enumerate);
-        if (ret) {
-            DEBUG(SSSDBG_MINOR_FAILURE, "Failed to parse subdom data, "
-                  "will try to use cached subdomain\n");
-        }
+        ipa_subdom_store_step(parent, ctx->id_ctx,
+                              ctx->sdap_id_ctx->opts->idmap_ctx,
+                              reply[c]);
     }
 
     ret = EOK;
-
 done:
     if (ret != EOK) {
         ctx->last_refreshed = 0;
@@ -851,8 +592,12 @@ struct ipa_subdomains_req_ctx {
 
 static void ipa_subdomains_get_conn_done(struct tevent_req *req);
 static errno_t
-ipa_subdomains_handler_get(struct ipa_subdomains_req_ctx *ctx,
-                           enum ipa_subdomains_req_type type);
+ipa_subdomains_handler_get_start(struct ipa_subdomains_req_ctx *ctx,
+                                 struct sdap_search_base **search_bases,
+                                 enum ipa_subdomains_req_type type);
+static errno_t
+ipa_subdomains_handler_get_cont(struct ipa_subdomains_req_ctx *ctx,
+                                enum ipa_subdomains_req_type type);
 static void ipa_subdomains_handler_done(struct tevent_req *req);
 static void ipa_subdomains_handler_master_done(struct tevent_req *req);
 static void ipa_subdomains_handler_ranges_done(struct tevent_req *req);
@@ -864,7 +609,8 @@ static struct ipa_subdomains_req_params subdomain_requests[] = {
     },
     { SUBDOMAINS_FILTER,
       ipa_subdomains_handler_done,
-      { IPA_CN, IPA_FLATNAME, IPA_TRUSTED_DOMAIN_SID, NULL }
+      { IPA_CN, IPA_FLATNAME, IPA_TRUSTED_DOMAIN_SID,
+        IPA_TRUST_DIRECTION, NULL }
     },
     { RANGE_FILTER,
       ipa_subdomains_handler_ranges_done,
@@ -947,7 +693,9 @@ static void ipa_subdomains_get_conn_done(struct tevent_req *req)
         goto fail;
     }
 
-    ret = ipa_subdomains_handler_get(ctx, IPA_SUBDOMAINS_RANGES);
+    ret = ipa_subdomains_handler_get_start(ctx,
+                                           ctx->sd_ctx->ranges_search_bases,
+                                           IPA_SUBDOMAINS_RANGES);
     if (ret != EOK && ret != EAGAIN) {
         goto fail;
     }
@@ -978,8 +726,8 @@ ipa_subdomains_handler_get(struct ipa_subdomains_req_ctx *ctx,
     }
 
     talloc_free(ctx->current_filter);
-    ctx->current_filter = sdap_get_id_specific_filter(ctx, params->filter,
-                                                            base->filter);
+    ctx->current_filter = sdap_combine_filters(ctx, params->filter,
+                                               base->filter);
     if (ctx->current_filter == NULL) {
         return ENOMEM;
     }
@@ -1002,7 +750,26 @@ ipa_subdomains_handler_get(struct ipa_subdomains_req_ctx *ctx,
     return EAGAIN;
 }
 
+static errno_t
+ipa_subdomains_handler_get_start(struct ipa_subdomains_req_ctx *ctx,
+                                 struct sdap_search_base **search_bases,
+                                 enum ipa_subdomains_req_type type)
+{
+    ctx->search_base_iter = 0;
+    ctx->search_bases = search_bases;
+    return ipa_subdomains_handler_get(ctx, type);
+}
+
+static errno_t
+ipa_subdomains_handler_get_cont(struct ipa_subdomains_req_ctx *ctx,
+                                enum ipa_subdomains_req_type type)
+{
+    ctx->search_base_iter++;
+    return ipa_subdomains_handler_get(ctx, type);
+}
+
 static void ipa_get_view_name_done(struct tevent_req *req);
+static void ipa_server_create_trusts_done(struct tevent_req *trust_req);
 static errno_t ipa_check_master(struct ipa_subdomains_req_ctx *ctx);
 
 static errno_t ipa_get_view_name(struct ipa_subdomains_req_ctx *ctx)
@@ -1025,6 +792,9 @@ static errno_t ipa_get_view_name(struct ipa_subdomains_req_ctx *ctx)
         return EOK;
     }
 
+    /* We add SDAP_DEREF_FLG_SILENT because old IPA servers don't have
+     * the attribute we dereference, causing the deref call to fail
+     */
     req = sdap_deref_search_with_filter_send(ctx, ctx->sd_ctx->be_ctx->ev,
                         ctx->sd_ctx->sdap_id_ctx->opts,
                         sdap_id_op_handle(ctx->sdap_op),
@@ -1032,7 +802,8 @@ static errno_t ipa_get_view_name(struct ipa_subdomains_req_ctx *ctx)
                         ctx->current_filter, IPA_ASSIGNED_ID_VIEW, attrs,
                         1, maps,
                         dp_opt_get_int(ctx->sd_ctx->sdap_id_ctx->opts->basic,
-                                       SDAP_SEARCH_TIMEOUT));
+                                       SDAP_SEARCH_TIMEOUT),
+                        SDAP_DEREF_FLG_SILENT);
 
     if (req == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "sdap_get_generic_send failed.\n");
@@ -1108,7 +879,7 @@ static void ipa_get_view_name_done(struct tevent_req *req)
     }
 
     DEBUG(SSSDBG_TRACE_FUNC, "Found view name [%s].\n", view_name);
-    if (strcmp(view_name, IPA_DEFAULT_VIEW_NAME) == 0) {
+    if (is_default_view(view_name)) {
         DEBUG(SSSDBG_TRACE_ALL,
               "Found IPA default view name, replacing with sysdb default.\n");
         view_name = SYSDB_DEFAULT_VIEW_NAME;
@@ -1127,9 +898,19 @@ static void ipa_get_view_name_done(struct tevent_req *req)
     } else {
         if (ctx->sd_ctx->id_ctx->view_name == NULL
             || strcmp(ctx->sd_ctx->id_ctx->view_name, view_name) != 0) {
-            /* View name changed */
+            /* View name changed. If there was a non-default non-local view
+             * was used the tree in cache containing the override values is
+             * removed. In all cases sysdb_invalidate_overrides() is called to
+             * remove the override attribute from the cached user objects.
+             *
+             * Typically ctx->sd_ctx->id_ctx->view_name == NULL means that the
+             * cache was empty but there was a bug in with caused that the
+             * view name was not written to the cache at all. In this case the
+             * cache must be invalidated if the new view is not the
+             * default-view as well. */
 
-            if (ctx->sd_ctx->id_ctx->view_name != NULL) {
+            if (ctx->sd_ctx->id_ctx->view_name != NULL
+                    || !is_default_view(view_name)) {
                 ret = sysdb_transaction_start(
                                             ctx->sd_ctx->be_ctx->domain->sysdb);
                 if (ret != EOK) {
@@ -1138,8 +919,8 @@ static void ipa_get_view_name_done(struct tevent_req *req)
                     goto done;
                 }
 
-                if (strcmp(ctx->sd_ctx->id_ctx->view_name,
-                           SYSDB_DEFAULT_VIEW_NAME) != 0) {
+                if (!is_default_view(ctx->sd_ctx->id_ctx->view_name)
+                        && !is_local_view(ctx->sd_ctx->id_ctx->view_name)) {
                     /* Old view was not the default view, delete view tree */
                     ret = sysdb_delete_view_tree(
                                              ctx->sd_ctx->be_ctx->domain->sysdb,
@@ -1220,13 +1001,7 @@ static void ipa_get_view_name_done(struct tevent_req *req)
 
     }
 
-    ret = ipa_check_master(ctx);
-    if (ret == EAGAIN) {
-        return;
-    } else if (ret != EOK) {
-        goto done;
-    }
-
+    ret = EOK;
 done:
     if (ret == EOK) {
         dp_error = DP_ERR_OK;
@@ -1243,6 +1018,7 @@ static void ipa_subdomains_handler_done(struct tevent_req *req)
     struct sss_domain_info *domain;
     bool refresh_has_changes = false;
     int dp_error = DP_ERR_FATAL;
+    struct tevent_req *trust_req;
 
     ctx = tevent_req_callback_data(req, struct ipa_subdomains_req_ctx);
     domain = ctx->sd_ctx->be_ctx->domain;
@@ -1266,8 +1042,7 @@ static void ipa_subdomains_handler_done(struct tevent_req *req)
         ctx->reply_count += reply_count;
     }
 
-    ctx->search_base_iter++;
-    ret = ipa_subdomains_handler_get(ctx, IPA_SUBDOMAINS_SLAVE);
+    ret = ipa_subdomains_handler_get_cont(ctx, IPA_SUBDOMAINS_SLAVE);
     if (ret == EAGAIN) {
         return;
     } else if (ret != EOK) {
@@ -1288,11 +1063,17 @@ static void ipa_subdomains_handler_done(struct tevent_req *req)
             goto done;
         }
 
-        ret = ipa_ad_subdom_refresh(ctx->sd_ctx->be_ctx, ctx->sd_ctx->id_ctx,
-                                    domain);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE, "ipa_ad_subdom_refresh failed.\n");
-            goto done;
+        if (ctx->sd_ctx->id_ctx->server_mode != NULL) {
+            trust_req = ipa_server_create_trusts_send(ctx, ctx->sd_ctx->be_ctx->ev,
+                                                      ctx->sd_ctx->be_ctx,
+                                                      ctx->sd_ctx->id_ctx,
+                                                      domain);
+            if (trust_req == NULL) {
+                ret = ENOMEM;
+                goto done;
+            }
+            tevent_req_set_callback(trust_req, ipa_server_create_trusts_done, ctx);
+            return;
         }
     }
 
@@ -1320,13 +1101,7 @@ static void ipa_subdomains_handler_done(struct tevent_req *req)
         }
     }
 
-    ret = ipa_check_master(ctx);
-    if (ret == EAGAIN) {
-        return;
-    } else if (ret != EOK) {
-        goto done;
-    }
-
+    ret = EOK;
 done:
     if (ret == EOK) {
         dp_error = DP_ERR_OK;
@@ -1334,6 +1109,23 @@ done:
     ipa_subdomains_done(ctx->sd_ctx, ctx->be_req, dp_error, ret, NULL);
 }
 
+static void ipa_server_create_trusts_done(struct tevent_req *trust_req)
+{
+    errno_t ret;
+    int dp_error = DP_ERR_FATAL;
+    struct ipa_subdomains_req_ctx *ctx;
+
+    ctx = tevent_req_callback_data(trust_req, struct ipa_subdomains_req_ctx);
+
+    ret = ipa_server_create_trusts_recv(trust_req);
+    talloc_zfree(trust_req);
+    if (ret == EOK) {
+        dp_error = DP_ERR_OK;
+    }
+
+    ipa_subdomains_done(ctx->sd_ctx, ctx->be_req, dp_error, ret, NULL);
+}
+
 static errno_t ipa_check_master(struct ipa_subdomains_req_ctx *ctx)
 {
     int ret;
@@ -1350,9 +1142,9 @@ static errno_t ipa_check_master(struct ipa_subdomains_req_ctx *ctx)
         domain->domain_id == NULL ||
         domain->realm == NULL) {
 
-        ctx->search_base_iter = 0;
-        ctx->search_bases = ctx->sd_ctx->master_search_bases;
-        ret = ipa_subdomains_handler_get(ctx, IPA_SUBDOMAINS_MASTER);
+        ret = ipa_subdomains_handler_get_start(ctx,
+                                               ctx->sd_ctx->master_search_bases,
+                                               IPA_SUBDOMAINS_MASTER);
         if (ret == EAGAIN) {
             return EAGAIN;
         } else if (ret != EOK) {
@@ -1401,9 +1193,19 @@ static void ipa_subdomains_handler_ranges_done(struct tevent_req *req)
         goto done;
     }
 
-    ctx->search_base_iter = 0;
-    ctx->search_bases = ctx->sd_ctx->search_bases;
-    ret = ipa_subdomains_handler_get(ctx, IPA_SUBDOMAINS_SLAVE);
+    ret = ipa_check_master(ctx);
+    if (ret == EAGAIN) {
+        DEBUG(SSSDBG_TRACE_ALL, "Checking master record..\n");
+        return;
+    } else if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_check_master failed.\n");
+        goto done;
+    }
+    /* Master domain is up-to-date. Continue checking subdomains */
+
+    DEBUG(SSSDBG_TRACE_ALL, "Master record up2date, checking subdomains\n");
+    ret = ipa_subdomains_handler_get_start(ctx, ctx->sd_ctx->search_bases,
+                                           IPA_SUBDOMAINS_SLAVE);
     if (ret == EAGAIN) {
         return;
     } else if (ret != EOK) {
@@ -1427,6 +1229,9 @@ static void ipa_subdomains_handler_master_done(struct tevent_req *req)
     size_t reply_count = 0;
     struct sysdb_attrs **reply = NULL;
     struct ipa_subdomains_req_ctx *ctx;
+    const char *flat = NULL;
+    const char *id = NULL;
+    const char *realm = NULL;
 
     ctx = tevent_req_callback_data(req, struct ipa_subdomains_req_ctx);
 
@@ -1438,9 +1243,6 @@ static void ipa_subdomains_handler_master_done(struct tevent_req *req)
     }
 
     if (reply_count) {
-        const char *flat = NULL;
-        const char *id = NULL;
-
         ret = sysdb_attrs_get_string(reply[0], IPA_FLATNAME, &flat);
         if (ret != EOK) {
             goto done;
@@ -1451,28 +1253,59 @@ static void ipa_subdomains_handler_master_done(struct tevent_req *req)
             goto done;
         }
 
-        ret = sysdb_master_domain_add_info(ctx->sd_ctx->be_ctx->domain,
-                                           flat, id, NULL);
+        /* There is only one master record. Don't bother checking other IPA
+         * search bases; move to checking subdomains instead
+         */
     } else {
-        ctx->search_base_iter++;
-        ret = ipa_subdomains_handler_get(ctx, IPA_SUBDOMAINS_MASTER);
+        ret = ipa_subdomains_handler_get_cont(ctx, IPA_SUBDOMAINS_MASTER);
         if (ret == EAGAIN) {
             return;
         } else if (ret != EOK) {
             goto done;
         }
 
-        /* Right now we know there has been an error
-         * and we don't have the master domain record
-         */
-        DEBUG(SSSDBG_CRIT_FAILURE, "Master domain record not found!\n");
+        /* All search paths are searched and no master domain record was
+         * found.
+         *
+         * A default IPA installation will not have a master domain record,
+         * this is only created by ipa-adtrust-install. Nevertheless we should
+         * continue to read other data like the idview on IPA clients. */
 
-        if (!ctx->sd_ctx->configured_explicit) {
-            ctx->sd_ctx->disabled_until = time(NULL) +
-                                          IPA_SUBDOMAIN_DISABLED_PERIOD;
+        DEBUG(SSSDBG_TRACE_INTERNAL, "Master domain record not found!\n");
+
+    }
+
+    realm = dp_opt_get_string(ctx->sd_ctx->id_ctx->ipa_options->basic,
+                              IPA_KRB5_REALM);
+    if (realm == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "No Kerberos realm for IPA?\n");
+        ret = EINVAL;
+        goto done;
+    }
+
+    ret = sysdb_master_domain_add_info(ctx->sd_ctx->be_ctx->domain,
+                                       realm, flat, id, NULL);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    ret = ipa_subdomains_handler_get_start(ctx,
+                                           ctx->sd_ctx->search_bases,
+                                           IPA_SUBDOMAINS_SLAVE);
+    if (ret == EAGAIN) {
+        return;
+    } else if (ret == EOK) {
+        /* If there are no search bases defined for subdomains try to get the
+         * idview before ending the request */
+        if (ctx->sd_ctx->id_ctx->server_mode == NULL) {
+            /* Only get view on clients, on servers it is always 'default' */
+            ret = ipa_get_view_name(ctx);
+            if (ret == EAGAIN) {
+                return;
+            } else if (ret != EOK) {
+                goto done;
+            }
         }
-
-        ret = EIO;
     }
 
 done:
@@ -1688,62 +1521,9 @@ int ipa_subdom_init(struct be_ctx *be_ctx,
               "Users from trusted domains might not be resolved correctly\n");
     }
 
-    return EOK;
-}
-
-int ipa_ad_subdom_init(struct be_ctx *be_ctx,
-                       struct ipa_id_ctx *id_ctx)
-{
-    char *realm;
-    char *hostname;
-    errno_t ret;
-
-    if (dp_opt_get_bool(id_ctx->ipa_options->basic,
-                        IPA_SERVER_MODE) == false) {
-        return EOK;
-    }
-
-    /* The IPA code relies on the default FQDN format to unparse user
-     * names. Warn loudly if the full_name_format was customized on the
-     * IPA server
-     */
-    if ((strcmp(be_ctx->domain->names->fq_fmt,
-               CONFDB_DEFAULT_FULL_NAME_FORMAT) != 0)
-            && (strcmp(be_ctx->domain->names->fq_fmt,
-                       CONFDB_DEFAULT_FULL_NAME_FORMAT_INTERNAL) != 0)) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "%s is set to a non-default value [%s] " \
-              "lookups of subdomain users will likely fail!\n",
-              CONFDB_FULL_NAME_FORMAT, be_ctx->domain->names->fq_fmt);
-        sss_log(SSS_LOG_ERR, "%s is set to a non-default value [%s] " \
-                "lookups of subdomain users will likely fail!\n",
-                CONFDB_FULL_NAME_FORMAT, be_ctx->domain->names->fq_fmt);
-        /* Attempt to continue */
-    }
-
-    realm = dp_opt_get_string(id_ctx->ipa_options->basic, IPA_KRB5_REALM);
-    if (realm == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "No Kerberos realm for IPA?\n");
-        return EINVAL;
-    }
-
-    hostname = dp_opt_get_string(id_ctx->ipa_options->basic, IPA_HOSTNAME);
-    if (hostname == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "No host name for IPA?\n");
-        return EINVAL;
-    }
-
-    id_ctx->server_mode = talloc_zero(id_ctx, struct ipa_server_mode_ctx);
-    if (id_ctx->server_mode == NULL) {
-        return ENOMEM;
-    }
-    id_ctx->server_mode->realm = realm;
-    id_ctx->server_mode->hostname = hostname;
-    id_ctx->server_mode->trusts = NULL;
-    id_ctx->server_mode->ext_groups = NULL;
-
-    ret = ipa_ad_subdom_refresh(be_ctx, id_ctx, be_ctx->domain);
+    ret = ipa_ad_subdom_init(be_ctx, id_ctx);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "ipa_ad_subdom_refresh failed.\n");
+        DEBUG(SSSDBG_CRIT_FAILURE, "ipa_ad_subdom_init failed.\n");
         return ret;
     }
 
