@@ -34,6 +34,16 @@
 #include "providers/ad/ad_id.h"
 #include "providers/ipa/ipa_subdomains.h"
 
+static struct tevent_req *
+ipa_srv_ad_acct_send(TALLOC_CTX *mem_ctx,
+                     struct tevent_context *ev,
+                     struct ipa_id_ctx *ipa_ctx,
+                     struct be_req *be_req,
+                     struct sysdb_attrs *override_attrs,
+                     struct be_acct_req *ar);
+static errno_t
+ipa_srv_ad_acct_recv(struct tevent_req *req, int *dp_error_out);
+
 struct ipa_subdomain_account_state {
     struct tevent_context *ev;
     struct ipa_id_ctx *ipa_ctx;
@@ -45,6 +55,7 @@ struct ipa_subdomain_account_state {
     struct be_acct_req *ar;
 
     bool ipa_server_mode;
+    bool server_retry;
     int entry_type;
     const char *filter;
     int filter_type;
@@ -263,7 +274,7 @@ static errno_t ipa_subdomain_account_get_original_step(struct tevent_req *req,
     struct tevent_req *subreq;
 
     if (state->ipa_server_mode) {
-        subreq = ipa_get_ad_acct_send(state, state->ev, state->ipa_ctx,
+        subreq = ipa_srv_ad_acct_send(state, state->ev, state->ipa_ctx,
                                       state->be_req, state->override_attrs, ar);
     } else {
         subreq = ipa_get_subdom_acct_send(state, state->ev, state->ipa_ctx,
@@ -291,13 +302,14 @@ static void ipa_subdomain_account_done(struct tevent_req *subreq)
     int ret;
 
     if (state->ipa_server_mode) {
-        ret = ipa_get_ad_acct_recv(subreq, &dp_error);
+        ret = ipa_srv_ad_acct_recv(subreq, &dp_error);
     } else {
         ret = ipa_get_subdom_acct_recv(subreq, &dp_error);
     }
     talloc_zfree(subreq);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_*_acct request failed: %d\n", ret);
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_*_acct request failed: [%d]: %s.\n",
+              ret, sss_strerror(ret));
         state->dp_error = dp_error;
         tevent_req_error(req, ret);
         return;
@@ -575,7 +587,7 @@ static void ipa_get_ad_acct_done(struct tevent_req *subreq);
 static struct ad_id_ctx *ipa_get_ad_id_ctx(struct ipa_id_ctx *ipa_ctx,
                                            struct sss_domain_info *dom);
 
-struct tevent_req *
+static struct tevent_req *
 ipa_get_ad_acct_send(TALLOC_CTX *mem_ctx,
                      struct tevent_context *ev,
                      struct ipa_id_ctx *ipa_ctx,
@@ -629,19 +641,16 @@ ipa_get_ad_acct_send(TALLOC_CTX *mem_ctx,
     case BE_REQ_BY_SECID:
     case BE_REQ_GROUP:
         clist = ad_gc_conn_list(req, ad_id_ctx, state->obj_dom);
-        if (clist == NULL) {
-            ret = ENOMEM;
-            goto fail;
-        }
         break;
     default:
-        clist = talloc_zero_array(req, struct sdap_id_conn_ctx *, 2);
-        if (clist == NULL) {
-            ret = ENOMEM;
-            goto fail;
-        }
-        clist[0] = ad_id_ctx->ldap_ctx;
-        clist[1] = NULL;
+        clist = ad_ldap_conn_list(req, ad_id_ctx, state->obj_dom);
+        break;
+    }
+
+    if (clist == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot generate AD connection list!\n");
+        ret = ENOMEM;
+        goto fail;
     }
 
     /* Now we already need ad_id_ctx in particular sdap_id_conn_ctx */
@@ -683,7 +692,7 @@ ipa_get_ad_id_ctx(struct ipa_id_ctx *ipa_ctx,
 static errno_t
 get_subdomain_homedir_of_user(TALLOC_CTX *mem_ctx, struct sss_domain_info *dom,
                               const char *fqname, uint32_t uid,
-                              const char **_homedir)
+                              const char *original, const char **_homedir)
 {
     errno_t ret;
     const char *name;
@@ -697,12 +706,20 @@ get_subdomain_homedir_of_user(TALLOC_CTX *mem_ctx, struct sss_domain_info *dom,
         goto done;
     }
 
+    if (strstr(dom->subdomain_homedir, "%o") != NULL && original == NULL) {
+        DEBUG(SSSDBG_TRACE_ALL,
+              "Original home directory for user: %s is empty.\n", fqname);
+        ret = ERR_HOMEDIR_IS_NULL;
+        goto done;
+    }
+
     ZERO_STRUCT(homedir_ctx);
 
     homedir_ctx.uid = uid;
     homedir_ctx.domain = dom->name;
     homedir_ctx.flatname = dom->flat_name;
     homedir_ctx.config_homedir_substr = dom->homedir_substr;
+    homedir_ctx.original = original;
     ret = sss_parse_name_const(tmp_ctx, dom->names, fqname,
                                NULL, &name);
     if (ret != EOK) {
@@ -809,6 +826,7 @@ apply_subdomain_homedir(TALLOC_CTX *mem_ctx, struct sss_domain_info *dom,
     errno_t ret;
     uint32_t uid;
     const char *fqname;
+    const char *original;
     const char *homedir = NULL;
     struct ldb_message_element *msg_el = NULL;
     size_t c;
@@ -833,10 +851,6 @@ apply_subdomain_homedir(TALLOC_CTX *mem_ctx, struct sss_domain_info *dom,
         goto done;
     }
 
-    /*
-     * Homedir is always overriden by subdomain_homedir even if it was
-     * explicitly set by user.
-     */
     fqname = ldb_msg_find_attr_as_string(msg, SYSDB_NAME, NULL);
     if (fqname == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Missing user name.\n");
@@ -852,11 +866,21 @@ apply_subdomain_homedir(TALLOC_CTX *mem_ctx, struct sss_domain_info *dom,
         goto done;
     }
 
-    ret = get_subdomain_homedir_of_user(mem_ctx, dom, fqname, uid, &homedir);
+    original = ldb_msg_find_attr_as_string(msg, SYSDB_HOMEDIR, NULL);
+    if (original == NULL) {
+        DEBUG(SSSDBG_TRACE_ALL, "Missing homedir of %s.\n", fqname);
+    }
+
+    ret = get_subdomain_homedir_of_user(mem_ctx, dom, fqname, uid, original,
+                                        &homedir);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE,
               "get_subdomain_homedir_of_user failed: [%d]: [%s]\n",
-               ret, sss_strerror(ret));
+              ret, sss_strerror(ret));
+        if (ret == ERR_HOMEDIR_IS_NULL) {
+            /* This is not fatal, fallback_homedir will be used. */
+            ret = EOK;
+        }
         goto done;
     }
 
@@ -887,6 +911,7 @@ errno_t get_object_from_cache(TALLOC_CTX *mem_ctx,
                             SYSDB_OBJECTCLASS,
                             SYSDB_UUID,
                             SYSDB_GHOST,
+                            SYSDB_HOMEDIR,
                             NULL };
     char *name;
 
@@ -916,6 +941,20 @@ errno_t get_object_from_cache(TALLOC_CTX *mem_ctx,
 
         *_msg = res->msgs[0];
 
+        ret = EOK;
+        goto done;
+    } else if (ar->filter_type == BE_FILTER_CERT) {
+        ret = sysdb_search_object_by_cert(mem_ctx, dom, ar->filter_value, attrs,
+                                          &res);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "Failed to make request to our cache: [%d]: [%s]\n",
+                   ret, sss_strerror(ret));
+            goto done;
+        }
+
+        *_msg = res->msgs[0];
+
         ret = EOK;
         goto done;
     } else if (ar->filter_type == BE_FILTER_IDNUM) {
@@ -1006,7 +1045,10 @@ ipa_get_ad_acct_ad_part_done(struct tevent_req *subreq)
 
     ret = ad_handle_acct_info_recv(subreq, &state->dp_error, NULL);
     talloc_zfree(subreq);
-    if (ret != EOK) {
+    if (ret == ERR_SUBDOM_INACTIVE) {
+        tevent_req_error(req, ret);
+        return;
+    } else if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "AD lookup failed: %d\n", ret);
         tevent_req_error(req, ret);
         return;
@@ -1188,6 +1230,7 @@ static errno_t ipa_get_ad_apply_override_step(struct tevent_req *req)
      * attributes set, i.e. where overrides might not have been applied. */
     ret = sysdb_asq_search(state, state->obj_dom, state->obj_msg->dn,
                           "(&("SYSDB_GC")("SYSDB_GIDNUM"=*)" \
+                            "("SYSDB_POSIX"=TRUE)" \
                             "(!("ORIGINALAD_PREFIX SYSDB_GIDNUM"=*))" \
                             "(!("ORIGINALAD_PREFIX SYSDB_NAME"=*)))",
                           SYSDB_INITGR_ATTR,
@@ -1288,7 +1331,7 @@ ipa_get_ad_acct_done(struct tevent_req *subreq)
     tevent_req_done(req);
 }
 
-errno_t
+static errno_t
 ipa_get_ad_acct_recv(struct tevent_req *req, int *dp_error_out)
 {
     struct ipa_get_ad_acct_state *state = tevent_req_data(req,
@@ -1302,3 +1345,173 @@ ipa_get_ad_acct_recv(struct tevent_req *req, int *dp_error_out)
 
     return EOK;
 }
+
+struct ipa_srv_ad_acct_state {
+    struct tevent_context *ev;
+    struct ipa_id_ctx *ipa_ctx;
+    struct be_req *be_req;
+    struct sysdb_attrs *override_attrs;
+    struct be_acct_req *ar;
+
+    struct sss_domain_info *obj_dom;
+    struct be_ctx *be_ctx;
+    bool retry;
+
+    int dp_error;
+};
+
+static int ipa_srv_ad_acct_lookup_step(struct tevent_req *req);
+static void ipa_srv_ad_acct_lookup_done(struct tevent_req *subreq);
+static void ipa_srv_ad_acct_retried(struct tevent_req *subreq);
+
+static struct tevent_req *
+ipa_srv_ad_acct_send(TALLOC_CTX *mem_ctx,
+                     struct tevent_context *ev,
+                     struct ipa_id_ctx *ipa_ctx,
+                     struct be_req *be_req,
+                     struct sysdb_attrs *override_attrs,
+                     struct be_acct_req *ar)
+{
+    errno_t ret;
+    struct tevent_req *req;
+    struct ipa_srv_ad_acct_state *state;
+
+    req = tevent_req_create(mem_ctx, &state, struct ipa_srv_ad_acct_state);
+    if (req == NULL) {
+        return NULL;
+    }
+
+    state->ev = ev;
+    state->ipa_ctx = ipa_ctx;
+    state->be_req = be_req;
+    state->override_attrs = override_attrs;
+    state->ar = ar;
+    state->retry = true;
+    state->dp_error = DP_ERR_FATAL;
+    state->be_ctx = be_req_get_be_ctx(state->be_req);
+
+    state->obj_dom = find_domain_by_name(
+                                  state->ipa_ctx->sdap_id_ctx->be->domain,
+                                  state->ar->domain, true);
+    if (state->obj_dom == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "Domain not found\n");
+        ret = ERR_DOMAIN_NOT_FOUND;
+        goto fail;
+    }
+
+    ret = ipa_srv_ad_acct_lookup_step(req);
+    if (ret != EOK) {
+        goto fail;
+    }
+
+    return req;
+
+fail:
+    tevent_req_error(req, ret);
+    tevent_req_post(req, ev);
+    return req;
+}
+
+static int ipa_srv_ad_acct_lookup_step(struct tevent_req *req)
+{
+    struct tevent_req *subreq;
+    struct ipa_srv_ad_acct_state *state = tevent_req_data(req,
+                                            struct ipa_srv_ad_acct_state);
+
+    DEBUG(SSSDBG_TRACE_FUNC, "Looking up AD account\n");
+    subreq = ipa_get_ad_acct_send(state, state->ev, state->ipa_ctx,
+                                  state->be_req, state->override_attrs,
+                                  state->ar);
+    if (subreq == NULL) {
+        return ENOMEM;
+    }
+    tevent_req_set_callback(subreq, ipa_srv_ad_acct_lookup_done, req);
+
+    return EOK;
+}
+
+static void ipa_srv_ad_acct_lookup_done(struct tevent_req *subreq)
+{
+    errno_t ret;
+    int dp_error = DP_ERR_FATAL;
+    struct tevent_req *req = tevent_req_callback_data(subreq,
+                                                struct tevent_req);
+    struct ipa_srv_ad_acct_state *state = tevent_req_data(req,
+                                            struct ipa_srv_ad_acct_state);
+
+    ret = ipa_get_ad_acct_recv(subreq, &dp_error);
+    talloc_free(subreq);
+    if (ret == ERR_SUBDOM_INACTIVE && state->retry == true) {
+
+        state->retry = false;
+
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "Sudomain lookup failed, will try to reset sudomain..\n");
+        subreq = ipa_server_trusted_dom_setup_send(state, state->ev,
+                                                   state->be_ctx,
+                                                   state->ipa_ctx,
+                                                   state->obj_dom);
+        if (subreq == NULL) {
+            goto fail;
+        }
+        tevent_req_set_callback(subreq, ipa_srv_ad_acct_retried, req);
+        return;
+    } else if (ret != EOK) {
+        be_mark_dom_offline(state->obj_dom, state->be_ctx);
+
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_*_acct request failed: [%d]: %s.\n",
+              ret, sss_strerror(ret));
+        goto fail;
+    }
+
+    state->dp_error = DP_ERR_OK;
+    tevent_req_done(req);
+    return;
+
+fail:
+    state->dp_error = dp_error;
+    tevent_req_error(req, ret);
+}
+
+static void ipa_srv_ad_acct_retried(struct tevent_req *subreq)
+{
+    errno_t ret;
+    struct tevent_req *req = tevent_req_callback_data(subreq,
+                                                struct tevent_req);
+    struct ipa_srv_ad_acct_state *state = tevent_req_data(req,
+                                            struct ipa_srv_ad_acct_state);
+
+    ret = ipa_server_trusted_dom_setup_recv(subreq);
+    talloc_free(subreq);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Failed to re-set subdomain [%d]: %s\n", ret, sss_strerror(ret));
+        state->dp_error = DP_ERR_FATAL;
+        tevent_req_error(req, ret);
+    }
+
+    DEBUG(SSSDBG_TRACE_FUNC, "Sudomain re-set, will retry lookup\n");
+    be_fo_reset_svc(state->be_ctx, state->obj_dom->name);
+
+    ret = ipa_srv_ad_acct_lookup_step(req);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Failed to look up AD acct [%d]: %s\n", ret, sss_strerror(ret));
+        state->dp_error = DP_ERR_FATAL;
+        tevent_req_error(req, ret);
+    }
+}
+
+static errno_t
+ipa_srv_ad_acct_recv(struct tevent_req *req, int *dp_error_out)
+{
+    struct ipa_srv_ad_acct_state *state = tevent_req_data(req,
+                                                struct ipa_srv_ad_acct_state);
+
+    if (dp_error_out) {
+        *dp_error_out = state->dp_error;
+    }
+
+    TEVENT_REQ_RETURN_ON_ERROR(req);
+    return EOK;
+}
