@@ -77,6 +77,10 @@ static errno_t pack_authtok(struct io_buffer *buf, size_t *rp,
         ret = sss_authtok_get_ccfile(tok, &data, &len);
         auth_token_length = len + 1;
         break;
+    case SSS_AUTHTOK_TYPE_2FA:
+        data = (char *) sss_authtok_get_data(tok);
+        auth_token_length = sss_authtok_get_size(tok);
+        break;
     default:
         ret = EINVAL;
     }
@@ -308,7 +312,7 @@ static errno_t fork_child(struct tevent_req *req)
         err = exec_child_ex(state,
                             pipefd_to_child, pipefd_from_child,
                             KRB5_CHILD, state->kr->krb5_ctx->child_debug_fd,
-                            k5c_extra_args, STDIN_FILENO, STDOUT_FILENO);
+                            k5c_extra_args, false, STDIN_FILENO, STDOUT_FILENO);
         if (err != EOK) {
             DEBUG(SSSDBG_CRIT_FAILURE, "Could not exec KRB5 child: [%d][%s].\n",
                       err, strerror(err));
@@ -320,8 +324,8 @@ static errno_t fork_child(struct tevent_req *req)
         close(pipefd_from_child[1]);
         state->io->write_to_child_fd = pipefd_to_child[1];
         close(pipefd_to_child[0]);
-        fd_nonblocking(state->io->read_from_child_fd);
-        fd_nonblocking(state->io->write_to_child_fd);
+        sss_fd_nonblocking(state->io->read_from_child_fd);
+        sss_fd_nonblocking(state->io->write_to_child_fd);
 
         ret = child_handler_setup(state->ev, pid, NULL, NULL, NULL);
         if (ret != EOK) {
@@ -528,9 +532,9 @@ parse_krb5_child_response(TALLOC_CTX *mem_ctx, uint8_t *buf, ssize_t len,
         DEBUG(SSSDBG_TRACE_LIBS, "child response [%d][%d][%d].\n",
               msg_status, msg_type, msg_len);
 
-        if ((p + msg_len) > len) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "message format error [%zu] > [%zd].\n",
-                  p+msg_len, len);
+        if (msg_len > len - p) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "message format error [%d] > [%zu].\n",
+                  msg_len, len - p);
             return EINVAL;
         }
 
