@@ -83,36 +83,18 @@ static errno_t check_pwexpire_kerberos(const char *expire_date, time_t now,
                                        struct pam_data *pd,
                                        int pwd_exp_warning)
 {
-    char *end;
-    struct tm tm;
     time_t expire_time;
     int expiration_warning;
     int ret = ERR_INTERNAL;
 
-    memset(&tm, 0, sizeof(tm));
-
-    end = strptime(expire_date, "%Y%m%d%H%M%SZ", &tm);
-    if (end == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "Kerberos expire date [%s] invalid.\n", expire_date);
-        return EINVAL;
-    }
-    if (*end != '\0') {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "Kerberos expire date [%s] contains extra characters.\n",
-                  expire_date);
-        return EINVAL;
+    ret = sss_utc_to_time_t(expire_date, "%Y%m%d%H%M%SZ",
+                            &expire_time);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "sss_utc_to_time_t failed with %d:%s.\n",
+              ret, sss_strerror(ret));
+        return ret;
     }
 
-    expire_time = mktime(&tm);
-    if (expire_time == -1) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "mktime failed to convert [%s].\n", expire_date);
-        return EINVAL;
-    }
-
-    tzset();
-    expire_time -= timezone;
     DEBUG(SSSDBG_TRACE_ALL,
           "Time info: tzname[0] [%s] tzname[1] [%s] timezone [%ld] "
            "daylight [%d] now [%ld] expire_time [%ld].\n", tzname[0],
@@ -436,7 +418,7 @@ static struct tevent_req *get_user_dn_send(TALLOC_CTX *memctx,
                                    sh, attrs, filter,
                                    dp_opt_get_int(opts->basic,
                                                   SDAP_SEARCH_TIMEOUT),
-                                   false);
+                                   SDAP_LOOKUP_SINGLE);
     if (!subreq) {
         ret = ENOMEM;
         goto done;
@@ -713,7 +695,7 @@ static void auth_resolve_done(struct tevent_req *subreq)
     int ret;
     bool use_tls;
 
-    ret = be_resolve_server_recv(subreq, &state->srv);
+    ret = be_resolve_server_recv(subreq, state, &state->srv);
     talloc_zfree(subreq);
     if (ret) {
         /* all servers have been tried and none
@@ -780,6 +762,18 @@ static void auth_connect_done(struct tevent_req *subreq)
                               state->srv, PORT_WORKING);
     }
 
+    /* In case the ID provider is set to proxy, this might be the first
+     * LDAP operation at all, so we need to set the connection status
+     */
+    if (state->sh->connected == false) {
+        ret = sdap_set_connected(state->sh, state->ev);
+        if (ret) {
+            DEBUG(SSSDBG_OP_FAILURE, "Cannot set connected status\n");
+            tevent_req_error(req, ret);
+            return;
+        }
+    }
+
     ret = get_user_dn(state, state->ctx->be->domain,
                       state->ctx->opts, state->username, &state->dn,
                       &state->pw_expire_type, &state->pw_expire_data);
@@ -831,7 +825,9 @@ static void auth_do_bind(struct tevent_req *req)
 
     subreq = sdap_auth_send(state, state->ev, state->sh,
                             NULL, NULL, state->dn,
-                            state->authtok);
+                            state->authtok,
+                            dp_opt_get_int(state->ctx->opts->basic,
+                                           SDAP_OPT_TIMEOUT));
     if (!subreq) {
         tevent_req_error(req, ENOMEM);
         return;
@@ -1044,6 +1040,7 @@ static void sdap_auth4chpass_done(struct tevent_req *req)
         } else {
             const char *password;
             const char *new_password;
+            int timeout;
 
             ret = sss_authtok_get_password(state->pd->authtok,
                                            &password, NULL);
@@ -1058,9 +1055,12 @@ static void sdap_auth4chpass_done(struct tevent_req *req)
                 goto done;
             }
 
+            timeout = dp_opt_get_int(state->ctx->opts->basic, SDAP_OPT_TIMEOUT);
+
             subreq = sdap_exop_modify_passwd_send(state, be_ctx->ev,
                                                   state->sh, state->dn,
-                                                  password, new_password);
+                                                  password, new_password,
+                                                  timeout);
             if (!subreq) {
                 DEBUG(SSSDBG_OP_FAILURE,
                       "Failed to change password for %s\n", state->username);
@@ -1302,6 +1302,10 @@ static void sdap_pam_auth_done(struct tevent_req *req)
     case ERR_PASSWORD_EXPIRED:
         state->pd->pam_status = PAM_NEW_AUTHTOK_REQD;
         break;
+    case ERR_ACCOUNT_LOCKED:
+        state->pd->account_locked = true;
+        state->pd->pam_status = PAM_PERM_DENIED;
+        break;
     default:
         state->pd->pam_status = PAM_SYSTEM_ERR;
         dp_err = DP_ERR_FATAL;
