@@ -44,11 +44,6 @@ void sdap_handler_done(struct be_req *req, int dp_err,
     return be_req_terminate(req, dp_err, error, errstr);
 }
 
-void sdap_mark_offline(struct sdap_id_ctx *ctx)
-{
-    be_mark_offline(ctx->be);
-}
-
 int ldap_id_setup_tasks(struct sdap_id_ctx *ctx)
 {
     return sdap_id_setup_tasks(ctx->be, ctx, ctx->opts->sdom,
@@ -399,6 +394,18 @@ done:
     return realm;
 }
 
+const char *sdap_gssapi_realm(struct dp_option *opts)
+{
+    const char *realm;
+
+    realm = dp_opt_get_cstring(opts, SDAP_SASL_REALM);
+    if (!realm) {
+        realm = dp_opt_get_cstring(opts, SDAP_KRB5_REALM);
+    }
+
+    return realm;
+}
+
 int sdap_gssapi_init(TALLOC_CTX *mem_ctx,
                      struct dp_option *opts,
                      struct be_ctx *bectx,
@@ -419,7 +426,7 @@ int sdap_gssapi_init(TALLOC_CTX *mem_ctx,
     krb5_servers = dp_opt_get_string(opts, SDAP_KRB5_KDC);
     krb5_backup_servers = dp_opt_get_string(opts, SDAP_KRB5_BACKUP_KDC);
 
-    krb5_opt_realm = dp_opt_get_string(opts, SDAP_KRB5_REALM);
+    krb5_opt_realm = sdap_gssapi_realm(opts);
     if (krb5_opt_realm == NULL) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Missing krb5_realm option, will use libkrb default\n");
@@ -773,7 +780,7 @@ errno_t list_missing_attrs(TALLOC_CTX *mem_ctx,
     /* Allocate the maximum possible values for missing_attrs, to
      * be on the safe side
      */
-    missing = talloc_array(tmp_ctx, char *, attr_count);
+    missing = talloc_array(tmp_ctx, char *, attr_count + 2);
     if (!missing) {
         ret = ENOMEM;
         goto done;
@@ -824,6 +831,12 @@ errno_t list_missing_attrs(TALLOC_CTX *mem_ctx,
             /* Attribute could not be found. Add to the missing list */
             missing[k] = talloc_steal(missing, sysdb_name);
             k++;
+
+            /* Remove originalMemberOf as well if MemberOf is missing */
+            if (strcmp(sysdb_name, SYSDB_MEMBEROF) == 0) {
+                missing[k] = talloc_strdup(missing, SYSDB_ORIG_MEMBEROF);
+                k++;
+            }
         }
     }
 
