@@ -59,7 +59,7 @@ errno_t ldap_setup_cleanup(struct sdap_id_ctx *id_ctx,
     struct ldap_id_cleanup_ctx *cleanup_ctx = NULL;
     char *name = NULL;
 
-    period = dp_opt_get_int(id_ctx->opts->basic, SDAP_CACHE_PURGE_TIMEOUT);
+    period = dp_opt_get_int(id_ctx->opts->basic, SDAP_PURGE_CACHE_TIMEOUT);
     if (period == 0) {
         /* Cleanup has been explicitly disabled, so we won't
          * create any cleanup tasks. */
@@ -169,12 +169,14 @@ done:
 static int cleanup_users_logged_in(hash_table_t *table,
                                    const struct ldb_message *msg);
 
+static errno_t expire_memberof_target_groups(struct sss_domain_info *dom,
+                                             struct ldb_message *user);
+
 static int cleanup_users(struct sdap_options *opts,
                          struct sss_domain_info *dom)
 {
     TALLOC_CTX *tmpctx;
-    struct sysdb_ctx *sysdb = dom->sysdb;
-    const char *attrs[] = { SYSDB_NAME, SYSDB_UIDNUM, NULL };
+    const char *attrs[] = { SYSDB_NAME, SYSDB_UIDNUM, SYSDB_MEMBEROF, NULL };
     time_t now = time(NULL);
     char *subfilter = NULL;
     int account_cache_expiration;
@@ -217,15 +219,13 @@ static int cleanup_users(struct sdap_options *opts,
         goto done;
     }
 
-    ret = sysdb_search_users(tmpctx, sysdb, dom,
-                             subfilter, attrs, &count, &msgs);
-    if (ret) {
-        if (ret == ENOENT) {
-            ret = EOK;
-        }
+    ret = sysdb_search_users(tmpctx, dom, subfilter, attrs, &count, &msgs);
+    if (ret == ENOENT) {
+        count = 0;
+    } else if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "sysdb_search_users failed: %d\n", ret);
         goto done;
     }
-
     DEBUG(SSSDBG_FUNC_DATA, "Found %zu expired user entries!\n", count);
 
     if (count == 0) {
@@ -238,6 +238,7 @@ static int cleanup_users(struct sdap_options *opts,
      * the cleanup in that case
      */
     if (ret != EOK && ret != ENOSYS) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "get_uid_table failed: %d\n", ret);
         goto done;
     }
 
@@ -249,6 +250,7 @@ static int cleanup_users(struct sdap_options *opts,
             ret = EFAULT;
             goto done;
         }
+        DEBUG(SSSDBG_TRACE_ALL, "Processing user %s\n", name);
 
         if (uid_table) {
             ret = cleanup_users_logged_in(uid_table, msgs[i]);
@@ -259,14 +261,28 @@ static int cleanup_users(struct sdap_options *opts,
                           "keeping data\n", name);
                 continue;
             } else if (ret != ENOENT) {
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "Cannot check if user is logged in: %d\n", ret);
                 goto done;
             }
         }
 
         /* If not logged in or cannot check the table, delete him */
         DEBUG(SSSDBG_TRACE_ALL, "About to delete user %s\n", name);
-        ret = sysdb_delete_user(sysdb, dom, name, 0);
+        ret = sysdb_delete_user(dom, name, 0);
         if (ret) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "sysdb_delete_user failed: %d\n", ret);
+            goto done;
+        }
+
+        /* Mark all groups of which user was a member as expired in cache,
+         * so that its ghost/member attributes are refreshed on next
+         * request. */
+        ret = expire_memberof_target_groups(dom, msgs[i]);
+        if (ret != EOK && ret != ENOENT) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "expire_memberof_target_groups failed: [%d]:%s\n",
+                  ret, sss_strerror(ret));
             goto done;
         }
     }
@@ -276,6 +292,43 @@ done:
     return ret;
 }
 
+static errno_t expire_memberof_target_groups(struct sss_domain_info *dom,
+                                             struct ldb_message *user)
+{
+    struct ldb_message_element *memberof_el = NULL;
+    errno_t ret;
+    TALLOC_CTX *tmp_ctx;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        return ENOMEM;
+    }
+
+    memberof_el = ldb_msg_find_element(user, SYSDB_MEMBEROF);
+    if (memberof_el == NULL) {
+        /* User has no cached groups. Nothing to be marked as expired. */
+        ret = EOK;
+        goto done;
+    }
+
+    for (unsigned int i = 0; i < memberof_el->num_values; i++) {
+        ret = sysdb_mark_entry_as_expired_ldb_val(dom,
+                                                  &memberof_el->values[i]);
+        if (ret != EOK && ret != ENOENT) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "sysdb_mark_entry_as_expired_ldb_val failed: [%d]: %s\n",
+                  ret, sss_strerror(ret));
+            goto done;
+        }
+    }
+
+    ret = EOK;
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
 static int cleanup_users_logged_in(hash_table_t *table,
                                    const struct ldb_message *msg)
 {
@@ -302,6 +355,7 @@ static int cleanup_users_logged_in(hash_table_t *table,
         return ENOENT;
     }
 
+    DEBUG(SSSDBG_OP_FAILURE, "hash_lookup failed: %d\n", ret);
     return EIO;
 }
 
@@ -340,12 +394,11 @@ static int cleanup_groups(TALLOC_CTX *memctx,
         goto done;
     }
 
-    ret = sysdb_search_groups(tmpctx, sysdb, domain,
-                              subfilter, attrs, &count, &msgs);
-    if (ret) {
-        if (ret == ENOENT) {
-            ret = EOK;
-        }
+    ret = sysdb_search_groups(tmpctx, domain, subfilter, attrs, &count, &msgs);
+    if (ret == ENOENT) {
+        count = 0;
+    } else if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "sysdb_search_groups failed: %d\n", ret);
         goto done;
     }
 
@@ -361,6 +414,7 @@ static int cleanup_groups(TALLOC_CTX *memctx,
 
         dn = ldb_dn_get_linearized(msgs[i]->dn);
         if (!dn) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "Cannot linearize DN!\n");
             ret = EFAULT;
             goto done;
         }
@@ -404,6 +458,8 @@ static int cleanup_groups(TALLOC_CTX *memctx,
             goto done;
         }
 
+        DEBUG(SSSDBG_TRACE_LIBS, "Searching with: %s\n", subfilter);
+
         ret = sysdb_search_entry(tmpctx, sysdb, base_dn,
                                  LDB_SCOPE_SUBTREE, subfilter, NULL,
                                  &u_count, &u_msgs);
@@ -419,14 +475,16 @@ static int cleanup_groups(TALLOC_CTX *memctx,
             }
 
             DEBUG(SSSDBG_TRACE_INTERNAL, "About to delete group %s\n", name);
-            ret = sysdb_delete_group(sysdb, domain, name, 0);
+            ret = sysdb_delete_group(domain, name, 0);
             if (ret) {
                 DEBUG(SSSDBG_OP_FAILURE, "Group delete returned %d (%s)\n",
                           ret, strerror(ret));
                 goto done;
             }
-        }
-        if (ret != EOK) {
+        } else if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Failed to search sysdb using %s: [%d] %s\n",
+                  subfilter, ret, sss_strerror(ret));
             goto done;
         }
         talloc_zfree(u_msgs);
