@@ -176,7 +176,7 @@ enum sdap_basic_opt {
     SDAP_OFFLINE_TIMEOUT,
     SDAP_FORCE_UPPER_CASE_REALM,
     SDAP_ENUM_REFRESH_TIMEOUT,
-    SDAP_CACHE_PURGE_TIMEOUT,
+    SDAP_PURGE_CACHE_TIMEOUT,
     SDAP_TLS_CACERT,
     SDAP_TLS_CACERTDIR,
     SDAP_TLS_CERT,
@@ -223,6 +223,7 @@ enum sdap_basic_opt {
     SDAP_IDMAP_AUTORID_COMPAT,
     SDAP_IDMAP_DEFAULT_DOMAIN,
     SDAP_IDMAP_DEFAULT_DOMAIN_SID,
+    SDAP_IDMAP_EXTRA_SLICE_INIT,
     SDAP_AD_MATCHING_RULE_GROUPS,
     SDAP_AD_MATCHING_RULE_INITGROUPS,
     SDAP_AD_USE_TOKENGROUPS,
@@ -231,6 +232,7 @@ enum sdap_basic_opt {
     SDAP_MIN_ID,
     SDAP_MAX_ID,
     SDAP_PWDLOCKOUT_DN,
+    SDAP_WILDCARD_LIMIT,
 
     SDAP_OPTS_BASIC /* opts counter */
 };
@@ -281,6 +283,7 @@ enum sdap_user_attrs {
     SDAP_AT_NDS_LOGIN_ALLOWED_TIME_MAP,
     SDAP_AT_USER_SSH_PUBLIC_KEY,
     SDAP_AT_USER_AUTH_TYPE,
+    SDAP_AT_USER_CERT,
 
     SDAP_OPTS_USER /* attrs counter */
 };
@@ -301,6 +304,7 @@ enum sdap_group_attrs {
     SDAP_AT_GROUP_MODSTAMP,
     SDAP_AT_GROUP_USN,
     SDAP_AT_GROUP_TYPE,
+    SDAP_AT_GROUP_EXT_MEMBER,
 
     SDAP_OPTS_GROUP /* attrs counter */
 };
@@ -371,6 +375,13 @@ struct sdap_search_base {
     const char *filter;
 };
 
+errno_t
+sdap_create_search_base(TALLOC_CTX *mem_ctx,
+                        const char *unparsed_base,
+                        int scope,
+                        const char *filter,
+                        struct sdap_search_base **_base);
+
 /* Values from
  * http://msdn.microsoft.com/en-us/library/cc223272%28v=prot.13%29.aspx
  */
@@ -380,7 +391,8 @@ enum dc_functional_level {
     DS_BEHAVIOR_WIN2008 = 3,
     DS_BEHAVIOR_WIN2008R2 = 4,
     DS_BEHAVIOR_WIN2012 = 5,
-    DS_BEHAVIOR_WIN2012R2 = 6
+    DS_BEHAVIOR_WIN2012R2 = 6,
+    DS_BEHAVIOR_WIN2016 = 7,
 };
 
 struct sdap_domain {
@@ -412,6 +424,26 @@ struct sdap_domain {
     void *pvt;
 };
 
+typedef struct tevent_req *
+(*ext_member_send_fn_t)(TALLOC_CTX *mem_ctx,
+                        struct tevent_context *ev,
+                        const char *ext_member,
+                        void *pvt);
+typedef errno_t
+(*ext_member_recv_fn_t)(TALLOC_CTX *mem_ctx,
+                        struct tevent_req *req,
+                        enum sysdb_member_type *member_type,
+                        struct sss_domain_info **_dom,
+                        struct sysdb_attrs **_member);
+
+struct sdap_ext_member_ctx {
+    /* Typically ID context of the external ID provider */
+    void *pvt;
+
+    ext_member_send_fn_t ext_member_resolve_send;
+    ext_member_recv_fn_t ext_member_resolve_recv;
+};
+
 struct sdap_options {
     struct dp_option *basic;
     struct sdap_attr_map *gen_map;
@@ -424,6 +456,9 @@ struct sdap_options {
     /* ID-mapping support */
     struct sdap_idmap_ctx *idmap_ctx;
 
+    /* Resolving external members */
+    struct sdap_ext_member_ctx *ext_ctx;
+
     /* FIXME - should this go to a special struct to avoid mixing with name-service-switch maps? */
     struct sdap_attr_map *sudorule_map;
     struct sdap_attr_map *autofs_mobject_map;
@@ -571,4 +606,13 @@ void sdap_steal_server_opts(struct sdap_id_ctx *id_ctx,
                             struct sdap_server_opts **srv_opts);
 
 char *sdap_make_oc_list(TALLOC_CTX *mem_ctx, struct sdap_attr_map *map);
+
+size_t sdap_steal_objects_in_dom(struct sdap_options *opts,
+                                 struct sysdb_attrs **dom_objects,
+                                 size_t offset,
+                                 struct sss_domain_info *dom,
+                                 struct sysdb_attrs **all_objects,
+                                 size_t count,
+                                 bool filter);
+
 #endif /* _SDAP_H_ */
