@@ -535,12 +535,11 @@ static errno_t sdap_account_expired_rhds(struct pam_data *pd,
 #define NDS_EXPIRED_MSG "The user account is expired"
 #define NDS_TIME_MAP_MSG "The user account is not allowed at this time"
 
-static bool nds_check_expired(const char *exp_time_str)
+bool nds_check_expired(const char *exp_time_str)
 {
-    char *end;
-    struct tm tm;
     time_t expire_time;
     time_t now;
+    errno_t ret;
 
     if (exp_time_str == NULL) {
         DEBUG(SSSDBG_TRACE_ALL,
@@ -548,30 +547,14 @@ static bool nds_check_expired(const char *exp_time_str)
         return false;
     }
 
-    memset(&tm, 0, sizeof(tm));
-
-    end = strptime(exp_time_str, "%Y%m%d%H%M%SZ", &tm);
-    if (end == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "NDS expire date [%s] invalid.\n", exp_time_str);
-        return true;
-    }
-    if (*end != '\0') {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "NDS expire date [%s] contains extra characters.\n",
-                  exp_time_str);
+    ret = sss_utc_to_time_t(exp_time_str, "%Y%m%d%H%M%SZ",
+                            &expire_time);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "sss_utc_to_time_t failed with %d:%s.\n",
+              ret, sss_strerror(ret));
         return true;
     }
 
-    expire_time = mktime(&tm);
-    if (expire_time == -1) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "mktime failed to convert [%s].\n", exp_time_str);
-        return true;
-    }
-
-    tzset();
-    expire_time -= timezone;
     now = time(NULL);
     DEBUG(SSSDBG_TRACE_ALL,
           "Time info: tzname[0] [%s] tzname[1] [%s] timezone [%ld] "
@@ -732,26 +715,38 @@ static errno_t sdap_account_expired(struct sdap_access_ctx *access_ctx,
     } else {
         if (strcasecmp(expire, LDAP_ACCOUNT_EXPIRE_SHADOW) == 0) {
             ret = sdap_account_expired_shadow(pd, user_entry);
-            if (ret != EOK) {
+            if (ret == ERR_ACCOUNT_EXPIRED) {
+                DEBUG(SSSDBG_TRACE_FUNC,
+                      "sdap_account_expired_shadow: %s.\n", sss_strerror(ret));
+            } else if (ret != EOK) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       "sdap_account_expired_shadow failed.\n");
             }
         } else if (strcasecmp(expire, LDAP_ACCOUNT_EXPIRE_AD) == 0) {
             ret = sdap_account_expired_ad(pd, user_entry);
-            if (ret != EOK) {
+            if (ret == ERR_ACCOUNT_EXPIRED || ret == ERR_ACCESS_DENIED) {
+                DEBUG(SSSDBG_TRACE_FUNC,
+                      "sdap_account_expired_ad: %s.\n", sss_strerror(ret));
+            } else if (ret != EOK) {
                 DEBUG(SSSDBG_CRIT_FAILURE, "sdap_account_expired_ad failed.\n");
             }
         } else if (strcasecmp(expire, LDAP_ACCOUNT_EXPIRE_RHDS) == 0 ||
                    strcasecmp(expire, LDAP_ACCOUNT_EXPIRE_IPA) == 0 ||
                    strcasecmp(expire, LDAP_ACCOUNT_EXPIRE_389DS) == 0) {
             ret = sdap_account_expired_rhds(pd, user_entry);
-            if (ret != EOK) {
+            if (ret == ERR_ACCESS_DENIED) {
+                DEBUG(SSSDBG_TRACE_FUNC,
+                      "sdap_account_expired_rhds: %s.\n", sss_strerror(ret));
+            } else if (ret != EOK) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       "sdap_account_expired_rhds failed.\n");
             }
         } else if (strcasecmp(expire, LDAP_ACCOUNT_EXPIRE_NDS) == 0) {
             ret = sdap_account_expired_nds(pd, user_entry);
-            if (ret != EOK) {
+            if (ret == ERR_ACCESS_DENIED) {
+                DEBUG(SSSDBG_TRACE_FUNC,
+                      "sdap_account_expired_nds: %s.\n", sss_strerror(ret));
+            } else if (ret != EOK) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       "sdap_account_expired_nds failed.\n");
             }
