@@ -299,15 +299,10 @@ static void sdap_process_message(struct tevent_context *ev,
 
     switch (msgtype) {
     case LDAP_RES_SEARCH_ENTRY:
+    case LDAP_RES_SEARCH_REFERENCE:
         /* go and process entry */
         break;
 
-    case LDAP_RES_SEARCH_REFERENCE:
-        /* more ops to come with this msgid */
-        /* just ignore */
-        ldap_msgfree(msg);
-        return;
-
     case LDAP_RES_BIND:
     case LDAP_RES_SEARCH_RESULT:
     case LDAP_RES_MODIFY:
@@ -409,12 +404,16 @@ static int sdap_op_destructor(void *mem)
 
     DLIST_REMOVE(op->sh->ops, op);
 
-    if (op->done) return 0;
+    if (op->done) {
+        DEBUG(SSSDBG_TRACE_INTERNAL, "Operation %d finished\n", op->msgid);
+        return 0;
+    }
 
     /* we don't check the result here, if a message was really abandoned,
      * hopefully the server will get an abandon.
      * If the operation was already fully completed, this is going to be
      * just a noop */
+    DEBUG(SSSDBG_TRACE_LIBS, "Abandoning operation %d\n", op->msgid);
     ldap_abandon_ext(op->sh->ldap, op->msgid, NULL, NULL);
 
     return 0;
@@ -431,6 +430,7 @@ static void sdap_op_timeout(struct tevent_req *req)
     }
 
     /* signal the caller that we have a timeout */
+    DEBUG(SSSDBG_TRACE_LIBS, "Issuing timeout for %d\n", op->msgid);
     op->callback(op, NULL, ETIMEDOUT, op->data);
 }
 
@@ -450,6 +450,9 @@ int sdap_op_add(TALLOC_CTX *memctx, struct tevent_context *ev,
     op->data = data;
     op->ev = ev;
 
+    DEBUG(SSSDBG_TRACE_INTERNAL,
+          "New operation %d timeout %d\n", op->msgid, timeout);
+
     /* check if we need to set a timeout */
     if (timeout) {
         struct tevent_req *req;
@@ -494,7 +497,8 @@ struct tevent_req *sdap_exop_modify_passwd_send(TALLOC_CTX *memctx,
                                            struct sdap_handle *sh,
                                            char *user_dn,
                                            const char *password,
-                                           const char *new_password)
+                                           const char *new_password,
+                                           int timeout)
 {
     struct tevent_req *req = NULL;
     struct sdap_exop_modify_passwd_state *state;
@@ -562,9 +566,8 @@ struct tevent_req *sdap_exop_modify_passwd_send(TALLOC_CTX *memctx,
     DEBUG(SSSDBG_TRACE_INTERNAL,
           "ldap_extended_operation sent, msgid = %d\n", msgid);
 
-    /* FIXME: get timeouts from configuration, for now 5 secs. */
     ret = sdap_op_add(state, ev, state->sh, msgid,
-                      sdap_exop_modify_passwd_done, req, 5, &state->op);
+                      sdap_exop_modify_passwd_done, req, timeout, &state->op);
     if (ret) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to set up operation!\n");
         ret = ERR_INTERNAL;
@@ -645,8 +648,14 @@ static void sdap_exop_modify_passwd_done(struct sdap_op *op,
         ret = EOK;
         break;
     case LDAP_CONSTRAINT_VIOLATION:
-        state->user_error_message = talloc_strdup(state,
-            "Please make sure the password meets the complexity constraints.");
+        if (errmsg && strlen(errmsg) != 0) {
+            state->user_error_message = talloc_strdup(state, errmsg);
+        } else {
+            state->user_error_message = talloc_strdup(state,
+                "Please make sure the password meets the "
+                "complexity constraints.");
+        }
+
         if (state->user_error_message == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed\n");
             ret = ENOMEM;
@@ -1137,7 +1146,6 @@ struct sdap_get_generic_ext_state {
     const char *filter;
     const char **attrs;
     int timeout;
-    int attrsonly;
     int sizelimit;
 
     struct sdap_op *op;
@@ -1148,10 +1156,13 @@ struct sdap_get_generic_ext_state {
     int nserverctrls;
     LDAPControl **clientctrls;
 
+    size_t ref_count;
+    char **refs;
+
     sdap_parse_cb parse_cb;
     void *cb_data;
 
-    bool allow_paging;
+    unsigned int flags;
 };
 
 static errno_t sdap_get_generic_ext_step(struct tevent_req *req);
@@ -1160,6 +1171,17 @@ static void sdap_get_generic_op_finished(struct sdap_op *op,
                                          struct sdap_msg *reply,
                                          int error, void *pvt);
 
+enum {
+    /* Be silent about exceeded size limit */
+    SDAP_SRCH_FLG_SIZELIMIT_SILENT = 1 << 0,
+
+    /* Allow paging */
+    SDAP_SRCH_FLG_PAGING           = 1 << 1,
+
+    /* Only attribute descriptions are requested */
+    SDAP_SRCH_FLG_ATTRS_ONLY       = 1 << 2,
+};
+
 static struct tevent_req *
 sdap_get_generic_ext_send(TALLOC_CTX *memctx,
                           struct tevent_context *ev,
@@ -1169,14 +1191,13 @@ sdap_get_generic_ext_send(TALLOC_CTX *memctx,
                           int scope,
                           const char *filter,
                           const char **attrs,
-                          int attrsonly,
                           LDAPControl **serverctrls,
                           LDAPControl **clientctrls,
                           int sizelimit,
                           int timeout,
-                          bool allow_paging,
                           sdap_parse_cb parse_cb,
-                          void *cb_data)
+                          void *cb_data,
+                          unsigned int flags)
 {
     errno_t ret;
     struct sdap_get_generic_ext_state *state;
@@ -1194,7 +1215,6 @@ sdap_get_generic_ext_send(TALLOC_CTX *memctx,
     state->scope = scope;
     state->filter = filter;
     state->attrs = attrs;
-    state->attrsonly = attrsonly;
     state->op = NULL;
     state->sizelimit = sizelimit;
     state->timeout = timeout;
@@ -1203,6 +1223,7 @@ sdap_get_generic_ext_send(TALLOC_CTX *memctx,
     state->parse_cb = parse_cb;
     state->cb_data = cb_data;
     state->clientctrls = clientctrls;
+    state->flags = flags;
 
     if (state->sh == NULL || state->sh->ldap == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE,
@@ -1217,10 +1238,11 @@ sdap_get_generic_ext_send(TALLOC_CTX *memctx,
     /* Be extra careful and never allow paging for BASE searches,
      * even if requested.
      */
-    if (scope == LDAP_SCOPE_BASE) {
-        state->allow_paging = false;
-    } else {
-        state->allow_paging = allow_paging;
+    if (scope == LDAP_SCOPE_BASE && (flags & SDAP_SRCH_FLG_PAGING)) {
+        /* Disable paging */
+        flags &= ~SDAP_SRCH_FLG_PAGING;
+        DEBUG(SSSDBG_TRACE_FUNC,
+              "WARNING: Disabling paging because scope is set to base.\n");
     }
 
     /* Also check for deref/asq requests and force
@@ -1231,7 +1253,7 @@ sdap_get_generic_ext_send(TALLOC_CTX *memctx,
                                 serverctrls,
                                 NULL);
     if (control) {
-        state->allow_paging = true;
+        flags |= SDAP_SRCH_FLG_PAGING;
     }
 
     /* ASQ */
@@ -1239,7 +1261,7 @@ sdap_get_generic_ext_send(TALLOC_CTX *memctx,
                                 serverctrls,
                                 NULL);
     if (control) {
-        state->allow_paging = true;
+        flags |= SDAP_SRCH_FLG_PAGING;
     }
 
     for (state->nserverctrls=0;
@@ -1307,7 +1329,7 @@ static errno_t sdap_get_generic_ext_step(struct tevent_req *req)
     disable_paging = dp_opt_get_bool(state->opts->basic, SDAP_DISABLE_PAGING);
 
     if (!disable_paging
-            && state->allow_paging
+            && (state->flags & SDAP_SRCH_FLG_PAGING)
             && sdap_is_control_supported(state->sh,
                                          LDAP_CONTROL_PAGEDRESULTS)) {
         lret = ldap_create_page_control(state->sh->ldap,
@@ -1328,7 +1350,8 @@ static errno_t sdap_get_generic_ext_step(struct tevent_req *req)
     lret = ldap_search_ext(state->sh->ldap, state->search_base,
                            state->scope, state->filter,
                            discard_const(state->attrs),
-                           state->attrsonly, state->serverctrls,
+                           (state->flags & SDAP_SRCH_FLG_ATTRS_ONLY),
+                           state->serverctrls,
                            state->clientctrls, NULL, state->sizelimit, &msgid);
     ldap_control_free(page_control);
     state->serverctrls[state->nserverctrls] = NULL;
@@ -1369,6 +1392,47 @@ done:
     return ret;
 }
 
+static errno_t
+sdap_get_generic_ext_add_references(struct sdap_get_generic_ext_state *state,
+                                    char **refs)
+{
+    int i;
+
+    if (refs == NULL) {
+        /* Rare, but it's possible that we might get a reference result with
+         * no references attached.
+         */
+        return EOK;
+    }
+
+    for (i = 0; refs[i]; i++) {
+        DEBUG(SSSDBG_TRACE_LIBS, "Additional References: %s\n", refs[i]);
+    }
+
+    /* Extend the size of the ref array */
+    state->refs = talloc_realloc(state, state->refs, char *,
+                                 state->ref_count + i);
+    if (state->refs == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "talloc_realloc failed extending ref_array.\n");
+        return ENOMEM;
+    }
+
+    /* Copy in all the references */
+    for (i = 0; refs[i]; i++) {
+        state->refs[state->ref_count + i] =
+                talloc_strdup(state->refs, refs[i]);
+
+        if (state->refs[state->ref_count + i] == NULL) {
+            return ENOMEM;
+        }
+    }
+
+    state->ref_count += i;
+
+    return EOK;
+}
+
 static void sdap_get_generic_op_finished(struct sdap_op *op,
                                          struct sdap_msg *reply,
                                          int error, void *pvt)
@@ -1377,7 +1441,6 @@ static void sdap_get_generic_op_finished(struct sdap_op *op,
     struct sdap_get_generic_ext_state *state = tevent_req_data(req,
                                             struct sdap_get_generic_ext_state);
     char *errmsg = NULL;
-    int i;
     char **refs = NULL;
     int result;
     int ret;
@@ -1394,8 +1457,27 @@ static void sdap_get_generic_op_finished(struct sdap_op *op,
 
     switch (ldap_msgtype(reply->msg)) {
     case LDAP_RES_SEARCH_REFERENCE:
-        /* ignore references for now */
-        talloc_free(reply);
+        ret = ldap_parse_reference(state->sh->ldap, reply->msg,
+                                   &refs, NULL, 0);
+        if (ret != LDAP_SUCCESS) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "ldap_parse_reference failed (%d)\n", state->op->msgid);
+            tevent_req_error(req, EIO);
+            return;
+        }
+
+        ret = sdap_get_generic_ext_add_references(state, refs);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "sdap_get_generic_ext_add_references failed: %s(%d)\n",
+                  sss_strerror(ret), ret);
+            ldap_memvfree((void **)refs);
+            tevent_req_error(req, ret);
+            return;
+        }
+
+        /* Remove the original strings */
+        ldap_memvfree((void **)refs);
 
         /* unlock the operation so that we can proceed with the next result */
         sdap_unlock_next_reply(state->op);
@@ -1429,8 +1511,12 @@ static void sdap_get_generic_op_finished(struct sdap_op *op,
 
         if (result == LDAP_SIZELIMIT_EXCEEDED) {
             /* Try to return what we've got */
-            DEBUG(SSSDBG_MINOR_FAILURE,
-                  "LDAP sizelimit was exceeded, returning incomplete data\n");
+
+            if ( ! (state->flags & SDAP_SRCH_FLG_SIZELIMIT_SILENT)) {
+                DEBUG(SSSDBG_MINOR_FAILURE,
+                      "LDAP sizelimit was exceeded, "
+                      "returning incomplete data\n");
+            }
         } else if (result == LDAP_INAPPROPRIATE_MATCHING) {
             /* This error should only occur when we're testing for
              * specialized functionality like the ldap matching rule
@@ -1448,15 +1534,14 @@ static void sdap_get_generic_op_finished(struct sdap_op *op,
             tevent_req_error(req, ENOTSUP);
             return;
         } else if (result == LDAP_REFERRAL) {
-            if (refs != NULL) {
-                for (i = 0; refs[i]; i++) {
-                    DEBUG(SSSDBG_TRACE_LIBS, "Ref: %s\n", refs[i]);
-                }
-                ldap_memvfree((void **) refs);
+            ret = sdap_get_generic_ext_add_references(state, refs);
+            if (ret != EOK) {
+                DEBUG(SSSDBG_OP_FAILURE,
+                      "sdap_get_generic_ext_add_references failed: %s(%d)\n",
+                      sss_strerror(ret), ret);
+                tevent_req_error(req, ret);
             }
-            ldap_memfree(errmsg);
-            tevent_req_error(req, ERR_REFERRAL);
-            return;
+            /* For referrals, we need to fall through as if it was LDAP_SUCCESS */
         } else if (result != LDAP_SUCCESS && result != LDAP_NO_SUCH_OBJECT) {
             DEBUG(SSSDBG_OP_FAILURE,
                   "Unexpected result from ldap: %s(%d), %s\n",
@@ -1526,27 +1611,40 @@ static void sdap_get_generic_op_finished(struct sdap_op *op,
 }
 
 static int
-sdap_get_generic_ext_recv(struct tevent_req *req)
+sdap_get_generic_ext_recv(struct tevent_req *req,
+                          TALLOC_CTX *mem_ctx,
+                          size_t *ref_count,
+                          char ***refs)
 {
+    struct sdap_get_generic_ext_state *state =
+            tevent_req_data(req, struct sdap_get_generic_ext_state);
+
     TEVENT_REQ_RETURN_ON_ERROR(req);
+
+    if (ref_count) {
+        *ref_count = state->ref_count;
+    }
+
+    if (refs) {
+        *refs = talloc_steal(mem_ctx, state->refs);
+    }
+
     return EOK;
 }
 
+/* This search handler can be used by most calls */
 static void generic_ext_search_handler(struct tevent_req *subreq,
                                        struct sdap_options *opts)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                       struct tevent_req);
     int ret;
+    size_t ref_count, i;
+    char **refs;
 
-    ret = sdap_get_generic_ext_recv(subreq);
+    ret = sdap_get_generic_ext_recv(subreq, req, &ref_count, &refs);
     talloc_zfree(subreq);
-    if (ret == ERR_REFERRAL) {
-        if (dp_opt_get_bool(opts->basic, SDAP_REFERRALS)) {
-            tevent_req_error(req, ret);
-            return;
-        }
-    } else if (ret) {
+    if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE,
               "sdap_get_generic_ext_recv failed [%d]: %s\n",
               ret, sss_strerror(ret));
@@ -1554,11 +1652,24 @@ static void generic_ext_search_handler(struct tevent_req *subreq,
         return;
     }
 
+    if (ref_count > 0) {
+        /* We will ignore referrals in the generic handler */
+        DEBUG(SSSDBG_TRACE_ALL,
+              "Request included referrals which were ignored.\n");
+        if (debug_level & SSSDBG_TRACE_ALL) {
+            for(i = 0; i < ref_count; i++) {
+                DEBUG(SSSDBG_TRACE_ALL,
+                      "    Ref: %s\n", refs[i]);
+            }
+        }
+    }
+
+    talloc_free(refs);
     tevent_req_done(req);
 }
 
-/* ==Generic Search============================================ */
-struct sdap_get_generic_state {
+/* ==Generic Search exposing all options======================= */
+struct sdap_get_and_parse_generic_state {
     struct sdap_attr_map *map;
     int map_num_attrs;
 
@@ -1566,56 +1677,71 @@ struct sdap_get_generic_state {
     struct sdap_options *opts;
 };
 
-static void sdap_get_generic_done(struct tevent_req *subreq);
-static errno_t sdap_get_generic_parse_entry(struct sdap_handle *sh,
-                                            struct sdap_msg *msg,
-                                            void *pvt);
+static void sdap_get_and_parse_generic_done(struct tevent_req *subreq);
+static errno_t sdap_get_and_parse_generic_parse_entry(struct sdap_handle *sh,
+                                                      struct sdap_msg *msg,
+                                                      void *pvt);
 
-struct tevent_req *sdap_get_generic_send(TALLOC_CTX *memctx,
-                                         struct tevent_context *ev,
-                                         struct sdap_options *opts,
-                                         struct sdap_handle *sh,
-                                         const char *search_base,
-                                         int scope,
-                                         const char *filter,
-                                         const char **attrs,
-                                         struct sdap_attr_map *map,
-                                         int map_num_attrs,
-                                         int timeout,
-                                         bool allow_paging)
+struct tevent_req *sdap_get_and_parse_generic_send(TALLOC_CTX *memctx,
+                                                   struct tevent_context *ev,
+                                                   struct sdap_options *opts,
+                                                   struct sdap_handle *sh,
+                                                   const char *search_base,
+                                                   int scope,
+                                                   const char *filter,
+                                                   const char **attrs,
+                                                   struct sdap_attr_map *map,
+                                                   int map_num_attrs,
+                                                   int attrsonly,
+                                                   LDAPControl **serverctrls,
+                                                   LDAPControl **clientctrls,
+                                                   int sizelimit,
+                                                   int timeout,
+                                                   bool allow_paging)
 {
     struct tevent_req *req = NULL;
     struct tevent_req *subreq = NULL;
-    struct sdap_get_generic_state *state = NULL;
+    struct sdap_get_and_parse_generic_state *state = NULL;
+    unsigned int flags = 0;
 
-    req = tevent_req_create(memctx, &state, struct sdap_get_generic_state);
+    req = tevent_req_create(memctx, &state,
+                            struct sdap_get_and_parse_generic_state);
     if (!req) return NULL;
 
     state->map = map;
     state->map_num_attrs = map_num_attrs;
     state->opts = opts;
 
+    if (allow_paging) {
+        flags |= SDAP_SRCH_FLG_PAGING;
+    }
+
+    if (attrsonly) {
+        flags |= SDAP_SRCH_FLG_ATTRS_ONLY;
+    }
+
     subreq = sdap_get_generic_ext_send(state, ev, opts, sh, search_base,
-                                       scope, filter, attrs, false, NULL,
-                                       NULL, 0, timeout, allow_paging,
-                                       sdap_get_generic_parse_entry, state);
+                                       scope, filter, attrs, serverctrls,
+                                       clientctrls, sizelimit, timeout,
+                                       sdap_get_and_parse_generic_parse_entry,
+                                       state, flags);
     if (!subreq) {
         talloc_zfree(req);
         return NULL;
     }
-    tevent_req_set_callback(subreq, sdap_get_generic_done, req);
+    tevent_req_set_callback(subreq, sdap_get_and_parse_generic_done, req);
 
     return req;
 }
 
-static errno_t sdap_get_generic_parse_entry(struct sdap_handle *sh,
-                                            struct sdap_msg *msg,
-                                            void *pvt)
+static errno_t sdap_get_and_parse_generic_parse_entry(struct sdap_handle *sh,
+                                                      struct sdap_msg *msg,
+                                                      void *pvt)
 {
     errno_t ret;
     struct sysdb_attrs *attrs;
-    struct sdap_get_generic_state *state =
-                talloc_get_type(pvt, struct sdap_get_generic_state);
+    struct sdap_get_and_parse_generic_state *state =
+                talloc_get_type(pvt, struct sdap_get_and_parse_generic_state);
 
     bool disable_range_rtrvl = dp_opt_get_bool(state->opts->basic,
                                                SDAP_DISABLE_RANGE_RETRIEVAL);
@@ -1640,14 +1766,89 @@ static errno_t sdap_get_generic_parse_entry(struct sdap_handle *sh,
     return EOK;
 }
 
+static void sdap_get_and_parse_generic_done(struct tevent_req *subreq)
+{
+    struct tevent_req *req = tevent_req_callback_data(subreq,
+                                                      struct tevent_req);
+    struct sdap_get_and_parse_generic_state *state =
+                tevent_req_data(req, struct sdap_get_and_parse_generic_state);
+
+    return generic_ext_search_handler(subreq, state->opts);
+}
+
+int sdap_get_and_parse_generic_recv(struct tevent_req *req,
+                                    TALLOC_CTX *mem_ctx,
+                                    size_t *reply_count,
+                                    struct sysdb_attrs ***reply)
+{
+    struct sdap_get_and_parse_generic_state *state = tevent_req_data(req,
+                                     struct sdap_get_and_parse_generic_state);
+
+    TEVENT_REQ_RETURN_ON_ERROR(req);
+
+    *reply_count = state->sreply.reply_count;
+    *reply = talloc_steal(mem_ctx, state->sreply.reply);
+
+    return EOK;
+}
+
+
+/* ==Simple generic search============================================== */
+struct sdap_get_generic_state {
+    size_t reply_count;
+    struct sysdb_attrs **reply;
+};
+
+static void sdap_get_generic_done(struct tevent_req *subreq);
+
+struct tevent_req *sdap_get_generic_send(TALLOC_CTX *memctx,
+                                         struct tevent_context *ev,
+                                         struct sdap_options *opts,
+                                         struct sdap_handle *sh,
+                                         const char *search_base,
+                                         int scope,
+                                         const char *filter,
+                                         const char **attrs,
+                                         struct sdap_attr_map *map,
+                                         int map_num_attrs,
+                                         int timeout,
+                                         bool allow_paging)
+{
+    struct tevent_req *req = NULL;
+    struct tevent_req *subreq = NULL;
+    struct sdap_get_generic_state *state = NULL;
+
+    req = tevent_req_create(memctx, &state, struct sdap_get_generic_state);
+    if (!req) return NULL;
+
+    subreq = sdap_get_and_parse_generic_send(memctx, ev, opts, sh, search_base,
+                                             scope, filter, attrs,
+                                             map, map_num_attrs,
+                                             false, NULL, NULL, 0, timeout,
+                                             allow_paging);
+    if (subreq == NULL) {
+        return NULL;
+    }
+    tevent_req_set_callback(subreq, sdap_get_generic_done, req);
+
+    return req;
+}
+
 static void sdap_get_generic_done(struct tevent_req *subreq)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                       struct tevent_req);
     struct sdap_get_generic_state *state =
                 tevent_req_data(req, struct sdap_get_generic_state);
+    errno_t ret;
 
-    return generic_ext_search_handler(subreq, state->opts);
+    ret = sdap_get_and_parse_generic_recv(subreq, state,
+                                          &state->reply_count, &state->reply);
+    if (ret != EOK) {
+        tevent_req_error(req, ret);
+        return;
+    }
+    tevent_req_done(req);
 }
 
 int sdap_get_generic_recv(struct tevent_req *req,
@@ -1655,13 +1856,13 @@ int sdap_get_generic_recv(struct tevent_req *req,
                           size_t *reply_count,
                           struct sysdb_attrs ***reply)
 {
-    struct sdap_get_generic_state *state = tevent_req_data(req,
-                                            struct sdap_get_generic_state);
+    struct sdap_get_generic_state *state =
+                tevent_req_data(req, struct sdap_get_generic_state);
 
     TEVENT_REQ_RETURN_ON_ERROR(req);
 
-    *reply_count = state->sreply.reply_count;
-    *reply = talloc_steal(mem_ctx, state->sreply.reply);
+    *reply_count = state->reply_count;
+    *reply = talloc_steal(mem_ctx, state->reply);
 
     return EOK;
 }
@@ -1732,9 +1933,9 @@ sdap_x_deref_search_send(TALLOC_CTX *memctx, struct tevent_context *ev,
                                        filter == NULL ? LDAP_SCOPE_BASE
                                                       : LDAP_SCOPE_SUBTREE,
                                        filter, attrs,
-                                       false, state->ctrls, NULL, 0, timeout,
-                                       true, sdap_x_deref_parse_entry,
-                                       state);
+                                       state->ctrls, NULL, 0, timeout,
+                                       sdap_x_deref_parse_entry,
+                                       state, SDAP_SRCH_FLG_PAGING);
     if (!subreq) {
         talloc_zfree(req);
         return NULL;
@@ -1908,6 +2109,10 @@ struct sdap_sd_search_state {
   size_t reply_count;
   struct sysdb_attrs **reply;
   struct sdap_reply sreply;
+
+  /* Referrals returned by the search */
+  size_t ref_count;
+  char **refs;
 };
 
 static int sdap_sd_search_create_control(struct sdap_handle *sh,
@@ -1952,9 +2157,9 @@ sdap_sd_search_send(TALLOC_CTX *memctx, struct tevent_context *ev,
     DEBUG(SSSDBG_TRACE_FUNC, "Searching entry [%s] using SD\n", base_dn);
     subreq = sdap_get_generic_ext_send(state, ev, opts, sh, base_dn,
                                        LDAP_SCOPE_BASE, "(objectclass=*)", attrs,
-                                       false, state->ctrls, NULL, 0, timeout,
-                                       true, sdap_sd_search_parse_entry,
-                                       state);
+                                       state->ctrls, NULL, 0, timeout,
+                                       sdap_sd_search_parse_entry,
+                                       state, SDAP_SRCH_FLG_PAGING);
     if (!subreq) {
         ret = EIO;
         goto fail;
@@ -2039,12 +2244,26 @@ static errno_t sdap_sd_search_parse_entry(struct sdap_handle *sh,
 
 static void sdap_sd_search_done(struct tevent_req *subreq)
 {
+    int ret;
+
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                       struct tevent_req);
     struct sdap_sd_search_state *state =
                 tevent_req_data(req, struct sdap_sd_search_state);
 
-    return generic_ext_search_handler(subreq, state->opts);
+    ret = sdap_get_generic_ext_recv(subreq, state,
+                                    &state->ref_count,
+                                    &state->refs);
+    talloc_zfree(subreq);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "sdap_get_generic_ext_recv failed [%d]: %s\n",
+              ret, sss_strerror(ret));
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    tevent_req_done(req);
 }
 
 static int sdap_sd_search_ctrls_destructor(void *ptr)
@@ -2060,7 +2279,9 @@ static int sdap_sd_search_ctrls_destructor(void *ptr)
 int sdap_sd_search_recv(struct tevent_req *req,
                         TALLOC_CTX *mem_ctx,
                         size_t *_reply_count,
-                        struct sysdb_attrs ***_reply)
+                        struct sysdb_attrs ***_reply,
+                        size_t *_ref_count,
+                        char ***_refs)
 {
     struct sdap_sd_search_state *state = tevent_req_data(req,
                                             struct sdap_sd_search_state);
@@ -2069,6 +2290,14 @@ int sdap_sd_search_recv(struct tevent_req *req,
     *_reply_count = state->sreply.reply_count;
     *_reply = talloc_steal(mem_ctx, state->sreply.reply);
 
+    if(_ref_count) {
+        *_ref_count = state->ref_count;
+    }
+
+    if (_refs) {
+        *_refs = talloc_steal(mem_ctx, state->refs);
+    }
+
     return EOK;
 }
 
@@ -2127,9 +2356,9 @@ sdap_asq_search_send(TALLOC_CTX *memctx, struct tevent_context *ev,
     DEBUG(SSSDBG_TRACE_FUNC, "Dereferencing entry [%s] using ASQ\n", base_dn);
     subreq = sdap_get_generic_ext_send(state, ev, opts, sh, base_dn,
                                        LDAP_SCOPE_BASE, NULL, attrs,
-                                       false, state->ctrls, NULL, 0, timeout,
-                                       true, sdap_asq_search_parse_entry,
-                                       state);
+                                       state->ctrls, NULL, 0, timeout,
+                                       sdap_asq_search_parse_entry,
+                                       state, SDAP_SRCH_FLG_PAGING);
     if (!subreq) {
         talloc_zfree(req);
         return NULL;
@@ -2373,9 +2602,12 @@ sdap_posix_check_send(TALLOC_CTX *memctx, struct tevent_context *ev,
     state->attrs[2] = opts->group_map[SDAP_AT_GROUP_GID].name;
     state->attrs[3] = NULL;
 
-    state->filter = talloc_asprintf(state, "(|(%s=*)(%s=*))",
+    state->filter = talloc_asprintf(state,
+        "(|(&(%s=*)(objectclass=%s))(&(%s=*)(objectclass=%s)))",
                                     opts->user_map[SDAP_AT_USER_UID].name,
-                                    opts->group_map[SDAP_AT_GROUP_GID].name);
+                                    opts->user_map[SDAP_OC_USER].name,
+                                    opts->group_map[SDAP_AT_GROUP_GID].name,
+                                    opts->group_map[SDAP_OC_GROUP].name);
     if (state->filter == NULL) {
         ret = ENOMEM;
         goto fail;
@@ -2408,10 +2640,10 @@ static errno_t sdap_posix_check_next(struct tevent_req *req)
                                  state->sh,
                                  state->search_bases[state->base_iter]->basedn,
                                  LDAP_SCOPE_SUBTREE, state->filter,
-                                 state->attrs, false,
+                                 state->attrs,
                                  NULL, NULL, 1, state->timeout,
-                                 false, sdap_posix_check_parse,
-                                 state);
+                                 sdap_posix_check_parse, state,
+                                 SDAP_SRCH_FLG_SIZELIMIT_SILENT);
     if (subreq == NULL) {
         return ENOMEM;
     }
@@ -2458,9 +2690,8 @@ static errno_t sdap_posix_check_parse(struct sdap_handle *sh,
     errno = 0;
     strtouint32(vals[0]->bv_val, &endptr, 10);
     if (errno || *endptr || (vals[0]->bv_val == endptr)) {
-        DEBUG(SSSDBG_OP_FAILURE,
+        DEBUG(SSSDBG_MINOR_FAILURE,
               "POSIX attribute is not a number: %s\n", vals[0]->bv_val);
-        goto done;
     }
 
     state->has_posix = true;
@@ -2477,7 +2708,7 @@ static void sdap_posix_check_done(struct tevent_req *subreq)
         tevent_req_data(req, struct sdap_posix_check_state);
     errno_t ret;
 
-    ret = sdap_get_generic_ext_recv(subreq);
+    ret = sdap_get_generic_ext_recv(subreq, NULL, NULL, NULL);
     talloc_zfree(subreq);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE,
@@ -2532,6 +2763,7 @@ struct sdap_deref_search_state {
     size_t reply_count;
     struct sdap_deref_attrs **reply;
     enum sdap_deref_type deref_type;
+    unsigned flags;
 };
 
 static void sdap_deref_search_done(struct tevent_req *subreq);
@@ -2548,7 +2780,8 @@ sdap_deref_search_with_filter_send(TALLOC_CTX *memctx,
                                    const char **attrs,
                                    int num_maps,
                                    struct sdap_attr_map_info *maps,
-                                   int timeout)
+                                   int timeout,
+                                   unsigned flags)
 {
     struct tevent_req *req = NULL;
     struct tevent_req *subreq = NULL;
@@ -2560,6 +2793,7 @@ sdap_deref_search_with_filter_send(TALLOC_CTX *memctx,
     state->sh = sh;
     state->reply_count = 0;
     state->reply = NULL;
+    state->flags = flags;
 
     if (sdap_is_control_supported(sh, LDAP_CONTROL_X_DEREF)) {
         DEBUG(SSSDBG_TRACE_INTERNAL, "Server supports OpenLDAP deref\n");
@@ -2686,14 +2920,20 @@ static void sdap_deref_search_done(struct tevent_req *subreq)
         DEBUG(SSSDBG_OP_FAILURE,
               "dereference processing failed [%d]: %s\n", ret, strerror(ret));
         if (ret == ENOTSUP) {
-            sss_log(SSS_LOG_WARNING,
-                "LDAP server claims to support deref, but deref search failed. "
-                "Disabling deref for further requests. You can permanently "
-                "disable deref by setting ldap_deref_threshold to 0 in domain "
-                "configuration.");
             state->sh->disable_deref = true;
-        } else {
-            sss_log(SSS_LOG_WARNING, "dereference processing failed : %s", strerror(ret));
+        }
+
+        if (!(state->flags & SDAP_DEREF_FLG_SILENT)) {
+            if (ret == ENOTSUP) {
+                sss_log(SSS_LOG_WARNING,
+                        "LDAP server claims to support deref, but deref search "
+                        "failed. Disabling deref for further requests. You can "
+                        "permanently disable deref by setting "
+                        "ldap_deref_threshold to 0 in domain configuration.");
+            } else {
+                sss_log(SSS_LOG_WARNING,
+                        "dereference processing failed : %s", strerror(ret));
+            }
         }
         tevent_req_error(req, ret);
         return;
