@@ -75,6 +75,12 @@ struct tevent_req *sdap_connect_send(TALLOC_CTX *memctx,
     req = tevent_req_create(memctx, &state, struct sdap_connect_state);
     if (!req) return NULL;
 
+    if (uri == NULL || sockaddr == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Invalid uri or sockaddr\n");
+        ret = EINVAL;
+        goto fail;
+    }
+
     state->reply = talloc(state, struct sdap_msg);
     if (!state->reply) {
         talloc_zfree(req);
@@ -146,14 +152,18 @@ static void sdap_sys_connect_done(struct tevent_req *subreq)
     ret = sss_ldap_init_recv(subreq, &state->sh->ldap, &sd);
     talloc_zfree(subreq);
     if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "sdap_async_connect_call request failed.\n");
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "sdap_async_connect_call request failed: [%d]: %s.\n",
+              ret, sss_strerror(ret));
         tevent_req_error(req, ret);
         return;
     }
 
     ret = setup_ldap_connection_callbacks(state->sh, state->ev);
     if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "setup_ldap_connection_callbacks failed.\n");
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "setup_ldap_connection_callbacks failed: [%d]: %s.\n",
+              ret, sss_strerror(ret));
         goto fail;
     }
 
@@ -314,9 +324,10 @@ static void sdap_sys_connect_done(struct tevent_req *subreq)
     ret = sdap_set_connected(state->sh, state->ev);
     if (ret) goto fail;
 
-    /* FIXME: get timeouts from configuration, for now 5 secs. */
     ret = sdap_op_add(state, state->ev, state->sh, msgid,
-                      sdap_connect_done, req, 5, &state->op);
+                      sdap_connect_done, req,
+                      dp_opt_get_int(state->opts->basic, SDAP_OPT_TIMEOUT),
+                      &state->op);
     if (ret) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to set up operation!\n");
         goto fail;
@@ -625,6 +636,7 @@ static void simple_bind_done(struct sdap_op *op,
 static struct tevent_req *simple_bind_send(TALLOC_CTX *memctx,
                                            struct tevent_context *ev,
                                            struct sdap_handle *sh,
+                                           int timeout,
                                            const char *user_dn,
                                            struct berval *pw)
 {
@@ -686,9 +698,8 @@ static struct tevent_req *simple_bind_send(TALLOC_CTX *memctx,
         if (ret) goto fail;
     }
 
-    /* FIXME: get timeouts from configuration, for now 5 secs. */
     ret = sdap_op_add(state, ev, sh, msgid,
-                      simple_bind_done, req, 5, &state->op);
+                      simple_bind_done, req, timeout, &state->op);
     if (ret) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to set up operation!\n");
         goto fail;
@@ -743,6 +754,12 @@ static void simple_bind_done(struct sdap_op *op,
 
     if (result == LDAP_SUCCESS) {
         ret = EOK;
+    } else if (result == LDAP_INVALID_CREDENTIALS
+                   && errmsg != NULL && strstr(errmsg, "data 775,") != NULL) {
+        /* Value 775 is described in
+         * https://msdn.microsoft.com/en-us/library/windows/desktop/ms681386%28v=vs.85%29.aspx
+         * for more details please see commit message. */
+        ret = ERR_ACCOUNT_LOCKED;
     } else {
         ret = ERR_AUTH_FAILED;
     }
@@ -1091,16 +1108,6 @@ struct tevent_req *sdap_kinit_send(TALLOC_CTX *memctx,
     state->lifetime = lifetime;
     state->krb_service_name = krb_service_name;
 
-    if (keytab) {
-        ret = setenv("KRB5_KTNAME", keytab, 1);
-        if (ret == -1) {
-            DEBUG(SSSDBG_OP_FAILURE,
-                  "Failed to set KRB5_KTNAME to %s\n", keytab);
-            talloc_free(req);
-            return NULL;
-        }
-    }
-
     if (canonicalize) {
         ret = setenv("KRB5_CANONICALIZE", "true", 1);
     } else {
@@ -1153,7 +1160,7 @@ static void sdap_kinit_kdc_resolved(struct tevent_req *subreq)
     struct tevent_req *tgtreq;
     int ret;
 
-    ret = be_resolve_server_recv(subreq, &state->kdc_srv);
+    ret = be_resolve_server_recv(subreq, state, &state->kdc_srv);
     talloc_zfree(subreq);
     if (ret != EOK) {
         /* all servers have been tried and none
@@ -1276,7 +1283,8 @@ struct tevent_req *sdap_auth_send(TALLOC_CTX *memctx,
                                   const char *sasl_mech,
                                   const char *sasl_user,
                                   const char *user_dn,
-                                  struct sss_auth_token *authtok)
+                                  struct sss_auth_token *authtok,
+                                  int simple_bind_timeout)
 {
     struct tevent_req *req, *subreq;
     struct sdap_auth_state *state;
@@ -1312,7 +1320,7 @@ struct tevent_req *sdap_auth_send(TALLOC_CTX *memctx,
         pw.bv_len = pwlen;
 
         state->is_sasl = false;
-        subreq = simple_bind_send(state, ev, sh, user_dn, &pw);
+        subreq = simple_bind_send(state, ev, sh, simple_bind_timeout, user_dn, &pw);
         if (!subreq) {
             tevent_req_error(req, ENOMEM);
             return tevent_req_post(req, ev);
@@ -1512,7 +1520,7 @@ static void sdap_cli_resolve_done(struct tevent_req *subreq)
                                              struct sdap_cli_connect_state);
     int ret;
 
-    ret = be_resolve_server_recv(subreq, &state->srv);
+    ret = be_resolve_server_recv(subreq, state, &state->srv);
     talloc_zfree(subreq);
     if (ret) {
         state->srv = NULL;
@@ -1716,12 +1724,6 @@ static void sdap_cli_kinit_step(struct tevent_req *req)
     struct sdap_cli_connect_state *state = tevent_req_data(req,
                                              struct sdap_cli_connect_state);
     struct tevent_req *subreq;
-    const char *realm;
-
-    realm = dp_opt_get_string(state->opts->basic, SDAP_SASL_REALM);
-    if (!realm) {
-        realm = dp_opt_get_string(state->opts->basic, SDAP_KRB5_REALM);
-    }
 
     subreq = sdap_kinit_send(state, state->ev,
                              state->be,
@@ -1733,7 +1735,7 @@ static void sdap_cli_kinit_step(struct tevent_req *req)
                                                    SDAP_KRB5_KEYTAB),
                         dp_opt_get_string(state->opts->basic,
                                                    SDAP_SASL_AUTHID),
-                        realm,
+                        sdap_gssapi_realm(state->opts->basic),
                         dp_opt_get_bool(state->opts->basic,
                                                    SDAP_KRB5_CANONICALIZE),
                         dp_opt_get_int(state->opts->basic,
@@ -1853,7 +1855,9 @@ static void sdap_cli_auth_step(struct tevent_req *req)
                             state->sh, sasl_mech,
                             dp_opt_get_string(state->opts->basic,
                                               SDAP_SASL_AUTHID),
-                            user_dn, authtok);
+                            user_dn, authtok,
+                            dp_opt_get_int(state->opts->basic,
+                                           SDAP_OPT_TIMEOUT));
     if (!subreq) {
         tevent_req_error(req, ENOMEM);
         return;
