@@ -64,7 +64,7 @@ static int sdap_find_entry_by_origDN(TALLOC_CTX *memctx,
         goto done;
     }
 
-    base_dn = sysdb_domain_dn(ctx, tmpctx, domain);
+    base_dn = sysdb_domain_dn(tmpctx, domain);
     if (!base_dn) {
         ret = ENOMEM;
         goto done;
@@ -109,7 +109,7 @@ done:
 }
 
 static errno_t
-sdap_get_members_with_primary_gid(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
+sdap_get_members_with_primary_gid(TALLOC_CTX *mem_ctx,
                                   struct sss_domain_info *domain,
                                   gid_t gid, char ***_localdn, size_t *_ndn)
 {
@@ -130,7 +130,7 @@ sdap_get_members_with_primary_gid(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
         return ENOMEM;
     }
 
-    ret = sysdb_search_users(mem_ctx, sysdb, domain, filter,
+    ret = sysdb_search_users(mem_ctx, domain, filter,
                              search_attrs, &count, &msgs);
     talloc_free(filter);
     if (ret == ENOENT) {
@@ -165,7 +165,7 @@ sdap_get_members_with_primary_gid(TALLOC_CTX *mem_ctx, struct sysdb_ctx *sysdb,
 
 static errno_t
 sdap_dn_by_primary_gid(TALLOC_CTX *mem_ctx, struct sysdb_attrs *ldap_attrs,
-                       struct sysdb_ctx *sysdb, struct sss_domain_info *domain,
+                       struct sss_domain_info *domain,
                        struct sdap_options *opts,
                        char ***_dn_list, size_t *_count)
 {
@@ -184,7 +184,7 @@ sdap_dn_by_primary_gid(TALLOC_CTX *mem_ctx, struct sysdb_attrs *ldap_attrs,
         return ret;
     }
 
-    ret = sdap_get_members_with_primary_gid(mem_ctx, sysdb, domain, gid,
+    ret = sdap_get_members_with_primary_gid(mem_ctx, domain, gid,
                                             _dn_list, _count);
     if (ret) return ret;
 
@@ -341,8 +341,7 @@ done:
     /* FIXME: support storing additional attributes */
 
 static errno_t
-sdap_store_group_with_gid(struct sysdb_ctx *ctx,
-                          struct sss_domain_info *domain,
+sdap_store_group_with_gid(struct sss_domain_info *domain,
                           const char *name,
                           gid_t gid,
                           struct sysdb_attrs *group_attrs,
@@ -363,8 +362,8 @@ sdap_store_group_with_gid(struct sysdb_ctx *ctx,
         }
     }
 
-    ret = sysdb_store_group(ctx, domain, name, gid,
-                            group_attrs, cache_timeout, now);
+    ret = sysdb_store_group(domain, name, gid, group_attrs,
+                            cache_timeout, now);
     if (ret) {
         DEBUG(SSSDBG_OP_FAILURE, "Could not store group %s\n", name);
         return ret;
@@ -493,7 +492,6 @@ sdap_process_ghost_members(struct sysdb_attrs *attrs,
 }
 
 static int sdap_save_group(TALLOC_CTX *memctx,
-                           struct sysdb_ctx *ctx,
                            struct sdap_options *opts,
                            struct sss_domain_info *dom,
                            struct sysdb_attrs *attrs,
@@ -512,9 +510,9 @@ static int sdap_save_group(TALLOC_CTX *memctx,
     TALLOC_CTX *tmpctx = NULL;
     bool posix_group;
     bool use_id_mapping;
+    bool need_filter;
     char *sid_str;
     struct sss_domain_info *subdomain;
-    int32_t ad_group_type;
 
     tmpctx = talloc_new(NULL);
     if (!tmpctx) {
@@ -536,7 +534,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
         ret = sysdb_attrs_add_string(group_attrs, SYSDB_SID_STR, sid_str);
         if (ret != EOK) {
             DEBUG(SSSDBG_MINOR_FAILURE, "Could not add SID string: [%s]\n",
-                                         strerror(ret));
+                                         sss_strerror(ret));
             goto done;
         }
     } else if (ret == ENOENT) {
@@ -545,10 +543,21 @@ static int sdap_save_group(TALLOC_CTX *memctx,
         sid_str = NULL;
     } else {
         DEBUG(SSSDBG_MINOR_FAILURE, "Could not identify objectSID: [%s]\n",
-                                     strerror(ret));
+                                     sss_strerror(ret));
         sid_str = NULL;
     }
 
+    /* Always store UUID if available */
+    ret = sysdb_handle_original_uuid(
+                                   opts->group_map[SDAP_AT_GROUP_UUID].def_name,
+                                   attrs,
+                                   opts->group_map[SDAP_AT_GROUP_UUID].sys_name,
+                                   group_attrs, SYSDB_UUID);
+    if (ret != EOK) {
+        DEBUG((ret == ENOENT) ? SSSDBG_TRACE_ALL : SSSDBG_MINOR_FAILURE,
+              "Failed to retrieve UUID [%d][%s].\n", ret, sss_strerror(ret));
+    }
+
     /* If this object has a SID available, we will determine the correct
      * domain by its SID. */
     if (sid_str != NULL) {
@@ -570,32 +579,20 @@ static int sdap_save_group(TALLOC_CTX *memctx,
     DEBUG(SSSDBG_TRACE_FUNC, "Processing group %s\n", group_name);
 
     posix_group = true;
-    if (opts->schema_type == SDAP_SCHEMA_AD) {
-        ret = sysdb_attrs_get_int32_t(attrs, SYSDB_GROUP_TYPE, &ad_group_type);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE, "sysdb_attrs_get_int32_t failed.\n");
-            goto done;
-        }
+    ret = sdap_check_ad_group_type(dom, opts, attrs, group_name,
+                                   &need_filter);
+    if (ret != EOK) {
+        goto done;
+    }
+    if (need_filter) {
+        posix_group = false;
+        gid = 0;
 
-        DEBUG(SSSDBG_TRACE_ALL, "AD group [%s] has type flags %#x.\n",
-                                 group_name, ad_group_type);
-        /* Only security groups from AD are considered for POSIX groups.
-         * Additionally only global and universal group are taken to account
-         * for trusted domains. */
-        if (!(ad_group_type & SDAP_AD_GROUP_TYPE_SECURITY)
-                || (IS_SUBDOMAIN(dom)
-                    && (!((ad_group_type & SDAP_AD_GROUP_TYPE_GLOBAL)
-                        || (ad_group_type & SDAP_AD_GROUP_TYPE_UNIVERSAL))))) {
-            posix_group = false;
-            gid = 0;
-            DEBUG(SSSDBG_TRACE_FUNC, "Filtering AD group [%s].\n",
-                                      group_name);
-            ret = sysdb_attrs_add_bool(group_attrs, SYSDB_POSIX, false);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_OP_FAILURE,
-                      "Error: Failed to mark group as non-posix!\n");
-                return ret;
-            }
+        ret = sysdb_attrs_add_bool(group_attrs, SYSDB_POSIX, false);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "Error: Failed to mark group as non-posix!\n");
+            return ret;
         }
     }
 
@@ -628,7 +625,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
             } else if (ret != EOK) {
                 DEBUG(SSSDBG_MINOR_FAILURE,
                       "Could not convert SID string: [%s]\n",
-                       strerror(ret));
+                       sss_strerror(ret));
                 goto done;
             }
 
@@ -647,7 +644,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
             } else if (ret != EOK) {
                 DEBUG(SSSDBG_MINOR_FAILURE,
                       "Error reading posix attribute: [%s]\n",
-                       strerror(ret));
+                       sss_strerror(ret));
                 goto done;
             }
 
@@ -657,7 +654,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
             if (ret != EOK) {
                 DEBUG(SSSDBG_MINOR_FAILURE,
                       "Error setting posix attribute: [%s]\n",
-                       strerror(ret));
+                       sss_strerror(ret));
                 goto done;
             }
 
@@ -690,7 +687,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE,
               "Error setting original DN: [%s]\n",
-               strerror(ret));
+               sss_strerror(ret));
         goto done;
     }
 
@@ -701,7 +698,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE,
               "Error setting mod timestamp: [%s]\n",
-               strerror(ret));
+               sss_strerror(ret));
         goto done;
     }
 
@@ -710,7 +707,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
     if (ret) {
         DEBUG(SSSDBG_MINOR_FAILURE,
               "Error looking up group USN: [%s]\n",
-               strerror(ret));
+               sss_strerror(ret));
         goto done;
     }
     if (el->num_values == 0) {
@@ -723,7 +720,7 @@ static int sdap_save_group(TALLOC_CTX *memctx,
         if (ret) {
             DEBUG(SSSDBG_MINOR_FAILURE,
                   "Error setting group USN: [%s]\n",
-                   strerror(ret));
+                   sss_strerror(ret));
             goto done;
         }
         usn_value = talloc_strdup(tmpctx, (const char*)el->values[0].data);
@@ -748,14 +745,13 @@ static int sdap_save_group(TALLOC_CTX *memctx,
     }
     DEBUG(SSSDBG_TRACE_FUNC, "Storing info for group %s\n", group_name);
 
-    ret = sdap_store_group_with_gid(ctx, dom,
-                                    group_name, gid, group_attrs,
+    ret = sdap_store_group_with_gid(dom, group_name, gid, group_attrs,
                                     dom->group_timeout,
                                     posix_group, now);
     if (ret) {
         DEBUG(SSSDBG_MINOR_FAILURE,
               "Could not store group with GID: [%s]\n",
-               strerror(ret));
+               sss_strerror(ret));
         goto done;
     }
 
@@ -771,7 +767,7 @@ done:
         DEBUG(SSSDBG_MINOR_FAILURE,
               "Failed to save group [%s]: [%s]\n",
                group_name ? group_name : "Unknown",
-               strerror(ret));
+               sss_strerror(ret));
     }
     talloc_free(tmpctx);
     return ret;
@@ -878,9 +874,45 @@ static int sdap_save_grpmem(TALLOC_CTX *memctx,
     const char *group_name;
     char **userdns = NULL;
     size_t nuserdns = 0;
+    struct sss_domain_info *group_dom = NULL;
     int ret;
 
-    ret = sdap_get_group_primary_name(memctx, opts, attrs, dom, &group_name);
+    if (dom->ignore_group_members) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Group members are ignored, nothing to do. If you see this " \
+              "message it might indicate an error in the group processing " \
+              "logic.\n");
+        return EOK;
+    }
+
+    ret = sysdb_attrs_get_string(attrs, SYSDB_SID_STR, &group_sid);
+    if (ret != EOK) {
+        /* Try harder. */
+        ret = sdap_attrs_get_sid_str(memctx, opts->idmap_ctx, attrs,
+                              opts->group_map[SDAP_AT_GROUP_OBJECTSID].sys_name,
+                              discard_const(&group_sid));
+        if (ret != EOK) {
+            DEBUG(SSSDBG_TRACE_FUNC, "Failed to get group sid\n");
+            group_sid = NULL;
+        }
+    }
+
+    if (group_sid != NULL) {
+        group_dom = sss_get_domain_by_sid_ldap_fallback(get_domains_head(dom),
+                                                        group_sid);
+        if (group_dom == NULL) {
+            DEBUG(SSSDBG_TRACE_FUNC, "SID [%s] does not belong to any known "
+                                     "domain, using [%s].\n", group_sid,
+                                                              dom->name);
+        }
+    }
+
+    if (group_dom == NULL) {
+        group_dom = dom;
+    }
+
+    ret = sdap_get_group_primary_name(memctx, opts, attrs, group_dom,
+                                      &group_name);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Failed to get group name\n");
         goto fail;
@@ -891,12 +923,12 @@ static int sdap_save_grpmem(TALLOC_CTX *memctx,
      * are reported with tokenGroups, too
      */
     if (opts->schema_type == SDAP_SCHEMA_AD) {
-        ret = sdap_dn_by_primary_gid(memctx, attrs, ctx, dom, opts,
+        ret = sdap_dn_by_primary_gid(memctx, attrs, group_dom, opts,
                                      &userdns, &nuserdns);
         if (ret != EOK) {
             DEBUG(SSSDBG_MINOR_FAILURE,
                   "sdap_dn_by_primary_gid failed: [%d][%s].\n",
-                   ret, strerror(ret));
+                  ret, strerror(ret));
             goto fail;
         }
     }
@@ -906,15 +938,9 @@ static int sdap_save_grpmem(TALLOC_CTX *memctx,
      * https://fedorahosted.org/sssd/ticket/2522
      */
     if (opts->schema_type == SDAP_SCHEMA_IPA_V1) {
-        ret = sysdb_attrs_get_string(attrs, SYSDB_SID_STR, &group_sid);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_TRACE_FUNC, "Failed to get group sid\n");
-            group_sid = NULL;
-        }
-
         if (group_sid != NULL) {
-            ret = retain_extern_members(memctx, dom, group_name, group_sid,
-                                        &userdns, &nuserdns);
+            ret = retain_extern_members(memctx, group_dom, group_name,
+                                        group_sid, &userdns, &nuserdns);
             if (ret != EOK) {
                 DEBUG(SSSDBG_TRACE_INTERNAL,
                       "retain_extern_members failed: %d:[%s].\n",
@@ -927,7 +953,7 @@ static int sdap_save_grpmem(TALLOC_CTX *memctx,
                     opts->group_map[SDAP_AT_GROUP_MEMBER].sys_name, &el);
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE, "sysdb_attrs_get_el failed: [%d][%s].\n",
-                                     ret, strerror(ret));
+              ret, strerror(ret));
         goto fail;
     }
 
@@ -945,7 +971,7 @@ static int sdap_save_grpmem(TALLOC_CTX *memctx,
             goto fail;
         }
 
-        ret = sdap_fill_memberships(opts, group_attrs, ctx, dom, ghosts,
+        ret = sdap_fill_memberships(opts, group_attrs, ctx, group_dom, ghosts,
                                     el->values, el->num_values,
                                     userdns, nuserdns);
         if (ret) {
@@ -956,11 +982,11 @@ static int sdap_save_grpmem(TALLOC_CTX *memctx,
         }
     }
 
-    ret = sysdb_store_group(ctx, dom, group_name, 0, group_attrs,
-                            dom->group_timeout, now);
+    ret = sysdb_store_group(group_dom, group_name, 0, group_attrs,
+                            group_dom->group_timeout, now);
     if (ret) {
         DEBUG(SSSDBG_MINOR_FAILURE, "sysdb_store_group failed: [%d][%s].\n",
-                                     ret, strerror(ret));
+              ret, strerror(ret));
         goto fail;
     }
 
@@ -1041,7 +1067,7 @@ static int sdap_save_groups(TALLOC_CTX *memctx,
         usn_value = NULL;
 
         /* if 2 pass savemembers = false */
-        ret = sdap_save_group(tmpctx, sysdb, opts, dom, groups[i],
+        ret = sdap_save_group(tmpctx, opts, dom, groups[i],
                               populate_members,
                               has_nesting && save_orig_member,
                               ghosts, &usn_value, now);
@@ -1449,7 +1475,6 @@ sdap_process_group_members_2307bis(struct tevent_req *req,
 
 static int
 sdap_add_group_member_2307(struct ldb_message_element *sysdb_dns,
-                           struct sss_domain_info *dom,
                            const char *username)
 {
     sysdb_dns->values[sysdb_dns->num_values].data =
@@ -1466,7 +1491,7 @@ sdap_add_group_member_2307(struct ldb_message_element *sysdb_dns,
 
 static int
 sdap_process_missing_member_2307(struct sdap_process_group_state *state,
-                                 char *member_name, time_t now)
+                                 char *member_name)
 {
     int ret;
     TALLOC_CTX *tmp_ctx;
@@ -1487,7 +1512,7 @@ sdap_process_missing_member_2307(struct sdap_process_group_state *state,
         goto done;
     }
 
-    ret = sysdb_search_users(tmp_ctx, state->sysdb, state->dom, filter,
+    ret = sysdb_search_users(tmp_ctx, state->dom, filter,
                              attrs, &count, &msgs);
     if (ret == EOK && count > 0) {
         /* Entry exists but the group references it with an alias. */
@@ -1512,14 +1537,14 @@ sdap_process_missing_member_2307(struct sdap_process_group_state *state,
             return ENOMEM;
         }
 
-        ret = sdap_add_group_member_2307(state->sysdb_dns, state->dom, user_dn);
+        ret = sdap_add_group_member_2307(state->sysdb_dns, user_dn);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE, "Could not add group member %s\n", username);
         }
     } else if (ret == ENOENT || count == 0) {
         /* The entry really does not exist, add a ghost */
         DEBUG(SSSDBG_TRACE_FUNC, "Adding a ghost entry\n");
-        ret = sdap_add_group_member_2307(state->ghost_dns, state->dom, member_name);
+        ret = sdap_add_group_member_2307(state->ghost_dns, member_name);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE, "Could not add group member %s\n", member_name);
         }
@@ -1541,18 +1566,16 @@ sdap_process_group_members_2307(struct sdap_process_group_state *state,
     char *member_name;
     char *userdn;
     int ret;
-    time_t now;
     int i;
 
-    now = time(NULL);
     for (i=0; i < memberel->num_values; i++) {
         member_name = (char *)memberel->values[i].data;
 
         /* We need to skip over zero-length usernames */
         if (member_name[0] == '\0') continue;
 
-        ret = sysdb_search_user_by_name(state, state->sysdb, state->dom,
-                                        member_name, NULL, &msg);
+        ret = sysdb_search_user_by_name(state, state->dom, member_name,
+                                        NULL, &msg);
         if (ret == EOK) {
             /*
              * User already cached in sysdb. Remember the sysdb DN for later
@@ -1566,7 +1589,7 @@ sdap_process_group_members_2307(struct sdap_process_group_state *state,
                 return ENOMEM;
             }
 
-            ret = sdap_add_group_member_2307(state->sysdb_dns, state->dom, userdn);
+            ret = sdap_add_group_member_2307(state->sysdb_dns, userdn);
             if (ret != EOK) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       "Could not add member %s into sysdb\n", member_name);
@@ -1577,7 +1600,7 @@ sdap_process_group_members_2307(struct sdap_process_group_state *state,
             DEBUG(SSSDBG_TRACE_LIBS, "member #%d (%s): not found in sysdb\n",
                        i, member_name);
 
-            ret = sdap_process_missing_member_2307(state, member_name, now);
+            ret = sdap_process_missing_member_2307(state, member_name);
             if (ret != EOK) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       "Error processing missing member #%d (%s):\n",
@@ -1728,12 +1751,14 @@ struct sdap_get_groups_state {
     const char *base_filter;
     char *filter;
     int timeout;
-    bool enumeration;
+    enum sdap_entry_lookup_type lookup_type;
+    bool no_members;
 
     char *higher_usn;
     struct sysdb_attrs **groups;
     size_t count;
     size_t check_count;
+    hash_table_t *missing_external;
 
     hash_table_t *user_hash;
     hash_table_t *group_hash;
@@ -1758,7 +1783,8 @@ struct tevent_req *sdap_get_groups_send(TALLOC_CTX *memctx,
                                        const char **attrs,
                                        const char *filter,
                                        int timeout,
-                                       bool enumeration)
+                                       enum sdap_entry_lookup_type lookup_type,
+                                       bool no_members)
 {
     errno_t ret;
     struct tevent_req *req;
@@ -1780,7 +1806,8 @@ struct tevent_req *sdap_get_groups_send(TALLOC_CTX *memctx,
     state->groups =  NULL;
     state->count = 0;
     state->timeout = timeout;
-    state->enumeration = enumeration;
+    state->lookup_type = lookup_type;
+    state->no_members = no_members;
     state->base_filter = filter;
     state->base_iter = 0;
     state->search_bases = sdom->group_search_bases;
@@ -1859,12 +1886,13 @@ static errno_t sdap_get_groups_next_base(struct tevent_req *req)
 {
     struct tevent_req *subreq;
     struct sdap_get_groups_state *state;
+    bool need_paging = false;
+    int sizelimit = 0;
 
     state = tevent_req_data(req, struct sdap_get_groups_state);
 
     talloc_zfree(state->filter);
-    state->filter = sdap_get_id_specific_filter(state,
-                        state->base_filter,
+    state->filter = sdap_combine_filters(state, state->base_filter,
                         state->search_bases[state->base_iter]->filter);
     if (!state->filter) {
         return ENOMEM;
@@ -1874,15 +1902,30 @@ static errno_t sdap_get_groups_next_base(struct tevent_req *req)
           "Searching for groups with base [%s]\n",
            state->search_bases[state->base_iter]->basedn);
 
-    subreq = sdap_get_generic_send(
+    switch (state->lookup_type) {
+    case SDAP_LOOKUP_SINGLE:
+        break;
+    /* Only requests that can return multiple entries should require
+     * the paging control
+     */
+    case SDAP_LOOKUP_WILDCARD:
+        sizelimit = dp_opt_get_int(state->opts->basic, SDAP_WILDCARD_LIMIT);
+        need_paging = true;
+        break;
+    case SDAP_LOOKUP_ENUMERATE:
+        need_paging = true;
+        break;
+    }
+
+    subreq = sdap_get_and_parse_generic_send(
             state, state->ev, state->opts,
             state->ldap_sh != NULL ? state->ldap_sh : state->sh,
             state->search_bases[state->base_iter]->basedn,
             state->search_bases[state->base_iter]->scope,
             state->filter, state->attrs,
             state->opts->group_map, SDAP_OPTS_GROUP,
-            state->timeout,
-            state->enumeration); /* If we're enumerating, we need paging */
+            0, NULL, NULL, sizelimit, state->timeout,
+            need_paging);
     if (!subreq) {
         return ENOMEM;
     }
@@ -1892,6 +1935,9 @@ static errno_t sdap_get_groups_next_base(struct tevent_req *req)
 }
 
 static void sdap_nested_done(struct tevent_req *req);
+static void sdap_search_group_copy_batch(struct sdap_get_groups_state *state,
+                                         struct sysdb_attrs **groups,
+                                         size_t count);
 static void sdap_ad_match_rule_members_process(struct tevent_req *subreq);
 
 static void sdap_get_groups_process(struct tevent_req *subreq)
@@ -1905,9 +1951,10 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
     bool next_base = false;
     size_t count;
     struct sysdb_attrs **groups;
+    char **groupnamelist;
 
-    ret = sdap_get_generic_recv(subreq, state,
-                                &count, &groups);
+    ret = sdap_get_and_parse_generic_recv(subreq, state,
+                                          &count, &groups);
     talloc_zfree(subreq);
     if (ret) {
         tevent_req_error(req, ret);
@@ -1917,14 +1964,10 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
     DEBUG(SSSDBG_TRACE_FUNC,
           "Search for groups, returned %zu results.\n", count);
 
-    if (!state->enumeration && count > 1) {
-        DEBUG(SSSDBG_MINOR_FAILURE,
-              "Individual group search returned multiple results\n");
-        tevent_req_error(req, EINVAL);
-        return;
-    }
-
-    if (state->enumeration || count == 0) {
+    if (state->lookup_type == SDAP_LOOKUP_WILDCARD || \
+            state->lookup_type == SDAP_LOOKUP_ENUMERATE || \
+        count == 0) {
+        /* No users found in this search or looking up multiple entries */
         next_base = true;
     }
 
@@ -1940,15 +1983,7 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
             return;
         }
 
-        /* Copy the new groups into the list
-         */
-        for (i = 0; i < count; i++) {
-            state->groups[state->count + i] =
-                talloc_steal(state->groups, groups[i]);
-        }
-
-        state->count += count;
-        state->groups[state->count] = NULL;
+        sdap_search_group_copy_batch(state, groups, count);
     }
 
     if (next_base) {
@@ -1971,6 +2006,32 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
         return;
     }
 
+    if (state->no_members) {
+        ret = sysdb_attrs_primary_name_list(state->sysdb, state,
+                                state->groups, state->count,
+                                state->opts->group_map[SDAP_AT_GROUP_NAME].name,
+                                &groupnamelist);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "sysdb_attrs_primary_name_list failed.\n");
+            tevent_req_error(req, ret);
+            return;
+        }
+
+        ret = sdap_add_incomplete_groups(state->sysdb, state->dom, state->opts,
+                                         groupnamelist, state->groups,
+                                         state->count);
+        if (ret == EOK) {
+            DEBUG(SSSDBG_TRACE_LIBS,
+                  "Writing only group data without members was successful.\n");
+            tevent_req_done(req);
+        } else {
+            DEBUG(SSSDBG_OP_FAILURE, "sdap_add_incomplete_groups failed.\n");
+            tevent_req_error(req, ret);
+        }
+        return;
+    }
+
     /* Check whether we need to do nested searches
      * for RFC2307bis/FreeIPA/ActiveDirectory
      * We don't need to do this for enumeration,
@@ -1980,7 +2041,7 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
      * LDAP_MATCHING_RULE_IN_CHAIN available in
      * AD 2008 and later
      */
-    if (!state->enumeration) {
+    if (state->lookup_type == SDAP_LOOKUP_SINGLE) {
         if ((state->opts->schema_type != SDAP_SCHEMA_RFC2307)
                 && (dp_opt_get_int(state->opts->basic, SDAP_NESTING_LEVEL) != 0)
                 && !dp_opt_get_bool(state->opts->basic, SDAP_AD_MATCHING_RULE_GROUPS)) {
@@ -2003,7 +2064,7 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
     /* If we're using LDAP_MATCHING_RULE_IN_CHAIN, start a subreq to
      * retrieve the members so we can save them in a single step.
      */
-    if (!state->enumeration
+    if (state->lookup_type == SDAP_LOOKUP_SINGLE
             && (state->opts->schema_type != SDAP_SCHEMA_RFC2307)
             && state->opts->support_matching_rule
             && dp_opt_get_bool(state->opts->basic, SDAP_AD_MATCHING_RULE_GROUPS)) {
@@ -2027,7 +2088,8 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
         return;
     }
 
-    if (state->enumeration
+    if ((state->lookup_type == SDAP_LOOKUP_ENUMERATE
+                || state->lookup_type == SDAP_LOOKUP_WILDCARD)
             && state->opts->schema_type != SDAP_SCHEMA_RFC2307
             && dp_opt_get_int(state->opts->basic, SDAP_NESTING_LEVEL) != 0) {
         DEBUG(SSSDBG_TRACE_ALL, "Saving groups without members first "
@@ -2046,7 +2108,7 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
         subreq = sdap_process_group_send(state, state->ev, state->dom,
                                          state->sysdb, state->opts,
                                          state->sh, state->groups[i],
-                                         state->enumeration);
+                                         state->lookup_type == SDAP_LOOKUP_ENUMERATE);
 
         if (!subreq) {
             tevent_req_error(req, ENOMEM);
@@ -2056,6 +2118,26 @@ static void sdap_get_groups_process(struct tevent_req *subreq)
     }
 }
 
+static void sdap_search_group_copy_batch(struct sdap_get_groups_state *state,
+                                         struct sysdb_attrs **groups,
+                                         size_t count)
+{
+    size_t copied;
+    bool filter;
+
+    /* Always copy all objects for wildcard lookups. */
+    filter = state->lookup_type == SDAP_LOOKUP_SINGLE ? true : false;
+
+    copied = sdap_steal_objects_in_dom(state->opts,
+                                       state->groups,
+                                       state->count,
+                                       state->dom,
+                                       groups, count, filter);
+
+    state->count += copied;
+    state->groups[state->count] = NULL;
+}
+
 static void sdap_get_groups_done(struct tevent_req *subreq)
 {
     struct tevent_req *req =
@@ -2093,7 +2175,7 @@ static void sdap_get_groups_done(struct tevent_req *subreq)
         ret = sdap_save_groups(state, state->sysdb, state->dom, state->opts,
                                state->groups, state->count,
                                !state->dom->ignore_group_members, NULL,
-                               !state->enumeration,
+                               state->lookup_type == SDAP_LOOKUP_SINGLE,
                                &state->higher_usn);
         if (ret) {
             DEBUG(SSSDBG_OP_FAILURE, "Failed to store groups.\n");
@@ -2252,6 +2334,8 @@ int sdap_get_groups_recv(struct tevent_req *req,
     return EOK;
 }
 
+static void sdap_nested_ext_done(struct tevent_req *subreq);
+
 static void sdap_nested_done(struct tevent_req *subreq)
 {
     errno_t ret, tret;
@@ -2267,7 +2351,8 @@ static void sdap_nested_done(struct tevent_req *subreq)
                                             struct sdap_get_groups_state);
 
     ret = sdap_nested_group_recv(state, subreq, &user_count, &users,
-                                 &group_count, &groups);
+                                 &group_count, &groups,
+                                 &state->missing_external);
     talloc_zfree(subreq);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Nested group processing failed: [%d][%s]\n",
@@ -2306,8 +2391,25 @@ static void sdap_nested_done(struct tevent_req *subreq)
     }
     in_transaction = false;
 
-    /* Processing complete */
-    tevent_req_done(req);
+    if (hash_count(state->missing_external) == 0) {
+        /* No external members. Processing complete */
+        DEBUG(SSSDBG_TRACE_INTERNAL, "No external members, done");
+        tevent_req_done(req);
+        return;
+    }
+
+    /* At the moment, we need to save the direct groups & members in one
+     * transaction and then query the others in a separate requests
+     */
+    subreq = sdap_nested_group_lookup_external_send(state, state->ev,
+                                                    state->dom,
+                                                    state->opts->ext_ctx,
+                                                    state->missing_external);
+    if (subreq == NULL) {
+        ret = ENOMEM;
+        goto fail;
+    }
+    tevent_req_set_callback(subreq, sdap_nested_ext_done, req);
     return;
 
 fail:
@@ -2320,6 +2422,28 @@ fail:
     tevent_req_error(req, ret);
 }
 
+static void sdap_nested_ext_done(struct tevent_req *subreq)
+{
+    errno_t ret;
+    struct tevent_req *req = tevent_req_callback_data(subreq,
+                                                      struct tevent_req);
+    struct sdap_get_groups_state *state = tevent_req_data(req,
+                                            struct sdap_get_groups_state);
+
+    ret = sdap_nested_group_lookup_external_recv(state, subreq);
+    talloc_free(subreq);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Cannot resolve external members [%d]: %s\n",
+              ret, sss_strerror(ret));
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    tevent_req_done(req);
+    return;
+}
+
 static errno_t sdap_nested_group_populate_users(TALLOC_CTX *mem_ctx,
                                                 struct sysdb_ctx *sysdb,
                                                 struct sss_domain_info *domain,
@@ -2414,7 +2538,7 @@ static errno_t sdap_nested_group_populate_users(TALLOC_CTX *mem_ctx,
             ret = ENOMEM;
             goto done;
         }
-        ret = sysdb_search_users(tmp_ctx, user_dom->sysdb, user_dom, filter,
+        ret = sysdb_search_users(tmp_ctx, user_dom, filter,
                                  search_attrs, &count, &msgs);
         talloc_zfree(filter);
         talloc_zfree(clean_orig_dn);
@@ -2444,8 +2568,8 @@ static errno_t sdap_nested_group_populate_users(TALLOC_CTX *mem_ctx,
 
             ret = sysdb_attrs_add_string(attrs, SYSDB_NAME, username);
             if (ret) goto done;
-            ret = sysdb_set_user_attr(user_dom->sysdb, user_dom, sysdb_name,
-                                      attrs, SYSDB_MOD_REP);
+            ret = sysdb_set_entry_attr(user_dom->sysdb, msgs[0]->dn, attrs,
+                                       SYSDB_MOD_REP);
             if (ret != EOK) goto done;
         } else {
             key.type = HASH_KEY_STRING;
