@@ -170,8 +170,7 @@ sdap_get_ad_match_rule_initgroups_next_base(struct tevent_req *req)
     state = tevent_req_data(req, struct sdap_ad_match_rule_initgr_state);
 
     talloc_zfree(state->filter);
-    state->filter = sdap_get_id_specific_filter(state,
-                        state->base_filter,
+    state->filter = sdap_combine_filters(state, state->base_filter,
                         state->search_bases[state->base_iter]->filter);
     if (!state->filter) {
         return ENOMEM;
@@ -1445,7 +1444,7 @@ sdap_ad_tokengroups_initgroups_send(TALLOC_CTX *mem_ctx,
     state->use_id_mapping = use_id_mapping;
     state->domain = domain;
 
-    /* We can compute the the gidNumber attribute from SIDs obtained from
+    /* We can compute the gidNumber attribute from SIDs obtained from
      * the tokenGroups lookup in case ID mapping is used for a user from the
      * parent domain. For trusted domains, we need to know the group type
      * to be able to filter out domain-local groups. Additionally, as a
