@@ -33,6 +33,9 @@
 #include "db/sysdb.h"
 #include "providers/ldap/ldap_common.h"
 #include "providers/ldap/sdap_async.h"
+#include "providers/ldap/sdap_async_private.h"
+#include "providers/ldap/sdap_idmap.h"
+#include "providers/ipa/ipa_dn.h"
 
 #define sdap_nested_group_sysdb_search_users(domain, filter) \
     sdap_nested_group_sysdb_search((domain), (filter), true)
@@ -53,6 +56,15 @@ struct sdap_nested_group_member {
     const char *group_filter;
 };
 
+#ifndef EXTERNAL_MEMBERS_CHUNK
+#define EXTERNAL_MEMBERS_CHUNK  16
+#endif /* EXTERNAL_MEMBERS_CHUNK */
+
+struct sdap_external_missing_member {
+    const char **parent_group_dns;
+    size_t parent_dn_idx;
+};
+
 struct sdap_nested_group_ctx {
     struct sss_domain_info *domain;
     struct sdap_options *opts;
@@ -61,6 +73,7 @@ struct sdap_nested_group_ctx {
     struct sdap_handle *sh;
     hash_table_t *users;
     hash_table_t *groups;
+    hash_table_t *missing_external;
     bool try_deref;
     int deref_treshold;
     int max_nesting_level;
@@ -181,37 +194,32 @@ done:
     return ret;
 }
 
-static errno_t sdap_nested_group_hash_entry(hash_table_t *table,
-                                            struct sysdb_attrs *entry,
-                                            const char *table_name)
+static errno_t sdap_nested_group_hash_insert(hash_table_t *table,
+                                             const char *entry_key,
+                                             void *entry_value,
+                                             bool overwrite,
+                                             const char *table_name)
 {
     hash_key_t key;
     hash_value_t value;
-    const char *name = NULL;
-    errno_t ret;
     int hret;
 
-    ret = sysdb_attrs_get_string(entry, SYSDB_ORIG_DN, &name);
-    if (ret != EOK) {
-        return ret;
-    }
-
     DEBUG(SSSDBG_TRACE_ALL, "Inserting [%s] into hash table [%s]\n",
-                             name, table_name);
+                             entry_key, table_name);
 
     key.type = HASH_KEY_STRING;
-    key.str = talloc_strdup(NULL, name);
+    key.str = talloc_strdup(NULL, entry_key);
     if (key.str == NULL) {
         return ENOMEM;
     }
 
-    if (hash_has_key(table, &key)) {
+    if (overwrite == false && hash_has_key(table, &key)) {
         talloc_free(key.str);
         return EEXIST;
     }
 
     value.type = HASH_VALUE_PTR;
-    value.ptr = entry;
+    value.ptr = entry_value;
 
     hret = hash_enter(table, &key, &value);
     if (hret != HASH_SUCCESS) {
@@ -225,6 +233,21 @@ static errno_t sdap_nested_group_hash_entry(hash_table_t *table,
     return EOK;
 }
 
+static errno_t sdap_nested_group_hash_entry(hash_table_t *table,
+                                            struct sysdb_attrs *entry,
+                                            const char *table_name)
+{
+    const char *name = NULL;
+    errno_t ret;
+
+    ret = sysdb_attrs_get_string(entry, SYSDB_ORIG_DN, &name);
+    if (ret != EOK) {
+        return ret;
+    }
+
+    return sdap_nested_group_hash_insert(table, name, entry, false, table_name);
+}
+
 static errno_t
 sdap_nested_group_hash_user(struct sdap_nested_group_ctx *group_ctx,
                             struct sysdb_attrs *user)
@@ -239,38 +262,38 @@ sdap_nested_group_hash_group(struct sdap_nested_group_ctx *group_ctx,
     struct sdap_attr_map *map = group_ctx->opts->group_map;
     gid_t gid;
     errno_t ret;
-    int32_t ad_group_type;
     bool posix_group = true;
+    bool use_id_mapping;
+    bool can_find_gid;
+    bool need_filter;
 
-    if (group_ctx->opts->schema_type == SDAP_SCHEMA_AD) {
-        ret = sysdb_attrs_get_int32_t(group, SYSDB_GROUP_TYPE, &ad_group_type);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE, "sysdb_attrs_get_int32_t failed.\n");
-            return ret;
-        }
-
-        DEBUG(SSSDBG_TRACE_ALL, "AD group has type flags %#x.\n",
-                                 ad_group_type);
-        /* Only security groups from AD are considered for POSIX groups.
-         * Additionally only global and universal group are taken to account
-         * for trusted domains. */
-        if (!(ad_group_type & SDAP_AD_GROUP_TYPE_SECURITY)
-                || (IS_SUBDOMAIN(group_ctx->domain)
-                    && (!((ad_group_type & SDAP_AD_GROUP_TYPE_GLOBAL)
-                        || (ad_group_type & SDAP_AD_GROUP_TYPE_UNIVERSAL))))) {
-            posix_group = false;
-            gid = 0;
-            DEBUG(SSSDBG_TRACE_FUNC, "Filtering AD group.\n");
-        }
+    ret = sdap_check_ad_group_type(group_ctx->domain, group_ctx->opts,
+                                   group, "", &need_filter);
+    if (ret != EOK) {
+        return ret;
     }
 
-    ret = sysdb_attrs_get_uint32_t(group, map[SDAP_AT_GROUP_GID].sys_name,
-                                   &gid);
-    if (ret == ENOENT || (ret == EOK && gid == 0) || !posix_group) {
+    if (need_filter) {
+        posix_group = false;
+        gid = 0;
+    }
+
+    use_id_mapping = sdap_idmap_domain_has_algorithmic_mapping(
+                                                          group_ctx->opts->idmap_ctx,
+                                                          group_ctx->domain->name,
+                                                          group_ctx->domain->domain_id);
+
+    can_find_gid = posix_group && !use_id_mapping;
+    if (can_find_gid) {
+        ret = sysdb_attrs_get_uint32_t(group, map[SDAP_AT_GROUP_GID].sys_name,
+                                       &gid);
+    }
+    if (!can_find_gid || ret == ENOENT || (ret == EOK && gid == 0)) {
         DEBUG(SSSDBG_TRACE_ALL,
              "The group's gid was %s\n", ret == ENOENT ? "missing" : "zero");
         DEBUG(SSSDBG_TRACE_INTERNAL,
              "Marking group as non-posix and setting GID=0!\n");
+
         if (ret == ENOENT || !posix_group) {
             ret = sysdb_attrs_add_uint32(group,
                                          map[SDAP_AT_GROUP_GID].sys_name, 0);
@@ -294,6 +317,76 @@ sdap_nested_group_hash_group(struct sdap_nested_group_ctx *group_ctx,
     return sdap_nested_group_hash_entry(group_ctx->groups, group, "groups");
 }
 
+static errno_t sdap_nested_group_external_add(hash_table_t *table,
+                                              const char *ext_member,
+                                              const char *parent_group_dn)
+{
+    hash_key_t key;
+    hash_value_t value;
+    int hret;
+    int ret;
+    struct sdap_external_missing_member *ext_mem;
+
+    key.type = HASH_KEY_STRING;
+    key.str = discard_const(ext_member);
+
+    DEBUG(SSSDBG_TRACE_ALL,
+          "Inserting external member [%s] into external members hash table\n",
+          ext_member);
+
+    hret = hash_lookup(table, &key, &value);
+    switch (hret) {
+    case HASH_ERROR_KEY_NOT_FOUND:
+        ext_mem = talloc_zero(table, struct sdap_external_missing_member);
+        if (ext_mem == NULL) {
+            return ENOMEM;
+        }
+        ext_mem->parent_group_dns = talloc_zero_array(ext_mem,
+                                                      const char *,
+                                                      EXTERNAL_MEMBERS_CHUNK);
+        if (ext_mem->parent_group_dns == NULL) {
+            talloc_free(ext_mem);
+            return ENOMEM;
+        }
+
+        ret = sdap_nested_group_hash_insert(table, ext_member, ext_mem,
+                                            true, "missing external users");
+        if (ret != EOK) {
+            return ret;
+        }
+        break;
+
+    case HASH_SUCCESS:
+        ext_mem = talloc_get_type(value.ptr,
+                                  struct sdap_external_missing_member);
+        if (ext_mem->parent_dn_idx == \
+                talloc_array_length(ext_mem->parent_group_dns)) {
+            ext_mem->parent_group_dns = talloc_realloc(ext_mem,
+                                                ext_mem->parent_group_dns,
+                                                const char *,
+                                                ext_mem->parent_dn_idx + \
+                                                    EXTERNAL_MEMBERS_CHUNK);
+            if (ext_mem->parent_group_dns == NULL) {
+                talloc_free(ext_mem);
+                return ENOMEM;
+            }
+        }
+        break;
+    default:
+        return EIO;
+    }
+
+    ext_mem->parent_group_dns[ext_mem->parent_dn_idx] = \
+                                        talloc_strdup(ext_mem->parent_group_dns,
+                                                      parent_group_dn);
+    if (ext_mem->parent_group_dns[ext_mem->parent_dn_idx] == NULL) {
+        return ENOMEM;
+    }
+    ext_mem->parent_dn_idx++;
+
+    return EOK;
+}
+
 static errno_t sdap_nested_group_sysdb_search(struct sss_domain_info *domain,
                                               const char *filter,
                                               bool user)
@@ -309,10 +402,10 @@ static errno_t sdap_nested_group_sysdb_search(struct sss_domain_info *domain,
     errno_t ret;
 
     if (user) {
-        ret = sysdb_search_users(NULL, domain->sysdb, domain, filter, attrs,
+        ret = sysdb_search_users(NULL, domain, filter, attrs,
                                  &count, &msgs);
     } else {
-        ret = sysdb_search_groups(NULL, domain->sysdb, domain, filter, attrs,
+        ret = sysdb_search_groups(NULL, domain, filter, attrs,
                                   &count, &msgs);
     }
     if (ret != EOK) {
@@ -475,6 +568,13 @@ sdap_nested_group_split_members(TALLOC_CTX *mem_ctx,
     errno_t ret;
     int i;
 
+    if (members == NULL) {
+        *_missing = NULL;
+        *_num_missing = 0;
+        *_num_groups = 0;
+        return EOK;
+    }
+
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "talloc_new() failed\n");
@@ -588,7 +688,10 @@ sdap_nested_group_split_members(TALLOC_CTX *mem_ctx,
 
     missing = talloc_realloc(mem_ctx, missing,
                              struct sdap_nested_group_member, num_missing);
-    if (missing == NULL) {
+    /* talloc_realloc behaves as talloc_free if 3rd parameter (count) is 0,
+     * so it's OK to return NULL then
+     */
+    if (missing == NULL && num_missing > 0) {
         ret = ENOMEM;
         goto done;
     }
@@ -613,6 +716,65 @@ done:
     return ret;
 }
 
+static errno_t
+sdap_nested_group_add_ext_members(TALLOC_CTX *mem_ctx,
+                                  struct sdap_nested_group_ctx *group_ctx,
+                                  struct sysdb_attrs *group,
+                                  struct ldb_message_element *ext_members)
+{
+    errno_t ret;
+    const char *ext_member_attr;
+    const char *orig_dn;
+
+    if (ext_members == NULL) {
+        return EOK;
+    }
+
+    ret = sysdb_attrs_get_string(group, SYSDB_ORIG_DN, &orig_dn);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "A group with no originalDN!?!\n");
+        return ret;
+    }
+
+    for (size_t i = 0; i < ext_members->num_values; i++) {
+        ext_member_attr = (const char *) ext_members->values[i].data;
+
+        ret = sdap_nested_group_external_add(group_ctx->missing_external,
+                                             ext_member_attr,
+                                             orig_dn);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                    "Cannot add %s into external members [%d]: %s\n",
+                    ext_member_attr, ret, sss_strerror(ret));
+            return ret;
+        }
+    }
+
+    return EOK;
+}
+
+static struct ldb_message_element *
+sdap_nested_group_ext_members(struct sdap_options *opts,
+                              struct sysdb_attrs *group)
+{
+    errno_t ret;
+    struct ldb_message_element *ext_members = NULL;
+
+    if (opts->ext_ctx == NULL) {
+        return NULL;
+    }
+
+    ret = sysdb_attrs_get_el_ext(group,
+                 opts->group_map[SDAP_AT_GROUP_EXT_MEMBER].sys_name,
+                 false, &ext_members);
+    if (ret != EOK && ret != ENOENT) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to retrieve external member list "
+                                   "[%d]: %s\n", ret, sss_strerror(ret));
+    }
+
+    return ext_members;
+}
+
 
 struct sdap_nested_group_state {
     struct sdap_nested_group_ctx *group_ctx;
@@ -661,6 +823,14 @@ sdap_nested_group_send(TALLOC_CTX *mem_ctx,
         goto immediately;
     }
 
+    ret = sss_hash_create(state->group_ctx, 32,
+                          &state->group_ctx->missing_external);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to create hash table [%d]: %s\n",
+                                    ret, strerror(ret));
+        goto immediately;
+    }
+
     state->group_ctx->try_deref = true;
     state->group_ctx->deref_treshold = dp_opt_get_int(opts->basic,
                                                       SDAP_DEREF_THRESHOLD);
@@ -754,7 +924,8 @@ errno_t sdap_nested_group_recv(TALLOC_CTX *mem_ctx,
                                unsigned long *_num_users,
                                struct sysdb_attrs ***_users,
                                unsigned long *_num_groups,
-                               struct sysdb_attrs ***_groups)
+                               struct sysdb_attrs ***_groups,
+                               hash_table_t **_missing_external)
 {
     struct sdap_nested_group_state *state = NULL;
     struct sysdb_attrs **users = NULL;
@@ -801,6 +972,11 @@ errno_t sdap_nested_group_recv(TALLOC_CTX *mem_ctx,
         *_groups = talloc_steal(mem_ctx, groups);
     }
 
+    if (_missing_external) {
+        *_missing_external = talloc_steal(mem_ctx,
+                                          state->group_ctx->missing_external);
+    }
+
     return EOK;
 }
 
@@ -810,6 +986,7 @@ struct sdap_nested_group_process_state {
     struct sdap_nested_group_member *missing;
     int num_missing_total;
     int num_missing_groups;
+    struct ldb_message_element *ext_members;
     int nesting_level;
     char *group_dn;
     bool deref;
@@ -860,13 +1037,16 @@ sdap_nested_group_process_send(TALLOC_CTX *mem_ctx,
 
     DEBUG(SSSDBG_TRACE_INTERNAL, "About to process group [%s]\n", orig_dn);
 
-    /* get member list */
-    ret = sysdb_attrs_get_el(group, group_map[SDAP_AT_GROUP_MEMBER].sys_name,
-                             &members);
-    if (ret == ENOENT) {
-        ret = EOK; /* no members */
+    /* get member list, both direct and external */
+    state->ext_members = sdap_nested_group_ext_members(state->group_ctx->opts,
+                                                       group);
+
+    ret = sysdb_attrs_get_el_ext(group, group_map[SDAP_AT_GROUP_MEMBER].sys_name,
+                                 false, &members);
+    if (ret == ENOENT && state->ext_members == NULL) {
+        ret = EOK; /* no members, direct or external */
         goto immediately;
-    } else if (ret != EOK) {
+    } else if (ret != EOK && ret != ENOENT) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Unable to retrieve member list "
                                     "[%d]: %s\n", ret, strerror(ret));
         goto immediately;
@@ -878,15 +1058,37 @@ sdap_nested_group_process_send(TALLOC_CTX *mem_ctx,
                                           &state->missing,
                                           &state->num_missing_total,
                                           &state->num_missing_groups);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to split member list "
+                                    "[%d]: %s\n", ret, sss_strerror(ret));
+        goto immediately;
+    }
 
-    DEBUG(SSSDBG_TRACE_INTERNAL, "Looking up %d/%d members of group [%s]\n",
-          state->num_missing_total, members->num_values, orig_dn);
+    ret = sdap_nested_group_add_ext_members(state,
+                                            state->group_ctx,
+                                            group,
+                                            state->ext_members);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to split external member list "
+                                    "[%d]: %s\n", ret, sss_strerror(ret));
+        goto immediately;
+    }
 
-    if (state->num_missing_total == 0) {
+    if (state->num_missing_total == 0
+            && hash_count(state->group_ctx->missing_external) == 0) {
         ret = EOK; /* we're done */
         goto immediately;
     }
 
+    /* If there are only indirect members of the group, it's still safe to
+     * proceed and let the direct lookup code just fall through.
+     */
+
+    DEBUG(SSSDBG_TRACE_INTERNAL, "Looking up %d/%d members of group [%s]\n",
+                                 state->num_missing_total,
+                                 members ? members->num_values : 0,
+                                 orig_dn);
+
     /* process members */
     if (group_ctx->try_deref
             && state->num_missing_total > group_ctx->deref_treshold) {
@@ -1256,8 +1458,9 @@ sdap_nested_group_single_step_process(struct tevent_req *subreq)
         /* save user in hash table */
         ret = sdap_nested_group_hash_user(state->group_ctx, entry);
         if (ret == EEXIST) {
-            DEBUG(SSSDBG_TRACE_FUNC, "User was looked up twice, "
-                                      "this shouldn't have happened.\n");
+            /* the user is already present, skip it */
+            talloc_zfree(entry);
+            ret = EOK;
             goto done;
         } else if (ret != EOK) {
             DEBUG(SSSDBG_CRIT_FAILURE, "Unable to save user in hash table "
@@ -1298,8 +1501,9 @@ sdap_nested_group_single_step_process(struct tevent_req *subreq)
         /* save group in hash table */
         ret = sdap_nested_group_hash_group(state->group_ctx, entry);
         if (ret == EEXIST) {
-            DEBUG(SSSDBG_TRACE_FUNC, "Group was looked up twice, "
-                                      "this shouldn't have happened.\n");
+            /* the group is already present, skip it */
+            talloc_zfree(entry);
+            ret = EOK;
             goto done;
         } else if (ret != EOK) {
             DEBUG(SSSDBG_CRIT_FAILURE, "Unable to save group in hash table "
@@ -1388,7 +1592,7 @@ static void sdap_nested_group_single_done(struct tevent_req *subreq)
     talloc_zfree(subreq);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Error processing nested groups "
-                                    "[%d]: %s\n.", ret, strerror(ret));
+                                    "[%d]: %s.\n", ret, strerror(ret));
         tevent_req_error(req, ret);
         return;
     }
@@ -1405,96 +1609,33 @@ static errno_t sdap_nested_group_single_recv(struct tevent_req *req)
     return EOK;
 }
 
-/* This should be a function pointer set from the IPA provider */
 static errno_t sdap_nested_group_get_ipa_user(TALLOC_CTX *mem_ctx,
                                               const char *user_dn,
                                               struct sysdb_ctx *sysdb,
                                               struct sysdb_attrs **_user)
 {
-    errno_t ret;
-    struct sysdb_attrs *user = NULL;
-    char *name;
-    struct ldb_dn *dn = NULL;
-    const char *rdn_name;
-    const char *users_comp_name;
-    const char *acct_comp_name;
-    const struct ldb_val *rdn_val;
-    const struct ldb_val *users_comp_val;
-    const struct ldb_val *acct_comp_val;
     TALLOC_CTX *tmp_ctx;
+    struct sysdb_attrs *user;
+    char *name;
+    errno_t ret;
 
     tmp_ctx = talloc_new(NULL);
-    if (!tmp_ctx) return ENOMEM;
+    if (tmp_ctx == NULL) {
+        return ENOMEM;
+    }
 
-    /* return username if dn is in form:
-     * uid=username,cn=users,cn=accounts,dc=example,dc=com */
-
-    dn = ldb_dn_new(tmp_ctx, sysdb_ctx_get_ldb(sysdb), user_dn);
-    if (dn == NULL) {
-        ret = ENOMEM;
+    ret = ipa_get_rdn(tmp_ctx, sysdb, user_dn, &name, "uid",
+                      "cn", "users", "cn", "accounts");
+    if (ret != EOK) {
         goto done;
     }
 
-    /* rdn, users, accounts and least one domain component */
-    if (ldb_dn_get_comp_num(dn) < 4) {
-        ret = ENOENT;
-        goto done;
-    }
-
-    rdn_name = ldb_dn_get_rdn_name(dn);
-    if (rdn_name == NULL) {
-        ret = EINVAL;
-        goto done;
-    }
-
-    /* rdn must be 'uid' */
-    if (strcasecmp("uid", rdn_name) != 0) {
-        ret = ENOENT;
-        goto done;
-    }
-
-    /* second component must be 'cn=users' */
-    users_comp_name = ldb_dn_get_component_name(dn, 1);
-    if (strcasecmp("cn", users_comp_name) != 0) {
-        ret = ENOENT;
-        goto done;
-    }
-
-    users_comp_val = ldb_dn_get_component_val(dn, 1);
-    if (strncasecmp("users", (const char *) users_comp_val->data,
-                    users_comp_val->length) != 0) {
-        ret = ENOENT;
-        goto done;
-    }
-
-    /* third component must be 'cn=accounts' */
-    acct_comp_name = ldb_dn_get_component_name(dn, 2);
-    if (strcasecmp("cn", acct_comp_name) != 0) {
-        ret = ENOENT;
-        goto done;
-    }
-
-    acct_comp_val = ldb_dn_get_component_val(dn, 2);
-    if (strncasecmp("accounts", (const char *) acct_comp_val->data,
-                    acct_comp_val->length) != 0) {
-        ret = ENOENT;
-        goto done;
-    }
-
-    /* value of rdn is username */
     user = sysdb_new_attrs(tmp_ctx);
     if (user == NULL) {
         ret = ENOMEM;
         goto done;
     }
 
-    rdn_val = ldb_dn_get_rdn_val(dn);
-    name = talloc_strndup(user, (const char *)rdn_val->data, rdn_val->length);
-    if (name == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
-
     ret = sysdb_attrs_add_string(user, SYSDB_NAME, name);
     if (ret != EOK) {
         goto done;
@@ -1577,8 +1718,7 @@ sdap_nested_group_lookup_user_send(TALLOC_CTX *mem_ctx,
     }
 
     /* use search base filter if needed */
-    filter = sdap_get_id_specific_filter(state, base_filter,
-                                         member->user_filter);
+    filter = sdap_combine_filters(state, base_filter, member->user_filter);
     if (filter == NULL) {
         ret = ENOMEM;
         goto immediately;
@@ -1721,8 +1861,7 @@ sdap_nested_group_lookup_group_send(TALLOC_CTX *mem_ctx,
      }
 
      /* use search base filter if needed */
-     filter = sdap_get_id_specific_filter(state, base_filter,
-                                          member->group_filter);
+     filter = sdap_combine_filters(state, base_filter, member->group_filter);
      if (filter == NULL) {
          ret = ENOMEM;
          goto immediately;
@@ -2320,3 +2459,385 @@ static errno_t sdap_nested_group_deref_recv(struct tevent_req *req)
 
     return EOK;
 }
+
+struct sdap_ext_member {
+    struct sdap_external_missing_member *missing_mem;
+    const char *ext_member_attr;
+
+    enum sysdb_member_type member_type;
+    struct sss_domain_info *dom;
+    struct sysdb_attrs *attrs;
+};
+
+struct sdap_nested_group_lookup_external_state {
+    struct tevent_context *ev;
+    struct sdap_ext_member_ctx *ext_ctx;
+    struct sss_domain_info *group_dom;
+    hash_table_t *missing_external;
+
+    hash_entry_t *entries;
+    unsigned long n_entries;
+    unsigned long eniter;
+
+    struct sdap_ext_member *ext_members;
+
+    ext_member_send_fn_t ext_member_resolve_send;
+    ext_member_recv_fn_t ext_member_resolve_recv;
+};
+
+static errno_t
+sdap_nested_group_lookup_external_step(struct tevent_req *req);
+static void
+sdap_nested_group_lookup_external_done(struct tevent_req *subreq);
+static errno_t
+sdap_nested_group_lookup_external_link(struct tevent_req *req);
+static errno_t
+sdap_nested_group_lookup_external_link_member(
+                        struct sdap_nested_group_lookup_external_state *state,
+                        struct sdap_ext_member *member);
+static errno_t
+sdap_nested_group_memberof_dn_by_original_dn(
+                            TALLOC_CTX *mem_ctx,
+                            struct sss_domain_info *group_dom,
+                            const char *original_dn,
+                            const char ***_parents);
+
+struct tevent_req *
+sdap_nested_group_lookup_external_send(TALLOC_CTX *mem_ctx,
+                                       struct tevent_context *ev,
+                                       struct sss_domain_info *group_dom,
+                                       struct sdap_ext_member_ctx *ext_ctx,
+                                       hash_table_t *missing_external)
+{
+    struct sdap_nested_group_lookup_external_state *state = NULL;
+    struct tevent_req *req = NULL;
+    errno_t ret;
+
+    req = tevent_req_create(mem_ctx, &state,
+                            struct sdap_nested_group_lookup_external_state);
+    if (req == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "tevent_req_create() failed\n");
+        return NULL;
+    }
+
+    state->ev = ev;
+    state->group_dom = group_dom;
+    state->ext_ctx = ext_ctx;
+    state->missing_external = missing_external;
+
+    if (state->ext_ctx->ext_member_resolve_send == NULL
+            || state->ext_ctx->ext_member_resolve_recv == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Wrong private context\n");
+        ret = EINVAL;
+        goto immediately;
+    }
+
+    ret = hash_entries(state->missing_external,
+                       &state->n_entries, &state->entries);
+    if (ret != HASH_SUCCESS) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "hash_entries returned %d\n", ret);
+        ret = EIO;
+        goto immediately;
+    }
+    state->eniter = 0;
+
+    state->ext_members = talloc_zero_array(state,
+                                           struct sdap_ext_member,
+                                           state->n_entries);
+    if (state->ext_members == NULL) {
+        ret = ENOMEM;
+        goto immediately;
+    }
+
+    ret = sdap_nested_group_lookup_external_step(req);
+    if (ret != EAGAIN) {
+        goto immediately;
+    }
+
+    return req;
+
+immediately:
+    if (ret == EOK) {
+        tevent_req_done(req);
+    } else {
+        tevent_req_error(req, ret);
+    }
+    tevent_req_post(req, ev);
+    return req;
+}
+
+static errno_t
+sdap_nested_group_lookup_external_step(struct tevent_req *req)
+{
+    struct tevent_req *subreq = NULL;
+    struct sdap_nested_group_lookup_external_state *state = NULL;
+    state = tevent_req_data(req,
+                            struct sdap_nested_group_lookup_external_state);
+
+    subreq = state->ext_ctx->ext_member_resolve_send(state,
+                                        state->ev,
+                                        state->entries[state->eniter].key.str,
+                                        state->ext_ctx->pvt);
+    if (subreq == NULL) {
+        return ENOMEM;
+    }
+    DEBUG(SSSDBG_TRACE_FUNC, "Refreshing member %lu/%lu\n",
+                             state->eniter, state->n_entries);
+    tevent_req_set_callback(subreq,
+                            sdap_nested_group_lookup_external_done,
+                            req);
+
+    return EAGAIN;
+}
+
+static void
+sdap_nested_group_lookup_external_done(struct tevent_req *subreq)
+{
+    errno_t ret;
+    struct tevent_req *req = NULL;
+    struct sdap_nested_group_lookup_external_state *state = NULL;
+    enum sysdb_member_type member_type;
+    struct sysdb_attrs *member;
+    struct sss_domain_info *member_dom;
+
+    req = tevent_req_callback_data(subreq, struct tevent_req);
+    state = tevent_req_data(req,
+                            struct sdap_nested_group_lookup_external_state);
+
+    ret = state->ext_ctx->ext_member_resolve_recv(state, subreq,
+                                                  &member_type,
+                                                  &member_dom,
+                                                  &member);
+    talloc_free(subreq);
+    if (ret == EOK) {
+        DEBUG(SSSDBG_TRACE_FUNC, "Refreshed member %lu\n", state->eniter);
+        state->ext_members[state->eniter].missing_mem = \
+                                    state->entries[state->eniter].value.ptr;
+        state->ext_members[state->eniter].dom = member_dom;
+
+        state->ext_members[state->eniter].ext_member_attr = \
+                        talloc_steal(state->ext_members,
+                                     state->entries[state->eniter].key.str);
+        state->ext_members[state->eniter].member_type = member_type;
+        state->ext_members[state->eniter].attrs = \
+                            talloc_steal(state->ext_members, member);
+    }
+
+    state->eniter++;
+    if (state->eniter >= state->n_entries) {
+        DEBUG(SSSDBG_TRACE_FUNC, "All external members processed\n");
+        ret = sdap_nested_group_lookup_external_link(req);
+        if (ret != EOK) {
+            tevent_req_error(req, ret);
+            return;
+        }
+        tevent_req_done(req);
+        return;
+    }
+
+    ret = sdap_nested_group_lookup_external_step(req);
+    if (ret != EOK && ret != EAGAIN) {
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    return;
+}
+
+static errno_t
+sdap_nested_group_lookup_external_link(struct tevent_req *req)
+{
+    errno_t ret, tret;
+    bool in_transaction = false;
+    struct sdap_nested_group_lookup_external_state *state = NULL;
+    state = tevent_req_data(req,
+                            struct sdap_nested_group_lookup_external_state);
+
+    ret = sysdb_transaction_start(state->group_dom->sysdb);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to start transaction\n");
+        goto fail;
+    }
+    in_transaction = true;
+
+
+    for (size_t i = 0; i < state->eniter; i++) {
+        if (state->ext_members[i].attrs == NULL) {
+            DEBUG(SSSDBG_MINOR_FAILURE, "The member %s could not be resolved\n",
+                                        state->ext_members[i].ext_member_attr);
+            continue;
+        }
+
+        ret = sdap_nested_group_lookup_external_link_member(state,
+                                                    &state->ext_members[i]);
+        if (ret != EOK) {
+            goto fail;
+        }
+    }
+
+    ret = sysdb_transaction_commit(state->group_dom->sysdb);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to commit transaction\n");
+        goto fail;
+    }
+    in_transaction = false;
+
+    return EOK;
+
+fail:
+    if (in_transaction) {
+        tret = sysdb_transaction_cancel(state->group_dom->sysdb);
+        if (tret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "Failed to cancel transaction\n");
+        }
+    }
+    return EFAULT;
+}
+
+static errno_t
+sdap_nested_group_lookup_external_link_member(
+                        struct sdap_nested_group_lookup_external_state *state,
+                        struct sdap_ext_member *member)
+{
+    const char *name;
+    int ret;
+    const char **parents = NULL;
+    size_t i;
+    TALLOC_CTX *tmp_ctx;
+    const char *orig_dn;
+
+    tmp_ctx = talloc_new(state);
+    if (tmp_ctx == NULL) {
+        return ENOMEM;
+    }
+
+    ret = sysdb_attrs_get_string(member->attrs, SYSDB_NAME, &name);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "No name for a user\n");
+        goto done;
+    }
+
+    /* This only works because the groups were saved in a previous
+     * transaction */
+    for (i=0; i < member->missing_mem->parent_dn_idx; i++) {
+        orig_dn = member->missing_mem->parent_group_dns[i];
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              "Linking external members %s from domain %s to parents of %s\n",
+              name, member->dom->name, orig_dn);
+        ret = sdap_nested_group_memberof_dn_by_original_dn(tmp_ctx,
+                                                           state->group_dom,
+                                                           orig_dn,
+                                                           &parents);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "Cannot find parents of %s\n", orig_dn);
+            continue;
+        }
+
+        /* We don't have to remove the members here, since all members attributes
+         * are always written anew
+         */
+        ret = sysdb_update_members_dn(member->dom, name, member->member_type,
+                                      parents, NULL);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "Cannot link %s@%s to its parents\n",
+                                       name, member->dom->name);
+            goto done;
+        }
+
+    }
+
+    ret = EOK;
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
+static errno_t
+sdap_nested_group_memberof_dn_by_original_dn(
+                            TALLOC_CTX *mem_ctx,
+                            struct sss_domain_info *group_dom,
+                            const char *original_dn,
+                            const char ***_parents)
+{
+    errno_t ret;
+    char *sanitized_dn;
+    char *filter;
+    const char *attrs[] = { SYSDB_NAME,
+                            SYSDB_MEMBEROF,
+                            NULL };
+    struct ldb_message **msgs = NULL;
+    size_t count;
+    TALLOC_CTX *tmp_ctx;
+    struct ldb_message_element *memberof;
+    const char **parents;
+
+    tmp_ctx = talloc_new(mem_ctx);
+    if (tmp_ctx == NULL) {
+        return ENOMEM;
+    }
+
+    ret = sss_filter_sanitize(tmp_ctx, original_dn, &sanitized_dn);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+                "Cannot sanitize originalDN [%s]\n", original_dn);
+        goto done;
+    }
+
+    filter = talloc_asprintf(tmp_ctx, "(%s=%s)", SYSDB_ORIG_DN, sanitized_dn);
+    if (filter == NULL) {
+        goto done;
+    }
+
+    ret = sysdb_search_groups(tmp_ctx, group_dom, filter, attrs,
+                              &count, &msgs);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    if (count != 1) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "More than one entry found by originalDN?\n");
+        goto done;
+    }
+
+    memberof = ldb_msg_find_element(msgs[0], SYSDB_MEMBEROF);
+    if (memberof == NULL || memberof->num_values == 0) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "The external group is not a member of any groups\n");
+        ret = ENOENT;
+        goto done;
+    }
+
+    parents = talloc_zero_array(tmp_ctx,
+                                const char *,
+                                memberof->num_values + 1);
+    if (parents == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    for (size_t i = 0; i < memberof->num_values; i++) {
+        parents[i] = talloc_strdup(parents,
+                                   (const char *) memberof->values[i].data);
+        if (parents[i] == NULL) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    *_parents = talloc_steal(mem_ctx, parents);
+    ret = EOK;
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
+errno_t
+sdap_nested_group_lookup_external_recv(TALLOC_CTX *mem_ctx,
+                                       struct tevent_req *req)
+{
+    TEVENT_REQ_RETURN_ON_ERROR(req);
+
+    return EOK;
+}
