@@ -47,7 +47,6 @@ struct sdap_dyndns_update_state {
     struct dp_option *opts;
 
     const char *hostname;
-    const char *dns_zone;
     const char *realm;
     const char *servername;
     int ttl;
@@ -59,8 +58,10 @@ struct sdap_dyndns_update_state {
     bool update_ptr;
     bool check_diff;
     enum be_nsupdate_auth auth_type;
-    bool use_server_with_nsupdate;
+    bool fallback_mode;
     char *update_msg;
+    struct sss_iface_addr *ptr_addr_iter;
+    bool del_phase;
 };
 
 static void sdap_dyndns_update_addrs_done(struct tevent_req *subreq);
@@ -71,6 +72,12 @@ static errno_t sdap_dyndns_update_step(struct tevent_req *req);
 static errno_t sdap_dyndns_update_ptr_step(struct tevent_req *req);
 static void sdap_dyndns_update_done(struct tevent_req *subreq);
 static void sdap_dyndns_update_ptr_done(struct tevent_req *subreq);
+static errno_t
+sdap_dyndns_next_ptr_record(struct sdap_dyndns_update_state *state,
+                            struct tevent_req *req);
+static struct sss_iface_addr*
+sdap_get_address_to_delete(struct sss_iface_addr *address_it,
+                           uint8_t remove_af);
 
 struct tevent_req *
 sdap_dyndns_update_send(TALLOC_CTX *mem_ctx,
@@ -81,7 +88,6 @@ sdap_dyndns_update_send(TALLOC_CTX *mem_ctx,
                         enum be_nsupdate_auth auth_type,
                         const char *ifname,
                         const char *hostname,
-                        const char *dns_zone,
                         const char *realm,
                         const char *servername,
                         const int ttl,
@@ -91,6 +97,7 @@ sdap_dyndns_update_send(TALLOC_CTX *mem_ctx,
     struct tevent_req *req;
     struct tevent_req *subreq;
     struct sdap_dyndns_update_state *state;
+    const char *conf_servername;
 
     req = tevent_req_create(mem_ctx, &state, struct sdap_dyndns_update_state);
     if (req == NULL) {
@@ -99,15 +106,22 @@ sdap_dyndns_update_send(TALLOC_CTX *mem_ctx,
     state->check_diff = check_diff;
     state->update_ptr = dp_opt_get_bool(opts, DP_OPT_DYNDNS_UPDATE_PTR);
     state->hostname = hostname;
-    state->dns_zone = dns_zone;
     state->realm = realm;
     state->servername = servername;
-    state->use_server_with_nsupdate = false;
+    state->fallback_mode = false;
     state->ttl = ttl;
     state->be_res = be_ctx->be_res;
     state->ev = ev;
     state->opts = opts;
     state->auth_type = auth_type;
+    state->ptr_addr_iter = NULL;
+    state->del_phase = true;
+
+    /* fallback servername is overriden by user option */
+    conf_servername = dp_opt_get_string(opts, DP_OPT_DYNDNS_SERVER);
+    if (conf_servername != NULL) {
+        state->servername = conf_servername;
+    }
 
     if (ifname) {
        /* Unless one family is restricted, just replace all
@@ -312,15 +326,15 @@ sdap_dyndns_update_step(struct tevent_req *req)
     state = tevent_req_data(req, struct sdap_dyndns_update_state);
 
     servername = NULL;
-    if (state->use_server_with_nsupdate == true &&
+    if (state->fallback_mode == true &&
         state->servername) {
         servername = state->servername;
     }
 
-    ret = be_nsupdate_create_fwd_msg(state, state->realm, state->dns_zone,
+    ret = be_nsupdate_create_fwd_msg(state, state->realm,
                                      servername, state->hostname,
                                      state->ttl, state->remove_af,
-                                     state->addresses, state->dns_addrlist,
+                                     state->addresses,
                                      &state->update_msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Can't get addresses for DNS update\n");
@@ -355,9 +369,9 @@ sdap_dyndns_update_done(struct tevent_req *subreq)
     talloc_zfree(subreq);
     if (ret != EOK) {
         /* If the update didn't succeed, we can retry using the server name */
-        if (state->use_server_with_nsupdate == false && state->servername &&
+        if (state->fallback_mode == false && state->servername &&
             WIFEXITED(child_status) && WEXITSTATUS(child_status) != 0) {
-            state->use_server_with_nsupdate = true;
+            state->fallback_mode = true;
             DEBUG(SSSDBG_MINOR_FAILURE,
                    "nsupdate failed, retrying with server name\n");
             ret = sdap_dyndns_update_step(req);
@@ -377,6 +391,16 @@ sdap_dyndns_update_done(struct tevent_req *subreq)
     }
 
     talloc_free(state->update_msg);
+
+    /* init iterator for addresses to be deleted */
+    state->ptr_addr_iter = sdap_get_address_to_delete(state->dns_addrlist,
+                                                      state->remove_af);
+    if (state->ptr_addr_iter == NULL) {
+        /* init iterator for addresses to be added */
+        state->del_phase = false;
+        state->ptr_addr_iter = state->addresses;
+    }
+
     ret = sdap_dyndns_update_ptr_step(req);
     if (ret != EOK) {
         tevent_req_error(req, ret);
@@ -385,6 +409,50 @@ sdap_dyndns_update_done(struct tevent_req *subreq)
     /* Execution will resume in sdap_dyndns_update_ptr_done */
 }
 
+
+static bool remove_addr(int address_family, uint8_t remove_af)
+{
+    bool ret = false;
+
+    switch(address_family) {
+    case AF_INET:
+        if (remove_af & DYNDNS_REMOVE_A) {
+            ret = true;
+        }
+        break;
+    case AF_INET6:
+        if (remove_af & DYNDNS_REMOVE_AAAA) {
+            ret = true;
+        }
+        break;
+    default:
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unknown address family\n");
+        ret = false;
+    }
+
+    return ret;
+}
+
+static struct sss_iface_addr*
+sdap_get_address_to_delete(struct sss_iface_addr *address_it,
+                           uint8_t remove_af)
+{
+    struct sockaddr_storage* address;
+
+    while (address_it != NULL) {
+        address = sss_iface_addr_get_address(address_it);
+
+        /* skip addresses that are not to be deleted */
+        if (remove_addr(address->ss_family, remove_af)) {
+            break;
+        }
+
+        address_it = sss_iface_addr_get_next(address_it);
+    }
+
+    return address_it;
+}
+
 static errno_t
 sdap_dyndns_update_ptr_step(struct tevent_req *req)
 {
@@ -392,20 +460,24 @@ sdap_dyndns_update_ptr_step(struct tevent_req *req)
     struct sdap_dyndns_update_state *state;
     const char *servername;
     struct tevent_req *subreq;
+    struct sockaddr_storage *address;
 
     state = tevent_req_data(req, struct sdap_dyndns_update_state);
 
     servername = NULL;
-    if (state->use_server_with_nsupdate == true &&
+    if (state->fallback_mode == true &&
         state->servername) {
         servername = state->servername;
     }
 
-    ret = be_nsupdate_create_ptr_msg(state, state->realm,
-                                     servername, state->hostname,
-                                     state->ttl, state->remove_af,
-                                     state->addresses, state->dns_addrlist,
-                                     &state->update_msg);
+    address = sss_iface_addr_get_address(state->ptr_addr_iter);
+    if (address == NULL) {
+        return EIO;
+    }
+
+    ret = be_nsupdate_create_ptr_msg(state, state->realm, servername,
+                                     state->hostname, state->ttl, address,
+                                     state->del_phase, &state->update_msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Can't get addresses for DNS update\n");
         return ret;
@@ -439,9 +511,9 @@ sdap_dyndns_update_ptr_done(struct tevent_req *subreq)
     talloc_zfree(subreq);
     if (ret != EOK) {
         /* If the update didn't succeed, we can retry using the server name */
-        if (state->use_server_with_nsupdate == false && state->servername &&
+        if (state->fallback_mode == false && state->servername &&
             WIFEXITED(child_status) && WEXITSTATUS(child_status) != 0) {
-            state->use_server_with_nsupdate = true;
+            state->fallback_mode = true;
             DEBUG(SSSDBG_MINOR_FAILURE,
                    "nsupdate failed, retrying with server name\n");
             ret = sdap_dyndns_update_ptr_step(req);
@@ -450,13 +522,55 @@ sdap_dyndns_update_ptr_done(struct tevent_req *subreq)
             }
         }
 
+        ret = sdap_dyndns_next_ptr_record(state, req);
+        if (ret == EAGAIN) {
+            return;
+        }
+
         tevent_req_error(req, ret);
         return;
     }
 
+    ret = sdap_dyndns_next_ptr_record(state, req);
+    if (ret == EAGAIN) {
+        return;
+    }
+
     tevent_req_done(req);
 }
 
+static errno_t
+sdap_dyndns_next_ptr_record(struct sdap_dyndns_update_state *state,
+                            struct tevent_req *req)
+{
+    errno_t ret;
+
+    if (state->del_phase) {
+        /* iterate to next address to delete */
+        state->ptr_addr_iter = sdap_get_address_to_delete(
+            sss_iface_addr_get_next(state->ptr_addr_iter), state->remove_af);
+        if (state->ptr_addr_iter == NULL) {
+            /* init iterator for addresses to be added */
+            state->del_phase = false;
+            state->ptr_addr_iter = state->addresses;
+        }
+    } else {
+        /* iterate to next address to add */
+        state->ptr_addr_iter = sss_iface_addr_get_next(state->ptr_addr_iter);
+    }
+
+    if (state->ptr_addr_iter != NULL) {
+
+        state->fallback_mode = false;
+        ret = sdap_dyndns_update_ptr_step(req);
+        if (ret == EOK) {
+            return EAGAIN;
+        }
+    }
+
+    return EOK;
+}
+
 errno_t
 sdap_dyndns_update_recv(struct tevent_req *req)
 {
@@ -474,6 +588,65 @@ static void sdap_dyndns_get_addrs_done(struct tevent_req *subreq);
 static errno_t sdap_dyndns_add_ldap_conn(struct sdap_dyndns_get_addrs_state *state,
                                          struct sdap_handle *sh);
 
+static errno_t get_ifaces_addrs(TALLOC_CTX *mem_ctx,
+                                const char *iface,
+                                struct sss_iface_addr **_result)
+{
+    struct sss_iface_addr *result_addrs = NULL;
+    struct sss_iface_addr *intf_addrs;
+    TALLOC_CTX *tmp_ctx;
+    char **list_of_intfs;
+    int num_of_intfs;
+    errno_t ret;
+    int i;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = split_on_separator(tmp_ctx, iface, ',', true, true, &list_of_intfs,
+                             &num_of_intfs);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "Parsing names of interfaces failed - %d:[%s].\n",
+              ret, sss_strerror(ret));
+        goto done;
+    }
+
+    for (i = 0; i < num_of_intfs; i++) {
+        ret = sss_iface_addr_list_get(tmp_ctx, list_of_intfs[i], &intf_addrs);
+        if (ret == EOK) {
+            if (result_addrs != NULL) {
+                /* If there is already an existing list, head of this existing
+                 * list will be considered as parent talloc context for the
+                 * new list.
+                 */
+                talloc_steal(result_addrs, intf_addrs);
+            }
+            sss_iface_addr_concatenate(&result_addrs, intf_addrs);
+        } else if (ret == ENOENT) {
+            /* non-critical failure */
+            DEBUG(SSSDBG_TRACE_FUNC,
+                  "Cannot get interface %s or there are no addresses "
+                  "bind to it.\n", list_of_intfs[i]);
+        } else {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "Cannot get list of addresses from interface %s - %d:[%s]\n",
+                  list_of_intfs[i], ret, sss_strerror(ret));
+            goto done;
+        }
+    }
+
+    ret = EOK;
+    *_result = talloc_steal(mem_ctx, result_addrs);
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
 static struct tevent_req *
 sdap_dyndns_get_addrs_send(TALLOC_CTX *mem_ctx,
                            struct tevent_context *ev,
@@ -492,10 +665,11 @@ sdap_dyndns_get_addrs_send(TALLOC_CTX *mem_ctx,
     }
 
     if (iface) {
-        ret = sss_iface_addr_list_get(state, iface, &state->addresses);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE,
-                  "Cannot get list of addresses from interface %s\n", iface);
+        ret = get_ifaces_addrs(state, iface, &state->addresses);
+        if (ret != EOK || state->addresses == NULL) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "get_ifaces_addrs() failed: %d:[%s]\n",
+                  ret, sss_strerror(ret));
         }
         /* We're done. Just fake an async request completion */
         goto done;
@@ -576,7 +750,6 @@ sdap_dyndns_add_ldap_conn(struct sdap_dyndns_get_addrs_state *state,
 {
     int ret;
     int fd;
-    struct sss_iface_addr *address;
     struct sockaddr_storage ss;
     socklen_t ss_len = sizeof(ss);
 
@@ -598,20 +771,21 @@ sdap_dyndns_add_ldap_conn(struct sdap_dyndns_get_addrs_state *state,
         return ret;
     }
 
-    switch(ss.ss_family) {
-    case AF_INET:
-    case AF_INET6:
-        address = sss_iface_addr_add(state, &state->addresses, &ss);
-        if (address == NULL) {
-            return ENOMEM;
-        }
-        break;
-    default:
+    if (ss.ss_family != AF_INET && ss.ss_family != AF_INET6) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Connection to LDAP is neither IPv4 nor IPv6\n");
         return EIO;
     }
 
+    ret = sss_get_dualstack_addresses(state, (struct sockaddr *) &ss,
+                                      &state->addresses);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "sss_get_dualstack_addresses failed: %d:[%s]\n",
+              ret, sss_strerror(ret));
+        return ret;
+    }
+
     return EOK;
 }
 
@@ -691,7 +865,6 @@ fail:
     return req;
 }
 
-
 static void
 sdap_dyndns_timer_conn_done(struct tevent_req *subreq)
 {
