@@ -28,8 +28,7 @@
 #define BUFLEN  1024
 
 errno_t
-proxy_save_service(struct sysdb_ctx *sysdb,
-                   struct sss_domain_info *domain,
+proxy_save_service(struct sss_domain_info *domain,
                    struct servent *svc,
                    bool lowercase,
                    uint64_t cache_timeout)
@@ -39,12 +38,14 @@ proxy_save_service(struct sysdb_ctx *sysdb,
     const char **protocols;
     const char **cased_aliases;
     TALLOC_CTX *tmp_ctx;
+    char *lc_alias = NULL;
     time_t now = time(NULL);
 
     tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) return ENOMEM;
 
-    cased_name = sss_get_cased_name(tmp_ctx, svc->s_name, !lowercase);
+    cased_name = sss_get_cased_name(tmp_ctx, svc->s_name,
+                                    domain->case_preserve);
     if (!cased_name) {
         ret = ENOMEM;
         goto done;
@@ -72,7 +73,25 @@ proxy_save_service(struct sysdb_ctx *sysdb,
         goto done;
     }
 
-    ret = sysdb_store_service(sysdb, domain,
+    if (domain->case_preserve) {
+        /* Add lowercased alias to allow case-insensitive lookup */
+        lc_alias = sss_tc_utf8_str_tolower(tmp_ctx, svc->s_name);
+        if (lc_alias == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, "Cannot convert name to lowercase.\n");
+            ret = ENOMEM;
+            goto done;
+        }
+
+        ret = add_string_to_list(tmp_ctx, lc_alias,
+                                 discard_const_p(char **, &cased_aliases));
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "Failed to add lowercased name alias.\n");
+            goto done;
+        }
+    }
+
+    ret = sysdb_store_service(domain,
                               cased_name,
                               ntohs(svc->s_port),
                               cased_aliases,
@@ -87,7 +106,6 @@ done:
 
 errno_t
 get_serv_byname(struct proxy_id_ctx *ctx,
-                struct sysdb_ctx *sysdb,
                 struct sss_domain_info *dom,
                 const char *name,
                 const char *protocol)
@@ -117,11 +135,11 @@ get_serv_byname(struct proxy_id_ctx *ctx,
 
     if (status == NSS_STATUS_NOTFOUND) {
         /* Make sure we remove it from the cache */
-        ret = sysdb_svc_delete(sysdb, dom, name, 0, protocol);
+        ret = sysdb_svc_delete(dom, name, 0, protocol);
     } else {
 
         /* Results found. Save them into the cache */
-        ret = proxy_save_service(sysdb, dom, result,
+        ret = proxy_save_service(dom, result,
                                  !dom->case_sensitive,
                                  dom->service_timeout);
     }
@@ -133,7 +151,6 @@ done:
 
 errno_t
 get_serv_byport(struct proxy_id_ctx *ctx,
-                struct sysdb_ctx *sysdb,
                 struct sss_domain_info *dom,
                 const char *be_filter,
                 const char *protocol)
@@ -171,10 +188,10 @@ get_serv_byport(struct proxy_id_ctx *ctx,
 
     if (status == NSS_STATUS_NOTFOUND) {
         /* Make sure we remove it from the cache */
-        ret = sysdb_svc_delete(sysdb, dom, NULL, port, protocol);
+        ret = sysdb_svc_delete(dom, NULL, port, protocol);
     } else {
         /* Results found. Save them into the cache */
-        ret = proxy_save_service(sysdb, dom, result,
+        ret = proxy_save_service(dom, result,
                                  !dom->case_sensitive,
                                  dom->service_timeout);
     }
@@ -309,14 +326,14 @@ enum_services(struct proxy_id_ctx *ctx,
                     break;
                 }
 
-                ret = sysdb_store_service(sysdb, dom,
-                        svc->s_name,
-                        svc->s_port,
-                        cased_aliases,
-                        protocols,
-                        NULL, NULL,
-                        dom->service_timeout,
-                        now);
+                ret = sysdb_store_service(dom,
+                                          svc->s_name,
+                                          svc->s_port,
+                                          cased_aliases,
+                                          protocols,
+                                          NULL, NULL,
+                                          dom->service_timeout,
+                                          now);
                 if (ret) {
                     /* Do not fail completely on errors.
                      * Just report the failure to save and go on */
