@@ -24,6 +24,11 @@
 #include "util/sss_utf8.h"
 #include "db/sysdb.h"
 
+#define NON_EXIST_USR_ALLOW "The user %s does not exist. Possible typo in simple_allow_users.\n"
+#define NON_EXIST_USR_DENY  "The user %s does not exist. Possible typo in simple_deny_users.\n"
+#define NON_EXIST_GRP_ALLOW "The group %s does not exist. Possible typo in simple_allow_groups.\n"
+#define NON_EXIST_GRP_DENY  "The group %s does not exist. Possible typo in simple_deny_groups.\n"
+
 static bool
 is_posix(const struct ldb_message *group)
 {
@@ -50,12 +55,14 @@ simple_check_users(struct simple_ctx *ctx, const char *username,
     /* First, check whether the user is in the allowed users list */
     if (ctx->allow_users != NULL) {
         for(i = 0; ctx->allow_users[i] != NULL; i++) {
-            domain = find_subdomain_by_object_name(ctx->domain,
-                                                   ctx->allow_users[i]);
+            domain = find_domain_by_object_name(ctx->domain,
+                                                ctx->allow_users[i]);
             if (domain == NULL) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "Invalid user %s!\n",
-                                            ctx->allow_users[i]);
-                return EINVAL;
+                DEBUG(SSSDBG_CRIT_FAILURE, NON_EXIST_USR_ALLOW,
+                      ctx->allow_users[i]);
+                sss_log(SSS_LOG_CRIT, NON_EXIST_USR_ALLOW,
+                        ctx->allow_users[i]);
+                continue;
             }
 
             if (sss_string_equal(domain->case_sensitive, username,
@@ -66,9 +73,11 @@ simple_check_users(struct simple_ctx *ctx, const char *username,
 
                 /* Do not return immediately on explicit allow
                  * We need to make sure none of the user's groups
-                 * are denied.
+                 * are denied. But there's no need to check username
+                 * matches any more.
                  */
                 *access_granted = true;
+                break;
             }
         }
     } else if (!ctx->allow_groups) {
@@ -83,11 +92,13 @@ simple_check_users(struct simple_ctx *ctx, const char *username,
     /* Next check whether this user has been specifically denied */
     if (ctx->deny_users != NULL) {
         for(i = 0; ctx->deny_users[i] != NULL; i++) {
-            domain = find_subdomain_by_object_name(ctx->domain,
-                                                   ctx->deny_users[i]);
+            domain = find_domain_by_object_name(ctx->domain,
+                                                ctx->deny_users[i]);
             if (domain == NULL) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "Invalid user %s!\n",
-                                            ctx->deny_users[i]);
+                DEBUG(SSSDBG_CRIT_FAILURE, NON_EXIST_USR_DENY,
+                      ctx->deny_users[i]);
+                sss_log(SSS_LOG_CRIT, NON_EXIST_USR_DENY,
+                        ctx->deny_users[i]);
                 return EINVAL;
             }
 
@@ -122,12 +133,15 @@ simple_check_groups(struct simple_ctx *ctx, const char **group_names,
     if (ctx->allow_groups && !*access_granted) {
         matched = false;
         for (i = 0; ctx->allow_groups[i]; i++) {
-            domain = find_subdomain_by_object_name(ctx->domain,
-                                                   ctx->allow_groups[i]);
+            domain = find_domain_by_object_name(ctx->domain,
+                                                ctx->allow_groups[i]);
             if (domain == NULL) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "Invalid group %s!\n",
-                                            ctx->allow_groups[i]);
-                return EINVAL;
+                DEBUG(SSSDBG_CRIT_FAILURE, NON_EXIST_GRP_ALLOW,
+                      ctx->allow_groups[i]);
+                sss_log(SSS_LOG_CRIT, NON_EXIST_GRP_ALLOW,
+                        ctx->allow_groups[i]);
+
+                continue;
             }
 
             for(j = 0; group_names[j]; j++) {
@@ -155,11 +169,14 @@ simple_check_groups(struct simple_ctx *ctx, const char **group_names,
     if (ctx->deny_groups) {
         matched = false;
         for (i = 0; ctx->deny_groups[i]; i++) {
-            domain = find_subdomain_by_object_name(ctx->domain,
-                                                   ctx->deny_groups[i]);
+            domain = find_domain_by_object_name(ctx->domain,
+                                                ctx->deny_groups[i]);
             if (domain == NULL) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "Invalid group %s!\n",
-                                            ctx->deny_groups[i]);
+                DEBUG(SSSDBG_CRIT_FAILURE, NON_EXIST_GRP_DENY,
+                      ctx->deny_groups[i]);
+                sss_log(SSS_LOG_CRIT, NON_EXIST_GRP_DENY,
+                        ctx->deny_groups[i]);
+
                 return EINVAL;
             }
 
@@ -279,8 +296,7 @@ simple_resolve_group_check(struct simple_resolve_group_state *state)
                                   SYSDB_GIDNUM, NULL };
 
     /* Check the cache by GID again and fetch the name */
-    ret = sysdb_search_group_by_gid(state, state->domain->sysdb,
-                                    state->domain, state->gid,
+    ret = sysdb_search_group_by_gid(state, state->domain, state->gid,
                                     group_attrs, &group);
     if (ret == ENOENT) {
         /* The group is missing, we will try to update it. */
@@ -379,6 +395,8 @@ struct simple_check_groups_state {
 
     const char **group_names;
     size_t num_names;
+
+    bool failed_to_resolve_groups;
 };
 
 static void simple_check_get_groups_next(struct tevent_req *subreq);
@@ -414,19 +432,20 @@ simple_check_get_groups_send(TALLOC_CTX *mem_ctx,
 
     state->ev = ev;
     state->ctx = ctx;
+    state->failed_to_resolve_groups = false;
 
     DEBUG(SSSDBG_TRACE_LIBS, "Looking up groups for user %s\n", username);
 
     /* get domain from username */
-    state->domain = find_subdomain_by_object_name(ctx->domain, username);
+    state->domain = find_domain_by_object_name(ctx->domain, username);
     if (state->domain == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Invalid user %s!\n", username);
         ret = EINVAL;
         goto done;
     }
 
-    ret = sysdb_search_user_by_name(state, state->domain->sysdb, state->domain,
-                                    username, attrs, &user);
+    ret = sysdb_search_user_by_name(state, state->domain, username, attrs,
+                                    &user);
     if (ret == ENOENT) {
         DEBUG(SSSDBG_MINOR_FAILURE, "No such user %s\n", username);
         ret = ERR_ACCOUNT_UNKNOWN;
@@ -438,7 +457,7 @@ simple_check_get_groups_send(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ret = sysdb_asq_search(state, state->domain->sysdb,
+    ret = sysdb_asq_search(state, state->domain,
                            user->dn, NULL, SYSDB_MEMBEROF,
                            attrs, &group_count, &groups);
     if (ret != EOK) {
@@ -532,11 +551,10 @@ static void simple_check_get_groups_next(struct tevent_req *subreq)
         DEBUG(SSSDBG_OP_FAILURE,
               "Could not resolve name of group with GID %"SPRIgid"\n",
               state->lookup_groups[state->giter].gid);
-        tevent_req_error(req, ret);
-        return;
+        state->failed_to_resolve_groups = true;
+    } else {
+        state->num_names++;
     }
-
-    state->num_names++;
     state->giter++;
 
     if (state->giter < state->num_groups) {
@@ -612,7 +630,7 @@ simple_check_process_group(struct simple_check_groups_state *state,
         /* We will look it up in main domain. */
         domain = state->ctx->domain;
     } else {
-        domain = find_subdomain_by_sid(state->ctx->domain, group_sid);
+        domain = find_domain_by_sid(state->ctx->domain, group_sid);
         if (domain == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE, "There is no domain information for "
                                         "SID %s\n", group_sid);
@@ -637,8 +655,8 @@ simple_check_get_groups_primary(struct simple_check_groups_state *state,
                                   SYSDB_GIDNUM, SYSDB_SID_STR, NULL };
     struct ldb_message *msg;
 
-    ret = sysdb_search_group_by_gid(state, state->domain->sysdb, state->domain,
-                                    gid, group_attrs, &msg);
+    ret = sysdb_search_group_by_gid(state, state->domain, gid, group_attrs,
+                                    &msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Could not look up primary group [%"SPRIgid"]: [%d][%s]\n",
@@ -670,6 +688,9 @@ simple_check_get_groups_recv(struct tevent_req *req,
     TEVENT_REQ_RETURN_ON_ERROR(req);
 
     *_group_names = talloc_steal(mem_ctx, state->group_names);
+    if (state->failed_to_resolve_groups) {
+        return ERR_SIMPLE_GROUPS_MISSING;
+    }
     return EOK;
 }
 
@@ -759,12 +780,25 @@ static void simple_access_check_done(struct tevent_req *subreq)
 
     /* We know the names now. Run the check. */
     ret = simple_check_get_groups_recv(subreq, state, &state->group_names);
+
     talloc_zfree(subreq);
     if (ret == ENOENT) {
         /* If the user wasn't found, just shortcut */
         state->access_granted = false;
         tevent_req_done(req);
         return;
+    } else if (ret == ERR_SIMPLE_GROUPS_MISSING) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Could not collect groups of user %s\n", state->username);
+        if (state->ctx->deny_groups == NULL) {
+            DEBUG(SSSDBG_TRACE_FUNC,
+                  "But no deny groups were defined so we can continue.\n");
+        } else {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "Some deny groups were defined, we can't continue\n");
+            tevent_req_error(req, ret);
+            return;
+        }
     } else if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Could not collect groups of user %s\n", state->username);
