@@ -303,7 +303,7 @@ HbacRuleElement_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    self->category = sss_python_set_new();
+    self->category = PySet_New(NULL);
     self->names = PyList_New(0);
     self->groups = PyList_New(0);
     if (!self->names || !self->groups || !self->category) {
@@ -388,7 +388,7 @@ HbacRuleElement_init(HbacRuleElement *self, PyObject *args, PyObject *kwargs)
             return -1;
         }
 
-        if (sss_python_set_add(self->category, tmp) != 0) {
+        if (PySet_Add(self->category, tmp) != 0) {
             Py_DECREF(tmp);
             return -1;
         }
@@ -452,7 +452,7 @@ hbac_rule_element_set_category(HbacRuleElement *self,
 
     CHECK_ATTRIBUTE_DELETE(category, "category");
 
-    if (!sss_python_set_check(category)) {
+    if (!PySet_Check(category)) {
         PyErr_Format(PyExc_TypeError, "The category must be a set type\n");
         return -1;
     }
@@ -493,7 +493,7 @@ HbacRuleElement_repr(HbacRuleElement *self)
     uint32_t category;
     PyObject *o, *format, *args;
 
-    format = sss_python_unicode_from_string("<category %lu names [%s] groups [%s]>");
+    format = PyUnicode_FromString("<category %lu names [%s] groups [%s]>");
     if (format == NULL) {
         return NULL;
     }
@@ -651,7 +651,7 @@ HbacRule_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    self->name = sss_python_unicode_from_string("");
+    self->name = PyUnicode_FromString("");
     if (self->name == NULL) {
         Py_DECREF(self);
         PyErr_NoMemory();
@@ -794,7 +794,7 @@ hbac_rule_set_enabled(HbacRuleObject *self, PyObject *enabled, void *closure)
 
         Py_DECREF(utf8_str);
         return 0;
-    } else if (PyBool_Check(enabled)) {
+    } else if (PyBool_Check(enabled) == true) {
         self->enabled = (enabled == Py_True);
         return 0;
     } else if (PYNUMBER_CHECK(enabled)) {
@@ -869,7 +869,7 @@ HbacRule_repr(HbacRuleObject *self)
     PyObject *srchosts_repr;
     PyObject *o, *format, *args;
 
-    format = sss_python_unicode_from_string("<name %s enabled %d "
+    format = PyUnicode_FromString("<name %s enabled %d "
                                             "users %s services %s "
                                             "targethosts %s srchosts %s>");
     if (format == NULL) {
@@ -945,7 +945,7 @@ py_hbac_rule_validate(HbacRuleObject *self, PyObject *args)
     }
 
     py_is_valid = PyBool_FromLong(is_valid);
-    py_missing = sss_python_set_new();
+    py_missing = PySet_New(NULL);
     if (!py_missing || !py_is_valid) {
         PyErr_NoMemory();
         goto fail;
@@ -962,7 +962,7 @@ py_hbac_rule_validate(HbacRuleObject *self, PyObject *args)
             goto fail;
         }
 
-        if (sss_python_set_add(py_missing, py_attr) != 0) {
+        if (PySet_Add(py_missing, py_attr) != 0) {
             /* If the set-add succeeded, it would steal the reference */
             Py_DECREF(py_attr);
             goto fail;
@@ -1149,7 +1149,7 @@ HbacRequestElement_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    self->name = sss_python_unicode_from_string("");
+    self->name = PyUnicode_FromString("");
     if (self->name == NULL) {
         PyErr_NoMemory();
         Py_DECREF(self);
@@ -1291,7 +1291,7 @@ HbacRequestElement_repr(HbacRequestElement *self)
     char *strgroups;
     PyObject *o, *format, *args;
 
-    format = sss_python_unicode_from_string("<name %s groups [%s]>");
+    format = PyUnicode_FromString("<name %s groups [%s]>");
     if (format == NULL) {
         return NULL;
     }
@@ -1609,7 +1609,7 @@ py_hbac_evaluate(HbacRequest *self, PyObject *args)
     eres = hbac_evaluate(rules, hbac_req, &info);
     switch (eres) {
     case HBAC_EVAL_ALLOW:
-        self->rule_name = sss_python_unicode_from_string(info->rule_name);
+        self->rule_name = PyUnicode_FromString(info->rule_name);
         if (!self->rule_name) {
             PyErr_NoMemory();
             goto fail;
@@ -1662,7 +1662,7 @@ HbacRequest_repr(HbacRequest *self)
     PyObject *srchost_repr;
     PyObject *o, *format, *args;
 
-    format = sss_python_unicode_from_string("<user %s service %s "
+    format = PyUnicode_FromString("<user %s service %s "
                                             "targethost %s srchost %s>");
     if (format == NULL) {
         return NULL;
@@ -1853,7 +1853,7 @@ py_hbac_result_string(PyObject *module, PyObject *args)
         return Py_None;
     }
 
-    return sss_python_unicode_from_string(str);
+    return PyUnicode_FromString(str);
 }
 
 PyDoc_STRVAR(py_hbac_error_string__doc__,
@@ -1877,7 +1877,7 @@ py_hbac_error_string(PyObject *module, PyObject *args)
         return Py_None;
     }
 
-    return sss_python_unicode_from_string(str);
+    return PyUnicode_FromString(str);
 }
 
 static PyMethodDef pyhbac_module_methods[] = {
