@@ -661,7 +661,7 @@ lookup_automntmap_step(struct setautomntent_lookup_ctx *lookup_ctx)
             if (!dctx->check_provider) {
                 if (dctx->cmd_ctx->check_next) {
                     DEBUG(SSSDBG_TRACE_INTERNAL, "Moving on to next domain\n");
-                    dom = get_next_domain(dom, false);
+                    dom = get_next_domain(dom, 0);
                     continue;
                 }
                 else break;
@@ -868,8 +868,8 @@ static void lookup_automntmap_cache_updated(uint16_t err_maj, uint32_t err_min,
                "Will try to return what we have in cache\n",
                (unsigned int)err_maj, (unsigned int)err_min, err_msg);
         /* Loop to the next domain if possible */
-        if (dctx->cmd_ctx->check_next && get_next_domain(dctx->domain, false)) {
-            dctx->domain = get_next_domain(dctx->domain, false);
+        if (dctx->cmd_ctx->check_next
+                && (dctx->domain = get_next_domain(dctx->domain, 0))) {
             dctx->check_provider = NEED_CHECK_PROVIDER(dctx->domain->provider);
         }
     }
