@@ -22,8 +22,6 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <unistd.h>
-#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
@@ -45,27 +43,6 @@
 #include "monitor/monitor_interfaces.h"
 #include "sbus/sbus_client.h"
 
-static errno_t set_nonblocking(int fd)
-{
-    int v;
-    int ferr;
-    errno_t error;
-
-    /* Get the current flags for this file descriptor */
-    v = fcntl(fd, F_GETFL, 0);
-
-    errno = 0;
-    /* Set the non-blocking flag on this fd */
-    ferr = fcntl(fd, F_SETFL, v | O_NONBLOCK);
-    if (ferr < 0) {
-        error = errno;
-        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to set fd non-blocking: [%d][%s]\n",
-                  error, strerror(error));
-        return error;
-    }
-    return EOK;
-}
-
 static errno_t set_close_on_exec(int fd)
 {
     int v;
@@ -537,7 +514,6 @@ static int sss_dp_init(struct resp_ctx *rctx,
                        const char *cli_name,
                        struct sss_domain_info *domain)
 {
-    struct sbus_interface *intf;
     struct be_conn *be_conn;
     int ret;
 
@@ -562,12 +538,7 @@ static int sss_dp_init(struct resp_ctx *rctx,
         return ret;
     }
 
-    intf = sbus_new_interface(rctx, DP_PATH, dp_intf, rctx);
-    if (!intf) {
-        ret = ENOMEM;
-    } else {
-        ret = sbus_conn_add_interface(be_conn->conn, intf);
-    }
+    ret = sbus_conn_register_iface(be_conn->conn, dp_intf, DP_PATH, rctx);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "Failed to export data provider.\n");
         return ret;
@@ -601,7 +572,7 @@ int create_pipe_fd(const char *sock_name, int *_fd, mode_t umaskval)
 
     orig_umaskval = umask(umaskval);
 
-    ret = set_nonblocking(fd);
+    ret = sss_fd_nonblocking(fd);
     if (ret != EOK) {
         goto done;
     }
@@ -873,7 +844,7 @@ int sss_process_init(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    for (dom = rctx->domains; dom; dom = get_next_domain(dom, false)) {
+    for (dom = rctx->domains; dom; dom = get_next_domain(dom, 0)) {
         ret = sss_names_init(rctx->cdb, rctx->cdb, dom->name, &dom->names);
         if (ret != EOK) {
             DEBUG(SSSDBG_FATAL_FAILURE,
@@ -951,8 +922,9 @@ responder_get_domain(struct resp_ctx *rctx, const char *name)
     struct sss_domain_info *dom;
     struct sss_domain_info *ret_dom = NULL;
 
-    for (dom = rctx->domains; dom; dom = get_next_domain(dom, true)) {
-        if (dom->disabled) {
+    for (dom = rctx->domains; dom;
+            dom = get_next_domain(dom, SSS_GND_DESCEND)) {
+        if (sss_domain_get_state(dom) == DOM_DISABLED) {
             continue;
         }
 
@@ -986,8 +958,10 @@ errno_t responder_get_domain_by_id(struct resp_ctx *rctx, const char *id,
 
     id_len = strlen(id);
 
-    for (dom = rctx->domains; dom; dom = get_next_domain(dom, true)) {
-        if (dom->disabled || dom->domain_id == NULL) {
+    for (dom = rctx->domains; dom;
+            dom = get_next_domain(dom, SSS_GND_DESCEND)) {
+        if (sss_domain_get_state(dom) == DOM_DISABLED ||
+                dom->domain_id == NULL) {
             continue;
         }
 
