@@ -29,16 +29,11 @@ static DBusMessage *sss_dp_get_domains_msg(void *pvt);
 struct sss_dp_domains_info {
     struct sss_domain_info *dom;
     const char *hint;
-    /* The DBus API expects its own Boolean type when formatting argument
-     * with DBUS_TYPE_BOOLEAN
-     */
-    dbus_bool_t force;
 };
 
 static struct tevent_req *
 get_subdomains_send(TALLOC_CTX *mem_ctx, struct resp_ctx *rctx,
                     struct sss_domain_info *dom,
-                    const bool force,
                     const char *hint)
 {
     errno_t ret;
@@ -58,7 +53,6 @@ get_subdomains_send(TALLOC_CTX *mem_ctx, struct resp_ctx *rctx,
         goto fail;
     }
     info->hint = hint;
-    info->force = force ? TRUE : FALSE;
     info->dom = dom;
 
     key = talloc_asprintf(state, "domains@%s", dom->name);
@@ -102,15 +96,14 @@ sss_dp_get_domains_msg(void *pvt)
     }
 
     DEBUG(SSSDBG_TRACE_FUNC,
-          "Sending get domains request for [%s][%sforced][%s]\n",
-           info->dom->name, info->force ? "" : "not ", info->hint);
+          "Sending get domains request for [%s][%s]\n",
+           info->dom->name, info->hint);
 
     /* Send the hint argument to provider as well. This will
      * be useful for some cases of transitional trust where
      * the server might not know all trusted domains
      */
     dbret = dbus_message_append_args(msg,
-                                     DBUS_TYPE_BOOLEAN, &info->force,
                                      DBUS_TYPE_STRING, &info->hint,
                                      DBUS_TYPE_INVALID);
     if (!dbret) {
@@ -141,7 +134,6 @@ struct sss_dp_get_domains_state {
     struct resp_ctx *rctx;
     struct sss_domain_info *dom;
     const char *hint;
-    bool force;
 };
 
 static void
@@ -182,7 +174,6 @@ struct tevent_req *sss_dp_get_domains_send(TALLOC_CTX *mem_ctx,
     }
 
     state->rctx = rctx;
-    state->force = force;
     if (hint != NULL) {
         state->hint = hint;
     } else {
@@ -195,7 +186,7 @@ struct tevent_req *sss_dp_get_domains_send(TALLOC_CTX *mem_ctx,
 
     state->dom = rctx->domains;
     while(state->dom != NULL && !NEED_CHECK_PROVIDER(state->dom->provider)) {
-        state->dom = get_next_domain(state->dom, false);
+        state->dom = get_next_domain(state->dom, 0);
     }
 
     if (state->dom == NULL) {
@@ -204,8 +195,7 @@ struct tevent_req *sss_dp_get_domains_send(TALLOC_CTX *mem_ctx,
         goto immediately;
     }
 
-    subreq = get_subdomains_send(req, rctx, state->dom,
-                                 state->force, state->hint);
+    subreq = get_subdomains_send(req, rctx, state->dom, state->hint);
     if (subreq == NULL) {
         ret = ENOMEM;
         goto immediately;
@@ -252,11 +242,11 @@ sss_dp_get_domains_process(struct tevent_req *subreq)
     }
 
     /* Advance to the next domain */
-    state->dom = get_next_domain(state->dom, false);
+    state->dom = get_next_domain(state->dom, 0);
 
     /* Skip local domains */
     while(state->dom != NULL && !NEED_CHECK_PROVIDER(state->dom->provider)) {
-        state->dom = get_next_domain(state->dom, false);
+        state->dom = get_next_domain(state->dom, 0);
     }
 
     if (state->dom == NULL) {
@@ -266,7 +256,7 @@ sss_dp_get_domains_process(struct tevent_req *subreq)
         return;
     }
 
-    subreq = get_subdomains_send(req, state->rctx, state->dom, state->force, state->hint);
+    subreq = get_subdomains_send(req, state->rctx, state->dom, state->hint);
     if (subreq == NULL) {
         ret = ENOMEM;
         goto fail;
@@ -355,7 +345,8 @@ static errno_t check_last_request(struct resp_ctx *rctx, const char *hint)
     }
 
     if (hint != NULL) {
-        for (dom = rctx->domains; dom; dom = get_next_domain(dom, true)) {
+        for (dom = rctx->domains; dom;
+                dom = get_next_domain(dom, SSS_GND_DESCEND)) {
             if (!IS_SUBDOMAIN(dom)) {
                 diff = now - dom->subdomains_last_checked.tv_sec;
                 /* not a subdomain */
@@ -373,16 +364,33 @@ static errno_t check_last_request(struct resp_ctx *rctx, const char *hint)
     return EOK;
 }
 
+struct get_domains_state {
+    struct resp_ctx *rctx;
+    struct sss_nc_ctx *optional_ncache;
+};
+
 static void get_domains_at_startup_done(struct tevent_req *req)
 {
     int ret;
+    struct get_domains_state *state;
+
+    state = tevent_req_callback_data(req, struct get_domains_state);
 
     ret = sss_dp_get_domains_recv(req);
     talloc_free(req);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "sss_dp_get_domains request failed.\n");
+        DEBUG(SSSDBG_MINOR_FAILURE, "sss_dp_get_domains request failed.\n");
     }
 
+    if (state->optional_ncache != NULL) {
+        ret = sss_ncache_reset_repopulate_permanent(state->rctx,
+                                                    state->optional_ncache);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE, "sss_dp_get_domains request failed.\n");
+        }
+    }
+
+    talloc_free(state);
     return;
 }
 
@@ -391,33 +399,44 @@ static void get_domains_at_startup(struct tevent_context *ev,
                                    void *pvt)
 {
     struct tevent_req *req;
-    struct resp_ctx *rctx;
+    struct get_domains_state *state;
 
-    rctx = talloc_get_type(pvt, struct resp_ctx);
+    state = talloc_get_type(pvt, struct get_domains_state);
 
-    req = sss_dp_get_domains_send(rctx, rctx, true, NULL);
+    req = sss_dp_get_domains_send(state, state->rctx, true, NULL);
     if (req == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "sss_dp_get_domains_send failed.\n");
+        talloc_free(state);
         return;
     }
 
-    tevent_req_set_callback(req, get_domains_at_startup_done, NULL);
+    tevent_req_set_callback(req, get_domains_at_startup_done, state);
     return;
 }
 
 errno_t schedule_get_domains_task(TALLOC_CTX *mem_ctx,
                                   struct tevent_context *ev,
-                                  struct resp_ctx *rctx)
+                                  struct resp_ctx *rctx,
+                                  struct sss_nc_ctx *optional_ncache)
 {
     struct tevent_immediate *imm;
+    struct get_domains_state *state;
+
+    state = talloc(mem_ctx, struct get_domains_state);
+    if (state == NULL) {
+        return ENOMEM;
+    }
+    state->rctx = rctx;
+    state->optional_ncache = optional_ncache;
 
     imm = tevent_create_immediate(mem_ctx);
     if (imm == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "tevent_create_immediate failed.\n");
+        talloc_free(state);
         return ENOMEM;
     }
 
-    tevent_schedule_immediate(imm, ev, get_domains_at_startup, rctx);
+    tevent_schedule_immediate(imm, ev, get_domains_at_startup, state);
 
     return EOK;
 }
@@ -428,6 +447,7 @@ struct sss_parse_inp_state {
 
     char *name;
     char *domname;
+    errno_t error;
 };
 
 static void sss_parse_inp_done(struct tevent_req *subreq);
@@ -509,14 +529,24 @@ static void sss_parse_inp_done(struct tevent_req *subreq)
         return;
     }
 
+    state->error = ERR_OK;
+
     ret = sss_parse_name_for_domains(state, state->rctx->domains,
                                      state->rctx->default_domain,
                                      state->rawinp,
                                      &state->domname, &state->name);
-    if (ret != EOK) {
+    if (ret == EAGAIN && state->domname != NULL && state->name == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Unknown domain in [%s]\n", state->rawinp);
+        state->error = ERR_DOMAIN_NOT_FOUND;
+    } else if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Invalid name received [%s]\n", state->rawinp);
-        tevent_req_error(req, ERR_INPUT_PARSE);
+        state->error = ERR_INPUT_PARSE;
+    }
+
+    if (state->error != ERR_OK) {
+        tevent_req_error(req, state->error);
         return;
     }
 
@@ -530,7 +560,9 @@ errno_t sss_parse_inp_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
     struct sss_parse_inp_state *state = tevent_req_data(req,
                                                 struct sss_parse_inp_state);
 
-    TEVENT_REQ_RETURN_ON_ERROR(req);
+    if (state->error != ERR_DOMAIN_NOT_FOUND) {
+        TEVENT_REQ_RETURN_ON_ERROR(req);
+    }
 
     if (_name) {
         *_name = talloc_steal(mem_ctx, state->name);
@@ -540,5 +572,5 @@ errno_t sss_parse_inp_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
         *_domname = talloc_steal(mem_ctx, state->domname);
     }
 
-    return EOK;
+    return state->error;
 }
