@@ -29,7 +29,7 @@
 #include "providers/data_provider.h"
 #include "responder/ifp/ifp_iface_generated.h"
 
-#define INFOPIPE_PATH "/org/freedesktop/sssd/infopipe"
+#define IFP_PATH "/org/freedesktop/sssd/infopipe"
 
 struct sysbus_ctx {
     struct sbus_connection *conn;
@@ -44,8 +44,14 @@ struct ifp_ctx {
 
     struct sysbus_ctx *sysbus;
     const char **user_whitelist;
+    uint32_t wildcard_limit;
 };
 
+errno_t ifp_register_sbus_interface(struct sbus_connection *conn,
+                                    void *handler_data);
+
+void ifp_register_nodes(struct ifp_ctx *ctx, struct sbus_connection *conn);
+
 /* This is a throwaway method to ease the review of the patch.
  * It will be removed later */
 int ifp_ping(struct sbus_request *dbus_req, void *data);
@@ -68,22 +74,36 @@ errno_t ifp_req_create(struct sbus_request *dbus_req,
 /* Returns an appropriate DBus error for specific ifp_req_create failures */
 int ifp_req_create_handle_failure(struct sbus_request *dbus_req, errno_t err);
 
-const char *ifp_path_strip_prefix(const char *path, const char *prefix);
-
-char *ifp_bus_path_unescape(TALLOC_CTX *mem_ctx, const char *path);
-char *ifp_bus_path_escape(TALLOC_CTX *mem_ctx, const char *path);
-
-char *_ifp_reply_objpath(TALLOC_CTX *mem_ctx, const char *base,
-                         const char *part, ...);
-
-#define ifp_reply_objpath(mem_ctx, base, ...) \
-    _ifp_reply_objpath(mem_ctx, base, ##__VA_ARGS__, NULL)
-
 errno_t ifp_add_ldb_el_to_dict(DBusMessageIter *iter_dict,
                                struct ldb_message_element *el);
-const char **ifp_parse_attr_list(TALLOC_CTX *mem_ctx, const char *conf_str);
 const char **
-ifp_parse_attr_list_ex(TALLOC_CTX *mem_ctx, const char *conf_str,
-                       const char **defaults);
+ifp_parse_user_attr_list(TALLOC_CTX *mem_ctx, const char *conf_str);
+
+const char **
+ifp_get_user_extra_attributes(TALLOC_CTX *mem_ctx, struct ifp_ctx *ifp_ctx);
+
 bool ifp_attr_allowed(const char *whitelist[], const char *attr);
+bool ifp_is_user_attr_allowed(struct ifp_ctx *ifp_ctx, const char *attr);
+
+/* Used for list calls */
+struct ifp_list_ctx {
+    struct sbus_request *sbus_req;
+    const char *filter;
+    uint32_t limit;
+
+    struct sss_domain_info *dom;
+    struct ifp_ctx *ctx;
+
+    const char **paths;
+    size_t path_count;
+};
+
+struct ifp_list_ctx *ifp_list_ctx_new(struct sbus_request *sbus_req,
+                                      struct ifp_ctx *ctx,
+                                      const char *filter,
+                                      uint32_t limit);
+
+size_t ifp_list_ctx_remaining_capacity(struct ifp_list_ctx *list_ctx,
+                                       size_t entries);
+
 #endif /* _IFPSRV_PRIVATE_H_ */
