@@ -34,6 +34,7 @@
 #include <dbus/dbus.h>
 
 #include "util/util.h"
+#include "util/strtonum.h"
 #include "sbus/sssd_dbus.h"
 #include "monitor/monitor_interfaces.h"
 #include "confdb/confdb.h"
@@ -68,69 +69,6 @@ static struct data_provider_iface ifp_dp_methods = {
     .getAccountInfo = NULL,
 };
 
-struct infopipe_iface ifp_iface = {
-    { &infopipe_iface_meta, 0 },
-    .Ping = ifp_ping,
-
-    /* components */
-    .ListComponents = ifp_list_components,
-    .ListResponders = ifp_list_responders,
-    .ListBackends = ifp_list_backends,
-    .FindMonitor = ifp_find_monitor,
-    .FindResponderByName = ifp_find_responder_by_name,
-    .FindBackendByName = ifp_find_backend_by_name,
-
-    .GetUserAttr = ifp_user_get_attr,
-    .GetUserGroups = ifp_user_get_groups,
-    .ListDomains = ifp_list_domains,
-    .FindDomainByName = ifp_find_domain_by_name,
-};
-
-struct infopipe_component ifp_component = {
-    { &infopipe_component_meta, 0 },
-    .Enable = ifp_component_enable,
-    .Disable = ifp_component_disable,
-    .ChangeDebugLevel = ifp_component_change_debug_level,
-    .ChangeDebugLevelTemporarily = ifp_component_change_debug_level_tmp,
-    .infopipe_component_get_name = ifp_component_get_name,
-    .infopipe_component_get_debug_level = ifp_component_get_debug_level,
-    .infopipe_component_get_enabled = ifp_component_get_enabled,
-    .infopipe_component_get_type = ifp_component_get_type,
-    /* FIXME: This should be part of Components.Backends interface, onece
-     * SSSD supports multiple interfaces per object path. */
-    .infopipe_component_get_providers = ifp_backend_get_providers
-};
-
-struct infopipe_domain ifp_domain = {
-    { &infopipe_domain_meta, 0 },
-    .infopipe_domain_get_name = ifp_dom_get_name,
-    .infopipe_domain_get_provider = ifp_dom_get_provider,
-    .infopipe_domain_get_primary_servers = ifp_dom_get_primary_servers,
-    .infopipe_domain_get_backup_servers = ifp_dom_get_backup_servers,
-    .infopipe_domain_get_min_id = ifp_dom_get_min_id,
-    .infopipe_domain_get_max_id = ifp_dom_get_max_id,
-    .infopipe_domain_get_realm = ifp_dom_get_realm,
-    .infopipe_domain_get_forest = ifp_dom_get_forest,
-    .infopipe_domain_get_login_format = ifp_dom_get_login_format,
-    .infopipe_domain_get_fully_qualified_name_format = ifp_dom_get_fqdn_format,
-    .infopipe_domain_get_enumerable = ifp_dom_get_enumerable,
-    .infopipe_domain_get_use_fully_qualified_names = ifp_dom_get_use_fqdn,
-    .infopipe_domain_get_subdomain = ifp_dom_get_subdomain,
-    .infopipe_domain_get_parent_domain = ifp_dom_get_parent_domain
-};
-
-struct sysbus_iface {
-    const char *path;
-    struct sbus_vtable *iface_vtable;
-};
-
-static struct sysbus_iface ifp_ifaces[] = {
-    { INFOPIPE_PATH, &ifp_iface.vtable },
-    { INFOPIPE_DOMAIN_PATH, &ifp_domain.vtable },
-    { INFOPIPE_COMPONENT_PATH, &ifp_component.vtable },
-    { NULL, NULL },
-};
-
 struct sss_cmd_table *get_ifp_cmds(void)
 {
     static struct sss_cmd_table ifp_cmds[] = {
@@ -171,15 +109,12 @@ static errno_t
 sysbus_init(TALLOC_CTX *mem_ctx,
             struct tevent_context *ev,
             const char *dbus_name,
-            struct sysbus_iface *sysbus_ifaces,
             void *pvt,
             struct sysbus_ctx **sysbus)
 {
     DBusError dbus_error;
     DBusConnection *conn = NULL;
     struct sysbus_ctx *system_bus = NULL;
-    struct sbus_interface *sif;
-    int i;
     errno_t ret;
 
     system_bus = talloc_zero(mem_ctx, struct sysbus_ctx);
@@ -225,25 +160,14 @@ sysbus_init(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    for (i = 0; sysbus_ifaces[i].path != NULL; i++) {
-        sif = sbus_new_interface(system_bus->conn,
-                                 sysbus_ifaces[i].path,
-                                 sysbus_ifaces[i].iface_vtable,
-                                 pvt);
-        if (sif == NULL) {
-            DEBUG(SSSDBG_CRIT_FAILURE,
-                  "Could not add the sbus interface\n");
-            goto fail;
-        }
-
-        ret = sbus_conn_add_interface(system_bus->conn, sif);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_CRIT_FAILURE,
-                  "Could not add the interface\n");
-            goto fail;
-        }
+    ret = ifp_register_sbus_interface(system_bus->conn, pvt);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Could not register interfaces\n");
+        goto fail;
     }
 
+    ifp_register_nodes(pvt, system_bus->conn);
+
     *sysbus = system_bus;
     return EOK;
 
@@ -275,8 +199,7 @@ static int ifp_sysbus_reconnect(struct sbus_request *dbus_req, void *data)
 
     /* Connect to the D-BUS system bus and set up methods */
     ret = sysbus_init(ifp_ctx, ifp_ctx->rctx->ev,
-                      INFOPIPE_IFACE,
-                      ifp_ifaces,
+                      IFACE_IFP,
                       ifp_ctx, &ifp_ctx->sysbus);
     if (ret == ERR_NO_SYSBUS) {
         DEBUG(SSSDBG_MINOR_FAILURE,
@@ -306,6 +229,7 @@ int ifp_process_init(TALLOC_CTX *mem_ctx,
     int max_retries;
     char *uid_str;
     char *attr_list_str;
+    char *wildcard_limit_str;
 
     ifp_cmds = get_ifp_cmds();
     ret = sss_process_init(mem_ctx, ev, cdb,
@@ -380,7 +304,7 @@ int ifp_process_init(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    ifp_ctx->user_whitelist = ifp_parse_attr_list(ifp_ctx, attr_list_str);
+    ifp_ctx->user_whitelist = ifp_parse_user_attr_list(ifp_ctx, attr_list_str);
     talloc_free(attr_list_str);
     if (ifp_ctx->user_whitelist == NULL) {
         DEBUG(SSSDBG_FATAL_FAILURE,
@@ -399,6 +323,27 @@ int ifp_process_init(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
+    /* A bit convoluted way until we have a confdb_get_uint32 */
+    ret = confdb_get_string(ifp_ctx->rctx->cdb,
+                            ifp_ctx->rctx,
+                            CONFDB_IFP_CONF_ENTRY,
+                            CONFDB_IFP_WILDCARD_LIMIT,
+                            NULL, /* no limit by default */
+                            &wildcard_limit_str);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Failed to retrieve limit for a wildcard search\n");
+        goto fail;
+    }
+
+    if (wildcard_limit_str) {
+        ifp_ctx->wildcard_limit = strtouint32(wildcard_limit_str, NULL, 10);
+        ret = errno;
+        if (ret != EOK) {
+            goto fail;
+        }
+    }
+
     for (iter = ifp_ctx->rctx->be_conns; iter; iter = iter->next) {
         sbus_reconnect_init(iter->conn, max_retries,
                             ifp_dp_reconnect_init, iter);
@@ -406,8 +351,7 @@ int ifp_process_init(TALLOC_CTX *mem_ctx,
 
     /* Connect to the D-BUS system bus and set up methods */
     ret = sysbus_init(ifp_ctx, ifp_ctx->rctx->ev,
-                      INFOPIPE_IFACE,
-                      ifp_ifaces,
+                      IFACE_IFP,
                       ifp_ctx, &ifp_ctx->sysbus);
     if (ret == ERR_NO_SYSBUS) {
         DEBUG(SSSDBG_MINOR_FAILURE,
@@ -420,7 +364,7 @@ int ifp_process_init(TALLOC_CTX *mem_ctx,
         return EIO;
     }
 
-    ret = schedule_get_domains_task(rctx, rctx->ev, rctx);
+    ret = schedule_get_domains_task(rctx, rctx->ev, rctx, NULL);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               "schedule_get_domains_tasks failed.\n");
