@@ -21,13 +21,16 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <sys/param.h>
+
 #include "db/sysdb.h"
 #include "responder/ifp/ifp_private.h"
 
-#define IFP_DEFAULT_ATTRS {SYSDB_NAME, SYSDB_UIDNUM,   \
-                           SYSDB_GIDNUM, SYSDB_GECOS,  \
-                           SYSDB_HOMEDIR, SYSDB_SHELL, \
-                           NULL}
+#define IFP_USER_DEFAULT_ATTRS {SYSDB_NAME, SYSDB_UIDNUM,   \
+                                SYSDB_GIDNUM, SYSDB_GECOS,  \
+                                SYSDB_HOMEDIR, SYSDB_SHELL, \
+                                "groups", \
+                                NULL}
 
 errno_t ifp_req_create(struct sbus_request *dbus_req,
                        struct ifp_ctx *ifp_ctx,
@@ -97,187 +100,6 @@ int ifp_req_create_handle_failure(struct sbus_request *dbus_req, errno_t err)
                                             "Cannot create IFP request\n"));
 }
 
-const char *ifp_path_strip_prefix(const char *path, const char *prefix)
-{
-    if (strncmp(path, prefix, strlen(prefix)) == 0) {
-        return path + strlen(prefix);
-    }
-
-    return NULL;
-}
-
-/* The following path related functions are based on similar code in
- * storaged, just tailored to use talloc instead of glib
- */
-char *ifp_bus_path_escape(TALLOC_CTX *mem_ctx, const char *path)
-{
-    size_t n;
-    char *safe_path = NULL;
-    TALLOC_CTX *tmp_ctx = NULL;
-
-    /* The path must be valid */
-    if (path == NULL) {
-        return NULL;
-    }
-
-    tmp_ctx = talloc_new(NULL);
-    if (tmp_ctx == NULL) {
-        return NULL;
-    }
-
-    safe_path = talloc_strdup(tmp_ctx, "");
-    if (safe_path == NULL) {
-        goto done;
-    }
-
-    /* Special case for an empty string */
-    if (strcmp(path, "") == 0) {
-        /* the for loop would just fall through */
-        safe_path = talloc_asprintf_append_buffer(safe_path, "_");
-    }
-
-    for (n = 0; path[n]; n++) {
-        int c = path[n];
-        /* D-Bus spec says:
-         * *
-         * * Each element must only contain the ASCII characters
-         * "[A-Z][a-z][0-9]_"
-         * */
-        if ((c >= 'A' && c <= 'Z')
-                || (c >= 'a' && c <= 'z')
-                || (c >= '0' && c <= '9')) {
-            safe_path = talloc_asprintf_append_buffer(safe_path, "%c", c);
-            if (safe_path == NULL) {
-                goto done;
-            }
-        } else {
-            safe_path = talloc_asprintf_append_buffer(safe_path, "_%02x", c);
-            if (safe_path == NULL) {
-                goto done;
-            }
-        }
-    }
-
-    safe_path = talloc_steal(mem_ctx, safe_path);
-done:
-    talloc_free(tmp_ctx);
-    return safe_path;
-}
-
-static inline int unhexchar(char c)
-{
-    if (c >= '0' && c <= '9') {
-        return c - '0';
-    }
-
-    if (c >= 'a' && c <= 'f') {
-        return c - 'a' + 10;
-    }
-
-    if (c >= 'A' && c <= 'F') {
-        return c - 'A' + 10;
-    }
-
-    return -1;
-}
-
-char *ifp_bus_path_unescape(TALLOC_CTX *mem_ctx, const char *path)
-{
-    char *safe_path;
-    const char *p;
-    int a, b, c;
-    TALLOC_CTX *tmp_ctx = NULL;
-
-    tmp_ctx = talloc_new(NULL);
-    if (tmp_ctx == NULL) {
-        return NULL;
-    }
-
-    safe_path = talloc_strdup(tmp_ctx, "");
-    if (safe_path == NULL) {
-        goto done;
-    }
-
-    /* Special case for the empty string */
-    if (strcmp(path, "_") == 0) {
-        safe_path = talloc_steal(mem_ctx, safe_path);
-        goto done;
-    }
-
-    for (p = path; *p; p++) {
-        if (*p == '_') {
-            /* There must be at least two more chars after underscore */
-            if (p[1] == '\0' || p[2] == '\0') {
-                safe_path = NULL;
-                goto done;
-            }
-
-            if ((a = unhexchar(p[1])) < 0
-                    || (b = unhexchar(p[2])) < 0) {
-                /* Invalid escape code, let's take it literal then */
-                c = '_';
-            } else {
-                c = ((a << 4) | b);
-                p += 2;
-            }
-        } else  {
-            c = *p;
-        }
-
-        safe_path = talloc_asprintf_append_buffer(safe_path, "%c", c);
-        if (safe_path == NULL) {
-            goto done;
-        }
-    }
-
-    safe_path = talloc_steal(mem_ctx, safe_path);
-done:
-    talloc_free(tmp_ctx);
-    return safe_path;
-}
-
-char *
-_ifp_reply_objpath(TALLOC_CTX *mem_ctx, const char *base,
-                   const char *part, ...)
-{
-    char *safe_part;
-    char *path = NULL;
-    va_list va;
-
-    if (base == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "Wrong object path base!\n");
-        return NULL;
-    }
-
-    path = talloc_strdup(mem_ctx, base);
-    if (path == NULL) return NULL;
-
-    va_start(va, part);
-    while (part != NULL) {
-        safe_part = ifp_bus_path_escape(mem_ctx, part);
-        if (safe_part == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "Could not add [%s] to objpath\n", part);
-            goto fail;
-        }
-
-        path = talloc_asprintf_append(path, "/%s", safe_part);
-        talloc_free(safe_part);
-        if (path == NULL) {
-            goto fail;
-        }
-
-        part = va_arg(va, const char *);
-    }
-    va_end(va);
-
-    return path;
-
-fail:
-    va_end(va);
-    talloc_free(path);
-    return NULL;
-}
-
 errno_t ifp_add_ldb_el_to_dict(DBusMessageIter *iter_dict,
                                struct ldb_message_element *el)
 {
@@ -356,13 +178,6 @@ errno_t ifp_add_ldb_el_to_dict(DBusMessageIter *iter_dict,
     return EOK;
 }
 
-const char **
-ifp_parse_attr_list(TALLOC_CTX *mem_ctx, const char *conf_str)
-{
-    const char *defaults[] = IFP_DEFAULT_ATTRS;
-
-    return parse_attr_list_ex(mem_ctx, conf_str, defaults);
-}
 
 bool
 ifp_attr_allowed(const char *whitelist[], const char *attr)
@@ -381,3 +196,130 @@ ifp_attr_allowed(const char *whitelist[], const char *attr)
 
     return (whitelist[i]) ? true : false;
 }
+
+const char **
+ifp_parse_user_attr_list(TALLOC_CTX *mem_ctx, const char *csv)
+{
+    static const char *defaults[] = IFP_USER_DEFAULT_ATTRS;
+
+    return parse_attr_list_ex(mem_ctx, csv, defaults);
+}
+
+const char **
+ifp_get_user_extra_attributes(TALLOC_CTX *mem_ctx, struct ifp_ctx *ifp_ctx)
+{
+    TALLOC_CTX *tmp_ctx = NULL;
+    const char *std[] = IFP_USER_DEFAULT_ATTRS;
+    const char **whitelist = ifp_ctx->user_whitelist;
+    const char **extra;
+    bool found;
+    int extra_num;
+    int i, j;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "talloc_new() failed\n");
+        return NULL;
+    }
+
+    for (i = 0; whitelist[i] != NULL; i++) {
+        /* Just count number of attributes in whitelist. */
+    }
+
+    extra = talloc_zero_array(tmp_ctx, const char *, i + 1);
+    if (extra == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "talloc_zero_array() failed\n");
+        goto fail;
+    }
+
+    extra_num = 0;
+    for (i = 0; whitelist[i] != NULL; i++) {
+        found = false;
+        for (j = 0; std[j] != NULL; j++) {
+            if (strcmp(whitelist[i], std[j]) == 0) {
+                found = true;
+                break;
+            }
+        }
+
+        if (!found) {
+            extra[extra_num] = talloc_strdup(extra, whitelist[i]);
+            if (extra[extra_num] == NULL) {
+                goto fail;
+            }
+
+            extra_num++;
+        }
+    }
+
+    extra = talloc_realloc(tmp_ctx, extra, const char *, extra_num + 1);
+    if (extra == NULL) {
+        goto fail;
+    }
+
+    talloc_steal(mem_ctx, extra);
+    talloc_free(tmp_ctx);
+    return extra;
+
+fail:
+    talloc_free(tmp_ctx);
+    return NULL;
+}
+
+bool
+ifp_is_user_attr_allowed(struct ifp_ctx *ifp_ctx, const char *attr)
+{
+    return ifp_attr_allowed(ifp_ctx->user_whitelist, attr);
+}
+
+static uint32_t ifp_list_limit(struct ifp_ctx *ctx, uint32_t limit)
+{
+    if (limit == 0) {
+        return ctx->wildcard_limit;
+    } else if (ctx->wildcard_limit) {
+        return MIN(ctx->wildcard_limit, limit);
+    } else {
+        return limit;
+    }
+}
+
+struct ifp_list_ctx *ifp_list_ctx_new(struct sbus_request *sbus_req,
+                                      struct ifp_ctx *ctx,
+                                      const char *filter,
+                                      uint32_t limit)
+{
+    struct ifp_list_ctx *list_ctx;
+
+    list_ctx = talloc_zero(sbus_req, struct ifp_list_ctx);
+    if (list_ctx == NULL) {
+        return NULL;
+    }
+
+    list_ctx->sbus_req = sbus_req;
+    list_ctx->limit = ifp_list_limit(ctx, limit);
+    list_ctx->ctx = ctx;
+    list_ctx->dom = ctx->rctx->domains;
+    list_ctx->filter = filter;
+    list_ctx->paths = talloc_zero_array(list_ctx, const char *, limit);
+    if (list_ctx->paths == NULL) {
+        talloc_free(list_ctx);
+        return NULL;
+    }
+
+    return list_ctx;
+}
+
+size_t ifp_list_ctx_remaining_capacity(struct ifp_list_ctx *list_ctx,
+                                       size_t entries)
+{
+    size_t capacity = list_ctx->limit - list_ctx->path_count;
+
+    if (capacity < entries) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "IFP list request has limit of %"PRIu32" entries but back end "
+              "returned %zu entries\n", list_ctx->limit, entries);
+        return capacity;
+    } else {
+        return entries;
+    }
+}
