@@ -124,6 +124,15 @@ static int nss_clear_memcache(struct sbus_request *dbus_req, void *data)
         return ret;
     }
 
+    ret = sss_mmap_cache_reinit(nctx, SSS_MC_CACHE_ELEMENTS,
+                                (time_t)memcache_timeout,
+                                &nctx->initgr_mc_ctx);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "initgroups mmap cache invalidation failed\n");
+        return ret;
+    }
+
 done:
     return sbus_request_return_and_finish(dbus_req, DBUS_TYPE_INVALID);
 }
@@ -517,6 +526,13 @@ int nss_process_init(TALLOC_CTX *mem_ctx,
         DEBUG(SSSDBG_CRIT_FAILURE, "group mmap cache is DISABLED\n");
     }
 
+    ret = sss_mmap_cache_init(nctx, "initgroups", SSS_MC_INITGROUPS,
+                              SSS_MC_CACHE_ELEMENTS, (time_t)memcache_timeout,
+                              &nctx->initgr_mc_ctx);
+    if (ret) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "inigroups mmap cache is DISABLED\n");
+    }
+
     /* Set up file descriptor limits */
     ret = confdb_get_int(nctx->rctx->cdb,
                          CONFDB_NSS_CONF_ENTRY,
@@ -530,15 +546,15 @@ int nss_process_init(TALLOC_CTX *mem_ctx,
     }
     responder_set_fd_limit(fd_limit);
 
-    ret = schedule_get_domains_task(rctx, rctx->ev, rctx);
+    ret = schedule_get_domains_task(rctx, rctx->ev, rctx, nctx->ncache);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "schedule_get_domains_tasks failed.\n");
         goto fail;
     }
 
-    ret = sss_names_init(nctx, nctx->rctx->cdb, NULL, &nctx->global_names);
+    ret = sss_ad_default_names_ctx(nctx, &nctx->global_names);
     if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "sss_names_init failed.\n");
+        DEBUG(SSSDBG_CRIT_FAILURE, "sss_ad_default_names_ctx failed.\n");
         goto fail;
     }
 
