@@ -56,7 +56,7 @@ static errno_t get_netgroup_entry(struct nss_ctx *nctx,
     return EIO;
 }
 
-static int netgr_hash_remove (TALLOC_CTX *ctx);
+static int netgr_hash_remove(TALLOC_CTX *ctx);
 static errno_t set_netgroup_entry(struct nss_ctx *nctx,
                                   struct getent_ctx *netgr)
 {
@@ -76,9 +76,9 @@ static errno_t set_netgroup_entry(struct nss_ctx *nctx,
     hret = hash_enter(nctx->netgroups, &key, &value);
     if (hret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
-              "Unable to add hash table entry for [%s]", key.str);
+              "Unable to add hash table entry for [%s]\n", key.str);
         DEBUG(SSSDBG_CONF_SETTINGS,
-              "Hash error [%d][%s]", hret, hash_error_string(hret));
+              "Hash error [%d][%s]\n", hret, hash_error_string(hret));
         return EIO;
     }
     talloc_steal(nctx->netgroups, netgr);
@@ -138,7 +138,7 @@ done:
     return nss_cmd_done(cmdctx, ret);
 }
 
-static int netgr_hash_remove (TALLOC_CTX *ctx)
+static int netgr_hash_remove(TALLOC_CTX *ctx)
 {
     int hret;
     hash_key_t key;
@@ -483,7 +483,6 @@ static errno_t lookup_netgr_step(struct setent_step_ctx *step_ctx)
     errno_t ret;
     struct sss_domain_info *dom = step_ctx->dctx->domain;
     struct getent_ctx *netgr;
-    struct sysdb_ctx *sysdb;
     char *name = NULL;
     uint32_t lifetime;
     TALLOC_CTX *tmp_ctx;
@@ -522,8 +521,7 @@ static errno_t lookup_netgr_step(struct setent_step_ctx *step_ctx)
 
         DEBUG(SSSDBG_CONF_SETTINGS, "Requesting info for [%s@%s]\n",
                   name, dom->name);
-        sysdb = dom->sysdb;
-        if (sysdb == NULL) {
+        if (dom->sysdb == NULL) {
             DEBUG(SSSDBG_FATAL_FAILURE,
                   "Fatal: Sysdb CTX not found for this domain!\n");
             ret = EIO;
@@ -531,34 +529,31 @@ static errno_t lookup_netgr_step(struct setent_step_ctx *step_ctx)
         }
 
         /* Look up the netgroup in the cache */
-        ret = sysdb_getnetgr(step_ctx->dctx, sysdb, dom, name,
-                             &step_ctx->dctx->res);
-        if (ret == ENOENT) {
+        ret = sysdb_getnetgr(step_ctx->dctx, dom, name, &step_ctx->dctx->res);
+        if (ret == EOK) {
+            if (step_ctx->dctx->res->count > 1) {
+                DEBUG(SSSDBG_FATAL_FAILURE,
+                      "getnetgr call returned more than one result !?!\n");
+                ret = EMSGSIZE;
+                goto done;
+            }
+        } else if (ret == ENOENT) {
             /* This netgroup was not found in this domain */
             if (!step_ctx->dctx->check_provider) {
                 if (step_ctx->check_next) {
-                    dom = get_next_domain(dom, false);
+                    dom = get_next_domain(dom, 0);
                     continue;
+                } else {
+                    break;
                 }
-                else break;
             }
-            ret = EOK;
-        }
-
-        if (ret != EOK) {
+        } else {
             DEBUG(SSSDBG_CRIT_FAILURE,
                   "Failed to make request to our cache!\n");
             ret = EIO;
             goto done;
         }
 
-        if (step_ctx->dctx->res->count > 1) {
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                  "getnetgr call returned more than one result !?!\n");
-            ret = EMSGSIZE;
-            goto done;
-        }
-
         ret = get_netgroup_entry(step_ctx->nctx, step_ctx->name,
                                  &netgr);
         if (ret != EOK) {
@@ -577,7 +572,7 @@ static errno_t lookup_netgr_step(struct setent_step_ctx *step_ctx)
 
             if (!step_ctx->dctx->check_provider) {
                 if (step_ctx->check_next) {
-                    dom = get_next_domain(dom, false);
+                    dom = get_next_domain(dom, 0);
                     continue;
                 }
                 else break;
@@ -602,7 +597,7 @@ static errno_t lookup_netgr_step(struct setent_step_ctx *step_ctx)
                               step_ctx->nctx,
                               step_ctx->dctx->res,
                               SSS_DP_NETGR,
-                              name, 0,
+                              name, 0, NULL,
                               lookup_netgr_dp_callback,
                               step_ctx);
             if (ret != EOK) {
@@ -668,22 +663,24 @@ static void lookup_netgr_dp_callback(uint16_t err_maj, uint32_t err_min,
                   "Will try to return what we have in cache\n",
                   (unsigned int)err_maj, (unsigned int)err_min, err_msg);
         /* Loop to the next domain if possible */
-        if (cmdctx->check_next && get_next_domain(dctx->domain, false)) {
-            dctx->domain = get_next_domain(dctx->domain, false);
+        if (cmdctx->check_next
+                && (dctx->domain = get_next_domain(dctx->domain, 0))) {
             dctx->check_provider = NEED_CHECK_PROVIDER(dctx->domain->provider);
         }
     }
 
     /* ok the backend returned, search to see if we have updated results */
     ret = lookup_netgr_step(step_ctx);
-    if (ret != EOK) {
-        if (ret == EAGAIN) {
-            return;
-        }
+    if (ret == EAGAIN) {
+        return;
     }
 
     /* We have results to return */
-    nss_setent_notify_error(dctx->netgr, ret);
+    if (ret == EOK) {
+        nss_setent_notify_done(dctx->netgr);
+    } else {
+        nss_setent_notify_error(dctx->netgr, ret);
+    }
 }
 
 static void setnetgrent_result_timeout(struct tevent_context *ev,
@@ -747,8 +744,12 @@ static void nss_cmd_setnetgrent_done(struct tevent_req *req)
             }
 
             sss_packet_get_body(packet, &body, &blen);
-            ((uint32_t *)body)[0] = 1; /* Got some results */
-            ((uint32_t *)body)[1] = 0; /* reserved */
+
+            /* Got some results. */
+            SAFEALIGN_SETMEM_UINT32(body, 1, NULL);
+
+            /* reserved */
+            SAFEALIGN_SETMEM_UINT32(body + sizeof(uint32_t), 0, NULL);
         }
 
         sss_cmd_done(cmdctx->cctx, cmdctx);
@@ -915,7 +916,7 @@ static errno_t nss_cmd_getnetgrent_process(struct nss_cmd_ctx *cmdctx,
     if (blen != sizeof(uint32_t)) {
         return EINVAL;
     }
-    num = *((uint32_t *)body);
+    SAFEALIGN_COPY_UINT32(&num, body, NULL);
 
     /* create response packet */
     ret = sss_packet_new(client->creq, 0,
@@ -1056,8 +1057,12 @@ static errno_t nss_cmd_retnetgrent(struct cli_ctx *client,
     }
 
     sss_packet_get_body(packet, &body, &blen);
-    ((uint32_t *)body)[0] = num; /* num results */
-    ((uint32_t *)body)[1] = 0; /* reserved */
+
+    /* num results */
+    SAFEALIGN_COPY_UINT32(body, &num, NULL);
+
+    /* reserved */
+    SAFEALIGN_SETMEM_UINT32(body + sizeof(uint32_t), 0, NULL);
 
     return EOK;
 }
