@@ -88,7 +88,6 @@ getserv_send(TALLOC_CTX *mem_ctx,
     size_t dom_idx = 0;
     struct nss_ctx *nctx =
             talloc_get_type(cctx->rctx->pvt_ctx, struct nss_ctx);
-    struct sysdb_ctx *sysdb;
     time_t now = time(NULL);
     uint64_t lastUpdate;
     uint64_t cacheExpire;
@@ -98,7 +97,7 @@ getserv_send(TALLOC_CTX *mem_ctx,
     if (!req) return NULL;
     state->dctx = dctx;
 
-    for (dom = cctx->rctx->domains; dom; dom = get_next_domain(dom, false)) {
+    for (dom = cctx->rctx->domains; dom; dom = get_next_domain(dom, 0)) {
         num_domains++;
     }
 
@@ -161,12 +160,11 @@ getserv_send(TALLOC_CTX *mem_ctx,
         /* if it is a domainless search, skip domains that require fully
           * qualified names instead */
          while (dom && cmdctx->check_next && dom->fqnames) {
-             dom = get_next_domain(dom, false);
+             dom = get_next_domain(dom, 0);
          }
          if (!dom) break;
 
-         sysdb = dom->sysdb;
-         if (sysdb == NULL) {
+         if (dom->sysdb == NULL) {
              DEBUG(SSSDBG_CRIT_FAILURE,
                    "Critical: Sysdb CTX not found for [%s]!\n", dom->name);
              ret = EINVAL;
@@ -192,7 +190,7 @@ getserv_send(TALLOC_CTX *mem_ctx,
 
                  /* If this is a multi-domain search, try the next one */
                  if (cmdctx->check_next) {
-                     dom = get_next_domain(dom, false);
+                     dom = get_next_domain(dom, 0);
                  } else {
                      /* This was a single-domain search.
                       * exit the loop. Since it was negatively-
@@ -212,7 +210,7 @@ getserv_send(TALLOC_CTX *mem_ctx,
                     SVC_PROTO_CASED ? SVC_PROTO_CASED : "<ANY>",
                     dom->name);
 
-             ret = sysdb_getservbyname(state, sysdb, dom,
+             ret = sysdb_getservbyname(state, dom,
                                        SVC_NAME_CASED,
                                        SVC_PROTO_CASED,
                                        &state->res);
@@ -233,7 +231,7 @@ getserv_send(TALLOC_CTX *mem_ctx,
 
                  /* If this is a multi-domain search, try the next one */
                  if (cmdctx->check_next) {
-                     dom = get_next_domain(dom, false);
+                     dom = get_next_domain(dom, 0);
                  } else {
                      /* This was a single-domain search.
                       * exit the loop. Since it was negatively-
@@ -253,7 +251,7 @@ getserv_send(TALLOC_CTX *mem_ctx,
                     SVC_PROTO_CASED ? SVC_PROTO_CASED : "<ANY>",
                     dom->name);
 
-             ret = sysdb_getservbyport(state, sysdb, dom, port,
+             ret = sysdb_getservbyport(state, dom, port,
                                        SVC_PROTO_CASED,
                                        &state->res);
          }
@@ -300,7 +298,7 @@ getserv_send(TALLOC_CTX *mem_ctx,
 
              /* If this is a multi-domain search, try the next one */
              if (cmdctx->check_next) {
-                 dom = get_next_domain(dom, false);
+                 dom = get_next_domain(dom, 0);
              } else {
                  /* This was a single-domain search.
                   * exit the loop.
@@ -382,7 +380,7 @@ getserv_send(TALLOC_CTX *mem_ctx,
 
              /* If this is a multi-domain search, try the next one */
              if (cmdctx->check_next) {
-                 dom = get_next_domain(dom, false);
+                 dom = get_next_domain(dom, 0);
              } else {
                  /* This was a single-domain search.
                   * exit the loop.
@@ -450,7 +448,6 @@ static void lookup_service_done(struct tevent_req *subreq)
     dbus_uint16_t err_maj;
     dbus_uint32_t err_min;
     char *err_msg;
-    struct sysdb_ctx *sysdb;
 
     struct tevent_req *req =
             tevent_req_callback_data(subreq, struct tevent_req);
@@ -481,8 +478,7 @@ static void lookup_service_done(struct tevent_req *subreq)
      * be returned, if it exists. Otherwise, move to the
      * next provider.
      */
-    sysdb = dom->sysdb;
-    if (sysdb == NULL) {
+    if (dom->sysdb == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Critical: Sysdb CTX not found for [%s]!\n",
                 dom->name);
@@ -497,7 +493,7 @@ static void lookup_service_done(struct tevent_req *subreq)
                SVC_PROTO_CASED ? SVC_PROTO_CASED : "<ANY>",
                dom->name);
 
-        ret = sysdb_getservbyname(state, sysdb, dom,
+        ret = sysdb_getservbyname(state, dom,
                                   SVC_NAME_CASED,
                                   SVC_PROTO_CASED,
                                   &state->res);
@@ -508,7 +504,7 @@ static void lookup_service_done(struct tevent_req *subreq)
                SVC_PROTO_CASED ? SVC_PROTO_CASED : "<ANY>",
                dom->name);
 
-        ret = sysdb_getservbyport(state, sysdb, dom,
+        ret = sysdb_getservbyport(state, dom,
                                   state->port,
                                   SVC_PROTO_CASED,
                                   &state->res);
@@ -646,7 +642,7 @@ fill_service(struct sss_packet *packet,
 
         /* Get the service name */
         orig_name = ldb_msg_find_attr_as_string(msg, SYSDB_NAME, NULL);
-        tmpstr = sss_get_cased_name(tmp_ctx, orig_name, dom->case_sensitive);
+        tmpstr = sss_get_cased_name(tmp_ctx, orig_name, dom->case_preserve);
         if (tmpstr == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE,
                   "Could not identify service name, skipping\n");
@@ -659,7 +655,7 @@ fill_service(struct sss_packet *packet,
         if (!port) {
             DEBUG(SSSDBG_CRIT_FAILURE,
                   "No port for service [%s]. Skipping\n", tmpstr);
-
+            continue;
         }
 
         /* Get the service protocol.
@@ -681,7 +677,7 @@ fill_service(struct sss_packet *packet,
             orig_proto = (const char *)el->values[0].data;
         }
 
-        tmpstr = sss_get_cased_name(tmp_ctx, orig_proto, dom->case_sensitive);
+        tmpstr = sss_get_cased_name(tmp_ctx, orig_proto, dom->case_preserve);
         if (tmpstr == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE,
                   "sss_get_cased_name failed, skipping\n");
@@ -700,7 +696,7 @@ fill_service(struct sss_packet *packet,
         sss_packet_get_body(packet, &body, &blen);
 
         /* Store the port number */
-        SAFEALIGN_SET_UINT32(&body[rzero + rsize], (uint32_t)htons(port), &rsize);
+        SAFEALIGN_SETMEM_UINT32(&body[rzero + rsize], (uint32_t)htons(port), &rsize);
 
         /* Get the aliases */
         el = ldb_msg_find_element(msg, SYSDB_NAME_ALIAS);
@@ -756,7 +752,7 @@ fill_service(struct sss_packet *packet,
         /* We must not advance rsize here, the data has already been
          * allocated and skipped earlier when aptr was assigned to.
          */
-        SAFEALIGN_SET_UINT32(&body[aptr], written_aliases, NULL);
+        SAFEALIGN_SETMEM_UINT32(&body[aptr], written_aliases, NULL);
 
         num++;
     }
@@ -773,8 +769,11 @@ done:
         return ENOENT;
     }
 
-    ((uint32_t *)body)[0] = num; /* num results */
-    ((uint32_t *)body)[1] = 0; /* reserved */
+    /* num results */
+    SAFEALIGN_COPY_UINT32(body, &num, NULL);
+
+    /* reserved */
+    SAFEALIGN_SETMEM_UINT32(body + sizeof(uint32_t), 0, NULL);
 
     return ret;
 }
@@ -909,7 +908,7 @@ errno_t parse_getservbyname(TALLOC_CTX *mem_ctx,
     i = j = 0;
 
     /* Copy in the service name */
-    while (body[i] && i < (blen - 1)) {
+    while (i < (blen - 1) && body[i]) {
         rawname[j] = body[i];
         i++;
         j++;
@@ -944,7 +943,7 @@ errno_t parse_getservbyname(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        while (body[i] && i < blen) {
+        while (i < blen && body[i]) {
             protocol[j] = body[i];
             i++;
             j++;
@@ -1072,7 +1071,7 @@ errno_t parse_getservbyport(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        while (body[i] && i < blen) {
+        while (i < blen && body[i]) {
             protocol[j] = body[i];
             i++;
             j++;
@@ -1259,7 +1258,7 @@ setservent_send(TALLOC_CTX *mem_ctx, struct cli_ctx *cctx)
     num_domains = 0;
     for (dom = state->cctx->rctx->domains;
          dom;
-         dom = get_next_domain(dom, false)) {
+         dom = get_next_domain(dom, 0)) {
         num_domains++;
     }
 
@@ -1306,7 +1305,7 @@ setservent_send(TALLOC_CTX *mem_ctx, struct cli_ctx *cctx)
               "Error [%s] requesting info from domain [%s]. Skipping.\n",
                strerror(ret), step_ctx->dctx->domain->name);
 
-        step_ctx->dctx->domain = get_next_domain(step_ctx->dctx->domain, false);
+        step_ctx->dctx->domain = get_next_domain(step_ctx->dctx->domain, 0);
     }
 
     /* All domains failed */
@@ -1384,7 +1383,7 @@ lookup_servent_send(TALLOC_CTX *mem_ctx,
             goto immediate;
         }
 
-        ret = sysdb_enumservent(state, sysdb, dom, &state->res);
+        ret = sysdb_enumservent(state, dom, &state->res);
         /* Whatever the result, we're done, so report it */
         goto immediate;
     }
@@ -1446,7 +1445,7 @@ lookup_servent_done(struct tevent_req *subreq)
         goto done;
     }
 
-    ret = sysdb_enumservent(state, sysdb, state->dom, &state->res);
+    ret = sysdb_enumservent(state, state->dom, &state->res);
     /* Whatever the result, we're done, so report it */
 
 done:
@@ -1501,7 +1500,7 @@ setservent_step_done(struct tevent_req *req)
         svcctx->num++;
     }
 
-    step_ctx->dctx->domain = get_next_domain(step_ctx->dctx->domain, false);
+    step_ctx->dctx->domain = get_next_domain(step_ctx->dctx->domain, 0);
 
     while (step_ctx->dctx->domain) {
         /* There are more domains to check */
@@ -1515,7 +1514,7 @@ setservent_step_done(struct tevent_req *req)
               "Error [%s] requesting info from domain [%s]. Skipping.\n",
                strerror(ret), step_ctx->dctx->domain->name);
 
-        step_ctx->dctx->domain = get_next_domain(step_ctx->dctx->domain, false);
+        step_ctx->dctx->domain = get_next_domain(step_ctx->dctx->domain, 0);
     }
 
     /* All domains have been checked */
@@ -1738,7 +1737,7 @@ nss_cmd_getservent_immediate(struct nss_cmd_ctx *cmdctx)
     if (blen != sizeof(uint32_t)) {
         return EINVAL;
     }
-    num = *((uint32_t *)body);
+    SAFEALIGN_COPY_UINT32(&num, body, NULL);
 
     /* create response packet */
     ret = sss_packet_new(cctx->creq, 0,
