@@ -584,13 +584,16 @@ static errno_t save_pac_user(struct pac_req_ctx *pr_ctx)
                                    &msg);
     if (ret == ENOENT) {
         if (pwd->pw_gid == 0 && !pr_ctx->dom->mpg) {
-            DEBUG(SSSDBG_CRIT_FAILURE,
-                  "Primary group RID from the PAC cannot be translated into "
-                  "a GID for user [%s]. Typically this happens when UIDs and "
-                  "GIDs are read from AD and the primary AD group does not "
-                  "have a GID assigned. Make sure the user is created by the "
-                  "ID provider before GSSAPI based authentication "
-                  "is used in this case.\n", pwd->pw_name);
+            DEBUG(SSSDBG_CRIT_FAILURE, "Primary group RID from the PAC "
+                                       "cannot be translated into a GID for "
+                                       "user [%s]. Typically this happens "
+                                       "when UIDs and GIDs are read from AD "
+                                       "and the primary AD group does not "
+                                       "have a GID assigned. Make sure the "
+                                       "user is created by the ID provider "
+                                       "before GSSAPI based authentication "
+                                       "is used in this case.\n",
+                  pwd->pw_name);
             ret = EINVAL;
             goto done;
         }
