@@ -31,6 +31,7 @@
 #include "providers/data_provider.h"
 #include "responder/pam/pamsrv.h"
 #include "responder/pam/pam_helpers.h"
+#include "responder/common/responder_cache_req.h"
 #include "db/sysdb.h"
 
 enum pam_verbosity {
@@ -42,12 +43,20 @@ enum pam_verbosity {
 
 #define DEFAULT_PAM_VERBOSITY PAM_VERBOSITY_IMPORTANT
 
+static errno_t
+pam_null_last_online_auth_with_curr_token(struct sss_domain_info *domain,
+                                          const char *username);
+static errno_t
+pam_get_last_online_auth_with_curr_token(struct sss_domain_info *domain,
+                                         const char *name,
+                                         uint64_t *_value);
+
 static void pam_reply(struct pam_auth_req *preq);
 
-static errno_t pack_user_info_account_expired(TALLOC_CTX *mem_ctx,
-                                              const char *user_error_message,
-                                              size_t *resp_len,
-                                              uint8_t **_resp)
+static errno_t pack_user_info_msg(TALLOC_CTX *mem_ctx,
+                                  const char *user_error_message,
+                                  size_t *resp_len,
+                                  uint8_t **_resp)
 {
     uint32_t resp_type = SSS_PAM_USER_INFO_ACCOUNT_EXPIRED;
     size_t err_len;
@@ -74,14 +83,13 @@ static errno_t pack_user_info_account_expired(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
-static void inform_account_expired(struct pam_data* pd,
-                                   const char *pam_message)
+static void inform_user(struct pam_data* pd, const char *pam_message)
 {
     size_t msg_len;
     uint8_t *msg;
     errno_t ret;
 
-    ret = pack_user_info_account_expired(pd, pam_message, &msg_len, &msg);
+    ret = pack_user_info_msg(pd, pam_message, &msg_len, &msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "pack_user_info_account_expired failed.\n");
@@ -143,6 +151,17 @@ static int extract_authtok_v2(struct sss_auth_token *tok,
                                            auth_token_length);
         }
         break;
+    case SSS_AUTHTOK_TYPE_2FA:
+        ret = sss_authtok_set(tok, SSS_AUTHTOK_TYPE_2FA,
+                              auth_token_data, auth_token_length);
+        break;
+    case SSS_AUTHTOK_TYPE_SC_PIN:
+        ret = sss_authtok_set_sc_pin(tok, (const char *) auth_token_data,
+                                     auth_token_length);
+        break;
+    case SSS_AUTHTOK_TYPE_SC_KEYPAD:
+        sss_authtok_set_sc_keypad(tok);
+        break;
     default:
         return EINVAL;
     }
@@ -422,6 +441,13 @@ static errno_t set_last_login(struct pam_auth_req *preq)
         goto fail;
     }
 
+    ret = sysdb_attrs_add_time_t(attrs,
+                                 SYSDB_LAST_ONLINE_AUTH_WITH_CURR_TOKEN,
+                                 time(NULL));
+    if (ret != EOK) {
+        goto fail;
+    }
+
     ret = sysdb_attrs_add_time_t(attrs, SYSDB_LAST_LOGIN, time(NULL));
     if (ret != EOK) {
         goto fail;
@@ -524,9 +550,37 @@ static void pam_reply_delay(struct tevent_context *ev, struct tevent_timer *te,
     pam_reply(preq);
 }
 
+static errno_t get_password_for_cache_auth(struct sss_auth_token *authtok,
+                                           const char **password)
+{
+    int ret;
+    size_t pw_len;
+    const char *fa2;
+    size_t fa2_len;
+
+    switch (sss_authtok_get_type(authtok)) {
+    case SSS_AUTHTOK_TYPE_PASSWORD:
+        ret = sss_authtok_get_password(authtok, password, NULL);
+        break;
+    case SSS_AUTHTOK_TYPE_2FA:
+        ret = sss_authtok_get_2fa(authtok, password, &pw_len, &fa2, &fa2_len);
+        break;
+    default:
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unsupported auth token type [%d].\n",
+              sss_authtok_get_type(authtok));
+        ret = EINVAL;
+    }
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to get password.\n");
+        return ret;
+    }
+
+    return EOK;
+}
+
 static int pam_forwarder(struct cli_ctx *cctx, int pam_cmd);
 static void pam_handle_cached_login(struct pam_auth_req *preq, int ret,
-                                    time_t expire_date, time_t delayed_until);
+                                    time_t expire_date, time_t delayed_until, bool cached_auth);
 
 static void pam_reply(struct pam_auth_req *preq)
 {
@@ -546,6 +600,7 @@ static void pam_reply(struct pam_auth_req *preq)
     time_t exp_date = -1;
     time_t delay_until = -1;
     char* pam_account_expired_message;
+    char* pam_account_locked_message;
     int pam_verbosity;
 
     pd = preq->pd;
@@ -562,15 +617,23 @@ static void pam_reply(struct pam_auth_req *preq)
     }
 
     DEBUG(SSSDBG_FUNC_DATA,
-          "pam_reply called with result [%d].\n", pd->pam_status);
+          "pam_reply called with result [%d]: %s.\n",
+          pd->pam_status, pam_strerror(NULL, pd->pam_status));
+    if (pd->pam_status == PAM_AUTHINFO_UNAVAIL || preq->use_cached_auth) {
 
-    if (pd->pam_status == PAM_AUTHINFO_UNAVAIL) {
         switch(pd->cmd) {
         case SSS_PAM_AUTHENTICATE:
             if ((preq->domain != NULL) &&
                 (preq->domain->cache_credentials == true) &&
                 (pd->offline_auth == false)) {
                 const char *password = NULL;
+                bool use_cached_auth;
+
+                /* backup value of preq->use_cached_auth*/
+                use_cached_auth = preq->use_cached_auth;
+                /* set to false to avoid entering this branch when pam_reply()
+                 * is recursively called from pam_handle_cached_login() */
+                preq->use_cached_auth = false;
 
                 /* do auth with offline credentials */
                 pd->offline_auth = true;
@@ -582,9 +645,10 @@ static void pam_reply(struct pam_auth_req *preq)
                     goto done;
                 }
 
-                ret = sss_authtok_get_password(pd->authtok, &password, NULL);
-                if (ret) {
-                    DEBUG(SSSDBG_FATAL_FAILURE, "Failed to get password.\n");
+                ret = get_password_for_cache_auth(pd->authtok, &password);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_FATAL_FAILURE,
+                          "get_password_and_type_for_cache_auth failed.\n");
                     goto done;
                 }
 
@@ -593,7 +657,8 @@ static void pam_reply(struct pam_auth_req *preq)
                                        pctx->rctx->cdb, false,
                                        &exp_date, &delay_until);
 
-                pam_handle_cached_login(preq, ret, exp_date, delay_until);
+                pam_handle_cached_login(preq, ret, exp_date, delay_until,
+                                        use_cached_auth);
                 return;
             }
             break;
@@ -627,6 +692,17 @@ static void pam_reply(struct pam_auth_req *preq)
         }
     }
 
+    if (pd->pam_status == PAM_SUCCESS && pd->cmd == SSS_PAM_CHAUTHTOK) {
+        ret = pam_null_last_online_auth_with_curr_token(preq->domain,
+                                                        pd->user);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "sysdb_null_last_online_auth_with_curr_token failed: "
+                  "%s [%d].\n", sss_strerror(ret), ret);
+            goto done;
+        }
+    }
+
     if (pd->response_delay > 0) {
         ret = gettimeofday(&tv, NULL);
         if (ret != EOK) {
@@ -686,7 +762,22 @@ static void pam_reply(struct pam_auth_req *preq)
             goto done;
         }
 
-        inform_account_expired(pd, pam_account_expired_message);
+        inform_user(pd, pam_account_expired_message);
+    }
+
+    if (pd->account_locked) {
+
+        ret = confdb_get_string(pctx->rctx->cdb, pd, CONFDB_PAM_CONF_ENTRY,
+                                CONFDB_PAM_ACCOUNT_LOCKED_MESSAGE, "",
+                                &pam_account_locked_message);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "Failed to get expiration message: %d:[%s].\n",
+                  ret, sss_strerror(ret));
+            goto done;
+        }
+
+        inform_user(pd, pam_account_locked_message);
     }
 
     ret = filter_responses(pctx->rctx->cdb, pd->resp_list);
@@ -750,8 +841,11 @@ done:
     sss_cmd_done(cctx, preq);
 }
 
+static void pam_dom_forwarder(struct pam_auth_req *preq);
+
 static void pam_handle_cached_login(struct pam_auth_req *preq, int ret,
-                                    time_t expire_date, time_t delayed_until)
+                                    time_t expire_date, time_t delayed_until,
+                                    bool use_cached_auth)
 {
     uint32_t resp_type;
     size_t resp_len;
@@ -800,6 +894,18 @@ static void pam_handle_cached_login(struct pam_auth_req *preq, int ret,
                 }
             }
             break;
+        case PAM_AUTH_ERR:
+            /* Was this attempt to authenticate from cache? */
+            if (use_cached_auth) {
+                /* Don't try cached authentication again, try online check. */
+                DEBUG(SSSDBG_FUNC_DATA,
+                      "Cached authentication failed for: %s\n",
+                      preq->pd->user);
+                preq->cached_auth_failed = true;
+                pam_dom_forwarder(preq);
+                return;
+            }
+            break;
         default:
             DEBUG(SSSDBG_TRACE_LIBS,
                   "cached login returned: %d\n", preq->pd->pam_status);
@@ -810,11 +916,11 @@ static void pam_handle_cached_login(struct pam_auth_req *preq, int ret,
 }
 
 static void pam_forwarder_cb(struct tevent_req *req);
+static void pam_forwarder_cert_cb(struct tevent_req *req);
 static void pam_check_user_dp_callback(uint16_t err_maj, uint32_t err_min,
                                        const char *err_msg, void *ptr);
 static int pam_check_user_search(struct pam_auth_req *preq);
 static int pam_check_user_done(struct pam_auth_req *preq, int ret);
-static void pam_dom_forwarder(struct pam_auth_req *preq);
 
 /* TODO: we should probably return some sort of cookie that is set in the
  * PAM_ENVIRONMENT, so that we can save performing some calls and cache
@@ -858,9 +964,24 @@ static errno_t pam_forwarder_parse_data(struct cli_ctx *cctx, struct pam_data *p
         goto done;
     }
 
-    ret = sss_parse_name_for_domains(pd, cctx->rctx->domains,
-                                     cctx->rctx->default_domain, pd->logon_name,
-                                     &pd->domain, &pd->user);
+    if (pd->logon_name != NULL) {
+        ret = sss_parse_name_for_domains(pd, cctx->rctx->domains,
+                                         cctx->rctx->default_domain,
+                                         pd->logon_name,
+                                         &pd->domain, &pd->user);
+    } else {
+        /* Only SSS_PAM_PREAUTH request may have a missing name, e.g. if the
+         * name is determined with the help of a certificate */
+        if (pd->cmd == SSS_PAM_PREAUTH
+                && may_do_cert_auth(talloc_get_type(cctx->rctx->pvt_ctx,
+                                                    struct pam_ctx), pd)) {
+            ret = EOK;
+        } else {
+            DEBUG(SSSDBG_CRIT_FAILURE, "Missing logon name in PAM request.\n");
+            ret = ERR_NO_CREDS;
+            goto done;
+        }
+    }
 
     DEBUG_PAM_DATA(SSSDBG_CONF_SETTINGS, pd);
 
@@ -918,6 +1039,51 @@ static bool is_domain_public(char *name,
     return false;
 }
 
+static errno_t check_cert(TALLOC_CTX *mctx,
+                          struct tevent_context *ev,
+                          struct pam_ctx *pctx,
+                          struct pam_auth_req *preq,
+                          struct pam_data *pd)
+{
+    int p11_child_timeout;
+    const int P11_CHILD_TIMEOUT_DEFAULT = 10;
+    char *cert_verification_opts;
+    errno_t ret;
+    struct tevent_req *req;
+
+    ret = confdb_get_int(pctx->rctx->cdb, CONFDB_PAM_CONF_ENTRY,
+                         CONFDB_PAM_P11_CHILD_TIMEOUT,
+                         P11_CHILD_TIMEOUT_DEFAULT,
+                         &p11_child_timeout);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Failed to read p11_child_timeout from confdb: [%d]: %s\n",
+              ret, sss_strerror(ret));
+        return ret;
+    }
+
+    ret = confdb_get_string(pctx->rctx->cdb, mctx, CONFDB_MONITOR_CONF_ENTRY,
+                            CONFDB_MONITOR_CERT_VERIFICATION, NULL,
+                            &cert_verification_opts);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Failed to read certificate_verification from confdb: [%d]: %s\n",
+              ret, sss_strerror(ret));
+        return ret;
+    }
+
+    req = pam_check_cert_send(mctx, ev, pctx->p11_child_debug_fd,
+                              pctx->nss_db, p11_child_timeout,
+                              cert_verification_opts, pd);
+    if (req == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "pam_check_cert_send failed.\n");
+        return ENOMEM;
+    }
+
+    tevent_req_set_callback(req, pam_forwarder_cert_cb, preq);
+    return EAGAIN;
+}
+
 static int pam_forwarder(struct cli_ctx *cctx, int pam_cmd)
 {
     struct sss_domain_info *dom;
@@ -967,53 +1133,62 @@ static int pam_forwarder(struct cli_ctx *cctx, int pam_cmd)
         }
         goto done;
     } else if (ret != EOK) {
-        ret = EINVAL;
         goto done;
     }
 
-    /* now check user is valid */
-    if (pd->domain) {
-        preq->domain = responder_get_domain(cctx->rctx, pd->domain);
-        if (!preq->domain) {
-            ret = ENOENT;
-            goto done;
-        }
-
-        ncret = sss_ncache_check_user(pctx->ncache, pctx->neg_timeout,
-                                      preq->domain, pd->user);
-        if (ncret == EEXIST) {
-            /* User found in the negative cache */
-            ret = ENOENT;
-            goto done;
-        }
-    } else {
-        for (dom = preq->cctx->rctx->domains;
-             dom;
-             dom = get_next_domain(dom, false)) {
-            if (dom->fqnames) continue;
-
-            ncret = sss_ncache_check_user(pctx->ncache, pctx->neg_timeout,
-                                          dom, pd->user);
-            if (ncret == ENOENT) {
-                /* User not found in the negative cache
-                 * Proceed with PAM actions
-                 */
-                break;
+    if (pd->user != NULL) {
+        /* now check user is valid */
+        if (pd->domain) {
+            preq->domain = responder_get_domain(cctx->rctx, pd->domain);
+            if (!preq->domain) {
+                ret = ENOENT;
+                goto done;
             }
 
-            /* Try the next domain */
-            DEBUG(SSSDBG_TRACE_FUNC,
-                  "User [%s@%s] filtered out (negative cache). "
-                   "Trying next domain.\n", pd->user, dom->name);
-        }
+            ncret = sss_ncache_check_user(pctx->ncache, pctx->neg_timeout,
+                                          preq->domain, pd->user);
+            if (ncret == EEXIST) {
+                /* User found in the negative cache */
+                ret = ENOENT;
+                goto done;
+            }
+        } else {
+            for (dom = preq->cctx->rctx->domains;
+                 dom;
+                 dom = get_next_domain(dom, 0)) {
+                if (dom->fqnames) continue;
 
-        if (!dom) {
-            ret = ENOENT;
-            goto done;
+                ncret = sss_ncache_check_user(pctx->ncache, pctx->neg_timeout,
+                                              dom, pd->user);
+                if (ncret == ENOENT) {
+                    /* User not found in the negative cache
+                     * Proceed with PAM actions
+                     */
+                    break;
+                }
+
+                /* Try the next domain */
+                DEBUG(SSSDBG_TRACE_FUNC,
+                      "User [%s@%s] filtered out (negative cache). "
+                       "Trying next domain.\n", pd->user, dom->name);
+            }
+
+            if (!dom) {
+                ret = ENOENT;
+                goto done;
+            }
+            preq->domain = dom;
         }
-        preq->domain = dom;
     }
 
+
+    if (may_do_cert_auth(pctx, pd)) {
+        ret = check_cert(cctx, cctx->ev, pctx, preq, pd);
+        /* Finish here */
+        goto done;
+    }
+
+
     if (preq->domain->provider == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Domain [%s] has no auth provider.\n", preq->domain->name);
@@ -1032,6 +1207,142 @@ done:
     return pam_check_user_done(preq, ret);
 }
 
+static void pam_forwarder_lookup_by_cert_done(struct tevent_req *req);
+static void pam_forwarder_cert_cb(struct tevent_req *req)
+{
+    struct pam_auth_req *preq = tevent_req_callback_data(req,
+                                                         struct pam_auth_req);
+    struct cli_ctx *cctx = preq->cctx;
+    struct pam_data *pd;
+    errno_t ret = EOK;
+    char *cert;
+    struct pam_ctx *pctx =
+            talloc_get_type(preq->cctx->rctx->pvt_ctx, struct pam_ctx);
+
+    ret = pam_check_cert_recv(req, preq, &cert, &preq->token_name);
+    talloc_free(req);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "get_cert request failed.\n");
+        goto done;
+    }
+
+    pd = preq->pd;
+
+    if (cert == NULL) {
+        if (pd->logon_name == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "No certificate found and no logon name given, " \
+                  "authentication not possible.\n");;
+            ret = ENOENT;
+        } else {
+            if (pd->cmd == SSS_PAM_AUTHENTICATE) {
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "No certificate returned, authentication failed.\n");
+                ret = ENOENT;
+            } else {
+                ret = pam_check_user_search(preq);
+                if (ret == EOK) {
+                    pam_dom_forwarder(preq);
+                }
+            }
+
+        }
+        goto done;
+    }
+
+
+    req = cache_req_user_by_cert_send(preq, cctx->ev, cctx->rctx,
+                                      pctx->ncache, pctx->neg_timeout,
+                                      0, NULL, cert);
+    if (req == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "cache_req_user_by_cert_send failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+    tevent_req_set_callback(req, pam_forwarder_lookup_by_cert_done, preq);
+    return;
+
+done:
+    pam_check_user_done(preq, ret);
+}
+
+static void pam_forwarder_lookup_by_cert_done(struct tevent_req *req)
+{
+    int ret;
+    struct ldb_result *res;
+    struct sss_domain_info *domain;
+    struct pam_auth_req *preq = tevent_req_callback_data(req,
+                                                         struct pam_auth_req);
+    const char *cert_user;
+
+
+    ret = cache_req_user_by_cert_recv(preq, req, &res, &domain, NULL);
+    talloc_zfree(req);
+    if (ret != EOK && ret != ENOENT) {
+        DEBUG(SSSDBG_OP_FAILURE, "cache_req_user_by_cert request failed.\n");
+        goto done;
+    }
+
+    if (ret == EOK && res->count > 1) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Search by certificate returned more than one result.\n");
+        ret = EINVAL;
+        goto done;
+    }
+
+    if (ret == EOK) {
+        if (preq->domain == NULL) {
+            preq->domain = domain;
+        }
+
+        preq->cert_user_obj = talloc_steal(preq, res->msgs[0]);
+
+        if (preq->pd->logon_name == NULL) {
+            cert_user = ldb_msg_find_attr_as_string(preq->cert_user_obj,
+                                                    SYSDB_NAME, NULL);
+            if (cert_user == NULL) {
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "Certificate user object has not name.\n");
+                ret = ENOENT;
+                goto done;
+            }
+
+            DEBUG(SSSDBG_FUNC_DATA, "Found certificate user [%s].\n",
+                                    cert_user);
+
+            ret = add_pam_cert_response(preq->pd, cert_user, preq->token_name);
+            if (ret != EOK) {
+                DEBUG(SSSDBG_OP_FAILURE, "add_pam_cert_response failed.\n");
+            }
+
+            preq->pd->domain = talloc_strdup(preq->pd, domain->name);
+            if (preq->pd->domain == NULL) {
+                DEBUG(SSSDBG_OP_FAILURE, "talloc_strdup failed.\n");
+                ret = ENOMEM;
+                goto done;
+            }
+            preq->pd->pam_status = PAM_SUCCESS;
+            pam_reply(preq);
+            return;
+        }
+    } else {
+        if (preq->pd->logon_name == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Missing logon name and no certificate user found.\n");
+            ret = ENOENT;
+            goto done;
+        }
+    }
+
+    ret = pam_check_user_search(preq);
+    if (ret == EOK) {
+        pam_dom_forwarder(preq);
+    }
+
+done:
+    pam_check_user_done(preq, ret);
+}
+
 static void pam_forwarder_cb(struct tevent_req *req)
 {
     struct pam_auth_req *preq = tevent_req_callback_data(req,
@@ -1039,6 +1350,8 @@ static void pam_forwarder_cb(struct tevent_req *req)
     struct cli_ctx *cctx = preq->cctx;
     struct pam_data *pd;
     errno_t ret = EOK;
+    struct pam_ctx *pctx =
+            talloc_get_type(preq->cctx->rctx->pvt_ctx, struct pam_ctx);
 
     ret = sss_dp_get_domains_recv(req);
     talloc_free(req);
@@ -1077,6 +1390,12 @@ static void pam_forwarder_cb(struct tevent_req *req)
         }
     }
 
+    if (may_do_cert_auth(pctx, pd)) {
+        ret = check_cert(cctx, cctx->ev, pctx, preq, pd);
+        /* Finish here */
+        goto done;
+    }
+
     ret = pam_check_user_search(preq);
     if (ret == EOK) {
         pam_dom_forwarder(preq);
@@ -1106,7 +1425,7 @@ static int pam_check_user_search(struct pam_auth_req *preq)
         * qualified names instead */
         while (dom && !preq->pd->domain && !preq->pd->name_is_upn
                && dom->fqnames) {
-            dom = get_next_domain(dom, false);
+            dom = get_next_domain(dom, 0);
         }
 
         if (!dom) break;
@@ -1202,7 +1521,7 @@ static int pam_check_user_search(struct pam_auth_req *preq)
 
             /* if a multidomain search, try with next */
             if (!preq->pd->domain) {
-                dom = get_next_domain(dom, false);
+                dom = get_next_domain(dom, 0);
                 continue;
             }
 
@@ -1319,6 +1638,11 @@ static int pam_check_user_done(struct pam_auth_req *preq, int ret)
         pam_reply(preq);
         break;
 
+    case ERR_NO_CREDS:
+        preq->pd->pam_status = PAM_CRED_INSUFFICIENT;
+        pam_reply(preq);
+        break;
+
     default:
         preq->pd->pam_status = PAM_SYSTEM_ERR;
         pam_reply(preq);
@@ -1368,11 +1692,100 @@ static void pam_check_user_dp_callback(uint16_t err_maj, uint32_t err_min,
     }
 }
 
+static errno_t pam_is_last_online_login_fresh(struct sss_domain_info *domain,
+                                              const char* user,
+                                              struct confdb_ctx *cdb,
+                                              int cached_auth_timeout,
+                                              bool *_result)
+{
+    errno_t ret;
+    bool result;
+    uint64_t last_login;
+
+    ret = pam_get_last_online_auth_with_curr_token(domain, user, &last_login);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "sysdb_get_last_online_auth_with_curr_token failed: %s:[%d]\n",
+              sss_strerror(ret), ret);
+        goto done;
+    }
+
+    result = time(NULL) < (last_login + cached_auth_timeout);
+    ret = EOK;
+
+done:
+    if (ret == EOK) {
+        *_result = result;
+    }
+    return ret;
+}
+
+static bool pam_is_cmd_cachable(int cmd)
+{
+    bool is_cachable;
+
+    switch(cmd) {
+    case SSS_PAM_AUTHENTICATE:
+        is_cachable = true;
+        break;
+    default:
+        is_cachable = false;
+    }
+
+    return is_cachable;
+}
+
+static bool pam_is_authtok_cachable(struct sss_auth_token *authtok)
+{
+    enum sss_authtok_type type;
+    bool cachable = false;
+
+    type = sss_authtok_get_type(authtok);
+    if (type == SSS_AUTHTOK_TYPE_PASSWORD) {
+        cachable = true;
+    } else {
+        DEBUG(SSSDBG_TRACE_LIBS, "Authentication token can't be cached\n");
+    }
+
+    return cachable;
+}
+
+static bool pam_can_user_cache_auth(struct confdb_ctx *cdb,
+                                    struct sss_domain_info *domain,
+                                    int pam_cmd,
+                                    struct sss_auth_token *authtok,
+                                    const char* user,
+                                    bool cached_auth_failed)
+{
+    errno_t ret;
+    bool result = false;
+
+    if (!cached_auth_failed /* don't try cached auth again */
+            && domain->cache_credentials
+            && domain->cached_auth_timeout > 0
+            && pam_is_authtok_cachable(authtok)
+            && pam_is_cmd_cachable(pam_cmd)) {
+
+        ret = pam_is_last_online_login_fresh(domain, user, cdb,
+                                             domain->cached_auth_timeout,
+                                             &result);
+        if (ret != EOK) {
+            /* non-critical, consider fail as 'non-fresh value' */
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "pam_is_last_online_login_fresh failed: %s:[%d]\n",
+                  sss_strerror(ret), ret);
+        }
+    }
+
+    return result;
+}
+
 static void pam_dom_forwarder(struct pam_auth_req *preq)
 {
     int ret;
     struct pam_ctx *pctx =
             talloc_get_type(preq->cctx->rctx->pvt_ctx, struct pam_ctx);
+    const char *cert_user;
 
     if (!preq->pd->domain) {
         preq->pd->domain = preq->domain->name;
@@ -1399,7 +1812,63 @@ static void pam_dom_forwarder(struct pam_auth_req *preq)
         return;
     }
 
-    if (!NEED_CHECK_PROVIDER(preq->domain->provider)) {
+    if (pam_can_user_cache_auth(pctx->rctx->cdb,
+                                preq->domain,
+                                preq->pd->cmd,
+                                preq->pd->authtok,
+                                preq->pd->user,
+                                preq->cached_auth_failed)) {
+        preq->use_cached_auth = true;
+        pam_reply(preq);
+        return;
+    }
+
+    if (may_do_cert_auth(pctx, preq->pd) && preq->cert_user_obj != NULL) {
+        /* Check if user matches certificate user */
+        cert_user = ldb_msg_find_attr_as_string(preq->cert_user_obj, SYSDB_NAME,
+                                                NULL);
+        if (cert_user == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Certificate user object has not name.\n");
+            preq->pd->pam_status = PAM_USER_UNKNOWN;
+            pam_reply(preq);
+            return;
+        }
+
+        /* pam_check_user_search() calls pd_set_primary_name() is the search
+         * was successful, so pd->user contains the canonical name as well */
+        if (strcmp(cert_user, preq->pd->user) == 0) {
+
+            preq->pd->pam_status = PAM_SUCCESS;
+
+            if (preq->pd->cmd == SSS_PAM_PREAUTH) {
+                ret = add_pam_cert_response(preq->pd, cert_user,
+                                            preq->token_name);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_OP_FAILURE, "add_pam_cert_response failed.\n");
+                    preq->pd->pam_status = PAM_AUTHINFO_UNAVAIL;
+                }
+            }
+
+            preq->callback = pam_reply;
+            pam_reply(preq);
+            return;
+        } else {
+            if (preq->pd->cmd == SSS_PAM_PREAUTH) {
+                DEBUG(SSSDBG_TRACE_FUNC,
+                      "User and certificate user do not match, " \
+                      "continue with other authentication methods.\n");
+            } else {
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      "User and certificate user do not match.\n");
+                preq->pd->pam_status = PAM_AUTH_ERR;
+                pam_reply(preq);
+                return;
+            }
+        }
+    }
+
+    if (!NEED_CHECK_PROVIDER(preq->domain->provider) ) {
         preq->callback = pam_reply;
         ret = LOCAL_pam_handler(preq);
     }
@@ -1450,6 +1919,12 @@ static int pam_cmd_chauthtok_prelim(struct cli_ctx *cctx) {
     return pam_forwarder(cctx, SSS_PAM_CHAUTHTOK_PRELIM);
 }
 
+static int pam_cmd_preauth(struct cli_ctx *cctx)
+{
+    DEBUG(SSSDBG_CONF_SETTINGS, "entering pam_cmd_preauth\n");
+    return pam_forwarder(cctx, SSS_PAM_PREAUTH);
+}
+
 struct cli_protocol_version *register_cli_protocol_version(void)
 {
     static struct cli_protocol_version pam_cli_protocol_version[] = {
@@ -1473,8 +1948,106 @@ struct sss_cmd_table *get_pam_cmds(void)
         {SSS_PAM_CLOSE_SESSION, pam_cmd_close_session},
         {SSS_PAM_CHAUTHTOK, pam_cmd_chauthtok},
         {SSS_PAM_CHAUTHTOK_PRELIM, pam_cmd_chauthtok_prelim},
+        {SSS_PAM_PREAUTH, pam_cmd_preauth},
         {SSS_CLI_NULL, NULL}
     };
 
     return sss_cmds;
 }
+
+errno_t
+pam_set_last_online_auth_with_curr_token(struct sss_domain_info *domain,
+                                         const char *username,
+                                         uint64_t value)
+{
+    TALLOC_CTX *tmp_ctx;
+    struct sysdb_attrs *attrs;
+    int ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    attrs = sysdb_new_attrs(tmp_ctx);
+    if (attrs == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = sysdb_attrs_add_time_t(attrs,
+                                 SYSDB_LAST_ONLINE_AUTH_WITH_CURR_TOKEN,
+                                 value);
+    if (ret != EOK) { goto done; }
+
+    ret = sysdb_set_user_attr(domain, username, attrs, SYSDB_MOD_REP);
+    if (ret != EOK) { goto done; }
+
+done:
+    if (ret != EOK) {
+        DEBUG(SSSDBG_TRACE_FUNC, "Error: %d (%s)\n", ret, sss_strerror(ret));
+    }
+
+    talloc_zfree(tmp_ctx);
+    return ret;
+}
+
+static errno_t
+pam_null_last_online_auth_with_curr_token(struct sss_domain_info *domain,
+                                          const char *username)
+{
+    return pam_set_last_online_auth_with_curr_token(domain, username, 0);
+}
+
+static errno_t
+pam_get_last_online_auth_with_curr_token(struct sss_domain_info *domain,
+                                         const char *name,
+                                         uint64_t *_value)
+{
+    TALLOC_CTX *tmp_ctx = NULL;
+    const char *attrs[] = { SYSDB_LAST_ONLINE_AUTH_WITH_CURR_TOKEN, NULL };
+    struct ldb_message *ldb_msg;
+    uint64_t value;
+    errno_t ret;
+
+    if (name == NULL || *name == '\0') {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Missing user name.\n");
+        ret = EINVAL;
+        goto done;
+    }
+
+    if (domain->sysdb == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Missing sysdb db context.\n");
+        ret = EINVAL;
+        goto done;
+    }
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = sysdb_search_user_by_name(tmp_ctx, domain, name, attrs, &ldb_msg);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "sysdb_search_user_by_name failed [%d][%s].\n",
+              ret, strerror(ret));
+        goto done;
+    }
+
+    /* Check offline_auth_cache_timeout */
+    value = ldb_msg_find_attr_as_uint64(ldb_msg,
+                                        SYSDB_LAST_ONLINE_AUTH_WITH_CURR_TOKEN,
+                                        0);
+    ret = EOK;
+
+done:
+    if (ret == EOK) {
+        *_value = value;
+    }
+
+    talloc_free(tmp_ctx);
+    return ret;
+}
