@@ -28,6 +28,7 @@
 #include "util/util.h"
 #include "db/sysdb_sudo.h"
 #include "responder/sudo/sudosrv_private.h"
+#include "providers/data_provider.h"
 
 static errno_t sudosrv_get_user(struct sudo_dom_ctx *dctx);
 
@@ -77,6 +78,8 @@ static errno_t sudosrv_get_user(struct sudo_dom_ctx *dctx)
     struct tevent_req *dpreq;
     struct dp_callback_ctx *cb_ctx;
     const char *original_name = NULL;
+    const char *extra_flag = NULL;
+    const char *search_name = NULL;
     char *name = NULL;
     uid_t uid = 0;
     errno_t ret;
@@ -91,7 +94,7 @@ static errno_t sudosrv_get_user(struct sudo_dom_ctx *dctx)
        /* if it is a domainless search, skip domains that require fully
         * qualified names instead */
         while (dom && cmd_ctx->check_next && dom->fqnames) {
-            dom = get_next_domain(dom, false);
+            dom = get_next_domain(dom, 0);
         }
 
         if (!dom) break;
@@ -119,8 +122,7 @@ static errno_t sudosrv_get_user(struct sudo_dom_ctx *dctx)
         DEBUG(SSSDBG_FUNC_DATA, "Requesting info about [%s@%s]\n",
               name, dom->name);
 
-        ret = sysdb_getpwnam(dctx, dctx->domain->sysdb,
-                             dctx->domain, name, &user);
+        ret = sysdb_getpwnam_with_views(dctx, dctx->domain, name, &user);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE,
                   "Failed to make request to our cache!\n");
@@ -139,7 +141,7 @@ static errno_t sudosrv_get_user(struct sudo_dom_ctx *dctx)
             /* if a multidomain search, try with next */
             if (cmd_ctx->check_next) {
                 dctx->check_provider = true;
-                dom = get_next_domain(dom, false);
+                dom = get_next_domain(dom, 0);
                 if (dom) continue;
             }
 
@@ -158,9 +160,24 @@ static errno_t sudosrv_get_user(struct sudo_dom_ctx *dctx)
          * outdated, go to DP */
         if ((user->count == 0 || cache_expire < time(NULL))
             && dctx->check_provider) {
+
+            search_name = cmd_ctx->username;
+            if (is_local_view(dom->view_name)) {
+                /* Search with original name in case of local view. */
+                if (user->count != 0) {
+                    search_name = ldb_msg_find_attr_as_string(user->msgs[0],
+                                                              SYSDB_NAME, NULL);
+                }
+            } else if (DOM_HAS_VIEWS(dom) && (user->count == 0
+                || ldb_msg_find_attr_as_string(user->msgs[0],
+                                               OVERRIDE_PREFIX SYSDB_NAME,
+                                               NULL) != NULL)) {
+                extra_flag = EXTRA_INPUT_MAYBE_WITH_VIEW;
+            }
+
             dpreq = sss_dp_get_account_send(cli_ctx, cli_ctx->rctx,
                                             dom, false, SSS_DP_INITGROUPS,
-                                            cmd_ctx->username, 0, NULL);
+                                            search_name, 0, extra_flag);
             if (!dpreq) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       "Out of memory sending data provider request\n");
@@ -188,12 +205,13 @@ static errno_t sudosrv_get_user(struct sudo_dom_ctx *dctx)
         }
 
         /* check uid */
-        uid = ldb_msg_find_attr_as_int(user->msgs[0], SYSDB_UIDNUM, 0);
+        uid = sss_view_ldb_msg_find_attr_as_uint64(dom, user->msgs[0],
+                                                   SYSDB_UIDNUM, 0);
         if (uid != cmd_ctx->uid) {
             /* if a multidomain search, try with next */
             if (cmd_ctx->check_next) {
                 dctx->check_provider = true;
-                dom = get_next_domain(dom, false);
+                dom = get_next_domain(dom, 0);
                 if (dom) continue;
             }
 
@@ -312,7 +330,6 @@ static void
 sudosrv_dp_req_done(struct tevent_req *req);
 
 static errno_t sudosrv_get_sudorules_query_cache(TALLOC_CTX *mem_ctx,
-                                                 struct sysdb_ctx *sysdb,
                                                  struct sss_domain_info *domain,
                                                  const char **attrs,
                                                  unsigned int flags,
@@ -365,7 +382,7 @@ errno_t sudosrv_get_rules(struct sudo_cmd_ctx *cmd_ctx)
      * expired rules for this user and defaults at once we will save one
      * provider call
      */
-    ret = sysdb_get_sudo_user_info(tmp_ctx, cmd_ctx->domain->sysdb, cmd_ctx->domain,
+    ret = sysdb_get_sudo_user_info(tmp_ctx, cmd_ctx->domain,
                                    cmd_ctx->orig_username, NULL, &groupnames);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
@@ -377,7 +394,7 @@ errno_t sudosrv_get_rules(struct sudo_cmd_ctx *cmd_ctx)
             | SYSDB_SUDO_FILTER_INCLUDE_DFL
             | SYSDB_SUDO_FILTER_ONLY_EXPIRED
             | SYSDB_SUDO_FILTER_USERINFO;
-    ret = sudosrv_get_sudorules_query_cache(tmp_ctx, cmd_ctx->domain->sysdb,
+    ret = sudosrv_get_sudorules_query_cache(tmp_ctx,
                                             cmd_ctx->domain, attrs, flags,
                                             cmd_ctx->orig_username,
                                             cmd_ctx->uid, groupnames,
@@ -571,7 +588,7 @@ static errno_t sudosrv_get_sudorules_from_cache(TALLOC_CTX *mem_ctx,
     switch (cmd_ctx->type) {
     case SSS_SUDO_USER:
         debug_name = cmd_ctx->cased_username;
-        ret = sysdb_get_sudo_user_info(tmp_ctx, cmd_ctx->domain->sysdb,
+        ret = sysdb_get_sudo_user_info(tmp_ctx,
                                        cmd_ctx->domain,
                                        cmd_ctx->orig_username,
                                        NULL, &groupnames);
@@ -589,7 +606,7 @@ static errno_t sudosrv_get_sudorules_from_cache(TALLOC_CTX *mem_ctx,
         break;
     }
 
-    ret = sudosrv_get_sudorules_query_cache(tmp_ctx, cmd_ctx->domain->sysdb,
+    ret = sudosrv_get_sudorules_query_cache(tmp_ctx,
                                             cmd_ctx->domain, attrs, flags,
                                             cmd_ctx->orig_username,
                                             cmd_ctx->uid, groupnames,
@@ -622,7 +639,6 @@ static errno_t
 sort_sudo_rules(struct sysdb_attrs **rules, size_t count, bool higher_wins);
 
 static errno_t sudosrv_get_sudorules_query_cache(TALLOC_CTX *mem_ctx,
-                                                 struct sysdb_ctx *sysdb,
                                                  struct sss_domain_info *domain,
                                                  const char **attrs,
                                                  unsigned int flags,
@@ -659,11 +675,11 @@ static errno_t sudosrv_get_sudorules_query_cache(TALLOC_CTX *mem_ctx,
         domain = domain->parent;
     }
 
-    ret = sysdb_search_custom(tmp_ctx, sysdb, domain, filter,
+    ret = sysdb_search_custom(tmp_ctx, domain, filter,
                               SUDORULE_SUBDIR, attrs,
                               &count, &msgs);
     if (ret != EOK && ret != ENOENT) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Error looking up SUDO rules");
+        DEBUG(SSSDBG_CRIT_FAILURE, "Error looking up SUDO rules\n");
         goto done;
     } if (ret == ENOENT) {
        *_rules = NULL;
