@@ -43,10 +43,14 @@ enum sss_sudo_type {
 struct sudo_ctx {
     struct resp_ctx *rctx;
 
+    int neg_timeout;
+    struct sss_nc_ctx *ncache;
+
     /*
      * options
      */
     bool timed;
+    bool inverse_order;
 };
 
 struct sudo_cmd_ctx {
