@@ -54,8 +54,15 @@ struct sbus_request;
 #define DBUS_ERROR_INIT { NULL, NULL, TRUE, 0, 0, 0, 0, NULL }
 #endif /* DBUS_ERROR_INIT */
 
+/**
+ * Note: internal functions do not rely on the value of this constant to
+ * simplify implementation. If this connstant change, some functions in
+ * sssd_dbus_interface.c needs to be amended.
+ */
+#define SBUS_SUBTREE_SUFFIX "/*"
+
 typedef int (*sbus_msg_handler_fn)(struct sbus_request *dbus_req,
-                                   void *instance_data);
+                                   void *handler_data);
 
 /*
  * sbus_conn_destructor_fn
@@ -73,6 +80,10 @@ typedef void (*sbus_conn_reconn_callback_fn)(struct sbus_connection *, int, void
  */
 typedef int (*sbus_server_conn_init_fn)(struct sbus_connection *, void *);
 
+typedef const char ** (* sbus_nodes_fn)(TALLOC_CTX *mem_ctx,
+                                        const char *path,
+                                        void *data);
+
 enum {
     SBUS_CONN_TYPE_PRIVATE = 1,
     SBUS_CONN_TYPE_SHARED,
@@ -110,24 +121,9 @@ struct sbus_vtable {
 struct sbus_interface {
     const char *path;
     struct sbus_vtable *vtable;
-    void *instance_data;
+    void *handler_data;
 };
 
-/*
- * Creates a new struct sbus_interface instance to be exported by a DBus
- * service.
- *
- * Pass the result to sbus_conn_add_interface(). The interface
- * will be exported at @object_path. The method handlers are represented by
- * @iface_vtable. @instance_data contains additional caller specific data
- * which is made available to handlers.
- */
-struct sbus_interface *
-sbus_new_interface(TALLOC_CTX *mem_ctx,
-                   const char *object_path,
-                   struct sbus_vtable *iface_vtable,
-                   void *instance_data);
-
 /* Server Functions */
 int sbus_new_server(TALLOC_CTX *mem_ctx,
                     struct tevent_context *ev,
@@ -170,9 +166,69 @@ int sbus_init_connection(TALLOC_CTX *ctx,
                          struct sbus_connection **_conn);
 
 DBusConnection *sbus_get_connection(struct sbus_connection *conn);
+
 void sbus_disconnect(struct sbus_connection *conn);
-int sbus_conn_add_interface(struct sbus_connection *conn,
-                            struct sbus_interface *intf);
+
+/*
+ * Register a new interface to be available at given object path.
+ *
+ * The interface will be exported at @object_path. The method handlers are
+ * represented by @iface_vtable. @pvt contains additional caller specific data
+ * which is made available to handlers.
+ */
+int sbus_conn_register_iface(struct sbus_connection *conn,
+                             struct sbus_vtable *iface_vtable,
+                             const char *object_path,
+                             void *handler_data);
+
+void
+sbus_conn_register_nodes(struct sbus_connection *conn,
+                         const char *path,
+                         sbus_nodes_fn nodes_fn,
+                         void *data);
+
+errno_t
+sbus_conn_reregister_paths(struct sbus_connection *conn);
+
+char *
+sbus_opath_escape_part(TALLOC_CTX *mem_ctx,
+                       const char *object_path_part);
+
+char *
+sbus_opath_unescape_part(TALLOC_CTX *mem_ctx,
+                         const char *object_path_part);
+
+char *
+_sbus_opath_compose(TALLOC_CTX *mem_ctx,
+                    const char *base,
+                    const char *part, ...);
+
+#define sbus_opath_compose(mem_ctx, base, ...) \
+    _sbus_opath_compose(mem_ctx, base, ##__VA_ARGS__, NULL)
+
+errno_t
+sbus_opath_decompose(TALLOC_CTX *mem_ctx,
+                     const char *object_path,
+                     const char *prefix,
+                     char ***_components,
+                     size_t *_len);
+
+errno_t
+sbus_opath_decompose_exact(TALLOC_CTX *mem_ctx,
+                           const char *object_path,
+                           const char *prefix,
+                           size_t expected,
+                           char ***_components);
+
+const char *
+sbus_opath_strip_prefix(const char *object_path,
+                        const char *prefix);
+
+char *
+sbus_opath_get_object_name(TALLOC_CTX *mem_ctx,
+                           const char *object_path,
+                           const char *base_path);
+
 bool sbus_conn_disconnecting(struct sbus_connection *conn);
 
 /* max_retries < 0: retry forever
@@ -184,12 +240,6 @@ void sbus_reconnect_init(struct sbus_connection *conn,
                          sbus_conn_reconn_callback_fn callback,
                          void *pvt);
 
-/* Default message handler
- * Should be usable for most cases */
-DBusHandlerResult sbus_message_handler(DBusConnection *conn,
-                                  DBusMessage *message,
-                                  void *user_data);
-
 /*
  * Send a message across the SBUS
  * If requested, the DBusPendingCall object will
@@ -225,6 +275,7 @@ struct sbus_request {
     DBusMessage *message;
     struct sbus_interface *intf;
     const struct sbus_method_meta *method;
+    const char *path;
 };
 
 /*
@@ -278,28 +329,6 @@ int sbus_request_return_and_finish(struct sbus_request *dbus_req,
                                    int first_arg_type,
                                    ...);
 
-int sbus_add_variant_to_dict(DBusMessageIter *iter_dict,
-                             const char *key,
-                             int type,
-                             const void *value);
-
-int sbus_add_array_as_variant_to_dict(DBusMessageIter *iter_dict,
-                                      const char *key,
-                                      int type,
-                                      uint8_t *values,
-                                      const int len,
-                                      const unsigned int item_size);
-
-int sbus_request_return_as_variant(struct sbus_request *dbus_req,
-                                   int type,
-                                   const void *value);
-
-int sbus_request_return_array_as_variant(struct sbus_request *dbus_req,
-                                         int type,
-                                         uint8_t *values,
-                                         const int len,
-                                         const size_t item_size);
-
 /*
 
  * Return an error for a DBus method call request. The @error is a normal
@@ -375,4 +404,24 @@ bool sbus_request_parse_or_finish(struct sbus_request *request,
                                   int first_arg_type,
                                   ...);
 
+struct sbus_incoming_signal {
+    struct sbus_connection *conn;
+    DBusMessage *message;
+    int64_t client;
+    const char *interface;
+    const char *signal;
+    const char *path;
+};
+
+typedef void
+(*sbus_incoming_signal_fn)(struct sbus_incoming_signal *sbus_signal,
+                           void *handler_data);
+
+errno_t
+sbus_signal_listen(struct sbus_connection *conn,
+                   const char *iface,
+                   const char *signal,
+                   sbus_incoming_signal_fn handler_fn,
+                   void *handler_data);
+
 #endif /* _SSSD_DBUS_H_*/
