@@ -30,21 +30,6 @@
 /* Types */
 struct dbus_ctx_list;
 
-static DBusObjectPathVTable dbus_object_path_vtable =
-    { NULL, sbus_message_handler, NULL, NULL, NULL, NULL };
-
-struct sbus_interface_p {
-    struct sbus_interface_p *prev, *next;
-    struct sbus_connection *conn;
-    struct sbus_interface *intf;
-
-    const char *reg_path;
-};
-
-static bool path_in_interface_list(struct sbus_interface_p *list,
-                                   const char *path);
-static void sbus_unreg_object_paths(struct sbus_connection *conn);
-
 static int sbus_auto_reconnect(struct sbus_connection *conn);
 
 static void sbus_dispatch(struct tevent_context *ev,
@@ -166,6 +151,7 @@ int sbus_init_connection(TALLOC_CTX *ctx,
                          struct sbus_connection **_conn)
 {
     struct sbus_connection *conn;
+    dbus_bool_t dbret;
     int ret;
 
     DEBUG(SSSDBG_TRACE_FUNC,"Adding connection %p\n", dbus_conn);
@@ -176,6 +162,28 @@ int sbus_init_connection(TALLOC_CTX *ctx,
     conn->dbus.conn = dbus_conn;
     conn->connection_type = connection_type;
 
+    ret = sbus_opath_hash_init(conn, conn, &conn->managed_paths);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Cannot create object paths hash table\n");
+        talloc_free(conn);
+        return EIO;
+    }
+
+    ret = sbus_nodes_hash_init(conn, conn, &conn->nodes_fns);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Cannot create node functions hash table\n");
+        talloc_free(conn);
+        return EIO;
+    }
+
+    ret = sbus_incoming_signal_hash_init(conn, &conn->incoming_signals);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Cannot create incoming singals "
+              "hash table\n");
+        talloc_free(conn);
+        return EIO;
+    }
+
     ret = sss_hash_create(conn, 32, &conn->clients);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Cannot create clients hash table\n");
@@ -189,6 +197,17 @@ int sbus_init_connection(TALLOC_CTX *ctx,
         return ret;
     }
 
+    /* Set up signal handler. */
+    dbret = dbus_connection_add_filter(dbus_conn, sbus_signal_handler, conn,
+                                       NULL);
+    if (dbret == false) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Cannot register signal handler\n");
+        talloc_free(conn);
+        return EIO;
+    }
+
+    sbus_register_common_signals(conn, conn);
+
     *_conn = conn;
     return ret;
 }
@@ -328,7 +347,7 @@ void sbus_disconnect(struct sbus_connection *conn)
     conn->disconnect = 1;
 
     /* Unregister object paths */
-    sbus_unreg_object_paths(conn);
+    talloc_zfree(conn->managed_paths);
 
     /* Disable watch functions */
     dbus_connection_set_watch_functions(conn->dbus.conn,
@@ -357,354 +376,12 @@ void sbus_disconnect(struct sbus_connection *conn)
     DEBUG(SSSDBG_TRACE_FUNC ,"Disconnected %p\n", conn->dbus.conn);
 }
 
-static bool sbus_fb_path_has_prefix(const char *path, const char *prefix)
-{
-    /* strlen-1 because we don't want to match the trailing '*' */
-    if (strncmp(path, prefix, strlen(prefix)-1) == 0) {
-        return true;
-    }
-
-    return false;
-}
-
-static bool sbus_path_has_fallback(const char *path)
-{
-    char *wildcard;
-
-    wildcard = strrchr(path, '*');
-    if (wildcard != NULL) {
-        /* This path was registered as fallback */
-        if (*(wildcard + 1) != '\0') {
-            /* Wildcard is only allowed as the last character in the path */
-            return false;
-        }
-        return true;
-    }
-
-    return false;
-}
-
-static bool sbus_iface_handles_path(struct sbus_interface_p *intf_p,
-                                    const char *path)
-{
-    if (sbus_path_has_fallback(intf_p->intf->path)) {
-        return sbus_fb_path_has_prefix(path, intf_p->intf->path);
-    }
-
-    return strcmp(path, intf_p->intf->path) == 0;
-}
-
-static void sbus_handler_got_caller_id(struct tevent_req *req);
-
-/* messsage_handler
- * Receive messages and process them
- */
-DBusHandlerResult sbus_message_handler(DBusConnection *dbus_conn,
-                                       DBusMessage *message,
-                                       void *user_data)
-{
-    struct sbus_interface_p *intf_p;
-    const char *msg_method;
-    const char *path;
-    const char *msg_interface;
-    const char *sender;
-    struct sbus_request *dbus_req = NULL;
-    struct tevent_req *req;
-
-    if (!user_data) {
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-    }
-    intf_p = talloc_get_type(user_data, struct sbus_interface_p);
-
-    msg_method = dbus_message_get_member(message);
-    DEBUG(SSSDBG_TRACE_ALL, "Received SBUS method [%s]\n", msg_method);
-    path = dbus_message_get_path(message);
-    msg_interface = dbus_message_get_interface(message);
-    sender = dbus_message_get_sender(message);
-
-    if (!msg_method || !path || !msg_interface) {
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-    }
-
-    /* Validate the D-BUS path */
-    if (!sbus_iface_handles_path(intf_p, path)) {
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-    }
-
-    /* Validate the method interface */
-    if (strcmp(msg_interface, intf_p->intf->vtable->meta->name) == 0 ||
-             strcmp(msg_interface, DBUS_PROPERTIES_INTERFACE) == 0 ||
-            (strcmp(msg_interface, DBUS_INTROSPECT_INTERFACE) == 0 &&
-                strcmp(msg_method, DBUS_INTROSPECT_METHOD) == 0)) {
-
-        /* OK, this message for us. Get the sender ID if applicable */
-        dbus_req = sbus_new_request(intf_p->conn, intf_p->intf, message);
-        if (dbus_req == NULL) {
-            return DBUS_HANDLER_RESULT_NEED_MEMORY;
-        }
-
-        req = sbus_get_sender_id_send(dbus_req, dbus_req->conn->ev,
-                                      dbus_req->conn, sender);
-        if (req == NULL) {
-            return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-        }
-        tevent_req_set_callback(req, sbus_handler_got_caller_id, dbus_req);
-        return DBUS_HANDLER_RESULT_HANDLED;
-    }
-
-    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-}
-
-static void sbus_handler_got_caller_id(struct tevent_req *req)
-{
-    struct sbus_request *dbus_req = \
-                      tevent_req_callback_data(req,
-                                               struct sbus_request);
-    errno_t ret;
-    DBusError *dberr;
-    DBusMessage *reply = NULL;
-    const struct sbus_method_meta *method;
-    const struct sbus_interface_meta *interface;
-    const char *msg_method;
-    const char *msg_interface;
-    sbus_msg_handler_fn handler_fn = NULL;
-    void *handler_data = NULL; /* Must be a talloc pointer! */
-    struct sbus_introspect_ctx *ictx = NULL;
-    const char *dbus_error = NULL;
-
-    ret = sbus_get_sender_id_recv(req, &dbus_req->client);
-    if (ret != EOK) {
-        dberr = sbus_error_new(dbus_req,
-                               DBUS_ERROR_FAILED,
-                               "Failed to retrieve called ID: %s\n",
-                               sss_strerror(ret));
-        sbus_request_fail_and_finish(dbus_req, dberr);
-        return;
-    }
-
-    msg_method = dbus_message_get_member(dbus_req->message);
-    msg_interface = dbus_message_get_interface(dbus_req->message);
-    DEBUG(SSSDBG_TRACE_ALL, "Received SBUS method [%s]\n", msg_method);
-
-    /* Prepare the handler */
-    interface = dbus_req->intf->vtable->meta;
-    if (strcmp(msg_interface, interface->name) == 0) {
-        method = sbus_meta_find_method(interface, msg_method);
-        if (method && method->vtable_offset)
-            handler_fn = VTABLE_FUNC(dbus_req->intf->vtable,
-                                     method->vtable_offset);
-
-        if (!method) {
-            /* Reply DBUS_ERROR_UNKNOWN_METHOD */
-            DEBUG(SSSDBG_CRIT_FAILURE,
-                  "No matching method found for %s.\n", msg_method);
-            dbus_error = DBUS_ERROR_UNKNOWN_METHOD;
-            goto fail;
-        } else if (!handler_fn) {
-            /* Reply DBUS_ERROR_NOT_SUPPORTED */
-            DEBUG(SSSDBG_CRIT_FAILURE,
-                  "No handler provided found for %s.\n", msg_method);
-            dbus_error = DBUS_ERROR_NOT_SUPPORTED;
-            goto fail;
-        }
-    } else if (strcmp(msg_interface, DBUS_INTROSPECT_INTERFACE) == 0 &&
-                strcmp(msg_method, DBUS_INTROSPECT_METHOD) == 0) {
-            DEBUG(SSSDBG_TRACE_LIBS, "Got introspection request\n");
-            ictx = talloc(dbus_req->conn, struct sbus_introspect_ctx);
-            if (ictx == NULL) {
-                dbus_error = DBUS_ERROR_NO_MEMORY;
-                goto fail;
-            }
-
-            handler_fn = sbus_introspect;
-            ictx->iface = interface;
-            handler_data = ictx;
-            method = &introspect_method;
-    } else if (strcmp(msg_interface, DBUS_PROPERTIES_INTERFACE) == 0) {
-        ret = sbus_properties_dispatch(dbus_req);
-        if (ret == ERR_SBUS_NOSUP) {
-            /* No known method matched */
-            dbus_error = DBUS_ERROR_NOT_SUPPORTED;
-            goto fail;
-        }
-        /* sbus_properties_dispatch handles all other errors
-         * or success internally
-         */
-        return;
-    }
-
-    if (handler_fn == NULL) {
-        DEBUG(SSSDBG_MINOR_FAILURE, "No handler matched!\n");
-        dbus_error = DBUS_ERROR_NOT_SUPPORTED;
-        goto fail;
-    }
-
-    dbus_req->method = method;
-    if (handler_data) {
-        /* If the handler uses private instance data, make
-         * sure they go away when the request does
-         */
-        talloc_steal(dbus_req, handler_data);
-    } else {
-        /* If no custom handler data is set, pass on the
-         * interface data
-         */
-        handler_data = dbus_req->intf->instance_data;
-    }
-
-    sbus_request_invoke_or_finish(dbus_req,
-                                  handler_fn,
-                                  handler_data,
-                                  method->invoker);
-    return;
-
-fail:
-    reply = dbus_message_new_error(dbus_req->message,
-                                   dbus_error ? dbus_error : DBUS_ERROR_FAILED,
-                                   NULL);
-    sbus_request_finish(dbus_req, reply);
-}
-
-struct sbus_interface *
-sbus_new_interface(TALLOC_CTX *mem_ctx,
-                   const char *object_path,
-                   struct sbus_vtable *iface_vtable,
-                   void *instance_data)
-{
-    struct sbus_interface *intf;
-
-    intf = talloc_zero(mem_ctx, struct sbus_interface);
-    if (intf == NULL) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "Cannot allocate a new sbus_interface.\n");
-        return NULL;
-    }
-
-    intf->path = talloc_strdup(intf, object_path);
-    if (intf->path == NULL) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "Cannot duplicate object path.\n");
-        talloc_free(intf);
-        return NULL;
-    }
-
-    intf->vtable = iface_vtable;
-    intf->instance_data = instance_data;
-    return intf;
-}
-
-static char *sbus_iface_get_reg_path(TALLOC_CTX *mem_ctx,
-                                     const char *path,
-                                     bool fallback)
-{
-    char *reg_path;
-
-    reg_path = talloc_strdup(mem_ctx, path);
-    if (reg_path == NULL) return NULL;
-
-    if (fallback) {
-        reg_path[strlen(path)-1] = '\0';
-    }
-    return reg_path;
-}
-
-/* Adds a new D-BUS path message handler to the connection
- * Note: this must be a unique path.
- */
-int sbus_conn_add_interface(struct sbus_connection *conn,
-                            struct sbus_interface *intf)
-{
-    struct sbus_interface_p *intf_p;
-    dbus_bool_t dbret;
-    const char *path;
-    bool fallback;
-
-    if (!conn || !intf || !intf->vtable || !intf->vtable->meta) {
-        return EINVAL;
-    }
-
-    path = intf->path;
-    fallback = sbus_path_has_fallback(path);
-
-    if (path_in_interface_list(conn->intf_list, path)) {
-        DEBUG(SSSDBG_FATAL_FAILURE,
-              "Cannot add method context with identical path.\n");
-        return EINVAL;
-    }
-
-    intf_p = talloc_zero(conn, struct sbus_interface_p);
-    if (!intf_p) {
-        return ENOMEM;
-    }
-    intf_p->conn = conn;
-    intf_p->intf = intf;
-    intf_p->reg_path = sbus_iface_get_reg_path(intf_p, path, fallback);
-    if (intf_p->reg_path == NULL) {
-        return ENOMEM;
-    }
-
-    DLIST_ADD(conn->intf_list, intf_p);
-
-    DEBUG(SSSDBG_TRACE_LIBS, "Will register path %s with%s fallback\n",
-                             intf_p->reg_path, fallback ? "" : "out");
-
-    if (fallback) {
-        dbret = dbus_connection_register_fallback(conn->dbus.conn,
-                                                  intf_p->reg_path,
-                                                  &dbus_object_path_vtable,
-                                                  intf_p);
-    } else {
-        dbret = dbus_connection_register_object_path(conn->dbus.conn,
-                                                     intf_p->reg_path,
-                                                     &dbus_object_path_vtable,
-                                                     intf_p);
-    }
-    if (!dbret) {
-        DEBUG(SSSDBG_FATAL_FAILURE,
-              "Could not register object path to the connection.\n");
-        return ENOMEM;
-    }
-
-    return EOK;
-}
-
-static bool path_in_interface_list(struct sbus_interface_p *list,
-                                   const char *path)
-{
-    struct sbus_interface_p *iter;
-
-    if (!list || !path) {
-        return false;
-    }
-
-    iter = list;
-    while (iter != NULL) {
-        if (strcmp(iter->intf->path, path) == 0) {
-            return true;
-        }
-        iter = iter->next;
-    }
-
-    return false;
-}
-
-static void sbus_unreg_object_paths(struct sbus_connection *conn)
-{
-    struct sbus_interface_p *iter = conn->intf_list;
-
-    while (iter != NULL) {
-        dbus_connection_unregister_object_path(conn->dbus.conn,
-                                               iter->intf->path);
-        iter = iter->next;
-    }
-}
-
 static void sbus_reconnect(struct tevent_context *ev,
                            struct tevent_timer *te,
                            struct timeval tv, void *data)
 {
     struct sbus_connection *conn;
-    struct sbus_interface_p *iter;
     DBusError dbus_error;
-    dbus_bool_t dbret;
     int ret;
 
     conn = talloc_get_type(data, struct sbus_connection);
@@ -723,20 +400,7 @@ static void sbus_reconnect(struct tevent_context *ev,
         }
 
         /* Re-register object paths */
-        iter = conn->intf_list;
-        while (iter) {
-            dbret = dbus_connection_register_object_path(conn->dbus.conn,
-                                                         iter->intf->path,
-                                                         &dbus_object_path_vtable,
-                                                         iter);
-            if (!dbret) {
-                DEBUG(SSSDBG_FATAL_FAILURE,
-                      "Could not register object path.\n");
-                dbus_connection_unref(conn->dbus.conn);
-                goto failed;
-            }
-            iter = iter->next;
-        }
+        sbus_conn_reregister_paths(conn);
 
         /* Reset retries to 0 to resume dispatch processing */
         conn->retries = 0;
