@@ -35,7 +35,6 @@ enum dbus_conn_type {
     SBUS_CONNECTION
 };
 
-struct sbus_interface_p;
 struct sbus_watch_ctx;
 
 struct sbus_connection {
@@ -48,8 +47,9 @@ struct sbus_connection {
     int connection_type;
     int disconnect;
 
-    /* dbus tables and handlers */
-    struct sbus_interface_p *intf_list;
+    hash_table_t *managed_paths;
+    hash_table_t *nodes_fns;
+    hash_table_t *incoming_signals;
 
     /* reconnect settings */
     int retries;
@@ -68,6 +68,11 @@ struct sbus_connection {
     struct sbus_watch_ctx *watch_list;
 };
 
+/* =Standard=interfaces=================================================== */
+
+struct sbus_vtable *sbus_introspect_vtable(void);
+struct sbus_vtable *sbus_properties_vtable(void);
+
 /* =Watches=============================================================== */
 
 struct sbus_watch_ctx {
@@ -103,14 +108,38 @@ struct sbus_request *
 sbus_new_request(struct sbus_connection *conn, struct sbus_interface *intf,
                  DBusMessage *message);
 
-/* =Interface=introspection=============================================== */
-extern const struct sbus_method_meta introspect_method;
+/* =Interface=and=object=paths============================================ */
 
-struct sbus_introspect_ctx {
-    const struct sbus_interface_meta *iface;
+struct sbus_interface_list {
+    struct sbus_interface_list *prev, *next;
+    struct sbus_interface *interface;
 };
 
-int sbus_introspect(struct sbus_request *dbus_req, void *pvt);
+errno_t
+sbus_opath_hash_init(TALLOC_CTX *mem_ctx,
+                     struct sbus_connection *conn,
+                     hash_table_t **_table);
+
+struct sbus_interface *
+sbus_opath_hash_lookup_iface(hash_table_t *table,
+                             const char *object_path,
+                             const char *iface_name);
+
+errno_t
+sbus_opath_hash_lookup_supported(TALLOC_CTX *mem_ctx,
+                                 hash_table_t *table,
+                                 const char *object_path,
+                                 struct sbus_interface_list **_list);
+
+errno_t
+sbus_nodes_hash_init(TALLOC_CTX *mem_ctx,
+                     struct sbus_connection *conn,
+                     hash_table_t **_table);
+
+const char **
+sbus_nodes_hash_lookup(TALLOC_CTX *mem_ctx,
+                       hash_table_t *table,
+                       const char *object_path);
 
 void
 sbus_request_invoke_or_finish(struct sbus_request *dbus_req,
@@ -137,4 +166,21 @@ struct tevent_req *sbus_get_sender_id_send(TALLOC_CTX *mem_ctx,
                                            const char *sender);
 int sbus_get_sender_id_recv(struct tevent_req *req, int64_t *_uid);
 
+/* =Properties============================================================ */
+
+int sbus_properties_dispatch(struct sbus_request *dbus_req);
+
+/* =Signals=============================================================== */
+
+DBusHandlerResult
+sbus_signal_handler(DBusConnection *conn,
+                    DBusMessage *message,
+                    void *handler_data);
+
+errno_t
+sbus_incoming_signal_hash_init(TALLOC_CTX *mem_ctx,
+                               hash_table_t **_table);
+
+void sbus_register_common_signals(struct sbus_connection *conn, void *pvt);
+
 #endif /* _SSSD_DBUS_PRIVATE_H_ */
