@@ -19,6 +19,7 @@
 */
 
 #include "util/util.h"
+#include "util/sss_utf8.h"
 #include "sbus/sssd_dbus.h"
 #include "sbus/sssd_dbus_private.h"
 
@@ -49,6 +50,7 @@ sbus_new_request(struct sbus_connection *conn,
     dbus_req->intf = intf;
     dbus_req->conn = conn;
     dbus_req->message = dbus_message_ref(message);
+    dbus_req->path = dbus_message_get_path(message);
     talloc_set_destructor(dbus_req, sbus_request_destructor);
 
     return dbus_req;
@@ -95,23 +97,66 @@ int sbus_request_finish(struct sbus_request *dbus_req,
     return talloc_free(dbus_req);
 }
 
+static int sbus_request_valist_check(va_list va, int first_arg_type)
+{
+    int ret = EOK;
+#ifdef HAVE_DBUSBASICVALUE
+    int type;
+    va_list va_check;
+    const DBusBasicValue *value;
+    bool ok;
+
+    va_copy(va_check, va);
+
+    type = first_arg_type;
+    while (type != DBUS_TYPE_INVALID) {
+        value = va_arg(va_check, const DBusBasicValue*);
+
+        if (type == DBUS_TYPE_STRING) {
+             ok = sss_utf8_check((const uint8_t *) value->str,
+                                  strlen(value->str));
+             if (!ok) {
+                   DEBUG(SSSDBG_MINOR_FAILURE,
+                         "sbus message argument [%s] contains invalid "
+                         "non-UTF8 characters\n", value->str);
+                 ret = EINVAL;
+                 break;
+             }
+        }
+        type = va_arg(va_check, int);
+    }
+
+    va_end(va_check);
+#endif /* HAVE_DBUSBASICVALUE */
+    return ret;
+}
+
 int sbus_request_return_and_finish(struct sbus_request *dbus_req,
                                    int first_arg_type,
                                    ...)
 {
     DBusMessage *reply;
+    DBusError error = DBUS_ERROR_INIT;
     dbus_bool_t dbret;
     va_list va;
     int ret;
 
+    va_start(va, first_arg_type);
+    ret = sbus_request_valist_check(va, first_arg_type);
+    if (ret != EOK) {
+        va_end(va);
+        dbus_set_error_const(&error, DBUS_ERROR_INVALID_ARGS, INTERNAL_ERROR);
+        return sbus_request_fail_and_finish(dbus_req, &error);
+    }
+
     reply = dbus_message_new_method_return(dbus_req->message);
     if (!reply) {
+        va_end(va);
         DEBUG(SSSDBG_CRIT_FAILURE, "Out of memory allocating DBus message\n");
         sbus_request_finish(dbus_req, NULL);
         return ENOMEM;
     }
 
-    va_start(va, first_arg_type);
     dbret = dbus_message_append_args_valist(reply, first_arg_type, va);
     va_end(va);
 
@@ -128,190 +173,6 @@ int sbus_request_return_and_finish(struct sbus_request *dbus_req,
     return ret;
 }
 
-int sbus_request_return_as_variant(struct sbus_request *dbus_req,
-                                   int type,
-                                   const void *value)
-{
-    DBusMessage *reply;
-    dbus_bool_t dbret;
-    DBusMessageIter iter;
-    DBusMessageIter valiter;
-    char strtype[2];
-    int ret;
-
-    snprintf(strtype, 2, "%c", type);
-
-    reply = dbus_message_new_method_return(dbus_req->message);
-    if (reply == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Out of memory allocating DBus message\n");
-        sbus_request_finish(dbus_req, NULL);
-        return ENOMEM;
-    }
-
-    dbus_message_iter_init_append(reply, &iter);
-    dbret = dbus_message_iter_open_container(&iter, DBUS_TYPE_VARIANT,
-                                             strtype, &valiter);
-    if (!dbret) {
-        sbus_request_fail_and_finish(
-                            dbus_req,
-                            sbus_error_new(dbus_req,
-                                        DBUS_ERROR_FAILED,
-                                        "Could not open variant for [%s]\n",
-                                        strtype));
-        ret = EINVAL;
-        goto done;
-    }
-
-    dbret = dbus_message_iter_append_basic(&valiter, type, value);
-    if (!dbret) {
-        sbus_request_fail_and_finish(
-                            dbus_req,
-                            sbus_error_new(dbus_req,
-                                        DBUS_ERROR_FAILED,
-                                        "Could not append [%s] to variant\n",
-                                        strtype));
-        ret = EINVAL;
-        goto done;
-    }
-
-    dbret = dbus_message_iter_close_container(&iter, &valiter);
-    if (dbret) {
-        ret = sbus_request_finish(dbus_req, reply);
-    } else {
-        sbus_request_fail_and_finish(
-                            dbus_req,
-                            sbus_error_new(dbus_req,
-                                        DBUS_ERROR_FAILED,
-                                        "Could not close variant\n"));
-        ret = EINVAL;
-    }
-
-done:
-    dbus_message_unref(reply);
-    return ret;
-}
-
-
-int sbus_request_return_array_as_variant(struct sbus_request *dbus_req,
-                                         int type,
-                                         uint8_t *values,
-                                         const int len,
-                                         const size_t item_size)
-{
-    TALLOC_CTX *tmp_ctx = NULL;
-    DBusMessage *reply = NULL;
-    dbus_bool_t dbret;
-    DBusMessageIter iter;
-    DBusMessageIter variant_iter;
-    DBusMessageIter array_iter;
-    char *variant_type = NULL;
-    char *array_type = NULL;
-    void *addr = NULL;
-    int ret;
-    int i;
-
-    tmp_ctx = talloc_new(NULL);
-    if (tmp_ctx == NULL) {
-        return ENOMEM;
-    }
-
-    variant_type = talloc_asprintf(tmp_ctx, DBUS_TYPE_ARRAY_AS_STRING "%c",
-                                   type);
-    if (variant_type == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
-
-    array_type = talloc_asprintf(tmp_ctx, "%c", type);
-    if (array_type == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
-
-    reply = dbus_message_new_method_return(dbus_req->message);
-    if (reply == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Out of memory allocating DBus message\n");
-        sbus_request_finish(dbus_req, NULL);
-        ret = ENOMEM;
-        goto done;
-    }
-
-    dbus_message_iter_init_append(reply, &iter);
-    dbret = dbus_message_iter_open_container(&iter, DBUS_TYPE_VARIANT,
-                                             variant_type, &variant_iter);
-    if (!dbret) {
-        sbus_request_fail_and_finish(
-                            dbus_req,
-                            sbus_error_new(dbus_req,
-                                        DBUS_ERROR_FAILED,
-                                        "Could not open variant for [%s]\n",
-                                        variant_type));
-        ret = EINVAL;
-        goto done;
-    }
-
-    /* Open container for values */
-    dbret = dbus_message_iter_open_container(&variant_iter, DBUS_TYPE_ARRAY,
-                                             array_type, &array_iter);
-    if (!dbret) {
-        sbus_request_fail_and_finish(
-                            dbus_req,
-                            sbus_error_new(dbus_req,
-                                        DBUS_ERROR_FAILED,
-                                        "Could not open array for [%s]\n",
-                                        array_type));
-        ret = EINVAL;
-        goto done;
-    }
-
-    for (i = 0; i < len; i++) {
-        addr = values + i * item_size;
-        dbret = dbus_message_iter_append_basic(&array_iter, type, addr);
-        if (!dbret) {
-            sbus_request_fail_and_finish(
-                                dbus_req,
-                                sbus_error_new(dbus_req,
-                                            DBUS_ERROR_FAILED,
-                                            "Could not append [%s] to variant\n",
-                                            array_type));
-            ret = EINVAL;
-            goto done;
-        }
-    }
-
-    dbret = dbus_message_iter_close_container(&variant_iter, &array_iter);
-    if (!dbret) {
-        sbus_request_fail_and_finish(
-                            dbus_req,
-                            sbus_error_new(dbus_req,
-                                        DBUS_ERROR_FAILED,
-                                        "Could not close array\n"));
-        ret = EINVAL;
-        goto done;
-    }
-
-    dbret = dbus_message_iter_close_container(&iter, &variant_iter);
-    if (dbret) {
-        ret = sbus_request_finish(dbus_req, reply);
-    } else {
-        sbus_request_fail_and_finish(
-                            dbus_req,
-                            sbus_error_new(dbus_req,
-                                        DBUS_ERROR_FAILED,
-                                        "Could not close variant\n"));
-        ret = EINVAL;
-        goto done;
-    }
-
-done:
-    if (reply != NULL) {
-        dbus_message_unref(reply);
-    }
-
-    talloc_free(tmp_ctx);
-    return ret;
-}
-
 int sbus_request_fail_and_finish(struct sbus_request *dbus_req,
                                  const DBusError *error)
 {
@@ -510,12 +371,20 @@ struct tevent_req *sbus_get_sender_id_send(TALLOC_CTX *mem_ctx,
         goto immediate;
     }
 
+    if (strcmp(sender, "org.freedesktop.DBus") == 0) {
+        ret = ERR_SBUS_SENDER_BUS;
+        goto immediate;
+    }
+
     state->sender = talloc_strdup(state, sender);
     if (state->sender == NULL) {
         ret = ENOMEM;
         goto immediate;
     }
 
+    DEBUG(SSSDBG_TRACE_INTERNAL,
+          "Looking for identity of sender [%s]\n", sender);
+
     key.type = HASH_KEY_STRING;
     key.str = discard_const(sender);
     ret = hash_lookup(conn->clients, &key, &value);
@@ -562,6 +431,7 @@ struct tevent_req *sbus_get_sender_id_send(TALLOC_CTX *mem_ctx,
                                      DBUS_TYPE_STRING, &sender,
                                      DBUS_TYPE_INVALID);
     if (!dbret) {
+        ret = ERR_INTERNAL;
         goto immediate;
     }
 
@@ -628,6 +498,7 @@ static void sbus_get_sender_id_done(DBusPendingCall *pending, void *ptr)
                                   DBUS_TYPE_UINT32, &uid,
                                   DBUS_TYPE_INVALID);
     if (!dbret) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Could not parse reply!\n");
         ret = EIO;
         goto done;
     }
@@ -640,6 +511,7 @@ static void sbus_get_sender_id_done(DBusPendingCall *pending, void *ptr)
     value.ul = state->uid;
     ret = hash_enter(state->conn->clients, &key, &value);
     if (ret != HASH_SUCCESS) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Could not add key to hash table!\n");
         ret = EIO;
         goto done;
     }
