@@ -21,10 +21,10 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <tevent.h>
+#include <dbus/dbus.h>
 
-#include "tevent.h"
 #include "util/util.h"
-#include "dbus/dbus.h"
 #include "sbus/sssd_dbus.h"
 #include "sbus/sssd_dbus_private.h"
 
@@ -56,7 +56,7 @@ static void sbus_server_init_new_connection(DBusServer *dbus_server,
                                SBUS_CONN_TYPE_PRIVATE, &conn);
     if (ret != 0) {
         dbus_connection_close(dbus_conn);
-        DEBUG(SSSDBG_FUNC_DATA,"Closing connection (failed setup)");
+        DEBUG(SSSDBG_FUNC_DATA, "Closing connection (failed setup)\n");
         return;
     }
 
@@ -145,7 +145,7 @@ remove_socket_symlink(const char *symlink_name)
     ret = snprintf(pidpath, PATH_MAX, "%s.%lu",
                    symlink_name, (unsigned long) getpid());
     if (ret < 0) {
-        DEBUG(SSSDBG_OP_FAILURE, "snprintf failed");
+        DEBUG(SSSDBG_OP_FAILURE, "snprintf failed\n");
         return EIO;
     } else if (ret >= PATH_MAX) {
         DEBUG(SSSDBG_OP_FAILURE, "path too long?!?!\n");
@@ -181,6 +181,7 @@ remove_socket_symlink(const char *symlink_name)
 int sbus_new_server(TALLOC_CTX *mem_ctx,
                     struct tevent_context *ev,
                     const char *address,
+                    uid_t uid, gid_t gid,
                     bool use_symlink,
                     struct sbus_connection **_server,
                     sbus_server_conn_init_fn init_fn,
@@ -191,7 +192,7 @@ int sbus_new_server(TALLOC_CTX *mem_ctx,
     DBusError dbus_error;
     dbus_bool_t dbret;
     char *tmp;
-    int ret;
+    int ret, tmp_ret;
     char *filename;
     char *symlink_filename = NULL;
     const char *socket_address;
@@ -249,19 +250,33 @@ int sbus_new_server(TALLOC_CTX *mem_ctx,
 
     /* Both check_file and chmod can handle both the symlink and
      * the socket */
-    ret = check_file(filename, getuid(), getgid(), -1, CHECK_SOCK, &stat_buf, true);
+    ret = check_file(filename,
+                     getuid(), getgid(), S_IFSOCK, S_IFMT, &stat_buf, true);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "check_file failed for [%s].\n", filename);
         ret = EIO;
         goto done;
     }
 
-    if ((stat_buf.st_mode & ~S_IFMT) != 0600) {
-        ret = chmod(filename, 0600);
+    if ((stat_buf.st_mode & ~S_IFMT) != (S_IRUSR|S_IWUSR)) {
+        ret = chmod(filename, (S_IRUSR|S_IWUSR));
         if (ret != EOK) {
+            ret = errno;
             DEBUG(SSSDBG_CRIT_FAILURE,
-                  "chmod failed for [%s]: [%d][%s].\n", filename, errno,
-                                                         strerror(errno));
+                  "chmod failed for [%s]: [%d][%s].\n", filename, ret,
+                                                        sss_strerror(ret));
+            ret = EIO;
+            goto done;
+        }
+    }
+
+    if (stat_buf.st_uid != uid || stat_buf.st_gid != gid) {
+        ret = chown(filename, uid, gid);
+        if (ret != EOK) {
+            ret = errno;
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "chown failed for [%s]: [%d][%s].\n", filename, ret,
+                                                        sss_strerror(ret));
             ret = EIO;
             goto done;
         }
@@ -328,9 +343,17 @@ int sbus_new_server(TALLOC_CTX *mem_ctx,
 
     *_server = talloc_steal(mem_ctx, server);
     ret = EOK;
+
 done:
     if (ret != EOK && symlink_filename) {
-        unlink(symlink_filename);
+        tmp_ret = unlink(symlink_filename);
+        /* non-fatal failure */
+        if (tmp_ret != EOK) {
+            tmp_ret = errno;
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "Failed to remove symbolic link: %d [%s]!\n",
+                  tmp_ret, sss_strerror(tmp_ret));
+        }
     }
     talloc_free(tmp_ctx);
     return ret;
