@@ -198,7 +198,7 @@ static enum sss_status sss_cli_recv_rep(enum sss_cli_command cmd,
     int len;
     int ret;
 
-    header[0] = SSS_NSS_HEADER_SIZE; /* unitl we know the real lenght */
+    header[0] = SSS_NSS_HEADER_SIZE; /* unitl we know the real length */
     header[1] = 0;
     header[2] = 0;
     header[3] = 0;
@@ -734,6 +734,22 @@ enum nss_status sss_nss_make_request(enum sss_cli_command cmd,
     }
 
     ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
+    if (ret == SSS_STATUS_UNAVAIL && *errnop == EPIPE) {
+        /* try reopen socket */
+        ret = sss_cli_check_socket(errnop, SSS_NSS_SOCKET_NAME);
+        if (ret != SSS_STATUS_SUCCESS) {
+#ifdef NONSTANDARD_SSS_NSS_BEHAVIOUR
+            *errnop = 0;
+            errno = 0;
+            return NSS_STATUS_NOTFOUND;
+#else
+            return NSS_STATUS_UNAVAIL;
+#endif
+        }
+
+        /* and make request one more time */
+        ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
+    }
     switch (ret) {
     case SSS_STATUS_TRYAGAIN:
         return NSS_STATUS_TRYAGAIN;
@@ -784,6 +800,16 @@ int sss_pac_make_request(enum sss_cli_command cmd,
     }
 
     ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
+    if (ret == SSS_STATUS_UNAVAIL && *errnop == EPIPE) {
+        /* try reopen socket */
+        ret = sss_cli_check_socket(errnop, SSS_PAC_SOCKET_NAME);
+        if (ret != SSS_STATUS_SUCCESS) {
+            return NSS_STATUS_UNAVAIL;
+        }
+
+        /* and make request one more time */
+        ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
+    }
     switch (ret) {
     case SSS_STATUS_TRYAGAIN:
         return NSS_STATUS_TRYAGAIN;
@@ -829,6 +855,7 @@ int sss_pam_make_request(enum sss_cli_command cmd,
     enum sss_status status;
     char *envval;
     struct stat stat_buf;
+    const char *socket_name;
 
     sss_pam_lock();
 
@@ -841,7 +868,8 @@ int sss_pam_make_request(enum sss_cli_command cmd,
 
     /* only root shall use the privileged pipe */
     if (getuid() == 0 && getgid() == 0) {
-        statret = stat(SSS_PAM_PRIV_SOCKET_NAME, &stat_buf);
+        socket_name = SSS_PAM_PRIV_SOCKET_NAME;
+        statret = stat(socket_name, &stat_buf);
         if (statret != 0) {
             ret = PAM_SERVICE_ERR;
             goto out;
@@ -854,10 +882,9 @@ int sss_pam_make_request(enum sss_cli_command cmd,
             ret = PAM_SERVICE_ERR;
             goto out;
         }
-
-        status = sss_cli_check_socket(errnop, SSS_PAM_PRIV_SOCKET_NAME);
     } else {
-        statret = stat(SSS_PAM_SOCKET_NAME, &stat_buf);
+        socket_name = SSS_PAM_SOCKET_NAME;
+        statret = stat(socket_name, &stat_buf);
         if (statret != 0) {
             ret = PAM_SERVICE_ERR;
             goto out;
@@ -870,9 +897,9 @@ int sss_pam_make_request(enum sss_cli_command cmd,
             ret = PAM_SERVICE_ERR;
             goto out;
         }
-
-        status = sss_cli_check_socket(errnop, SSS_PAM_SOCKET_NAME);
     }
+
+    status = sss_cli_check_socket(errnop, socket_name);
     if (status != SSS_STATUS_SUCCESS) {
         ret = PAM_SERVICE_ERR;
         goto out;
@@ -887,6 +914,18 @@ int sss_pam_make_request(enum sss_cli_command cmd,
     }
 
     status = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
+    if (status == SSS_STATUS_UNAVAIL && *errnop == EPIPE) {
+        /* try reopen socket */
+        status = sss_cli_check_socket(errnop, socket_name);
+        if (status != SSS_STATUS_SUCCESS) {
+            ret = PAM_SERVICE_ERR;
+            goto out;
+        }
+
+        /* and make request one more time */
+        status = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
+    }
+
     if (status == SSS_STATUS_SUCCESS) {
         ret = PAM_SUCCESS;
     } else {
@@ -910,38 +949,51 @@ void sss_pam_close_fd(void)
     sss_pam_unlock();
 }
 
-int sss_sudo_make_request(enum sss_cli_command cmd,
-                          struct sss_cli_req_data *rd,
-                          uint8_t **repbuf, size_t *replen,
-                          int *errnop)
+static enum sss_status
+sss_cli_make_request_with_checks(enum sss_cli_command cmd,
+                                 struct sss_cli_req_data *rd,
+                                 uint8_t **repbuf, size_t *replen,
+                                 int *errnop,
+                                 const char *socket_name)
 {
     enum sss_status ret = SSS_STATUS_UNAVAIL;
 
-    ret = sss_cli_check_socket(errnop, SSS_SUDO_SOCKET_NAME);
+    ret = sss_cli_check_socket(errnop, socket_name);
     if (ret != SSS_STATUS_SUCCESS) {
         return SSS_STATUS_UNAVAIL;
     }
 
     ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
+    if (ret == SSS_STATUS_UNAVAIL && *errnop == EPIPE) {
+        /* try reopen socket */
+        ret = sss_cli_check_socket(errnop, socket_name);
+        if (ret != SSS_STATUS_SUCCESS) {
+            return SSS_STATUS_UNAVAIL;
+        }
+
+        /* and make request one more time */
+        ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
+    }
 
     return ret;
 }
 
+int sss_sudo_make_request(enum sss_cli_command cmd,
+                          struct sss_cli_req_data *rd,
+                          uint8_t **repbuf, size_t *replen,
+                          int *errnop)
+{
+    return sss_cli_make_request_with_checks(cmd, rd, repbuf, replen, errnop,
+                                            SSS_SUDO_SOCKET_NAME);
+}
+
 int sss_autofs_make_request(enum sss_cli_command cmd,
                             struct sss_cli_req_data *rd,
                             uint8_t **repbuf, size_t *replen,
                             int *errnop)
 {
-    enum sss_status ret = SSS_STATUS_UNAVAIL;
-
-    ret = sss_cli_check_socket(errnop, SSS_AUTOFS_SOCKET_NAME);
-    if (ret != SSS_STATUS_SUCCESS) {
-        return SSS_STATUS_UNAVAIL;
-    }
-
-    ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
-
-    return ret;
+    return sss_cli_make_request_with_checks(cmd, rd, repbuf, replen, errnop,
+                                            SSS_AUTOFS_SOCKET_NAME);
 }
 
 int sss_ssh_make_request(enum sss_cli_command cmd,
@@ -949,16 +1001,8 @@ int sss_ssh_make_request(enum sss_cli_command cmd,
                          uint8_t **repbuf, size_t *replen,
                          int *errnop)
 {
-    enum sss_status ret = SSS_STATUS_UNAVAIL;
-
-    ret = sss_cli_check_socket(errnop, SSS_SSH_SOCKET_NAME);
-    if (ret != SSS_STATUS_SUCCESS) {
-        return SSS_STATUS_UNAVAIL;
-    }
-
-    ret = sss_cli_make_request_nochecks(cmd, rd, repbuf, replen, errnop);
-
-    return ret;
+    return sss_cli_make_request_with_checks(cmd, rd, repbuf, replen, errnop,
+                                            SSS_SSH_SOCKET_NAME);
 }
 
 
@@ -1032,6 +1076,7 @@ struct sss_mutex {
 
 static void sss_nss_mt_init(void);
 static void sss_pam_mt_init(void);
+static void sss_nss_mc_mt_init(void);
 
 static struct sss_mutex sss_nss_mtx = { .mtx  = PTHREAD_MUTEX_INITIALIZER,
                                         .once = PTHREAD_ONCE_INIT,
@@ -1041,6 +1086,9 @@ static struct sss_mutex sss_pam_mtx = { .mtx  = PTHREAD_MUTEX_INITIALIZER,
                                         .once = PTHREAD_ONCE_INIT,
                                         .init = sss_pam_mt_init };
 
+static struct sss_mutex sss_nss_mc_mtx = { .mtx  = PTHREAD_MUTEX_INITIALIZER,
+                                           .once = PTHREAD_ONCE_INIT,
+                                           .init = sss_nss_mc_mt_init };
 
 /* Wrappers for robust mutex support */
 static int sss_mutexattr_setrobust (pthread_mutexattr_t *attr)
@@ -1125,6 +1173,20 @@ void sss_pam_unlock(void)
     sss_mt_unlock(&sss_pam_mtx);
 }
 
+/* NSS mutex wrappers */
+static void sss_nss_mc_mt_init(void)
+{
+    sss_mt_init(&sss_nss_mc_mtx);
+}
+void sss_nss_mc_lock(void)
+{
+    sss_mt_lock(&sss_nss_mc_mtx);
+}
+void sss_nss_mc_unlock(void)
+{
+    sss_mt_unlock(&sss_nss_mc_mtx);
+}
+
 #else
 
 /* sorry no mutexes available */
@@ -1132,6 +1194,8 @@ void sss_nss_lock(void) { return; }
 void sss_nss_unlock(void) { return; }
 void sss_pam_lock(void) { return; }
 void sss_pam_unlock(void) { return; }
+void sss_nss_mc_lock(void) { return; }
+void sss_nss_mc_unlock(void) { return; }
 #endif
 
 
