@@ -48,7 +48,7 @@ void setup_check_and_open(void)
     filename = strdup(FILENAME_TEMPLATE);
     fail_unless(filename != NULL, "strdup failed");
 
-    old_umask = umask(077);
+    old_umask = umask(SSS_DFL_X_UMASK);
     ret = mkstemp(filename);
     umask(old_umask);
     fail_unless(ret != -1, "mkstemp failed [%d][%s]", errno, strerror(errno));
@@ -79,7 +79,8 @@ START_TEST(test_wrong_filename)
 {
     int ret;
 
-    ret = check_and_open_readonly("/bla/bla/bla", &fd, uid, gid, mode, CHECK_REG);
+    ret = check_and_open_readonly("/bla/bla/bla", &fd,
+                                  uid, gid, S_IFREG|mode, 0);
     fail_unless(ret == ENOENT,
                 "check_and_open_readonly succeeded on non-existing file");
     fail_unless(fd == -1, "check_and_open_readonly file descriptor not -1");
@@ -104,11 +105,12 @@ START_TEST(test_symlink)
     ret = symlink(filename, newpath);
     fail_unless(ret == 0, "symlink failed [%d][%s]", ret, strerror(errno));
 
-    ret = check_file(newpath, uid, gid, mode, CHECK_REG, NULL, false);
+    ret = check_file(newpath, uid, gid, S_IFREG|mode, 0, NULL, false);
     unlink(newpath);
 
     fail_unless(ret == EINVAL,
                 "check_and_open_readonly succeeded on symlink");
+    free(newpath);
 }
 END_TEST
 
@@ -130,11 +132,12 @@ START_TEST(test_follow_symlink)
     ret = symlink(filename, newpath);
     fail_unless(ret == 0, "symlink failed [%d][%s]", ret, strerror(errno));
 
-    ret = check_file(newpath, uid, gid, mode, CHECK_REG, NULL, true);
+    ret = check_file(newpath, uid, gid, S_IFREG|mode, 0, NULL, true);
     unlink(newpath);
 
     fail_unless(ret == EOK,
                 "check_and_open_readonly failed on symlink with follow=true");
+    free(newpath);
 }
 END_TEST
 
@@ -142,7 +145,7 @@ START_TEST(test_not_regular_file)
 {
     int ret;
 
-    ret = check_and_open_readonly("/dev/null", &fd, uid, gid, mode, CHECK_REG);
+    ret = check_and_open_readonly("/dev/null", &fd, uid, gid, S_IFREG|mode, 0);
     fail_unless(ret == EINVAL,
                 "check_and_open_readonly succeeded on non-regular file");
     fail_unless(fd == -1, "check_and_open_readonly file descriptor not -1");
@@ -153,7 +156,7 @@ START_TEST(test_wrong_uid)
 {
     int ret;
 
-    ret = check_and_open_readonly(filename, &fd, uid+1, gid, mode, CHECK_REG);
+    ret = check_and_open_readonly(filename, &fd, uid+1, gid, S_IFREG|mode, 0);
     fail_unless(ret == EINVAL,
                 "check_and_open_readonly succeeded with wrong uid");
     fail_unless(fd == -1, "check_and_open_readonly file descriptor not -1");
@@ -164,7 +167,7 @@ START_TEST(test_wrong_gid)
 {
     int ret;
 
-    ret = check_and_open_readonly(filename, &fd, uid, gid+1, mode, CHECK_REG);
+    ret = check_and_open_readonly(filename, &fd, uid, gid+1, S_IFREG|mode, 0);
     fail_unless(ret == EINVAL,
                 "check_and_open_readonly succeeded with wrong gid");
     fail_unless(fd == -1, "check_and_open_readonly file descriptor not -1");
@@ -175,8 +178,8 @@ START_TEST(test_wrong_permission)
 {
     int ret;
 
-    ret = check_and_open_readonly(filename, &fd, uid, gid, (mode|S_IWOTH),
-                                  CHECK_REG);
+    ret = check_and_open_readonly(filename, &fd,
+                                  uid, gid, S_IFREG|mode|S_IWOTH, 0);
     fail_unless(ret == EINVAL,
                 "check_and_open_readonly succeeded with wrong mode");
     fail_unless(fd == -1, "check_and_open_readonly file descriptor not -1");
@@ -187,7 +190,7 @@ START_TEST(test_ok)
 {
     int ret;
 
-    ret = check_and_open_readonly(filename, &fd, uid, gid, mode, CHECK_REG);
+    ret = check_and_open_readonly(filename, &fd, uid, gid, S_IFREG|mode, 0);
     fail_unless(ret == EOK,
                 "check_and_open_readonly failed");
     fail_unless(fd >= 0,
@@ -201,7 +204,7 @@ START_TEST(test_write)
     ssize_t size;
     errno_t my_errno;
 
-    ret = check_and_open_readonly(filename, &fd, uid, gid, mode, CHECK_REG);
+    ret = check_and_open_readonly(filename, &fd, uid, gid, S_IFREG|mode, 0);
     fail_unless(ret == EOK,
                 "check_and_open_readonly failed");
     fail_unless(fd >= 0,
