@@ -28,15 +28,18 @@
 #include "providers/dp_ptask_private.h"
 #include "providers/dp_ptask.h"
 #include "tests/cmocka/common_mock.h"
+#include "tests/cmocka/common_mock_be.h"
 #include "tests/common.h"
 
 #define DELAY  2
 #define PERIOD 1
+#define TIMEOUT 123
 
 #define new_test(test) \
     cmocka_unit_test_setup_teardown(test_ ## test, test_setup, test_teardown)
 
 struct test_ctx {
+    struct sss_test_ctx *tctx;
     struct be_ctx *be_ctx;
 
     time_t when;
@@ -274,19 +277,23 @@ static int test_setup(void **state)
     test_ctx = talloc_zero(global_talloc_context, struct test_ctx);
     assert_non_null(test_ctx);
 
-    /* create be_ctx, only ev and offline field should be used */
-    test_ctx->be_ctx = talloc_zero(test_ctx, struct be_ctx);
+    test_ctx->tctx = create_ev_test_ctx(test_ctx);
+    assert_non_null(test_ctx->tctx);
+
+    test_ctx->be_ctx = mock_be_ctx(test_ctx, test_ctx->tctx);
     assert_non_null(test_ctx->be_ctx);
 
     test_ctx->be_ctx->ev = tevent_context_init(test_ctx->be_ctx);
     assert_non_null(test_ctx->be_ctx->ev);
 
+    check_leaks_push(test_ctx);
     *state = test_ctx;
     return 0;
 }
 
 static int test_teardown(void **state)
 {
+    assert_true(check_leaks_pop(*state));
     talloc_zfree(*state);
     assert_true(leak_check_teardown());
     return 0;
@@ -791,6 +798,26 @@ void test_be_ptask_get_period(void **state)
     assert_null(ptask);
 }
 
+void test_be_ptask_get_timeout(void **state)
+{
+    struct test_ctx *test_ctx = (struct test_ctx *)(*state);
+    struct be_ptask *ptask = NULL;
+    time_t out_timeout;
+    errno_t ret;
+
+    ret = be_ptask_create(test_ctx, test_ctx->be_ctx, PERIOD, 0, 0, 0, TIMEOUT,
+                          BE_PTASK_OFFLINE_SKIP, 0, test_be_ptask_send,
+                          test_be_ptask_recv, test_ctx, "Test ptask", &ptask);
+    assert_int_equal(ret, ERR_OK);
+    assert_non_null(ptask);
+
+    out_timeout = be_ptask_get_timeout(ptask);
+    assert_true(TIMEOUT == out_timeout);
+
+    be_ptask_destroy(&ptask);
+    assert_null(ptask);
+}
+
 void test_be_ptask_create_sync(void **state)
 {
     struct test_ctx *test_ctx = (struct test_ctx *)(*state);
@@ -966,6 +993,7 @@ int main(int argc, const char *argv[])
         new_test(be_ptask_reschedule_timeout),
         new_test(be_ptask_reschedule_backoff),
         new_test(be_ptask_get_period),
+        new_test(be_ptask_get_timeout),
         new_test(be_ptask_create_sync),
         new_test(be_ptask_sync_reschedule_ok),
         new_test(be_ptask_sync_reschedule_error),
