@@ -30,10 +30,9 @@
 #define BLOB_DEFAULT    "blobval"
 #define INT_DEFAULT     123
 
-#define TESTS_PATH "tests_opts"
+#define TESTS_PATH "tp_" BASE_FILE_STEM
 #define TEST_CONF_DB "test_opt_conf.ldb"
 #define TEST_DOM_NAME "opt_test"
-#define TEST_SYSDB_FILE "cache_"TEST_DOM_NAME".ldb"
 #define TEST_ID_PROVIDER "ldap"
 
 enum test_opts {
@@ -518,12 +517,12 @@ int main(int argc, const char *argv[])
     /* Even though normally the tests should clean up after themselves
      * they might not after a failed run. Remove the old db to be sure */
     tests_set_cwd();
-    test_dom_suite_cleanup(TESTS_PATH, TEST_CONF_DB, TEST_SYSDB_FILE);
+    test_dom_suite_cleanup(TESTS_PATH, TEST_CONF_DB, TEST_DOM_NAME);
     test_dom_suite_setup(TESTS_PATH);
 
     ret = cmocka_run_group_tests(tests, NULL, NULL);
     if (ret == 0 && !no_cleanup) {
-        test_dom_suite_cleanup(TESTS_PATH, TEST_CONF_DB, TEST_SYSDB_FILE);
+        test_dom_suite_cleanup(TESTS_PATH, TEST_CONF_DB, TEST_DOM_NAME);
     }
     return ret;
 }
