@@ -134,14 +134,6 @@ void ifp_test_req_wrong_uid(void **state)
     assert_true(leak_check_teardown());
 }
 
-void test_path_prefix(void **state)
-{
-    const char *prefix = "foo";
-
-    assert_non_null(ifp_path_strip_prefix("foobar", prefix));
-    assert_null(ifp_path_strip_prefix("notfoo", prefix));
-}
-
 void test_el_to_dict(void **state)
 {
     static struct sbus_request *sr;
@@ -234,7 +226,7 @@ static void attr_parse_test(const char *expected[], const char *input)
     test_ctx = talloc_new(NULL);
     assert_non_null(test_ctx);
 
-    res = ifp_parse_attr_list(test_ctx, input);
+    res = ifp_parse_user_attr_list(test_ctx, input);
 
     if (expected) {
         /* Positive test */
@@ -277,7 +269,7 @@ void test_attr_acl(void **state)
     const char *exp_defaults[] = { SYSDB_NAME, SYSDB_UIDNUM,
                                    SYSDB_GIDNUM, SYSDB_GECOS,
                                    SYSDB_HOMEDIR, SYSDB_SHELL,
-                                   NULL };
+                                   "groups", NULL };
     attr_parse_test(exp_defaults, NULL);
 
     /* Test adding some attributes to the defaults */
@@ -285,13 +277,13 @@ void test_attr_acl(void **state)
                               SYSDB_NAME, SYSDB_UIDNUM,
                               SYSDB_GIDNUM, SYSDB_GECOS,
                               SYSDB_HOMEDIR, SYSDB_SHELL,
-                              NULL };
+                              "groups", NULL };
     attr_parse_test(exp_add, "+telephoneNumber, +streetAddress");
 
     /* Test removing some attributes to the defaults */
     const char *exp_rm[] = { SYSDB_NAME,
                              SYSDB_GIDNUM, SYSDB_GECOS,
-                             SYSDB_HOMEDIR,
+                             SYSDB_HOMEDIR, "groups",
                              NULL };
     attr_parse_test(exp_rm, "-"SYSDB_SHELL ",-"SYSDB_UIDNUM);
 
@@ -299,7 +291,7 @@ void test_attr_acl(void **state)
     const char *exp_add_rm[] = { "telephoneNumber",
                                  SYSDB_NAME, SYSDB_UIDNUM,
                                  SYSDB_GIDNUM, SYSDB_GECOS,
-                                 SYSDB_HOMEDIR,
+                                 SYSDB_HOMEDIR, "groups",
                                  NULL };
     attr_parse_test(exp_add_rm, "+telephoneNumber, -"SYSDB_SHELL);
 
@@ -307,7 +299,7 @@ void test_attr_acl(void **state)
     const char *exp_add_rm_override[] = { SYSDB_NAME, SYSDB_UIDNUM,
                                           SYSDB_GIDNUM, SYSDB_GECOS,
                                           SYSDB_HOMEDIR, SYSDB_SHELL,
-                                          NULL };
+                                          "groups", NULL };
     attr_parse_test(exp_add_rm_override,
                     "+telephoneNumber, -telephoneNumber, +telephoneNumber");
 
@@ -315,7 +307,7 @@ void test_attr_acl(void **state)
     const char *rm_all[] = { NULL };
     attr_parse_test(rm_all,  "-"SYSDB_NAME ", -"SYSDB_UIDNUM
                              ", -"SYSDB_GIDNUM ", -"SYSDB_GECOS
-                             ", -"SYSDB_HOMEDIR ", -"SYSDB_SHELL);
+                             ", -"SYSDB_HOMEDIR ", -"SYSDB_SHELL", -groups");
 
     /* Malformed list */
     attr_parse_test(NULL,  "missing_plus_or_minus");
@@ -362,64 +354,6 @@ void test_attr_allowed(void **state)
     assert_false(ifp_attr_allowed(NULL, "name"));
 }
 
-void test_path_escape_unescape(void **state)
-{
-    char *escaped;
-    char *raw;
-    TALLOC_CTX *mem_ctx;
-
-    assert_true(leak_check_setup());
-    mem_ctx = talloc_new(global_talloc_context);
-
-    escaped = ifp_bus_path_escape(mem_ctx, "noescape");
-    assert_non_null(escaped);
-    assert_string_equal(escaped, "noescape");
-    raw = ifp_bus_path_unescape(mem_ctx, escaped);
-    talloc_free(escaped);
-    assert_non_null(raw);
-    assert_string_equal(raw, "noescape");
-    talloc_free(raw);
-
-    escaped = ifp_bus_path_escape(mem_ctx, "redhat.com");
-    assert_non_null(escaped);
-    assert_string_equal(escaped, "redhat_2ecom"); /* dot is 0x2E in ASCII */
-    raw = ifp_bus_path_unescape(mem_ctx, escaped);
-    talloc_free(escaped);
-    assert_non_null(raw);
-    assert_string_equal(raw, "redhat.com");
-    talloc_free(raw);
-
-    escaped = ifp_bus_path_escape(mem_ctx, "path_with_underscore");
-    assert_non_null(escaped);
-    /* underscore is 0x5F in ascii */
-    assert_string_equal(escaped, "path_5fwith_5funderscore");
-    raw = ifp_bus_path_unescape(mem_ctx, escaped);
-    talloc_free(escaped);
-    assert_non_null(raw);
-    assert_string_equal(raw, "path_with_underscore");
-    talloc_free(raw);
-
-    /* empty string */
-    escaped = ifp_bus_path_escape(mem_ctx, "");
-    assert_non_null(escaped);
-    assert_string_equal(escaped, "_");
-    raw = ifp_bus_path_unescape(mem_ctx, escaped);
-    talloc_free(escaped);
-    assert_non_null(raw);
-    assert_string_equal(raw, "");
-    talloc_free(raw);
-
-    /* negative tests */
-    escaped = ifp_bus_path_escape(mem_ctx, NULL);
-    assert_null(escaped);
-    raw = ifp_bus_path_unescape(mem_ctx, "wrongpath_");
-    assert_null(raw);
-
-    assert_true(leak_check_teardown());
-}
-
-#define PATH_BASE "/some/path"
-
 struct ifp_test_req_ctx {
     struct ifp_req *ireq;
     struct sbus_request *sr;
@@ -464,32 +398,6 @@ static int ifp_test_req_teardown(void **state)
     return 0;
 }
 
-void test_reply_path(void **state)
-{
-    struct ifp_test_req_ctx *test_ctx = talloc_get_type_abort(*state,
-                                                struct ifp_test_req_ctx);
-    char *path;
-
-    /* Doesn't need escaping */
-    path = ifp_reply_objpath(test_ctx->ireq, PATH_BASE, "domname", NULL);
-    assert_non_null(path);
-    assert_string_equal(path, PATH_BASE"/domname");
-    talloc_free(path);
-}
-
-void test_reply_path_escape(void **state)
-{
-    struct ifp_test_req_ctx *test_ctx = talloc_get_type_abort(*state,
-                                                struct ifp_test_req_ctx);
-    char *path;
-
-    /* A dot needs escaping */
-    path = ifp_reply_objpath(test_ctx->ireq, PATH_BASE, "redhat.com", NULL);
-    assert_non_null(path);
-    assert_string_equal(path, PATH_BASE"/redhat_2ecom");
-    talloc_free(path);
-}
-
 int main(int argc, const char *argv[])
 {
     poptContext pc;
@@ -509,13 +417,6 @@ int main(int argc, const char *argv[])
         cmocka_unit_test(test_attr_acl),
         cmocka_unit_test(test_attr_acl_ex),
         cmocka_unit_test(test_attr_allowed),
-        cmocka_unit_test(test_path_escape_unescape),
-        cmocka_unit_test_setup_teardown(test_reply_path,
-                                        ifp_test_req_setup,
-                                        ifp_test_req_teardown),
-        cmocka_unit_test_setup_teardown(test_reply_path_escape,
-                                        ifp_test_req_setup,
-                                        ifp_test_req_teardown),
     };
 
     /* Set debug level to invalid value so we can deside if -d 0 was used. */
