@@ -28,10 +28,9 @@
 #include "tests/cmocka/common_mock.h"
 #include "tests/cmocka/common_mock_resp.h"
 
-#define TESTS_PATH "tests_responder"
+#define TESTS_PATH "tp_" BASE_FILE_STEM
 #define TEST_CONF_DB "test_responder_conf.ldb"
 #define TEST_DOM_NAME "responder_test"
-#define TEST_SYSDB_FILE "cache_"TEST_DOM_NAME".ldb"
 #define TEST_ID_PROVIDER "ldap"
 
 #define NAME "username"
@@ -162,8 +161,9 @@ void parse_inp_simple_done(struct tevent_req *req)
     char *domname = NULL;
 
     ret = sss_parse_inp_recv(req, parse_inp_ctx, &name, &domname);
-    parse_inp_ctx->tctx->done = true;
     assert_int_equal(ret, EOK);
+
+    test_ev_done(parse_inp_ctx->tctx, EOK);
     talloc_free(req);
 
     assert_string_equal(name, NAME);
@@ -240,8 +240,8 @@ void parse_inp_neg_done(struct tevent_req *req)
     char *domname = NULL;
 
     ret = sss_parse_inp_recv(req, parse_inp_ctx, &name, &domname);
-    parse_inp_ctx->tctx->done = true;
     assert_int_equal(ret, ERR_INPUT_PARSE);
+    test_ev_done(parse_inp_ctx->tctx, EOK);
     talloc_free(req);
 
     assert_null(name);
@@ -267,6 +267,39 @@ void parse_inp_call_neg(void **state)
     assert_int_equal(ret, EOK);
 }
 
+struct sss_nc_ctx {
+    struct parse_inp_test_ctx *pctx;
+};
+
+errno_t sss_ncache_reset_repopulate_permanent(struct resp_ctx *rctx,
+                                              struct sss_nc_ctx *dummy_ncache_ptr)
+{
+    test_ev_done(dummy_ncache_ptr->pctx->tctx, EOK);
+    return EOK;
+}
+
+void test_schedule_get_domains_task(void **state)
+{
+    struct parse_inp_test_ctx *parse_inp_ctx = talloc_get_type(*state,
+                                                   struct parse_inp_test_ctx);
+    errno_t ret;
+    struct sss_nc_ctx *dummy_ncache_ptr;
+
+    dummy_ncache_ptr = talloc(parse_inp_ctx, struct sss_nc_ctx);
+    assert_non_null(dummy_ncache_ptr);
+    dummy_ncache_ptr->pctx = parse_inp_ctx;
+
+    ret = schedule_get_domains_task(dummy_ncache_ptr,
+                                    parse_inp_ctx->rctx->ev,
+                                    parse_inp_ctx->rctx,
+                                    dummy_ncache_ptr);
+    assert_int_equal(ret, EOK);
+
+    ret = test_ev_loop(parse_inp_ctx->tctx);
+    assert_int_equal(ret, EOK);
+    talloc_free(dummy_ncache_ptr);
+}
+
 int main(int argc, const char *argv[])
 {
     int rv;
@@ -294,6 +327,9 @@ int main(int argc, const char *argv[])
         cmocka_unit_test_setup_teardown(parse_inp_call_neg,
                                         parse_inp_test_setup,
                                         parse_inp_test_teardown),
+        cmocka_unit_test_setup_teardown(test_schedule_get_domains_task,
+                                        parse_inp_test_setup,
+                                        parse_inp_test_teardown),
     };
 
     /* Set debug level to invalid value so we can deside if -d 0 was used. */
@@ -316,12 +352,12 @@ int main(int argc, const char *argv[])
     /* Even though normally the tests should clean up after themselves
      * they might not after a failed run. Remove the old db to be sure */
     tests_set_cwd();
-    test_dom_suite_cleanup(TESTS_PATH, TEST_CONF_DB, TEST_SYSDB_FILE);
+    test_dom_suite_cleanup(TESTS_PATH, TEST_CONF_DB, TEST_DOM_NAME);
     test_dom_suite_setup(TESTS_PATH);
 
     rv = cmocka_run_group_tests(tests, NULL, NULL);
     if (rv == 0 && !no_cleanup) {
-        test_dom_suite_cleanup(TESTS_PATH, TEST_CONF_DB, TEST_SYSDB_FILE);
+        test_dom_suite_cleanup(TESTS_PATH, TEST_CONF_DB, TEST_DOM_NAME);
     }
     return rv;
 }
