@@ -43,6 +43,9 @@
 #define TEST_OFFSET 1000000
 #define TEST_OFFSET_STR "1000000"
 
+const int TEST_2922_MIN_ID = 1842600000;
+const int TEST_2922_MAX_ID = 1842799999;
+
 struct test_ctx {
     TALLOC_CTX *mem_idmap;
     struct sss_idmap_ctx *idmap_ctx;
@@ -58,7 +61,7 @@ static void idmap_free(void *ptr, void *pvt)
     talloc_free(ptr);
 }
 
-void test_sss_idmap_setup(void **state)
+static int test_sss_idmap_setup(void **state)
 {
     struct test_ctx *test_ctx;
     enum idmap_error_code err;
@@ -78,10 +81,11 @@ void test_sss_idmap_setup(void **state)
     assert_int_equal(err, IDMAP_SUCCESS);
 
     *state = test_ctx;
+    return 0;
 }
 
-void setup_ranges(struct test_ctx *test_ctx, bool external_mapping,
-                  bool second_domain)
+static int setup_ranges(struct test_ctx *test_ctx, bool external_mapping,
+                        bool second_domain, bool sec_slices)
 {
     struct sss_idmap_range range;
     enum idmap_error_code err;
@@ -102,19 +106,63 @@ void setup_ranges(struct test_ctx *test_ctx, bool external_mapping,
         sid = TEST_DOM_SID;
     }
 
-    err = sss_idmap_add_domain_ex(test_ctx->idmap_ctx, name, sid, &range, NULL,
-                                  0, external_mapping);
+    if (sec_slices) {
+        err = sss_idmap_add_auto_domain_ex(test_ctx->idmap_ctx, name, sid,
+                                           &range, NULL, 0, external_mapping,
+                                           NULL, NULL);
+    } else {
+        err = sss_idmap_add_domain_ex(test_ctx->idmap_ctx, name, sid, &range,
+                                      NULL, 0, external_mapping);
+    }
     assert_int_equal(err, IDMAP_SUCCESS);
 
     range.min += TEST_OFFSET;
     range.max += TEST_OFFSET;
 
-    err = sss_idmap_add_domain_ex(test_ctx->idmap_ctx, name, sid, &range, NULL,
-                                  TEST_OFFSET, external_mapping);
+    if (sec_slices) {
+        err = sss_idmap_add_auto_domain_ex(test_ctx->idmap_ctx, name, sid,
+                                           &range, NULL, TEST_OFFSET,
+                                           external_mapping, NULL, NULL);
+    } else {
+        err = sss_idmap_add_domain_ex(test_ctx->idmap_ctx, name, sid, &range,
+                                      NULL, TEST_OFFSET, external_mapping);
+    }
     assert_int_equal(err, IDMAP_SUCCESS);
+    return 0;
 }
 
-void test_sss_idmap_setup_with_domains(void **state) {
+static int setup_ranges_2922(struct test_ctx *test_ctx)
+{
+    const int TEST_2922_DFL_SLIDE = 9212;
+    struct sss_idmap_range range;
+    enum idmap_error_code err;
+    const char *name;
+    const char *sid;
+    /* Pick a new slice. */
+    id_t slice_num = -1;
+
+    assert_non_null(test_ctx);
+
+    name = TEST_DOM_NAME;
+    sid = TEST_DOM_SID;
+
+    err = sss_idmap_calculate_range(test_ctx->idmap_ctx, sid, &slice_num,
+                                    &range);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    /* Range computation should be deterministic. Lets validate that.  */
+    assert_int_equal(range.min, TEST_2922_MIN_ID);
+    assert_int_equal(range.max, TEST_2922_MAX_ID);
+    assert_int_equal(slice_num, TEST_2922_DFL_SLIDE);
+
+    err = sss_idmap_add_domain_ex(test_ctx->idmap_ctx, name, sid, &range,
+                                  NULL, 0, false /* No external mapping */);
+    assert_int_equal(err, IDMAP_SUCCESS);
+
+    return 0;
+}
+
+static int test_sss_idmap_setup_with_domains(void **state)
+{
     struct test_ctx *test_ctx;
 
     test_sss_idmap_setup(state);
@@ -122,10 +170,12 @@ void test_sss_idmap_setup_with_domains(void **state) {
     test_ctx = talloc_get_type(*state, struct test_ctx);
     assert_non_null(test_ctx);
 
-    setup_ranges(test_ctx, false, false);
+    setup_ranges(test_ctx, false, false, false);
+    return 0;
 }
 
-void test_sss_idmap_setup_with_external_mappings(void **state) {
+static int test_sss_idmap_setup_with_domains_2922(void **state)
+{
     struct test_ctx *test_ctx;
 
     test_sss_idmap_setup(state);
@@ -133,10 +183,12 @@ void test_sss_idmap_setup_with_external_mappings(void **state) {
     test_ctx = talloc_get_type(*state, struct test_ctx);
     assert_non_null(test_ctx);
 
-    setup_ranges(test_ctx, true, false);
+    setup_ranges_2922(test_ctx);
+    return 0;
 }
 
-void test_sss_idmap_setup_with_both(void **state) {
+static int test_sss_idmap_setup_with_domains_sec_slices(void **state)
+{
     struct test_ctx *test_ctx;
 
     test_sss_idmap_setup(state);
@@ -144,11 +196,38 @@ void test_sss_idmap_setup_with_both(void **state) {
     test_ctx = talloc_get_type(*state, struct test_ctx);
     assert_non_null(test_ctx);
 
-    setup_ranges(test_ctx, false, false);
-    setup_ranges(test_ctx, true, true);
+    setup_ranges(test_ctx, false, false, true);
+    return 0;
 }
 
-void test_sss_idmap_teardown(void **state)
+static int test_sss_idmap_setup_with_external_mappings(void **state)
+{
+    struct test_ctx *test_ctx;
+
+    test_sss_idmap_setup(state);
+
+    test_ctx = talloc_get_type(*state, struct test_ctx);
+    assert_non_null(test_ctx);
+
+    setup_ranges(test_ctx, true, false, false);
+    return 0;
+}
+
+static int test_sss_idmap_setup_with_both(void **state)
+{
+    struct test_ctx *test_ctx;
+
+    test_sss_idmap_setup(state);
+
+    test_ctx = talloc_get_type(*state, struct test_ctx);
+    assert_non_null(test_ctx);
+
+    setup_ranges(test_ctx, false, false, false);
+    setup_ranges(test_ctx, true, true, false);
+    return 0;
+}
+
+static int test_sss_idmap_teardown(void **state)
 {
     struct test_ctx *test_ctx;
 
@@ -161,6 +240,7 @@ void test_sss_idmap_teardown(void **state)
     assert_true(check_leaks_pop(test_ctx) == true);
     talloc_free(test_ctx);
     assert_true(leak_check_teardown());
+    return 0;
 }
 
 void test_add_domain(void **state)
@@ -268,6 +348,100 @@ void test_map_id(void **state)
     sss_idmap_free_sid(test_ctx->idmap_ctx, sid);
 }
 
+/* https://fedorahosted.org/sssd/ticket/2922 */
+/* ID mapping - bug in computing max id for slice range */
+void test_map_id_2922(void **state)
+{
+    const char* TEST_2922_FIRST_SID = TEST_DOM_SID"-0";
+    /* Last SID = first SID + (default) rangesize -1 */
+    const char* TEST_2922_LAST_SID = TEST_DOM_SID"-199999";
+    /* Last SID = first SID + rangesize */
+    const char* TEST_2922_LAST_SID_PLUS_ONE = TEST_DOM_SID"-200000";
+    struct test_ctx *test_ctx;
+    enum idmap_error_code err;
+    uint32_t id;
+    char *sid = NULL;
+
+    test_ctx = talloc_get_type(*state, struct test_ctx);
+
+    assert_non_null(test_ctx);
+
+    /* Min UNIX ID to SID */
+    err = sss_idmap_unix_to_sid(test_ctx->idmap_ctx, TEST_2922_MIN_ID, &sid);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_string_equal(sid, TEST_2922_FIRST_SID);
+    sss_idmap_free_sid(test_ctx->idmap_ctx, sid);
+
+    /* First SID to UNIX ID */
+    err = sss_idmap_sid_to_unix(test_ctx->idmap_ctx, TEST_2922_FIRST_SID, &id);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_int_equal(id, TEST_2922_MIN_ID);
+
+    /* Max UNIX ID to SID */
+    err = sss_idmap_unix_to_sid(test_ctx->idmap_ctx, TEST_2922_MAX_ID, &sid);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_string_equal(sid, TEST_2922_LAST_SID);
+    sss_idmap_free_sid(test_ctx->idmap_ctx, sid);
+
+    /* Last SID to UNIX ID */
+    err = sss_idmap_sid_to_unix(test_ctx->idmap_ctx, TEST_2922_LAST_SID, &id);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_int_equal(id, TEST_2922_MAX_ID);
+
+    /* Max UNIX ID + 1 to SID */
+    err = sss_idmap_unix_to_sid(test_ctx->idmap_ctx, TEST_2922_MAX_ID + 1,
+                                &sid);
+    assert_int_equal(err, IDMAP_NO_DOMAIN);
+
+    /* Last SID + 1 to UNIX ID */
+    err = sss_idmap_sid_to_unix(test_ctx->idmap_ctx,
+                                TEST_2922_LAST_SID_PLUS_ONE, &id);
+    /* Auto adding new ranges is disable in this test.  */
+    assert_int_equal(err, IDMAP_NO_RANGE);
+}
+
+void test_map_id_sec_slices(void **state)
+{
+    struct test_ctx *test_ctx;
+    enum idmap_error_code err;
+    uint32_t id;
+    char *sid = NULL;
+
+    test_ctx = talloc_get_type(*state, struct test_ctx);
+
+    assert_non_null(test_ctx);
+
+    err = sss_idmap_sid_to_unix(test_ctx->idmap_ctx, TEST_DOM_SID"1-1", &id);
+    assert_int_equal(err, IDMAP_NO_DOMAIN);
+
+    err = sss_idmap_sid_to_unix(test_ctx->idmap_ctx, TEST_DOM_SID"-4000000",
+                                &id);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_int_equal(id, 575600000);
+
+    err = sss_idmap_unix_to_sid(test_ctx->idmap_ctx, TEST_OFFSET - 1, &sid);
+    assert_int_equal(err, IDMAP_NO_DOMAIN);
+
+    err = sss_idmap_sid_to_unix(test_ctx->idmap_ctx, TEST_DOM_SID"-0", &id);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_int_equal(id, TEST_RANGE_MIN);
+
+    err = sss_idmap_unix_to_sid(test_ctx->idmap_ctx, id, &sid);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_string_equal(sid, TEST_DOM_SID"-0");
+    sss_idmap_free_sid(test_ctx->idmap_ctx, sid);
+
+    err = sss_idmap_sid_to_unix(test_ctx->idmap_ctx,
+                                TEST_DOM_SID"-"TEST_OFFSET_STR, &id);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_int_equal(id, TEST_RANGE_MIN+TEST_OFFSET);
+
+    err = sss_idmap_unix_to_sid(test_ctx->idmap_ctx, id, &sid);
+    assert_int_equal(err, IDMAP_SUCCESS);
+    assert_string_equal(sid, TEST_DOM_SID"-"TEST_OFFSET_STR);
+    sss_idmap_free_sid(test_ctx->idmap_ctx, sid);
+}
+
 void test_map_id_external(void **state)
 {
     struct test_ctx *test_ctx;
@@ -510,28 +684,35 @@ int main(int argc, const char *argv[])
         POPT_TABLEEND
     };
 
-    const UnitTest tests[] = {
-        unit_test_setup_teardown(test_add_domain,
-                                 test_sss_idmap_setup, test_sss_idmap_teardown),
-        unit_test_setup_teardown(test_map_id,
-                                 test_sss_idmap_setup_with_domains,
-                                 test_sss_idmap_teardown),
-        unit_test_setup_teardown(test_map_id_external,
-                                 test_sss_idmap_setup_with_external_mappings,
-                                 test_sss_idmap_teardown),
-        unit_test_setup_teardown(test_check_sid_id,
-                                 test_sss_idmap_setup_with_domains,
-                                 test_sss_idmap_teardown),
-        unit_test_setup_teardown(test_check_sid_id,
-                                 test_sss_idmap_setup_with_external_mappings,
-                                 test_sss_idmap_teardown),
-        unit_test_setup_teardown(test_has_algorithmic,
-                                 test_sss_idmap_setup_with_both,
-                                 test_sss_idmap_teardown),
-        unit_test_setup_teardown(test_has_algorithmic_by_name,
-                                 test_sss_idmap_setup_with_both,
-                                 test_sss_idmap_teardown),
-        unit_test(test_sss_idmap_check_collision_ex),
+    const struct CMUnitTest tests[] = {
+        cmocka_unit_test_setup_teardown(test_add_domain,
+                                        test_sss_idmap_setup,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test_setup_teardown(test_map_id,
+                                        test_sss_idmap_setup_with_domains,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test_setup_teardown(test_map_id_2922,
+                                        test_sss_idmap_setup_with_domains_2922,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test_setup_teardown(test_map_id_sec_slices,
+                                        test_sss_idmap_setup_with_domains_sec_slices,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test_setup_teardown(test_map_id_external,
+                                        test_sss_idmap_setup_with_external_mappings,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test_setup_teardown(test_check_sid_id,
+                                        test_sss_idmap_setup_with_domains,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test_setup_teardown(test_check_sid_id,
+                                        test_sss_idmap_setup_with_external_mappings,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test_setup_teardown(test_has_algorithmic,
+                                        test_sss_idmap_setup_with_both,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test_setup_teardown(test_has_algorithmic_by_name,
+                                        test_sss_idmap_setup_with_both,
+                                        test_sss_idmap_teardown),
+        cmocka_unit_test(test_sss_idmap_check_collision_ex),
     };
 
     /* Set debug level to invalid value so we can deside if -d 0 was used. */
@@ -549,9 +730,9 @@ int main(int argc, const char *argv[])
     }
     poptFreeContext(pc);
 
-    DEBUG_INIT(debug_level);
+    DEBUG_CLI_INIT(debug_level);
 
     tests_set_cwd();
 
-    return run_tests(tests);
+    return cmocka_run_group_tests(tests, NULL, NULL);
 }
