@@ -50,6 +50,7 @@ struct test_ctx {
 struct task {
     struct test_ctx *test_ctx;
     const char *location;
+    struct fo_service *service;
     int recv;
     int port;
     int new_server_status;
@@ -147,6 +148,7 @@ test_resolve_service_callback(struct tevent_req *req)
     int port;
     struct task *task;
     struct fo_server *server = NULL;
+    struct fo_server *active_server = NULL;
     struct resolv_hostent *he;
     int i;
 
@@ -154,7 +156,7 @@ test_resolve_service_callback(struct tevent_req *req)
 
     task->test_ctx->tasks--;
 
-    recv_status = fo_resolve_service_recv(req, &server);
+    recv_status = fo_resolve_service_recv(req, req, &server);
     talloc_free(req);
     fail_if(recv_status != task->recv, "%s: Expected return of %d, got %d",
             task->location, task->recv, recv_status);
@@ -172,7 +174,7 @@ test_resolve_service_callback(struct tevent_req *req)
 
     if (fo_get_server_name(server) != NULL) {
         he = fo_get_server_hostent(server);
-        fail_if(he == NULL, "%s: fo_get_server_hostent() returned NULL");
+        fail_if(he == NULL, "fo_get_server_hostent() returned NULL");
         for (i = 0; he->addr_list[i]; i++) {
             char buf[256];
 
@@ -181,6 +183,13 @@ test_resolve_service_callback(struct tevent_req *req)
         }
     }
 
+    if (task->new_port_status == PORT_WORKING
+            && task->new_server_status == SERVER_WORKING) {
+        active_server = fo_get_active_server(task->service);
+        fail_if(active_server == NULL, "Missing active server");
+        fail_if(server != active_server, "Current server is not active server");
+    }
+
 }
 
 #define get_request(a, b, c, d, e, f) \
@@ -203,6 +212,7 @@ _get_request(struct test_ctx *test_ctx, struct fo_service *service,
     task->new_port_status = new_port_status;
     task->new_server_status = new_server_status;
     task->location = location;
+    task->service = service;
     test_ctx->tasks++;
 
     req = fo_resolve_service_send(test_ctx, test_ctx->ev,
@@ -242,6 +252,7 @@ START_TEST(test_fo_resolve_service)
 
     /* Make requests. */
     get_request(ctx, service[0], EOK, 20, PORT_WORKING, -1);
+    get_request(ctx, service[0], EOK, 20, PORT_WORKING, SERVER_WORKING);
     get_request(ctx, service[0], EOK, 20, -1, SERVER_NOT_WORKING);
     get_request(ctx, service[0], EOK, 80, PORT_WORKING, -1);
     get_request(ctx, service[0], EOK, 80, PORT_NOT_WORKING, -1);
@@ -311,7 +322,7 @@ main(int argc, const char *argv[])
     }
     poptFreeContext(pc);
 
-    DEBUG_INIT(debug_level);
+    DEBUG_CLI_INIT(debug_level);
 
     tests_set_cwd();
 
