@@ -164,16 +164,18 @@ static void get_test_srchost(TALLOC_CTX *mem_ctx,
     new_srchost = talloc_zero(mem_ctx, struct hbac_request_element);
     fail_if (new_srchost == NULL);
 
-    new_srchost->name = talloc_strdup(new_srchost, "client.example.com");
+    new_srchost->name = talloc_strdup(new_srchost, HBAC_TEST_SRCHOST);
     fail_if(new_srchost->name == NULL);
 
     new_srchost->groups = talloc_array(new_srchost, const char *, 3);
     fail_if(new_srchost->groups == NULL);
 
-    new_srchost->groups[0] = talloc_strdup(new_srchost->groups, "site_hosts");
+    new_srchost->groups[0] = talloc_strdup(new_srchost->groups,
+                                           HBAC_TEST_SRCHOSTGROUP1);
     fail_if(new_srchost->groups[0] == NULL);
 
-    new_srchost->groups[1] = talloc_strdup(new_srchost->groups, "corp_hosts");
+    new_srchost->groups[1] = talloc_strdup(new_srchost->groups,
+                                           HBAC_TEST_SRCHOSTGROUP2);
     fail_if(new_srchost->groups[1] == NULL);
 
     new_srchost->groups[2] = NULL;
@@ -187,7 +189,7 @@ START_TEST(ipa_hbac_test_allow_all)
     TALLOC_CTX *test_ctx;
     struct hbac_rule **rules;
     struct hbac_eval_req *eval_req;
-    struct hbac_info *info;
+    struct hbac_info *info = NULL;
     bool is_valid;
     uint32_t missing_attrs;
 
@@ -223,7 +225,8 @@ START_TEST(ipa_hbac_test_allow_all)
                 hbac_result_string(HBAC_EVAL_ALLOW),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
-
+    hbac_free_info(info);
+    info = NULL;
     talloc_free(test_ctx);
 }
 END_TEST
@@ -234,7 +237,7 @@ START_TEST(ipa_hbac_test_allow_user)
     TALLOC_CTX *test_ctx;
     struct hbac_rule **rules;
     struct hbac_eval_req *eval_req;
-    struct hbac_info *info;
+    struct hbac_info *info = NULL;
     bool is_valid;
     uint32_t missing_attrs;
 
@@ -280,6 +283,8 @@ START_TEST(ipa_hbac_test_allow_user)
                 hbac_result_string(HBAC_EVAL_ALLOW),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     /* Negative test */
     rules[0]->users->names[0] = HBAC_TEST_INVALID_USER;
@@ -297,6 +302,8 @@ START_TEST(ipa_hbac_test_allow_user)
                 hbac_result_string(HBAC_EVAL_DENY),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     talloc_free(test_ctx);
 }
@@ -308,7 +315,7 @@ START_TEST(ipa_hbac_test_allow_utf8)
     TALLOC_CTX *test_ctx;
     struct hbac_rule **rules;
     struct hbac_eval_req *eval_req;
-    struct hbac_info *info;
+    struct hbac_info *info = NULL;
     bool is_valid;
     uint32_t missing_attrs;
 
@@ -377,6 +384,9 @@ START_TEST(ipa_hbac_test_allow_utf8)
                 hbac_result_string(HBAC_EVAL_ALLOW),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
+
 
     /* Negative test - a different letter */
     rules[0]->users->names[0] = (const char *) &user_utf8_lowcase_neg;
@@ -389,6 +399,8 @@ START_TEST(ipa_hbac_test_allow_utf8)
                 hbac_result_string(HBAC_EVAL_DENY),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     /* Negative test - Turkish dotless i. We cannot know that capital I
      * casefolds into dotless i unless we know the language is Turkish */
@@ -408,6 +420,8 @@ START_TEST(ipa_hbac_test_allow_utf8)
                 hbac_result_string(HBAC_EVAL_DENY),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     talloc_free(test_ctx);
 }
@@ -419,7 +433,7 @@ START_TEST(ipa_hbac_test_allow_group)
     TALLOC_CTX *test_ctx;
     struct hbac_rule **rules;
     struct hbac_eval_req *eval_req;
-    struct hbac_info *info;
+    struct hbac_info *info = NULL;
     bool is_valid;
     uint32_t missing_attrs;
 
@@ -466,6 +480,8 @@ START_TEST(ipa_hbac_test_allow_group)
                 hbac_result_string(HBAC_EVAL_ALLOW),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     /* Negative test */
     rules[0]->users->groups[0] = HBAC_TEST_INVALID_GROUP;
@@ -483,6 +499,8 @@ START_TEST(ipa_hbac_test_allow_group)
                 hbac_result_string(HBAC_EVAL_DENY),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     talloc_free(test_ctx);
 }
@@ -494,7 +512,7 @@ START_TEST(ipa_hbac_test_allow_svc)
     TALLOC_CTX *test_ctx;
     struct hbac_rule **rules;
     struct hbac_eval_req *eval_req;
-    struct hbac_info *info;
+    struct hbac_info *info = NULL;
     bool is_valid;
     uint32_t missing_attrs;
 
@@ -540,6 +558,8 @@ START_TEST(ipa_hbac_test_allow_svc)
                 hbac_result_string(HBAC_EVAL_ALLOW),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     /* Negative test */
     rules[0]->services->names[0] = HBAC_TEST_INVALID_SERVICE;
@@ -557,6 +577,8 @@ START_TEST(ipa_hbac_test_allow_svc)
                 hbac_result_string(HBAC_EVAL_DENY),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     talloc_free(test_ctx);
 }
@@ -568,7 +590,7 @@ START_TEST(ipa_hbac_test_allow_svcgroup)
     TALLOC_CTX *test_ctx;
     struct hbac_rule **rules;
     struct hbac_eval_req *eval_req;
-    struct hbac_info *info;
+    struct hbac_info *info = NULL;
     bool is_valid;
     uint32_t missing_attrs;
 
@@ -615,6 +637,8 @@ START_TEST(ipa_hbac_test_allow_svcgroup)
                 hbac_result_string(HBAC_EVAL_ALLOW),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     /* Negative test */
     rules[0]->services->groups[0] = HBAC_TEST_INVALID_SERVICEGROUP;
@@ -632,6 +656,8 @@ START_TEST(ipa_hbac_test_allow_svcgroup)
                 hbac_result_string(HBAC_EVAL_DENY),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     talloc_free(test_ctx);
 }
@@ -643,7 +669,7 @@ START_TEST(ipa_hbac_test_allow_srchost)
     TALLOC_CTX *test_ctx;
     struct hbac_rule **rules;
     struct hbac_eval_req *eval_req;
-    struct hbac_info *info;
+    struct hbac_info *info = NULL;
     bool is_valid;
     uint32_t missing_attrs;
 
@@ -689,6 +715,8 @@ START_TEST(ipa_hbac_test_allow_srchost)
                 hbac_result_string(HBAC_EVAL_ALLOW),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     /* Negative test */
     rules[0]->srchosts->names[0] = HBAC_TEST_INVALID_SRCHOST;
@@ -702,10 +730,12 @@ START_TEST(ipa_hbac_test_allow_srchost)
     result = hbac_evaluate(rules, eval_req, &info);
     fail_unless(result == HBAC_EVAL_DENY,
                 "Expected [%s], got [%s]; "
-                "Error: [%s](%s)",
+                "Error: [%s]",
                 hbac_result_string(HBAC_EVAL_DENY),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     talloc_free(test_ctx);
 }
@@ -717,7 +747,7 @@ START_TEST(ipa_hbac_test_allow_srchostgroup)
     TALLOC_CTX *test_ctx;
     struct hbac_rule **rules;
     struct hbac_eval_req *eval_req;
-    struct hbac_info *info;
+    struct hbac_info *info = NULL;
     bool is_valid;
     uint32_t missing_attrs;
 
@@ -764,6 +794,8 @@ START_TEST(ipa_hbac_test_allow_srchostgroup)
                 hbac_result_string(HBAC_EVAL_ALLOW),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     /* Negative test */
     rules[0]->srchosts->groups[0] = HBAC_TEST_INVALID_SRCHOSTGROUP;
@@ -781,6 +813,8 @@ START_TEST(ipa_hbac_test_allow_srchostgroup)
                 hbac_result_string(HBAC_EVAL_DENY),
                 hbac_result_string(result),
                 info ? hbac_error_string(info->code):"Unknown");
+    hbac_free_info(info);
+    info = NULL;
 
     talloc_free(test_ctx);
 }
