@@ -32,7 +32,7 @@
 #include "util/sss_utf8.h"
 #include "tests/common.h"
 
-#define TESTS_PATH "tests_krb5_utils"
+#define TESTS_PATH "tp_" BASE_FILE_STEM
 
 #define BASE "/abc/def"
 #define FILENAME "ghi"
@@ -634,7 +634,7 @@ START_TEST(test_parse_krb5_map_user)
                     name_to_primary[0].krb_primary == NULL);
         talloc_free(name_to_primary);
 
-        check_leaks_pop(mem_ctx);
+        fail_unless(check_leaks_pop(mem_ctx));
     }
     /* valid input */
     {
@@ -653,7 +653,7 @@ START_TEST(test_parse_krb5_map_user)
         compare_map_id_name_to_krb_primary(name_to_primary,  expected,
                                          sizeof(expected)/sizeof(const char*)/2);
         talloc_free(name_to_primary);
-        check_leaks_pop(mem_ctx);
+        fail_unless(check_leaks_pop(mem_ctx));
     }
     /* invalid input */
     {
@@ -677,13 +677,26 @@ START_TEST(test_parse_krb5_map_user)
         ret = parse_krb5_map_user(mem_ctx, "joe:j:user", &name_to_primary);
         fail_unless(ret == EINVAL);
 
-        check_leaks_pop(mem_ctx);
+        fail_unless(check_leaks_pop(mem_ctx));
     }
 
     talloc_free(mem_ctx);
 }
 END_TEST
 
+START_TEST(test_sss_krb5_realm_has_proxy)
+{
+    fail_unless(sss_krb5_realm_has_proxy(NULL) == false);
+
+    setenv("KRB5_CONFIG", "/dev/null", 1);
+    fail_unless(sss_krb5_realm_has_proxy("REALM") == false);
+
+    setenv("KRB5_CONFIG", ABS_SRC_DIR"/src/tests/krb5_proxy_check_test_data.conf", 1);
+    fail_unless(sss_krb5_realm_has_proxy("REALM") == false);
+    fail_unless(sss_krb5_realm_has_proxy("REALM_PROXY") == true);
+}
+END_TEST
+
 Suite *krb5_utils_suite (void)
 {
     Suite *s = suite_create ("krb5_utils");
@@ -723,6 +736,7 @@ Suite *krb5_utils_suite (void)
     TCase *tc_krb5_helpers = tcase_create("Helper functions");
     tcase_add_test(tc_krb5_helpers, test_compare_principal_realm);
     tcase_add_test(tc_krb5_helpers, test_parse_krb5_map_user);
+    tcase_add_test(tc_krb5_helpers, test_sss_krb5_realm_has_proxy);
     suite_add_tcase(s, tc_krb5_helpers);
 
     return s;
