@@ -5,12 +5,12 @@ import unittest
 import sys
 import os
 import copy
-import sys
+import tempfile
+
+BUILD_DIR = os.getenv('builddir') or "."
+TEST_DIR = os.getenv('SSS_TEST_DIR') or "."
+MODPATH = tempfile.mkdtemp(prefix="tp_pyhbac_", dir=TEST_DIR)
 
-srcdir = os.getenv('builddir')
-if not srcdir:
-    srcdir = "."
-MODPATH = srcdir + "/.libs" #FIXME - is there a way to get this from libtool?
 
 if sys.version_info[0] > 2:
     unicode = str
@@ -41,6 +41,16 @@ class PyHbacImport(unittest.TestCase):
     def testImport(self):
         " Import the module and assert it comes from tree "
         try:
+            dest_module_path = MODPATH + "/pyhbac.so"
+
+            if sys.version_info[0] > 2:
+                src_module_path = BUILD_DIR + "/.libs/_py3hbac.so"
+            else:
+                src_module_path = BUILD_DIR + "/.libs/_py2hbac.so"
+
+            src_module_path = os.path.abspath(src_module_path)
+            os.symlink(src_module_path, dest_module_path)
+
             import pyhbac
         except ImportError as e:
             print("Could not load the pyhbac module. Please check if it is compiled", file=sys.stderr)
@@ -121,7 +131,6 @@ class PyHbacRuleElementTest(unittest.TestCase):
         el.names = ['foo']
         el.groups = ['bar, baz']
         self.assertEquals(el.__repr__(), u'<category 1 names [foo] groups [bar, baz]>')
-        
 
 class PyHbacRuleTest(unittest.TestCase):
     def testRuleGetSetName(self):
@@ -441,6 +450,11 @@ class PyHbacRequestTest(unittest.TestCase):
         self.assertRaises(TypeError, req.evaluate, (allow_rule, None))
 
 class PyHbacModuleTest(unittest.TestCase):
+    @classmethod
+    def tearDownClass(cls):
+        os.unlink(MODPATH + "/pyhbac.so")
+        os.rmdir(MODPATH)
+
     def testHasResultTypes(self):
         assert hasattr(pyhbac, "HBAC_EVAL_ALLOW")
         assert hasattr(pyhbac, "HBAC_EVAL_DENY")
