@@ -22,12 +22,12 @@ from __future__ import print_function
 import unittest
 import sys
 import os
-import copy
+import tempfile
+
+BUILD_DIR = os.getenv('builddir') or "."
+TEST_DIR = os.getenv('SSS_TEST_DIR') or "."
+MODPATH = tempfile.mkdtemp(prefix="tp_pysss_murmur_", dir=TEST_DIR)
 
-srcdir = os.getenv('builddir')
-if not srcdir:
-    srcdir = "."
-MODPATH = srcdir + "/.libs" #FIXME - is there a way to get this from libtool?
 
 def compat_assertItemsEqual(this, expected_seq, actual_seq, msg=None):
     return this.assertEqual(sorted(expected_seq), sorted(actual_seq))
@@ -57,6 +57,16 @@ class PySssMurmurImport(unittest.TestCase):
     def testImport(self):
         " Import the module and assert it comes from tree "
         try:
+            dest_module_path = MODPATH + "/pysss_murmur.so"
+
+            if sys.version_info[0] > 2:
+                src_module_path = BUILD_DIR + "/.libs/_py3sss_murmur.so"
+            else:
+                src_module_path = BUILD_DIR + "/.libs/_py2sss_murmur.so"
+
+            src_module_path = os.path.abspath(src_module_path)
+            os.symlink(src_module_path, dest_module_path)
+
             import pysss_murmur
         except ImportError as e:
             print("Could not load the pysss_murmur module. Please check if it is compiled", file=sys.stderr)
@@ -64,6 +74,11 @@ class PySssMurmurImport(unittest.TestCase):
         self.assertEqual(pysss_murmur.__file__, MODPATH + "/pysss_murmur.so")
 
 class PySssMurmurTest(unittest.TestCase):
+    @classmethod
+    def tearDownClass(cls):
+        os.unlink(MODPATH + "/pysss_murmur.so")
+        os.rmdir(MODPATH)
+
     def testExpectedHash(self):
         hash = pysss_murmur.murmurhash3("S-1-5-21-2153326666-2176343378-3404031434", 41, 0xdeadbeef)
         self.assertEqual(hash, 93103853)
