@@ -112,7 +112,7 @@ START_TEST(check_allowed_uids_test)
     int ret;
     size_t c;
 
-    for (c = 0; uid_check_data[c].exp_ret == -1; c++) {
+    for (c = 0; uid_check_data[c].exp_ret != -1; c++) {
         ret = check_allowed_uids(uid_check_data[c].uid,
                                  uid_check_data[c].allowed_uids_count,
                                  uid_check_data[c].allowed_uids);
@@ -163,7 +163,7 @@ int main(int argc, const char *argv[])
     }
     poptFreeContext(pc);
 
-    DEBUG_INIT(debug_level);
+    DEBUG_CLI_INIT(debug_level);
     tests_set_cwd();
 
     Suite *s = responder_test_suite();
