@@ -43,33 +43,38 @@
 #define PILOT_IFACE "test.Pilot"
 #define PILOT_BLINK "Blink"
 #define PILOT_EAT "Eat"
+#define PILOT_CRASH "Crash"
 
 #define PILOT_IFACE_INTROSPECT \
         "<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\"\n" \
-        "\"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n"                \
-        "<node>\n"                                                                           \
-        "  <interface name=\"test.Pilot\">\n"                                                \
-        "    <method name=\"Blink\">\n"                                                      \
-        "    </method>\n"                                                                    \
-        "    <method name=\"Eat\">\n"                                                        \
-        "    </method>\n"                                                                    \
-        "  </interface>\n"                                                                   \
-        " <interface name=\"org.freedesktop.DBus.Introspectable\">\n"                        \
-        "   <method name=\"Introspect\">\n"                                                  \
-        "     <arg name=\"data\" type=\"s\" direction=\"out\"/>\n"                           \
-        "   </method>\n"                                                                     \
-        " </interface>\n"                                                                    \
-        " <interface name=\"org.freedesktop.DBus.Properties\">\n"                            \
-        "   <method name=\"Get\">\n"                                                         \
-        "     <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n"                       \
-        "     <arg name=\"property\" direction=\"in\" type=\"s\"/>\n"                        \
-        "     <arg name=\"value\" direction=\"out\" type=\"v\"/>\n"                          \
-        "   </method>\n"                                                                     \
-        "   <method name=\"GetAll\">\n"                                                      \
-        "     <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n"                       \
-        "     <arg name=\"properties\" direction=\"out\" type=\"a{sv}\"/>\n"                 \
-        "   </method>\n"                                                                     \
-        " </interface>\n"                                                                    \
+        " \"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n" \
+        "<node name=\"/test/leela\">\n" \
+        "  <interface name=\"org.freedesktop.DBus.Introspectable\">\n" \
+        "    <method name=\"Introspect\">\n" \
+        "      <arg type=\"s\" name=\"data\" direction=\"out\" />\n" \
+        "    </method>\n" \
+        "  </interface>\n" \
+        "  <interface name=\"org.freedesktop.DBus.Properties\">\n" \
+        "    <method name=\"Get\">\n" \
+        "      <arg type=\"s\" name=\"interface_name\" direction=\"in\" />\n" \
+        "      <arg type=\"s\" name=\"property_name\" direction=\"in\" />\n" \
+        "      <arg type=\"v\" name=\"value\" direction=\"out\" />\n" \
+        "    </method>\n" \
+        "    <method name=\"Set\">\n" \
+        "      <arg type=\"s\" name=\"interface_name\" direction=\"in\" />\n" \
+        "      <arg type=\"s\" name=\"property_name\" direction=\"in\" />\n" \
+        "      <arg type=\"v\" name=\"value\" direction=\"in\" />\n" \
+        "    </method>\n" \
+        "    <method name=\"GetAll\">\n" \
+        "      <arg type=\"s\" name=\"interface_name\" direction=\"in\" />\n" \
+        "      <arg type=\"a{sv}\" name=\"props\" direction=\"out\" />\n" \
+        "    </method>\n" \
+        "  </interface>\n" \
+        "  <interface name=\"test.Pilot\">\n" \
+        "    <method name=\"Blink\" />\n" \
+        "    <method name=\"Eat\" />\n" \
+        "    <method name=\"Crash\" />\n" \
+        "  </interface>\n" \
         "</node>\n"
 
 /* our vtable */
@@ -77,6 +82,7 @@ struct pilot_vtable {
     struct sbus_vtable vtable;
     sbus_msg_handler_fn Blink;
     sbus_msg_handler_fn Eat;
+    sbus_msg_handler_fn Crash;
 };
 
 const struct sbus_method_meta pilot_methods[] = {
@@ -94,6 +100,13 @@ const struct sbus_method_meta pilot_methods[] = {
         offsetof(struct pilot_vtable, Eat),
         NULL
     },
+    {
+        PILOT_CRASH, /* method name */
+        NULL, /* in args: manually parsed */
+        NULL, /* out args: manually parsed */
+        offsetof(struct pilot_vtable, Crash),
+        NULL
+    },
     { NULL, }
 };
 
@@ -108,20 +121,18 @@ const struct sbus_interface_meta pilot_meta = {
 static int blink_handler(struct sbus_request *req, void *data)
 {
     DBusError error = DBUS_ERROR_INIT;
-    const char *path;
     dbus_int32_t duration = 0;
     dbus_bool_t crashed;
 
     ck_assert(req->intf->vtable->meta == &pilot_meta);
     ck_assert(data != NULL);
-    ck_assert(data == req->intf->instance_data);
+    ck_assert(data == req->intf->handler_data);
 
-    path = dbus_message_get_path(req->message);
-    ck_assert_str_eq(req->intf->path, path);
+    ck_assert_str_eq(req->intf->path, req->path);
 
-    if (strcmp(path, "/test/fry") == 0) {
+    if (strcmp(req->path, "/test/fry") == 0) {
         ck_assert_str_eq(data, "Don't crash");
-    } else if (strcmp(path, "/test/leela") == 0) {
+    } else if (strcmp(req->path, "/test/leela") == 0) {
         ck_assert_str_eq(data, "Crash into the billboard");
     } else {
         ck_abort();
@@ -168,27 +179,34 @@ static int eat_handler(struct sbus_request *req, void *data)
     return sbus_request_return_and_finish(req, DBUS_TYPE_INVALID);
 }
 
+static int crash_handler(struct sbus_request *req, void *data)
+{
+    /* Pilot crashes by returning a malformed UTF-8 string */
+    const char *invalid = "ad\351la\357d";
+
+    return sbus_request_return_and_finish(req,
+                                          DBUS_TYPE_STRING, &invalid,
+                                          DBUS_TYPE_INVALID);
+}
+
 struct pilot_vtable pilot_impl = {
     { &pilot_meta, 0 },
     .Blink = blink_handler,
     .Eat = eat_handler,
+    .Crash = crash_handler,
 };
 
 static int pilot_test_server_init(struct sbus_connection *server, void *unused)
 {
     int ret;
 
-    ret = sbus_conn_add_interface(server,
-                                  sbus_new_interface(server, "/test/leela",
-                                                     &pilot_impl.vtable,
-                                                     "Crash into the billboard"));
+    ret = sbus_conn_register_iface(server, &pilot_impl.vtable, "/test/leela",
+                                   "Crash into the billboard");
     ck_assert_int_eq(ret, EOK);
 
 
-    ret = sbus_conn_add_interface(server,
-                                  sbus_new_interface(server, "/test/fry",
-                                                     &pilot_impl.vtable,
-                                                     "Don't crash"));
+    ret = sbus_conn_register_iface(server, &pilot_impl.vtable, "/test/fry",
+                                   "Don't crash");
     ck_assert_int_eq(ret, EOK);
 
     return EOK;
@@ -307,6 +325,33 @@ START_TEST(test_request_parse_bad_args)
 }
 END_TEST
 
+START_TEST(test_request_dontcrash)
+{
+#ifdef HAVE_DBUSBASICVALUE
+    TALLOC_CTX *ctx;
+    DBusConnection *client;
+    DBusError error = DBUS_ERROR_INIT;
+    DBusMessage *reply;
+
+    ctx = talloc_new(NULL);
+    client = test_dbus_setup_mock(ctx, NULL, pilot_test_server_init, NULL);
+
+    reply = test_dbus_call_sync(client,
+                                "/test/leela",
+                                PILOT_IFACE,
+                                PILOT_CRASH,
+                                &error,
+                                DBUS_TYPE_INVALID); /* bad agruments */
+    ck_assert(reply == NULL);
+    ck_assert(dbus_error_is_set(&error));
+    ck_assert(dbus_error_has_name(&error, DBUS_ERROR_INVALID_ARGS));
+    dbus_error_free(&error);
+
+    talloc_free(ctx);
+#endif /* HAVE_DBUSBASICVALUE */
+}
+END_TEST
+
 START_TEST(test_introspection)
 {
     TALLOC_CTX *ctx;
@@ -376,6 +421,7 @@ TCase *create_sbus_tests(void)
     tcase_add_test(tc, test_raw_handler);
     tcase_add_test(tc, test_request_parse_ok);
     tcase_add_test(tc, test_request_parse_bad_args);
+    tcase_add_test(tc, test_request_dontcrash);
     tcase_add_test(tc, test_introspection);
     tcase_add_test(tc, test_sbus_new_error);
 
