@@ -29,6 +29,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <stdlib.h>
+#include <ctype.h>
 
 #include "util/util.h"
 #include "util/sss_utf8.h"
@@ -405,10 +406,39 @@ START_TEST(test_sss_filter_sanitize)
                 "Expected [%s], got [%s]",
                 has_all_expected, sanitized);
 
+    /* Input is reused from previous test - "\\(user)*name" */
+    const char has_all_allow_asterisk_expected[] = "\\5c\\28user\\29*name";
+    ret = sss_filter_sanitize_ex(test_ctx, has_all, &sanitized, "*");
+    fail_unless(ret == EOK, "has_all error [%d][%s]",
+                ret, strerror(ret));
+    fail_unless(strcmp(has_all_allow_asterisk_expected, sanitized)==0,
+                "Expected [%s], got [%s]",
+                has_all_expected, sanitized);
+
     talloc_free(test_ctx);
 }
 END_TEST
 
+START_TEST(test_fd_nonblocking)
+{
+    int fd;
+    int flags;
+    errno_t ret;
+
+    fd = open("/dev/null", O_RDONLY);
+    fail_unless(fd > 0);
+
+    flags = fcntl(fd, F_GETFL, 0);
+    fail_if(flags & O_NONBLOCK);
+
+    ret = sss_fd_nonblocking(fd);
+    fail_unless(ret == EOK);
+    flags = fcntl(fd, F_GETFL, 0);
+    fail_unless(flags & O_NONBLOCK);
+    close(fd);
+}
+END_TEST
+
 START_TEST(test_size_t_overflow)
 {
     fail_unless(!SIZE_T_OVERFLOW(1, 1), "unexpected overflow");
@@ -563,7 +593,7 @@ void setup_atomicio(void)
     fail_unless(filename != NULL, "strdup failed");
 
     atio_fd = -1;
-    old_umask = umask(077);
+    old_umask = umask(SSS_DFL_X_UMASK);
     ret = mkstemp(filename);
     umask(old_umask);
     fail_unless(ret != -1, "mkstemp failed [%d][%s]", errno, strerror(errno));
@@ -984,7 +1014,7 @@ END_TEST
 START_TEST(test_known_service)
 {
     const char * const * svcs;
-    bool found_nss;
+    bool found_nss = false;
     int i;
 
     /* Just make sure we can't find a bogus service and nss
@@ -993,7 +1023,7 @@ START_TEST(test_known_service)
     svcs = get_known_services();
     for (i = 0; svcs[i]; i++) {
         ck_assert_str_ne(svcs[i], "nosuchservice");
-        if (strcmp(svcs[i], "nss")) {
+        if (strcmp(svcs[i], "nss") == 0) {
             found_nss = true;
         }
     }
@@ -1056,6 +1086,28 @@ START_TEST(test_sss_strerror_err_last)
 }
 END_TEST
 
+START_TEST(test_sss_strerror_string_validation)
+{
+    enum sssd_errors idx;
+    const char *error;
+    size_t len;
+    char last_character;
+
+    for (idx = ERR_BASE; idx < ERR_LAST; ++idx) {
+        error = sss_strerror(idx);
+        fail_if(error == NULL, "sss_strerror returned NULL for valid index");
+
+        len = strlen(error);
+        fail_if(len == 0, "sss_strerror returned empty string");
+
+        last_character = error[len - 1];
+        fail_if(isalpha(last_character) == 0 && last_character != ')',
+                "Error string [%s] must finish with alphabetic character\n",
+                error);
+    }
+}
+END_TEST
+
 Suite *util_suite(void)
 {
     Suite *s = suite_create("util");
@@ -1076,6 +1128,7 @@ Suite *util_suite(void)
     tcase_add_test (tc_util, test_check_ipv6_addr);
     tcase_add_test (tc_util, test_is_host_in_domain);
     tcase_add_test (tc_util, test_known_service);
+    tcase_add_test (tc_util, test_fd_nonblocking);
     tcase_set_timeout(tc_util, 60);
 
     TCase *tc_utf8 = tcase_create("utf8");
@@ -1110,6 +1163,7 @@ Suite *util_suite(void)
 
     TCase *tc_sss_strerror = tcase_create("sss_strerror");
     tcase_add_test(tc_sss_strerror, test_sss_strerror_err_last);
+    tcase_add_test(tc_sss_strerror, test_sss_strerror_string_validation);
 
     suite_add_tcase (s, tc_util);
     suite_add_tcase (s, tc_utf8);
