@@ -62,7 +62,7 @@ struct seed_ctx {
 
 static int seed_prompt(const char *req)
 {
-    size_t len = 0;
+    ssize_t len = 0;
     size_t i = 0;
     char *prompt = NULL;
     int ret = EOK;
@@ -523,7 +523,7 @@ static int seed_init(TALLOC_CTX *mem_ctx,
         BAD_POPT_PARAMS(pc, poptStrerror(ret), ret, fini);
     }
 
-    DEBUG_INIT(pc_debug);
+    DEBUG_CLI_INIT(pc_debug);
 
     CHECK_ROOT(ret, argv[0]);
 
@@ -653,7 +653,6 @@ done:
 
 static int seed_domain_user_info(const char *name,
                                  const char *domain_name,
-                                 struct sysdb_ctx *sysdb,
                                  struct sss_domain_info *domain,
                                  bool *is_cached)
 {
@@ -685,7 +684,7 @@ static int seed_domain_user_info(const char *name,
     }
 
     /* look for user in cache */
-    ret = sysdb_getpwnam(tmp_ctx, sysdb, domain, name, &res);
+    ret = sysdb_getpwnam(tmp_ctx, domain, name, &res);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Couldn't lookup user (%s) in the cache\n", name);
@@ -740,7 +739,7 @@ static int seed_cache_user(struct seed_ctx *sctx)
     in_transaction = true;
 
     if (sctx->user_cached == false) {
-        ret = sysdb_add_user(sctx->sysdb, sctx->domain, sctx->uctx->name,
+        ret = sysdb_add_user(sctx->domain, sctx->uctx->name,
                              sctx->uctx->uid, sctx->uctx->gid,
                              sctx->uctx->gecos, sctx->uctx->home,
                              sctx->uctx->shell, NULL, NULL, 0, 0);
@@ -753,7 +752,7 @@ static int seed_cache_user(struct seed_ctx *sctx)
         }
     }
 
-    ret = sysdb_cache_password(sctx->sysdb, sctx->domain, sctx->uctx->name,
+    ret = sysdb_cache_password(sctx->domain, sctx->uctx->name,
                                sctx->uctx->password);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Failed to cache password. (%d)[%s]\n",
@@ -805,7 +804,7 @@ int main(int argc, const char **argv)
 
     /* get user info from domain */
     ret = seed_domain_user_info(sctx->uctx->name, sctx->uctx->domain_name,
-                                sctx->sysdb, sctx->domain, &sctx->user_cached);
+                                sctx->domain, &sctx->user_cached);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Failed lookup of user [%s] in domain [%s]\n",
                                   sctx->uctx->name, sctx->uctx->domain_name);
