@@ -35,25 +35,108 @@
 #define DFL_SKEL_DIR       "/etc/skel"
 #define DFL_MAIL_DIR       "/var/spool/mail"
 
+#define ATTR_NAME_SEP      '='
+#define ATTR_VAL_SEP       ','
 
 #define VAR_CHECK(var, val, attr, msg) do { \
         if (var != (val)) { \
-            DEBUG(SSSDBG_CRIT_FAILURE, msg" attribute: %s", attr); \
+            DEBUG(SSSDBG_CRIT_FAILURE, msg" attribute: %s\n", attr); \
             return val; \
         } \
 } while(0)
 
-struct sync_op_res {
-    struct ops_ctx *data;
-    int error;
-    bool done;
-};
+static int attr_name_val_split(TALLOC_CTX *mem_ctx, const char *nameval,
+                               char **_name, char ***_values, int *_nvals)
+{
+    char *name;
+    char **values;
+    const char *vals;
+    int nvals;
+    TALLOC_CTX *tmp_ctx;
+    errno_t ret;
 
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) return ENOMEM;
+
+    vals = strchr(nameval, ATTR_NAME_SEP);
+    if (vals == NULL) {
+        ret = EINVAL;
+        goto done;
+    }
+
+    name = talloc_strndup(tmp_ctx, nameval, vals-nameval);
+    if (name == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+    vals++;
+
+    ret = split_on_separator(tmp_ctx, vals, ATTR_VAL_SEP, true, true,
+                             &values, &nvals);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    *_name = talloc_steal(mem_ctx, name);
+    *_values = talloc_steal(mem_ctx, values);
+    *_nvals = nvals;
+    ret = EOK;
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
+static int attr_op(struct ops_ctx *octx, const char *nameval, int op)
+{
+    TALLOC_CTX *tmp_ctx;
+    errno_t ret;
+    struct sysdb_attrs *attrs;
+    char *name;
+    char **vals;
+    int nvals;
+    int i;
+
+    switch(op) {
+    case SYSDB_MOD_ADD:
+    case SYSDB_MOD_DEL:
+    case SYSDB_MOD_REP:
+        break;
+    default:
+        return EINVAL;
+    }
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) return ENOMEM;
+
+    attrs = sysdb_new_attrs(tmp_ctx);
+    if (attrs == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = attr_name_val_split(tmp_ctx, nameval, &name, &vals, &nvals);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    for (i=0; i < nvals; i++) {
+        ret = sysdb_attrs_add_string(attrs, name, vals[i]);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  "Could not add %s to %s\n", vals[i], name);
+            continue;
+        }
+    }
+
+    ret = sysdb_set_user_attr(octx->domain, octx->name, attrs, op);
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
 /*
  * Generic modify groups member
  */
-static int mod_groups_member(struct sysdb_ctx *sysdb,
-                             struct sss_domain_info *dom,
+static int mod_groups_member(struct sss_domain_info *dom,
                              char **grouplist,
                              struct ldb_dn *member_dn,
                              int optype)
@@ -71,13 +154,13 @@ static int mod_groups_member(struct sysdb_ctx *sysdb,
 /* FIXME: add transaction around loop */
     for (i = 0; grouplist[i]; i++) {
 
-        parent_dn = sysdb_group_dn(sysdb, tmpctx, dom, grouplist[i]);
+        parent_dn = sysdb_group_dn(tmpctx, dom, grouplist[i]);
         if (!parent_dn) {
             ret = ENOMEM;
             goto done;
         }
 
-        ret = sysdb_mod_group_member(sysdb, member_dn, parent_dn, optype);
+        ret = sysdb_mod_group_member(dom, member_dn, parent_dn, optype);
         if (ret) {
             goto done;
         }
@@ -90,11 +173,11 @@ done:
     return ret;
 }
 
-#define add_to_groups(sysdb, data, member_dn) \
-    mod_groups_member(sysdb, data->domain, data->addgroups, member_dn, \
+#define add_to_groups(data, member_dn) \
+    mod_groups_member(data->domain, data->addgroups, member_dn, \
                       LDB_FLAG_MOD_ADD)
-#define remove_from_groups(sysdb, data, member_dn) \
-    mod_groups_member(sysdb, data->domain, data->rmgroups, member_dn, \
+#define remove_from_groups(data, member_dn) \
+    mod_groups_member(data->domain, data->rmgroups, member_dn, \
                       LDB_FLAG_MOD_DELETE)
 
 /*
@@ -191,7 +274,6 @@ static int usermod_build_attrs(TALLOC_CTX *mem_ctx,
  * Public interface for modifying users
  */
 int usermod(TALLOC_CTX *mem_ctx,
-            struct sysdb_ctx *sysdb,
             struct ops_ctx *data)
 {
     struct sysdb_attrs *attrs = NULL;
@@ -199,7 +281,7 @@ int usermod(TALLOC_CTX *mem_ctx,
     int ret;
 
     if (data->addgroups || data->rmgroups) {
-        member_dn = sysdb_user_dn(sysdb, mem_ctx, data->domain, data->name);
+        member_dn = sysdb_user_dn(mem_ctx, data->domain, data->name);
         if (!member_dn) {
             return ENOMEM;
         }
@@ -218,7 +300,7 @@ int usermod(TALLOC_CTX *mem_ctx,
     }
 
     if (attrs->num != 0) {
-        ret = sysdb_set_user_attr(sysdb, data->domain, data->name,
+        ret = sysdb_set_user_attr(data->domain, data->name,
                                   attrs, SYSDB_MOD_REP);
         if (ret) {
             return ret;
@@ -226,14 +308,36 @@ int usermod(TALLOC_CTX *mem_ctx,
     }
 
     if (data->rmgroups != NULL) {
-        ret = remove_from_groups(sysdb, data, member_dn);
+        ret = remove_from_groups(data, member_dn);
         if (ret) {
             return ret;
         }
     }
 
     if (data->addgroups != NULL) {
-        ret = add_to_groups(sysdb, data, member_dn);
+        ret = add_to_groups(data, member_dn);
+        if (ret) {
+            return ret;
+        }
+    }
+
+    if (data->addattr) {
+        ret = attr_op(data, data->addattr, SYSDB_MOD_ADD);
+        if (ret) {
+            return ret;
+        }
+    }
+
+    if (data->setattr) {
+        ret = attr_op(data, data->setattr, SYSDB_MOD_REP);
+        if (ret) {
+            return ret;
+        }
+
+    }
+
+    if (data->delattr) {
+        ret = attr_op(data, data->delattr, SYSDB_MOD_DEL);
         if (ret) {
             return ret;
         }
@@ -249,7 +353,6 @@ int usermod(TALLOC_CTX *mem_ctx,
  * Public interface for modifying groups
  */
 int groupmod(TALLOC_CTX *mem_ctx,
-             struct sysdb_ctx *sysdb,
              struct ops_ctx *data)
 {
     struct sysdb_attrs *attrs = NULL;
@@ -257,7 +360,7 @@ int groupmod(TALLOC_CTX *mem_ctx,
     int ret;
 
     if (data->addgroups || data->rmgroups) {
-        member_dn = sysdb_group_dn(sysdb, mem_ctx, data->domain, data->name);
+        member_dn = sysdb_group_dn(mem_ctx, data->domain, data->name);
         if (!member_dn) {
             return ENOMEM;
         }
@@ -273,7 +376,7 @@ int groupmod(TALLOC_CTX *mem_ctx,
             return ret;
         }
 
-        ret = sysdb_set_group_attr(sysdb, data->domain, data->name,
+        ret = sysdb_set_group_attr(data->domain, data->name,
                                    attrs, SYSDB_MOD_REP);
         if (ret) {
             return ret;
@@ -281,14 +384,14 @@ int groupmod(TALLOC_CTX *mem_ctx,
     }
 
     if (data->rmgroups != NULL) {
-        ret = remove_from_groups(sysdb, data, member_dn);
+        ret = remove_from_groups(data, member_dn);
         if (ret) {
             return ret;
         }
     }
 
     if (data->addgroups != NULL) {
-        ret = add_to_groups(sysdb, data, member_dn);
+        ret = add_to_groups(data, member_dn);
         if (ret) {
             return ret;
         }
@@ -464,13 +567,11 @@ done:
  * Public interface for adding users
  */
 int useradd(TALLOC_CTX *mem_ctx,
-            struct sysdb_ctx *sysdb,
             struct ops_ctx *data)
 {
     int ret;
 
-    ret = sysdb_add_user(sysdb,
-                         data->domain, data->name, data->uid, data->gid,
+    ret = sysdb_add_user(data->domain, data->name, data->uid, data->gid,
                          data->gecos, data->home, data->shell,
                          NULL, NULL, 0, 0);
     if (ret) {
@@ -480,13 +581,13 @@ int useradd(TALLOC_CTX *mem_ctx,
     if (data->addgroups) {
         struct ldb_dn *member_dn;
 
-        member_dn = sysdb_user_dn(sysdb, mem_ctx, data->domain, data->name);
+        member_dn = sysdb_user_dn(mem_ctx, data->domain, data->name);
         if (!member_dn) {
             ret = ENOMEM;
             goto done;
         }
 
-        ret = add_to_groups(sysdb, data, member_dn);
+        ret = add_to_groups(data, member_dn);
         if (ret) {
             goto done;
         }
@@ -509,7 +610,7 @@ int userdel(TALLOC_CTX *mem_ctx,
     struct ldb_dn *user_dn;
     int ret;
 
-    user_dn = sysdb_user_dn(sysdb, mem_ctx, data->domain, data->name);
+    user_dn = sysdb_user_dn(mem_ctx, data->domain, data->name);
     if (!user_dn) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Could not construct a user DN\n");
         return ENOMEM;
@@ -530,13 +631,11 @@ int userdel(TALLOC_CTX *mem_ctx,
 /*
  * Public interface for adding groups
  */
-int groupadd(struct sysdb_ctx *sysdb,
-             struct ops_ctx *data)
+int groupadd(struct ops_ctx *data)
 {
     int ret;
 
-    ret = sysdb_add_group(sysdb, data->domain,
-                          data->name, data->gid, NULL, 0, 0);
+    ret = sysdb_add_group(data->domain, data->name, data->gid, NULL, 0, 0);
     if (ret == EOK) {
         flush_nscd_cache(NSCD_DB_GROUP);
     }
@@ -553,7 +652,7 @@ int groupdel(TALLOC_CTX *mem_ctx,
     struct ldb_dn *group_dn;
     int ret;
 
-    group_dn = sysdb_group_dn(sysdb, mem_ctx, data->domain, data->name);
+    group_dn = sysdb_group_dn(mem_ctx, data->domain, data->name);
     if (group_dn == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Could not construct a group DN\n");
         return ENOMEM;
@@ -574,7 +673,6 @@ int groupdel(TALLOC_CTX *mem_ctx,
  * getpwnam, getgrnam and friends
  */
 int sysdb_getpwnam_sync(TALLOC_CTX *mem_ctx,
-                        struct sysdb_ctx *sysdb,
                         const char *name,
                         struct ops_ctx *out)
 {
@@ -582,7 +680,7 @@ int sysdb_getpwnam_sync(TALLOC_CTX *mem_ctx,
     const char *str;
     int ret;
 
-    ret = sysdb_getpwnam(mem_ctx, sysdb, out->domain, name, &res);
+    ret = sysdb_getpwnam(mem_ctx, out->domain, name, &res);
     if (ret) {
         return ret;
     }
@@ -648,7 +746,6 @@ int sysdb_getpwnam_sync(TALLOC_CTX *mem_ctx,
 }
 
 int sysdb_getgrnam_sync(TALLOC_CTX *mem_ctx,
-                        struct sysdb_ctx *sysdb,
                         const char *name,
                         struct ops_ctx *out)
 {
@@ -656,7 +753,7 @@ int sysdb_getgrnam_sync(TALLOC_CTX *mem_ctx,
     const char *str;
     int ret;
 
-    ret = sysdb_getgrnam(mem_ctx, sysdb, out->domain, name, &res);
+    ret = sysdb_getgrnam(mem_ctx, out->domain, name, &res);
     if (ret) {
         return ret;
     }
