@@ -227,7 +227,6 @@ int check_group_names(struct tools_ctx *tctx,
     ret = EOK;
     for (i=0; grouplist[i]; ++i) {
         ret = sysdb_getgrnam_sync(tctx,
-                                  tctx->sysdb,
                                   grouplist[i],
                                   groupinfo);
         if (ret) {
@@ -260,7 +259,9 @@ int set_locale(void)
 
     c = setlocale(LC_ALL, "");
     if (c == NULL) {
-        return EIO;
+        /* If setlocale fails, continue with the default
+         * locale. */
+        DEBUG(SSSDBG_MINOR_FAILURE, "Unable to set locale\n");
     }
 
     errno = 0;
@@ -325,7 +326,7 @@ fini:
  * Check is path is owned by uid
  * returns  0 - owns
  *         -1 - does not own
- *         >0 - an error occured, error code
+ *         >0 - an error occurred, error code
  */
 static int is_owner(uid_t uid, const char *path)
 {
