@@ -46,6 +46,13 @@ struct sss_child_ctx {
     struct sss_sigchild_ctx *sigchld_ctx;
 };
 
+static void sss_child_handler(struct tevent_context *ev,
+                              struct tevent_signal *se,
+                              int signum,
+                              int count,
+                              void *siginfo,
+                              void *private_data);
+
 errno_t sss_sigchld_init(TALLOC_CTX *mem_ctx,
                          struct tevent_context *ev,
                          struct sss_sigchild_ctx **child_ctx)
@@ -177,12 +184,12 @@ static void sss_child_invoke_cb(struct tevent_context *ev,
     talloc_free(imm);
 }
 
-void sss_child_handler(struct tevent_context *ev,
-                       struct tevent_signal *se,
-                       int signum,
-                       int count,
-                       void *siginfo,
-                       void *private_data)
+static void sss_child_handler(struct tevent_context *ev,
+                              struct tevent_signal *se,
+                              int signum,
+                              int count,
+                              void *siginfo,
+                              void *private_data)
 {
     struct sss_sigchild_ctx *sigchld_ctx;
     struct tevent_immediate *imm;
@@ -262,6 +269,10 @@ struct sss_child_ctx_old {
     void *pvt;
 };
 
+static void child_sig_handler(struct tevent_context *ev,
+                              struct tevent_signal *sige, int signum,
+                              int count, void *__siginfo, void *pvt);
+
 int child_handler_setup(struct tevent_context *ev, int pid,
                         sss_child_callback_t cb, void *pvt,
                         struct sss_child_ctx_old **_child_ctx)
@@ -507,35 +518,12 @@ int read_pipe_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
-/* The pipes to communicate with the child must be nonblocking */
-void fd_nonblocking(int fd)
-{
-    int flags;
-    int ret;
-
-    flags = fcntl(fd, F_GETFL, 0);
-    if (flags == -1) {
-        ret = errno;
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "F_GETFL failed [%d][%s].\n", ret, strerror(ret));
-        return;
-    }
-
-    if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
-        ret = errno;
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "F_SETFL failed [%d][%s].\n", ret, strerror(ret));
-    }
-
-    return;
-}
-
 static void child_invoke_callback(struct tevent_context *ev,
                                   struct tevent_immediate *imm,
                                   void *pvt);
-void child_sig_handler(struct tevent_context *ev,
-                       struct tevent_signal *sige, int signum,
-                       int count, void *__siginfo, void *pvt)
+static void child_sig_handler(struct tevent_context *ev,
+                              struct tevent_signal *sige, int signum,
+                              int count, void *__siginfo, void *pvt)
 {
     int ret, err;
     struct sss_child_ctx_old *child_ctx;
@@ -580,7 +568,7 @@ void child_sig_handler(struct tevent_context *ev,
                       "child [%d] was stopped by signal [%d].\n", ret,
                           WSTOPSIG(child_ctx->child_status));
             }
-            if (WIFCONTINUED(child_ctx->child_status)) {
+            if (WIFCONTINUED(child_ctx->child_status) == true) {
                 DEBUG(SSSDBG_TRACE_LIBS,
                       "child [%d] was resumed by delivery of SIGCONT.\n",
                           ret);
@@ -624,6 +612,7 @@ static errno_t prepare_child_argv(TALLOC_CTX *mem_ctx,
                                   int child_debug_fd,
                                   const char *binary,
                                   const char *extra_argv[],
+                                  bool extra_args_only,
                                   char ***_argv)
 {
     /*
@@ -631,18 +620,24 @@ static errno_t prepare_child_argv(TALLOC_CTX *mem_ctx,
      * debug_microseconds and NULL
      */
     uint_t argc = 5;
-    char ** argv;
+    char ** argv = NULL;
     errno_t ret = EINVAL;
     size_t i;
 
+    if (extra_args_only) {
+        argc = 2; /* program name and NULL */
+    }
+
     /* Save the current state in case an interrupt changes it */
     bool child_debug_to_file = debug_to_file;
     bool child_debug_timestamps = debug_timestamps;
     bool child_debug_microseconds = debug_microseconds;
     bool child_debug_stderr = debug_to_stderr;
 
-    if (child_debug_to_file) argc++;
-    if (child_debug_stderr) argc++;
+    if (!extra_args_only) {
+        if (child_debug_to_file) argc++;
+        if (child_debug_stderr) argc++;
+    }
 
     if (extra_argv) {
         for (i = 0; extra_argv[i]; i++) argc++;
@@ -671,42 +666,44 @@ static errno_t prepare_child_argv(TALLOC_CTX *mem_ctx,
         }
     }
 
-    argv[--argc] = talloc_asprintf(argv, "--debug-level=%#.4x",
-                              debug_level);
-    if (argv[argc] == NULL) {
-        ret = ENOMEM;
-        goto fail;
-    }
-
-    if (child_debug_stderr) {
-        argv[--argc] = talloc_strdup(argv, "--debug-to-stderr");
+    if (!extra_args_only) {
+        argv[--argc] = talloc_asprintf(argv, "--debug-level=%#.4x",
+                                  debug_level);
         if (argv[argc] == NULL) {
             ret = ENOMEM;
             goto fail;
         }
-    }
 
-    if (child_debug_to_file) {
-        argv[--argc] = talloc_asprintf(argv, "--debug-fd=%d",
-                                       child_debug_fd);
+        if (child_debug_stderr) {
+            argv[--argc] = talloc_strdup(argv, "--debug-to-stderr");
+            if (argv[argc] == NULL) {
+                ret = ENOMEM;
+                goto fail;
+            }
+        }
+
+        if (child_debug_to_file) {
+            argv[--argc] = talloc_asprintf(argv, "--debug-fd=%d",
+                                           child_debug_fd);
+            if (argv[argc] == NULL) {
+                ret = ENOMEM;
+                goto fail;
+            }
+        }
+
+        argv[--argc] = talloc_asprintf(argv, "--debug-timestamps=%d",
+                                       child_debug_timestamps);
         if (argv[argc] == NULL) {
             ret = ENOMEM;
             goto fail;
         }
-    }
 
-    argv[--argc] = talloc_asprintf(argv, "--debug-timestamps=%d",
-                                   child_debug_timestamps);
-    if (argv[argc] == NULL) {
-        ret = ENOMEM;
-        goto fail;
-    }
-
-    argv[--argc] = talloc_asprintf(argv, "--debug-microseconds=%d",
-                                       child_debug_microseconds);
-    if (argv[argc] == NULL) {
-        ret = ENOMEM;
-        goto fail;
+        argv[--argc] = talloc_asprintf(argv, "--debug-microseconds=%d",
+                                           child_debug_microseconds);
+        if (argv[argc] == NULL) {
+            ret = ENOMEM;
+            goto fail;
+        }
     }
 
     argv[--argc] = talloc_strdup(argv, binary);
@@ -732,7 +729,7 @@ fail:
 errno_t exec_child_ex(TALLOC_CTX *mem_ctx,
                       int *pipefd_to_child, int *pipefd_from_child,
                       const char *binary, int debug_fd,
-                      const char *extra_argv[],
+                      const char *extra_argv[], bool extra_args_only,
                       int child_in_fd, int child_out_fd)
 {
     int ret;
@@ -758,7 +755,7 @@ errno_t exec_child_ex(TALLOC_CTX *mem_ctx,
     }
 
     ret = prepare_child_argv(mem_ctx, debug_fd,
-                             binary, extra_argv,
+                             binary, extra_argv, extra_args_only,
                              &argv);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "prepare_child_argv.\n");
@@ -776,32 +773,10 @@ errno_t exec_child(TALLOC_CTX *mem_ctx,
                    const char *binary, int debug_fd)
 {
     return exec_child_ex(mem_ctx, pipefd_to_child, pipefd_from_child,
-                         binary, debug_fd, NULL,
+                         binary, debug_fd, NULL, false,
                          STDIN_FILENO, STDOUT_FILENO);
 }
 
-void child_cleanup(int readfd, int writefd)
-{
-    int ret;
-
-    if (readfd != -1) {
-        ret = close(readfd);
-        if (ret != EOK) {
-            ret = errno;
-            DEBUG(SSSDBG_CRIT_FAILURE,
-                  "close failed [%d][%s].\n", ret, strerror(ret));
-        }
-    }
-    if (writefd != -1) {
-        ret = close(writefd);
-        if (ret != EOK) {
-            ret = errno;
-            DEBUG(SSSDBG_CRIT_FAILURE,
-                  "close failed [%d][%s].\n", ret, strerror(ret));
-        }
-    }
-}
-
 int child_io_destructor(void *ptr)
 {
     int ret;
