@@ -68,13 +68,6 @@ errno_t sss_child_register(TALLOC_CTX *mem_ctx,
                            void *pvt,
                            struct sss_child_ctx **child_ctx);
 
-void sss_child_handler(struct tevent_context *ev,
-                       struct tevent_signal *se,
-                       int signum,
-                       int count,
-                       void *siginfo,
-                       void *private_data);
-
 /* Callback to be invoked when a sigchld handler is called.
  * The tevent_signal * associated with the handler will be
  * freed automatically when this function returns.
@@ -107,15 +100,11 @@ int read_pipe_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 /* The pipes to communicate with the child must be nonblocking */
 void fd_nonblocking(int fd);
 
-void child_sig_handler(struct tevent_context *ev,
-                       struct tevent_signal *sige, int signum,
-                       int count, void *__siginfo, void *pvt);
-
 /* Never returns EOK, ether returns an error, or doesn't return on success */
 errno_t exec_child_ex(TALLOC_CTX *mem_ctx,
                       int *pipefd_to_child, int *pipefd_from_child,
                       const char *binary, int debug_fd,
-                      const char *extra_argv[],
+                      const char *extra_argv[], bool extra_args_only,
                       int child_in_fd, int child_out_fd);
 
 /* Same as exec_child_ex() except child_in_fd is set to STDIN_FILENO and
@@ -125,8 +114,6 @@ errno_t exec_child(TALLOC_CTX *mem_ctx,
                    int *pipefd_to_child, int *pipefd_from_child,
                    const char *binary, int debug_fd);
 
-void child_cleanup(int readfd, int writefd);
-
 int child_io_destructor(void *ptr);
 
 errno_t child_debug_init(const char *logfile, int *debug_fd);
