@@ -19,6 +19,14 @@ unsigned char *sss_base64_decode(TALLOC_CTX *mem_ctx,
                                  const char *in,
                                  size_t *outsize);
 
+#define SSS_SHA1_LENGTH 20
+
+int sss_hmac_sha1(const unsigned char *key,
+                  size_t key_len,
+                  const unsigned char *in,
+                  size_t in_len,
+                  unsigned char *out);
+
 int sss_password_encrypt(TALLOC_CTX *mem_ctx, const char *password, int plen,
                          enum obfmethod meth, char **obfpwd);
 
