@@ -35,9 +35,11 @@ struct sss_domain_info *get_domains_head(struct sss_domain_info *domain)
 }
 
 struct sss_domain_info *get_next_domain(struct sss_domain_info *domain,
-                                        bool descend)
+                                        uint32_t gnd_flags)
 {
     struct sss_domain_info *dom;
+    bool descend = gnd_flags & SSS_GND_DESCEND;
+    bool include_disabled = gnd_flags & SSS_GND_INCLUDE_DISABLED;
 
     dom = domain;
     while (dom) {
@@ -50,7 +52,16 @@ struct sss_domain_info *get_next_domain(struct sss_domain_info *domain,
         } else {
             dom = NULL;
         }
-        if (dom && !dom->disabled) break;
+
+        if (dom) {
+            if (sss_domain_get_state(dom) == DOM_DISABLED
+                    && !include_disabled) {
+                continue;
+            } else {
+                /* Next domain found. */
+                break;
+            }
+        }
     }
 
     return dom;
@@ -91,8 +102,8 @@ struct sss_domain_info *find_domain_by_name(struct sss_domain_info *domain,
         return NULL;
     }
 
-    while (dom && dom->disabled) {
-        dom = get_next_domain(dom, true);
+    while (dom && sss_domain_get_state(dom) == DOM_DISABLED) {
+        dom = get_next_domain(dom, SSS_GND_DESCEND);
     }
     while (dom) {
         if (strcasecmp(dom->name, name) == 0 ||
@@ -100,7 +111,7 @@ struct sss_domain_info *find_domain_by_name(struct sss_domain_info *domain,
              (strcasecmp(dom->flat_name, name) == 0))) {
             return dom;
         }
-        dom = get_next_domain(dom, true);
+        dom = get_next_domain(dom, SSS_GND_DESCEND);
     }
 
     return NULL;
@@ -119,8 +130,8 @@ struct sss_domain_info *find_domain_by_sid(struct sss_domain_info *domain,
 
     sid_len = strlen(sid);
 
-    while (dom && dom->disabled) {
-        dom = get_next_domain(dom, true);
+    while (dom && sss_domain_get_state(dom) == DOM_DISABLED) {
+        dom = get_next_domain(dom, SSS_GND_DESCEND);
     }
 
     while (dom) {
@@ -141,7 +152,7 @@ struct sss_domain_info *find_domain_by_sid(struct sss_domain_info *domain,
             }
         }
 
-        dom = get_next_domain(dom, true);
+        dom = get_next_domain(dom, SSS_GND_DESCEND);
     }
 
     return NULL;
@@ -195,135 +206,6 @@ done:
     return dom;
 }
 
-struct sss_domain_info *new_subdomain(TALLOC_CTX *mem_ctx,
-                                      struct sss_domain_info *parent,
-                                      const char *name,
-                                      const char *realm,
-                                      const char *flat_name,
-                                      const char *id,
-                                      bool mpg,
-                                      bool enumerate,
-                                      const char *forest)
-{
-    struct sss_domain_info *dom;
-    bool inherit_option;
-
-    DEBUG(SSSDBG_TRACE_FUNC,
-          "Creating [%s] as subdomain of [%s]!\n", name, parent->name);
-
-    dom = talloc_zero(mem_ctx, struct sss_domain_info);
-    if (dom == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "talloc_zero failed.\n");
-        return NULL;
-    }
-
-    dom->parent = parent;
-
-    /* Sub-domains always have the same view as the parent */
-    dom->has_views = parent->has_views;
-    if (parent->view_name != NULL) {
-        dom->view_name = talloc_strdup(dom, parent->view_name);
-        if (dom->view_name == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "Failed to copy parent's view name.\n");
-            goto fail;
-        }
-    }
-
-    dom->name = talloc_strdup(dom, name);
-    if (dom->name == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "Failed to copy domain name.\n");
-        goto fail;
-    }
-
-    dom->provider = talloc_strdup(dom, parent->provider);
-    if (dom->provider == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "Failed to copy provider name.\n");
-        goto fail;
-    }
-
-    dom->conn_name = talloc_strdup(dom, parent->conn_name);
-    if (dom->conn_name == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "Failed to copy connection name.\n");
-        goto fail;
-    }
-
-    if (realm != NULL) {
-        dom->realm = talloc_strdup(dom, realm);
-        if (dom->realm == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "Failed to copy realm name.\n");
-            goto fail;
-        }
-    }
-
-    if (flat_name != NULL) {
-        dom->flat_name = talloc_strdup(dom, flat_name);
-        if (dom->flat_name == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "Failed to copy flat name.\n");
-            goto fail;
-        }
-    }
-
-    if (id != NULL) {
-        dom->domain_id = talloc_strdup(dom, id);
-        if (dom->domain_id == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "Failed to copy id.\n");
-            goto fail;
-        }
-    }
-
-    if (forest != NULL) {
-        dom->forest = talloc_strdup(dom, forest);
-        if (dom->forest == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "Failed to copy forest.\n");
-            goto fail;
-        }
-    }
-
-    dom->enumerate = enumerate;
-    dom->fqnames = true;
-    dom->mpg = mpg;
-    /* If the parent domain filters out group members, the subdomain should
-     * as well if configured */
-    inherit_option = string_in_list(CONFDB_DOMAIN_IGNORE_GROUP_MEMBERS,
-                                    parent->sd_inherit, false);
-    if (inherit_option) {
-        dom->ignore_group_members = parent->ignore_group_members;
-    }
-
-    /* If the parent domain explicitly limits ID ranges, the subdomain
-     * should honour the limits as well.
-     */
-    dom->id_min = parent->id_min ? parent->id_min : 0;
-    dom->id_max = parent->id_max ? parent->id_max : 0xffffffff;
-    dom->pwd_expiration_warning = parent->pwd_expiration_warning;
-    dom->cache_credentials = parent->cache_credentials;
-    dom->case_sensitive = false;
-    dom->user_timeout = parent->user_timeout;
-    dom->group_timeout = parent->group_timeout;
-    dom->netgroup_timeout = parent->netgroup_timeout;
-    dom->service_timeout = parent->service_timeout;
-    dom->names = parent->names;
-
-    dom->override_homedir = parent->override_homedir;
-    dom->fallback_homedir = parent->fallback_homedir;
-    dom->subdomain_homedir = parent->subdomain_homedir;
-    dom->override_shell = parent->override_shell;
-    dom->default_shell = parent->default_shell;
-    dom->homedir_substr = parent->homedir_substr;
-
-    if (parent->sysdb == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "Missing sysdb context in parent domain.\n");
-        goto fail;
-    }
-    dom->sysdb = parent->sysdb;
-
-    return dom;
-
-fail:
-    talloc_free(dom);
-    return NULL;
-}
-
 errno_t sssd_domain_init(TALLOC_CTX *mem_ctx,
                          struct confdb_ctx *cdb,
                          const char *domain_name,
@@ -438,13 +320,13 @@ sss_write_domain_mappings(struct sss_domain_info *domain)
         goto done;
     }
 
-    old_mode = umask(077);
+    old_mode = umask(SSS_DFL_X_UMASK);
     fd = mkstemp(tmp_file);
     umask(old_mode);
     if (fd < 0) {
         DEBUG(SSSDBG_OP_FAILURE,
-              "creating the temp file [%s] for domain-realm mappings failed\n",
-              tmp_file);
+              "creating the temp file [%s] for domain-realm mappings "
+              "failed.\n", tmp_file);
         ret = EIO;
         talloc_zfree(tmp_ctx);
         goto done;
@@ -473,9 +355,9 @@ sss_write_domain_mappings(struct sss_domain_info *domain)
         goto done;
     }
 
-    for (dom = get_next_domain(domain, true);
+    for (dom = get_next_domain(domain, SSS_GND_DESCEND);
          dom && IS_SUBDOMAIN(dom); /* if we get back to a parent, stop */
-         dom = get_next_domain(dom, false)) {
+         dom = get_next_domain(dom, 0)) {
         ret = fprintf(fstream, ".%s = %s\n%s = %s\n",
                                dom->name, dom->realm, dom->name, dom->realm);
         if (ret < 0) {
@@ -492,9 +374,9 @@ sss_write_domain_mappings(struct sss_domain_info *domain)
         goto done;
     }
 
-    for (dom = get_next_domain(domain, true);
+    for (dom = get_next_domain(domain, SSS_GND_DESCEND);
             dom && IS_SUBDOMAIN(dom); /* if we get back to a parent, stop */
-            dom = get_next_domain(dom, false)) {
+            dom = get_next_domain(dom, 0)) {
 
         if (dom->forest == NULL) {
             continue;
@@ -608,7 +490,7 @@ errno_t get_dom_names(TALLOC_CTX *mem_ctx,
     dom = start_dom;
     while (dom) {
         count++;
-        dom = get_next_domain(dom, false);
+        dom = get_next_domain(dom, 0);
     }
 
     dom_names = talloc_array(tmp_ctx, char*, count);
@@ -626,7 +508,7 @@ errno_t get_dom_names(TALLOC_CTX *mem_ctx,
             ret = ENOMEM;
             goto done;
         }
-        dom = get_next_domain(dom, false);
+        dom = get_next_domain(dom, 0);
         i++;
     }
 
@@ -688,13 +570,12 @@ static errno_t sss_write_krb5_localauth_snippet(const char *path)
         goto done;
     }
 
-    old_mode = umask(077);
+    old_mode = umask(SSS_DFL_X_UMASK);
     fd = mkstemp(tmp_file);
     umask(old_mode);
     if (fd < 0) {
-        DEBUG(SSSDBG_OP_FAILURE,
-              "creating the temp file [%s] for domain-realm mappings failed\n",
-              tmp_file);
+        DEBUG(SSSDBG_OP_FAILURE, "creating the temp file [%s] for "
+                                 "domain-realm mappings failed.\n", tmp_file);
         ret = EIO;
         talloc_zfree(tmp_ctx);
         goto done;
@@ -860,3 +741,14 @@ done:
 
     return ret;
 }
+
+enum sss_domain_state sss_domain_get_state(struct sss_domain_info *dom)
+{
+    return dom->state;
+}
+
+void sss_domain_set_state(struct sss_domain_info *dom,
+                          enum sss_domain_state state)
+{
+    dom->state = state;
+}
