@@ -295,6 +295,14 @@ done:
     return ret;
 }
 
+int sss_ad_default_names_ctx(TALLOC_CTX *mem_ctx,
+                             struct sss_names_ctx **_out)
+{
+    return sss_names_init_from_args(mem_ctx, IPA_AD_DEFAULT_RE,
+                                    CONFDB_DEFAULT_FULL_NAME_FORMAT,
+                                    _out);
+}
+
 int sss_parse_name(TALLOC_CTX *memctx,
                    struct sss_names_ctx *snctx,
                    const char *orig, char **_domain, char **_name)
@@ -415,7 +423,7 @@ int sss_parse_name_for_domains(TALLOC_CTX *memctx,
     rname = NULL;
     rdomain = NULL;
 
-    for (dom = domains; dom != NULL; dom = get_next_domain(dom, false)) {
+    for (dom = domains; dom != NULL; dom = get_next_domain(dom, 0)) {
         ret = sss_parse_name(tmp_ctx, dom->names, orig, &dmatch, &nmatch);
         if (ret == EOK) {
             /*
@@ -473,7 +481,7 @@ int sss_parse_name_for_domains(TALLOC_CTX *memctx,
                     goto done;
                 }
 
-                for (dom = domains; dom != NULL; dom = get_next_domain(dom, false)) {
+                for (dom = domains; dom != NULL; dom = get_next_domain(dom, 0)) {
                     match = match_any_domain_or_subdomain_name(dom, rdomain);
                     if (match != NULL) {
                         break;
@@ -481,8 +489,7 @@ int sss_parse_name_for_domains(TALLOC_CTX *memctx,
                 }
                 if (match == NULL) {
                     DEBUG(SSSDBG_FUNC_DATA, "default domain [%s] is currently " \
-                                            "not known, trying to look it up.\n",
-                                            rdomain);
+                                            "not known\n", rdomain);
                     *domain = talloc_steal(memctx, rdomain);
                     ret = EAGAIN;
                     goto done;
