@@ -26,6 +26,7 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <string.h>
 #include <strings.h>
 #include <ctype.h>
@@ -50,6 +51,7 @@
 #include "util/util_errors.h"
 #include "util/util_safealign.h"
 #include "util/sss_format.h"
+#include "util/debug.h"
 
 #define _(STRING) gettext (STRING)
 
@@ -57,115 +59,11 @@
 
 #define CLEAR_MC_FLAG "clear_mc_flag"
 
-#ifdef HAVE_FUNCTION_ATTRIBUTE_FORMAT
-#define SSS_ATTRIBUTE_PRINTF(a1, a2) __attribute__ ((format (printf, a1, a2)))
-#else
-#define SSS_ATTRIBUTE_PRINTF(a1, a2)
-#endif
+/** Default secure umask */
+#define SSS_DFL_UMASK 0177
 
-extern const char *debug_prg_name;
-extern int debug_level;
-extern int debug_timestamps;
-extern int debug_microseconds;
-extern int debug_to_file;
-extern int debug_to_stderr;
-extern const char *debug_log_file;
-void debug_fn(const char *file,
-              long line,
-              const char *function,
-              int level,
-              const char *format, ...) SSS_ATTRIBUTE_PRINTF(5,6);
-int debug_convert_old_level(int old_level);
-errno_t set_debug_file_from_fd(const int fd);
-
-#define SSS_DOM_ENV           "_SSS_DOM"
-
-#define SSSDBG_FATAL_FAILURE  0x0010   /* level 0 */
-#define SSSDBG_CRIT_FAILURE   0x0020   /* level 1 */
-#define SSSDBG_OP_FAILURE     0x0040   /* level 2 */
-#define SSSDBG_MINOR_FAILURE  0x0080   /* level 3 */
-#define SSSDBG_CONF_SETTINGS  0x0100   /* level 4 */
-#define SSSDBG_FUNC_DATA      0x0200   /* level 5 */
-#define SSSDBG_TRACE_FUNC     0x0400   /* level 6 */
-#define SSSDBG_TRACE_LIBS     0x1000   /* level 7 */
-#define SSSDBG_TRACE_INTERNAL 0x2000   /* level 8 */
-#define SSSDBG_TRACE_ALL      0x4000   /* level 9 */
-#define SSSDBG_IMPORTANT_INFO SSSDBG_OP_FAILURE
-
-#define SSSDBG_INVALID        -1
-#define SSSDBG_UNRESOLVED     0
-#define SSSDBG_MASK_ALL       0xFFF0   /* enable all debug levels */
-#define SSSDBG_DEFAULT        SSSDBG_FATAL_FAILURE
-
-#define SSSDBG_TIMESTAMP_UNRESOLVED   -1
-#define SSSDBG_TIMESTAMP_DEFAULT       1
-
-#define SSSDBG_MICROSECONDS_UNRESOLVED   -1
-#define SSSDBG_MICROSECONDS_DEFAULT       0
-
-#define SSSD_DEBUG_OPTS \
-        {"debug-level", 'd', POPT_ARG_INT, &debug_level, 0, \
-         _("Debug level"), NULL}, \
-        {"debug-to-files", 'f', POPT_ARG_NONE, &debug_to_file, 0, \
-         _("Send the debug output to files instead of stderr"), NULL }, \
-        {"debug-to-stderr", 0, POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, &debug_to_stderr, 0, \
-         _("Send the debug output to stderr directly."), NULL }, \
-        {"debug-timestamps", 0, POPT_ARG_INT, &debug_timestamps, 0, \
-         _("Add debug timestamps"), NULL}, \
-         {"debug-microseconds", 0, POPT_ARG_INT, &debug_microseconds, 0, \
-          _("Show timestamps with microseconds"), NULL},
-
-/** \def DEBUG(level, format, ...)
-    \brief macro to generate debug messages
-
-    \param level the debug level, please use one of the SSSDBG_* macros
-    \param format the debug message format string, should result in a
-                  newline-terminated message
-    \param ... the debug message format arguments
-*/
-#define DEBUG(level, format, ...) do { \
-    int __debug_macro_level = level; \
-    if (DEBUG_IS_SET(__debug_macro_level)) { \
-        debug_fn(__FILE__, __LINE__, __FUNCTION__, \
-                 __debug_macro_level, \
-                 format, ##__VA_ARGS__); \
-    } \
-} while (0)
-
-/** \def DEBUG_IS_SET(level)
-    \brief checks whether level is set in debug_level
-
-    \param level the debug level, please use one of the SSSDBG*_ macros
-*/
-#define DEBUG_IS_SET(level) (debug_level & (level) || \
-                            (debug_level == SSSDBG_UNRESOLVED && \
-                                            (level & (SSSDBG_FATAL_FAILURE | \
-                                                      SSSDBG_CRIT_FAILURE))))
-
-#define DEBUG_INIT(dbg_lvl) do { \
-    if (dbg_lvl != SSSDBG_INVALID) { \
-        debug_level = debug_convert_old_level(dbg_lvl); \
-    } else { \
-        debug_level = SSSDBG_UNRESOLVED; \
-    } \
-\
-    talloc_set_log_fn(talloc_log_fn); \
-} while (0)
-
-/* CLI tools shall debug to stderr even when SSSD was compiled with journald
- * support
- */
-#define DEBUG_CLI_INIT(dbg_lvl) do { \
-    DEBUG_INIT(dbg_lvl);             \
-    debug_to_stderr = 1;             \
-} while (0)
-
-#define PRINT(fmt, ...) fprintf(stdout, gettext(fmt), ##__VA_ARGS__)
-#define ERROR(fmt, ...) fprintf(stderr, gettext(fmt), ##__VA_ARGS__)
-
-#ifndef discard_const
-#define discard_const(ptr) ((void *)((uintptr_t)(ptr)))
-#endif
+/** Secure mask with executable bit */
+#define SSS_DFL_X_UMASK 0077
 
 #ifndef NULL
 #define NULL 0
@@ -296,6 +194,9 @@ int sss_names_init(TALLOC_CTX *mem_ctx,
                    const char *domain,
                    struct sss_names_ctx **out);
 
+int sss_ad_default_names_ctx(TALLOC_CTX *mem_ctx,
+                             struct sss_names_ctx **_out);
+
 int sss_parse_name(TALLOC_CTX *memctx,
                    struct sss_names_ctx *snctx,
                    const char *orig, char **_domain, char **_name);
@@ -412,6 +313,9 @@ int split_on_separator(TALLOC_CTX *mem_ctx, const char *str,
 
 char **parse_args(const char *str);
 
+errno_t parse_cert_verify_opts(const char *verify_opts, bool *do_ocsp);
+
+
 errno_t sss_hash_create(TALLOC_CTX *mem_ctx,
                         unsigned long count,
                         hash_table_t **tbl);
@@ -444,6 +348,17 @@ errno_t sss_hash_create_ex(TALLOC_CTX *mem_ctx,
 errno_t add_strings_lists(TALLOC_CTX *mem_ctx, const char **l1, const char **l2,
                           bool copy_strings, char ***_new_list);
 
+/**
+ * @brief set file descriptor as nonblocking
+ *
+ * Set the O_NONBLOCK flag for the input fd
+ *
+ * @param[in] fd            The file descriptor to set as nonblocking
+ *
+ * @return                  EOK on success, errno code otherwise
+ */
+errno_t sss_fd_nonblocking(int fd);
+
 /* Copy a NULL-terminated string list
  * Returns NULL on out of memory error or invalid input
  */
@@ -472,6 +387,11 @@ errno_t sss_filter_sanitize(TALLOC_CTX *mem_ctx,
                             const char *input,
                             char **sanitized);
 
+errno_t sss_filter_sanitize_ex(TALLOC_CTX *mem_ctx,
+                               const char *input,
+                               char **sanitized,
+                               const char *ignore);
+
 errno_t sss_filter_sanitize_for_dom(TALLOC_CTX *mem_ctx,
                                     const char *input,
                                     struct sss_domain_info *dom,
@@ -537,13 +457,19 @@ void to_sized_string(struct sized_string *out, const char *in);
 /* from domain_info.c */
 struct sss_domain_info *get_domains_head(struct sss_domain_info *domain);
 
+#define SSS_GND_DESCEND 0x01
+#define SSS_GND_INCLUDE_DISABLED 0x02
+#define SSS_GND_ALL_DOMAINS (SSS_GND_DESCEND | SSS_GND_INCLUDE_DISABLED)
 struct sss_domain_info *get_next_domain(struct sss_domain_info *domain,
-                                        bool descend);
+                                        uint32_t gnd_flags);
 struct sss_domain_info *find_domain_by_name(struct sss_domain_info *domain,
                                             const char *name,
                                             bool match_any);
 struct sss_domain_info *find_domain_by_sid(struct sss_domain_info *domain,
                                            const char *sid);
+enum sss_domain_state sss_domain_get_state(struct sss_domain_info *dom);
+void sss_domain_set_state(struct sss_domain_info *dom,
+                          enum sss_domain_state state);
 
 struct sss_domain_info*
 sss_get_domain_by_sid_ldap_fallback(struct sss_domain_info *domain,
@@ -556,16 +482,6 @@ find_domain_by_object_name(struct sss_domain_info *domain,
 bool subdomain_enumerates(struct sss_domain_info *parent,
                           const char *sd_name);
 
-struct sss_domain_info *new_subdomain(TALLOC_CTX *mem_ctx,
-                                      struct sss_domain_info *parent,
-                                      const char *name,
-                                      const char *realm,
-                                      const char *flat_name,
-                                      const char *id,
-                                      bool mpg,
-                                      bool enumerate,
-                                      const char *forest);
-
 errno_t sssd_domain_init(TALLOC_CTX *mem_ctx,
                          struct confdb_ctx *cdb,
                          const char *domain_name,
@@ -625,6 +541,8 @@ char * sss_reverse_replace_space(TALLOC_CTX *mem_ctx,
 errno_t guid_blob_to_string_buf(const uint8_t *blob, char *str_buf,
                                 size_t buf_size);
 
+const char *get_last_x_chars(const char *str, size_t x);
+
 /* from become_user.c */
 errno_t become_user(uid_t uid, gid_t gid);
 struct sss_creds;
@@ -648,4 +566,25 @@ int get_seuser(TALLOC_CTX *mem_ctx, const char *login_name,
 /* convert time from generalized form to unix time */
 errno_t sss_utc_to_time_t(const char *str, const char *format, time_t *unix_time);
 
+/* Creates a unique file using mkstemp with provided umask. The template
+ * must end with XXXXXX. Returns the fd, sets _err to an errno value on error.
+ *
+ * Prefer using sss_unique_file() as it uses a secure umask internally.
+ */
+int sss_unique_file_ex(TALLOC_CTX *mem_ctx,
+                       char *path_tmpl,
+                       mode_t file_umask,
+                       errno_t *_err);
+int sss_unique_file(TALLOC_CTX *owner,
+                    char *path_tmpl,
+                    errno_t *_err);
+
+/* Creates a unique filename using mkstemp with secure umask. The template
+ * must end with XXXXXX
+ *
+ * path_tmpl must be a talloc context. Destructor would be set on the filename
+ * so that it's guaranteed the file is removed.
+ */
+int sss_unique_filename(TALLOC_CTX *owner, char *path_tmpl);
+
 #endif /* __SSSD_UTIL_H__ */
