@@ -84,6 +84,7 @@ enum sssd_errors {
     ERR_INVALID_EXTRA_ATTR,
     ERR_SBUS_GET_SENDER_ERROR,
     ERR_SBUS_NO_SENDER,
+    ERR_SBUS_INVALID_PATH,
     ERR_NO_SIDS,
     ERR_SBUS_NOSUP,
     ERR_NO_SYSBUS,
@@ -91,9 +92,20 @@ enum sssd_errors {
     ERR_SELINUX_CONTEXT,
     ERR_REGEX_NOMATCH,
     ERR_TIMESPEC_NOT_SUPPORTED,
+    ERR_INVALID_CONFIG,
     ERR_MALFORMED_ENTRY,
     ERR_UNEXPECTED_ENTRY_TYPE,
     ERR_SIMPLE_GROUPS_MISSING,
+    ERR_HOMEDIR_IS_NULL,
+    ERR_TRUST_NOT_SUPPORTED,
+    ERR_IPA_GETKEYTAB_FAILED,
+    ERR_TRUST_FOREST_UNKNOWN,
+    ERR_P11_CHILD,
+    ERR_ADDR_FAMILY_NOT_SUPPORTED,
+    ERR_SBUS_SENDER_BUS,
+    ERR_SUBDOM_INACTIVE,
+    ERR_ACCOUNT_LOCKED,
+    ERR_RENEWAL_CHILD,
     ERR_LAST            /* ALWAYS LAST */
 };
 
